#include <stdio.h>
#include <math.h>

#undef NX
#undef NY

#define NX 20
#define NY 20

typedef double real;

typedef real realmatrix[NX+1][NY+1];

realmatrix UX,UY;
realmatrix UXX,UYY,UXY;
realmatrix SXX,SYY,SXY;
realmatrix FX,FY;

real a = 1;
real b = 1;

int nx = NX;
int ny = NY;

real dx = a/nx;
real dy = b/ny;

real Y = 2;
real nu = 1.0/3;
real lambda = Y*nu/(1-2*nu)/(1+nu);
real mu = Y/2/(1+nu);
real rho0g0 = 1;
real scale = Y/(1-nu*nu)/rho0g0;

real omega = 1;
real eps = omega/(1/dx/dx+1/dy/dy)/Y;

int iters = 100000;
int ask = 1000;
int write = 1000;
int print = 1;
real accuracy = 0;
real precision;

int iter;

real force;
real energy;

char *outname = "elastic.data";

FILE *out;


#define DDXC(A) ((A[ix+1][iy]-A[ix-1][iy])/(2*dx))
#define DDXR(A) ((4*A[ix+1][iy]-A[ix+2][iy]-3*A[ix][iy])/(2*dx))
#define DDXL(A) ((4*A[ix-1][iy]-A[ix-2][iy]-3*A[ix][iy])/(-2*dx))
#define DDX(A) (( ix == 0 )? DDXR(A) : ( ix == nx )? DDXL(A) : DDXC(A))

#define DDYC(A) ((A[ix][iy+1]-A[ix][iy-1])/(2*dy))
#define DDYR(A) ((4*A[ix][iy+1]-A[ix][iy+2]-3*A[ix][iy])/(2*dy))
#define DDYL(A) ((4*A[ix][iy-1]-A[ix][iy-2]-3*A[ix][iy])/(-2*dy))
#define DDY(A)  (( iy == 0 )? DDYR(A) : ( iy == ny )? DDYL(A) : DDYC(A))

//---------------------------------------------------------------------------------

void Initialize()
{
	int ix,iy;

	for( ix = 0; ix <= nx; ix++ )
		for( iy = 0; iy <= ny; iy++ )
			UX[ix][iy] = UY[ix][iy] = 0;
}

//---------------------------------------------------------------------------------

void DoStrain()
{
	int ix,iy;

	// new strains

	for( ix = 0; ix <= nx; ix++ )
		for( iy = 0; iy <= ny; iy++ )
		{
			real duxdx = DDX(UX);
			real duydy = DDY(UY);
			real duydx = DDX(UY);
			real duxdy = DDY(UX);

			UXX[ix][iy] = duxdx;
			UYY[ix][iy] = duydy;
			UXY[ix][iy] = (duydx+duxdy)/2;
		}
}

//---------------------------------------------------------------------------------

void DoStress()
{
	int ix,iy;

	for( ix = 0; ix <= nx; ix++ )
		for( iy = 0; iy <= ny; iy++ )
		{
			SXX[ix][iy] = ( ix == nx )? 0 : (2*mu+lambda)*UXX[ix][iy]+lambda*UYY[ix][iy];
			SYY[ix][iy] = ( iy == ny )? 0 : (2*mu+lambda)*UYY[ix][iy]+lambda*UXX[ix][iy];
			SXY[ix][iy] = ( ix == 0 || iy == 0 || ix == nx || iy == ny )? 0 : 2*mu*UXY[ix][iy];
		}

}

//---------------------------------------------------------------------------------

void DoForce()
{
	int ix,iy;

	// inside


	real force=0;
	for( ix = 0; ix <= nx; ix++ )
		for( iy = 0; iy <= ny; iy++ )
		{
			real dsxxdx = DDX(SXX);
			real dsxydy = DDY(SXY);
			real dsxydx = DDX(SXY);
			real dsyydy = DDY(SYY);

			real fx = dsxxdx+dsxydy;
			real fy = dsxydx+dsyydy-rho0g0;

			FX[ix][iy] = fx;
			FY[ix][iy] = fy;

			real gx = ( ix == 0 || ix == nx )? 0.5 : 1.0;
			real gy = ( iy == 0 || iy == ny )? 0.5 : 1.0;
		
			force += gx*fx*fx+gy*fy*fy;

		}

	precision = sqrt(force/nx/ny)/rho0g0;
}
			
//---------------------------------------------------------------------------------

void DoEnergy()
{
	int ix,iy;
	
	energy = 0;
	for( ix = 0; ix <= nx; ix++ )
		for( iy = 0; iy <= ny; iy++ )
		{
			real de = (UXX[ix][iy]*SXX[ix][iy]+UYY[ix][iy]*SYY[ix][iy])/2+UXY[ix][iy]*SXY[ix][iy]+rho0g0*UY[ix][iy];

			if( ix == 0 || ix == nx ) de /= 2;
			if( iy == 0 || iy == ny ) de /= 2;

			energy += de;
		}

	energy *= dx*dy;
}

//---------------------------------------------------------------------------------

void Update()
{
	int ix,iy;
	
	for( ix = 0; ix <= nx; ix++ )
		for( iy = 0; iy <= ny; iy++ )
		{
			real dux = ( ix == 0 )? 0 : eps*FX[ix][iy];
			real duy = ( iy == 0 )? 0 : eps*FY[ix][iy];
			real ux = UX[ix][iy] += dux; 
			real uy = UY[ix][iy] += duy;

		}
}

//---------------------------------------------------------------------------------

#define realOUTPUT(x) fprintf(out," "#x" = %f;\n",x);
#define intOUTPUT(x) fprintf(out," "#x" = %d;\n",x);

void WriteInfo()
{
	int ix,iy;

	fopen_s(&out,outname,"w");

	// input parameters

	fprintf(out,"{\n");
	intOUTPUT(nx);
	intOUTPUT(ny);

	realOUTPUT(a);
	realOUTPUT(b);
	realOUTPUT(Y);
	realOUTPUT(nu);
	realOUTPUT(lambda);
	realOUTPUT(mu);
	realOUTPUT(rho0g0);
	realOUTPUT(scale);
	realOUTPUT(omega);
	realOUTPUT(eps);
	realOUTPUT(accuracy);

	intOUTPUT(iters);
		
	
	// x

	fprintf(out," x = {");
	for( ix = 0; ix <= nx; ix++ ) 
	{
		real x = ix*dx;
		if( ix > 0 ) fprintf(out,",");
		fprintf(out,"%f",x);
	}
	fprintf(out,"};\n");

	// y

	fprintf(out," y = {0");
	for( iy = 0; iy <= ny; iy++ ) 
	{
		real y = iy*dy;
		if( iy > 0 ) fprintf(out,",");
		fprintf(out,"%f",y);
	}
	fprintf(out,"};\n");

	fprintf(out,"};\n");
	fclose(out);
}

//---------------------------------------------------------------------------------

void WriteRealMatrix(char *name,realmatrix &matrix)
{
	int ix,iy;

	fprintf(out," %s =\n {\n  {",name);
	for( ix = 0; ix <= nx; ix++ )
	{
		if( ix > 0 ) fprintf(out,",\n  {");
		for( iy = 0; iy <= ny; iy++)
		{
			if( iy > 0 ) fprintf(out,",");
			fprintf(out,"%f",matrix[ix][iy]);
		}
		fprintf(out,"}");
	}
	fprintf(out,"\n };\n");
}


void WriteData()
{
	fopen_s(&out,outname,"a+");

	fprintf(out,"{\n");
	intOUTPUT(iter);
	realOUTPUT(precision);
	realOUTPUT(energy);

	WriteRealMatrix("ux",UX);
	WriteRealMatrix("uy",UY);
	WriteRealMatrix("uxx",UXX);
	WriteRealMatrix("uyy",UYY);
	WriteRealMatrix("uxy",UXY);
	WriteRealMatrix("sxx",SXX);
	WriteRealMatrix("syy",SYY);
	WriteRealMatrix("sxy",SXY);

	fprintf(out,"};\n");

	fclose(out);
}

//---------------------------------------------------------------------------------

int main()
{
	Initialize();
	WriteInfo();

	for( iter = 1; iter <= iters; iter++ )
	{
		DoStrain();
		DoStress();
		DoForce();
		Update();
		DoEnergy();
		if( iter%print == 0 ) 
			printf("%10d%20g%20g\n",iter,precision,energy);
		if( iter%write == 0) WriteData();
		if( iter%ask == 0 ) getc(stdin);
		if( precision < accuracy ) break;
	}

	if( iter%write != 0 ) WriteData();
	return 0;
}
