% CREATE NACA 4-DIGIT AIRFOIL GEOMETRY

% INPUT PARAMETERS

code=[2 4 12];						% standard shape for general aviation
f=code(1)/100;						% maximum camber in percent of chord
xf=code(2)/10;                      % maximum camber position in tenth of chord
t=code(3)/100;						% thickness fraction of chord
n1=10;								% number of points on left upper and lower halves
n2=10;								% number of points on right upper and lower halves

% CONSTRUCT CAMBER LINE AND DERIVATIVE, 

t1=linspace(0,1,n1);					% parameter values near leading edge
t2=linspace(0,1,n2);					% parameter values near trailing edge
x1=xf*t1.^2;							% first range of x values
x2=xf+(1-xf)*t2;						% second range of x values
x2=x2(2:numel(x2));						% prevent double counting
yc1=f*(2*xf*x1-x1.^2)/xf^2;             % camber line
yc2=f*(1-2*xf+2*xf*x2-x2.^2)/(1-xf)^2;  % 
dycdx1=2*f*(xf-x1)/xf^2;				% derivative
dycdx2=2*f*(xf-x2)/(1-xf)^2;
x=cat(2,x1,x2);							% unite
yc=cat(2,yc1,yc2);
dycdx=cat(2,dycdx1,dycdx2);

% CONSTRUCT THICKNESS 
% a correction has been applied to the last coefficient to get closure at
% x=1. The first form is authoritative but has a little gap. The second
% form has been corrected by about 0.2 percent.

%yt=5*t*(0.2969*x.^0.5-0.126*x-0.3516*x.^2+0.2843*x.^3-0.1015*x.^4);
yt=5*t*(0.2969*x.^0.5-0.126*x-0.3516*x.^2+0.2843*x.^3-(0.1015+0.0021)*x.^4);

% CONSTRUCT AIRFOIL OUTLINE

costheta= 1./sqrt(1+dycdx.^2);
sintheta=dycdx.*costheta;
ytsin=yt.*sintheta;
ytcos=yt.*costheta;
xupper=x-ytsin;
xlower=x+ytsin;
yupper=yc+ytcos;
ylower=yc-ytcos;

% make geometry object and plot it

length=numel(x);
plower=[xlower;ylower];
plower=flipdim(plower,2);
plower=plower(:,1:length-1);
pupper=[xupper;yupper];
pupper=pupper(:,2:length);
p=cat(2,plower,pupper);
airfoil=geomspline(p);
geomplot(airfoil,'pointmode','off');

% write to file

name=sprintf('naca%1d%1d%2d.dxf',code(1),code(2),code(3));
dxfwrite(name,airfoil);