(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     35824,       1081]
NotebookOptionsPosition[     31677,        936]
NotebookOutlinePosition[     32149,        954]
CellTagsIndexPosition[     32106,        951]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 3 Boyancy and stability (p. 41)\
\>", "Title",
 CellChangeTimes->{
  3.4157945451335983`*^9, {3.5016647880685863`*^9, 3.5016647919868107`*^9}, {
   3.5311194706752696`*^9, 3.531119473343422*^9}, {3.586836863682227*^9, 
   3.58683686664062*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"SetDirectory", "[", 
    RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{"<<", "Lautrup`"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.3961771022009134`*^9, 3.3961771027049136`*^9}, 
   3.3961786989609137`*^9, 3.479209070075*^9, 3.5885694710560145`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 3.2 The gentle art of ballooning (p. 44)\
\>", "Title",
 CellChangeTimes->{{3.501667933051469*^9, 3.501667942883031*^9}, {
  3.5868368817615013`*^9, 3.5868368843138237`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 3.2 La Montgolfiere (p. 44)\
\>", "Section",
 CellChangeTimes->{{3.4017986541399236`*^9, 3.4017986944419236`*^9}, {
  3.401801368603924*^9, 3.401801369193924*^9}, {3.5016679467162504`*^9, 
  3.5016679543996897`*^9}, {3.586836890833626*^9, 3.5868368933610425`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(26\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{{3.575474881265048*^9, 3.575474898821156*^9}, {
   3.575531179762336*^9, 3.5755312058734665`*^9}, 3.5755314219323916`*^9, 
   3.575536117355773*^9, 3.586581181505554*^9, {3.5868369314397087`*^9, 
   3.586836940453858*^9}, 3.5868379174635077`*^9, 3.5868380690065627`*^9, 
   3.586838872983475*^9, {3.5885694824504776`*^9, 3.588569515722703*^9}, 
   3.58857969304051*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Rho]", "=", 
   RowBox[{"1.1", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"density", " ", "of", " ", "air"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"T", "=", 
   RowBox[{
    RowBox[{"0.", " ", "Celsius"}], "//", 
    RowBox[{"ToUnit", "[", "Kelvin", "]"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"air", " ", "temperature", " ", "at", " ", "height"}], " ", "*)"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"M", "=", 
    RowBox[{"850.", " ", "Kilogram"}]}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{
    "1600", " ", "Pound", " ", "balloon", " ", "material", " ", "plus", " ", 
     "passengers"}], " ", "*)"}], ";"}], " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"V", "=", 
   RowBox[{"1700.", " ", 
    SuperscriptBox["Meter", "3"]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"60", ",", 
    RowBox[{"000", " ", "cubic", " ", "foot"}]}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"z", "=", " ", 
    RowBox[{"1000.", "Meter"}]}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"balloon", " ", "height"}], " ", "*)"}], ";"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["h", "2"], "=", 
   RowBox[{"30.", " ", "Kilometer"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"atmospheric", " ", "scale", " ", "height"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Gamma]", "=", 
   RowBox[{"7", "/", "5"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"adiabatic", " ", "index", " ", "of", " ", "air"}], " ", "*)"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.4016907101532173`*^9, 3.4016907405822177`*^9}, {
   3.4016949822212176`*^9, 3.4016949839242177`*^9}, {3.4016951332202177`*^9, 
   3.4016951731372175`*^9}, {3.4016954627382174`*^9, 
   3.4016954936952176`*^9}, {3.401696365013218*^9, 3.401696365381218*^9}, {
   3.4016970698162174`*^9, 3.4016970781882176`*^9}, {3.4016986410752172`*^9, 
   3.4016986419792175`*^9}, {3.4016991094322176`*^9, 3.4016991113922176`*^9}, 
   3.4017987065409236`*^9, {3.4017988298019238`*^9, 3.4017988306029234`*^9}, {
   3.4017988843949237`*^9, 3.4017988845869236`*^9}, 3.401799020891924*^9, {
   3.4017992118029237`*^9, 3.4017992147789235`*^9}, {3.4018042679159236`*^9, 
   3.4018042989069233`*^9}, {3.4018044241869235`*^9, 
   3.4018044251949234`*^9}, {3.501668017814317*^9, 3.5016680306370506`*^9}, {
   3.501728644448522*^9, 3.5017286994916697`*^9}, {3.5017290347588463`*^9, 
   3.501729149212393*^9}, {3.5017291978161726`*^9, 3.5017292385345016`*^9}, {
   3.5017300169800262`*^9, 3.5017300409713984`*^9}, {3.501730443156402*^9, 
   3.501730443437418*^9}, {3.501732061870987*^9, 3.501732077791898*^9}, {
   3.5017321481829243`*^9, 3.501732173358364*^9}, {3.50173249254062*^9, 
   3.501732539627313*^9}, {3.5868369209394016`*^9, 3.586836937150409*^9}, {
   3.5868379071782165`*^9, 3.586837914425116*^9}, {3.586837991272807*^9, 
   3.586838059685351*^9}, {3.5868387956398525`*^9, 3.5868388331124673`*^9}, {
   3.5868388787122364`*^9, 3.5868388899606056`*^9}}],

Cell["Hot air density", "Text",
 CellChangeTimes->{{3.50166843973345*^9, 3.5016684521621604`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]", "'"}], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Rho]", " ", "V"}], "-", "M"}], ")"}], "/", "V"}], "//", 
   "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.5016684649948945`*^9, 3.501668486977152*^9}, {
  3.5017296168581405`*^9, 3.501729632860056*^9}}],

Cell[BoxData[
 TemplateBox[{"0.6000000000000001`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575531206173504*^9, 3.5755314222164235`*^9, 3.5755361176438093`*^9, 
   3.5865811816345725`*^9, {3.5868369315867434`*^9, 3.5868369406228495`*^9}, 
   3.5868379176275578`*^9, 3.586838069168583*^9, 3.586838873151539*^9, {
   3.5885694825884824`*^9, 3.5885695158876743`*^9}, 3.588579693175584*^9}]
}, Open  ]],

Cell["Hot air temperature", "Text",
 CellChangeTimes->{{3.50166849953887*^9, 3.501668507834345*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"T", "'"}], "=", 
  RowBox[{"T", " ", 
   RowBox[{"\[Rho]", "/", 
    RowBox[{"\[Rho]", "'"}]}]}]}]], "Input",
 CellChangeTimes->{{3.501668494976609*^9, 3.501668526263399*^9}, {
  3.5017317690362377`*^9, 3.501731774723563*^9}}],

Cell[BoxData[
 TemplateBox[{"500.7749999999999`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kelvins"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755312063035192`*^9, 3.5755314223464403`*^9, 3.575536117789825*^9, 
   3.586581181698578*^9, {3.5868369316547513`*^9, 3.5868369406908555`*^9}, 
   3.586837917696562*^9, 3.58683806923855*^9, 3.5868388732195044`*^9, {
   3.5885694826584873`*^9, 3.588569515956699*^9}, 3.5885796932395754`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"T", "'"}], "//", 
  RowBox[{"ToUnit", "[", "Celsius", "]"}]}]], "Input",
 CellChangeTimes->{{3.575531224287675*^9, 3.575531233239752*^9}}],

Cell[BoxData[
 TemplateBox[{"227.62499999999994`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"\[Degree]C\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: degrees Celsius"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"DegreesCelsius\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575531233739809*^9, 3.575531422379444*^9, 3.57553611782683*^9, 
   3.586581181712576*^9, {3.58683693167074*^9, 3.586836940707887*^9}, 
   3.5868379177165623`*^9, 3.5868380692556133`*^9, 3.586838873239538*^9, {
   3.588569482674474*^9, 3.5885695159746866`*^9}, 3.588579693258569*^9}]
}, Open  ]],

Cell["Paper ignition temperature", "Text",
 CellChangeTimes->{{3.501732462344893*^9, 3.501732465977101*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"451.", "Fahrenheit"}], "//", 
  RowBox[{"ToUnit", "[", "Celsius", "]"}]}]], "Input",
 CellChangeTimes->{{3.5755312553994102`*^9, 3.575531286855183*^9}, 
   3.5755314311444955`*^9}],

Cell[BoxData[
 TemplateBox[{"232.77777777777789`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"\[Degree]C\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: degrees Celsius"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"DegreesCelsius\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575531268804019*^9, {3.575531422409455*^9, 3.575531432257629*^9}, 
   3.5755361178568335`*^9, 3.5865811817275424`*^9, {3.5868369316877317`*^9, 
   3.5868369407248507`*^9}, 3.586837917737531*^9, 3.5868380692735605`*^9, 
   3.586838873260541*^9, {3.5885694826925006`*^9, 3.588569515993688*^9}, 
   3.5885796932775736`*^9}]
}, Open  ]],

Cell["Air density at the ground", "Text",
 CellChangeTimes->{{3.501732471745431*^9, 3.5017324755766497`*^9}, {
  3.5017332035052853`*^9, 3.5017332128008165`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Rho]", "0"], "=", 
  RowBox[{
   RowBox[{"\[Rho]", "/", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"z", "/", 
        SubscriptBox["h", "2"]}]}], ")"}], 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.5017324263558345`*^9, 3.5017324493081474`*^9}, {
  3.5017325571413155`*^9, 3.501732558733406*^9}}],

Cell[BoxData[
 TemplateBox[{"1.1972940581176024`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575531206453535*^9, 3.575531422627475*^9, 3.5755361179458456`*^9, 
   3.586581181770587*^9, {3.5868369317337184`*^9, 3.586836940769909*^9}, 
   3.5868379177835617`*^9, 3.5868380693186035`*^9, 3.58683887330855*^9, {
   3.588569482742505*^9, 3.588569516040722*^9}, 3.5885796933255835`*^9}]
}, Open  ]],

Cell["Ground temperature", "Text",
 CellChangeTimes->{{3.501732591897303*^9, 3.501732595984537*^9}, {
  3.5017331980329723`*^9, 3.501733200337104*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["T", "0"], "=", 
  RowBox[{"T", "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     RowBox[{"z", "/", 
      SubscriptBox["h", "2"]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.5017325979606495`*^9, 3.5017326159636793`*^9}, 
   3.5868388684159327`*^9}],

Cell[BoxData[
 TemplateBox[{"282.5689655172414`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kelvins"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755312065385466`*^9, 3.575531422712484*^9, 3.5755361180368557`*^9, 
   3.5865811818135695`*^9, {3.586836931784768*^9, 3.5868369408149*^9}, 
   3.586837917831581*^9, 3.5868380693656044`*^9, {3.5868388688339925`*^9, 
   3.5868388733555603`*^9}, {3.588569482790513*^9, 3.588569516085703*^9}, 
   3.588579693372587*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["T", "0"], "//", 
  RowBox[{"ToUnit", "[", "Celsius", "]"}]}]], "Input",
 CellChangeTimes->{{3.5017326184908237`*^9, 3.5017326254512224`*^9}, {
  3.5755315466003504`*^9, 3.575531553961231*^9}}],

Cell[BoxData[
 TemplateBox[{"9.418965517241418`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"\[Degree]C\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: degrees Celsius"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"DegreesCelsius\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575531206565551*^9, 3.5755314227364883`*^9, 3.5755315548583393`*^9, 
   3.5755361181148643`*^9, 3.5865811818266096`*^9, {3.586836931801817*^9, 
   3.586836940830862*^9}, 3.5868379178496013`*^9, 3.586838069384591*^9, 
   3.5868388733755827`*^9, {3.588569482809472*^9, 3.588569516102753*^9}, 
   3.588579693390592*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 3.3 La Charliere (p. 45)\
\>", "Section",
 CellChangeTimes->{{3.4018013501859236`*^9, 3.4018013778339233`*^9}, 
   3.5016679627631683`*^9, {3.5016685914921293`*^9, 3.5016685950513334`*^9}, {
   3.5868379668407483`*^9, 3.586837969127014*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(7\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.575531566480735*^9, 3.5755317264689302`*^9, 3.575536118372896*^9, 
   3.5865811819776096`*^9, {3.588569482962493*^9, 3.5885695162537613`*^9}, 
   3.5885796935415697`*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Rho]", "=", 
   RowBox[{"1.1", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"air", " ", "density"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"0.5", "*", "27", "Foot"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"balloon", " ", "radius"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"d", "=", 
   RowBox[{"1.5", " ", "Millimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"balloon", " ", "skin", " ", "thickness"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "1"], "=", 
   RowBox[{"0.4", " ", 
    RowBox[{"Gram", "/", 
     SuperscriptBox["Centimeter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"skin", " ", "density"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4016793490793767`*^9, 3.401679352374377*^9}, {
   3.4018005277239237`*^9, 3.4018005278199234`*^9}, {3.4018009107959237`*^9, 
   3.401801010667924*^9}, {3.4018010414049234`*^9, 3.401801063931924*^9}, {
   3.401801099339924*^9, 3.4018011006829233`*^9}, {3.4018020948429236`*^9, 
   3.4018021197559233`*^9}, {3.401803898075924*^9, 3.4018039225559235`*^9}, {
   3.4018039586829233`*^9, 3.4018039589229236`*^9}, {3.401804110747924*^9, 
   3.4018041108269234`*^9}, {3.4018050018199234`*^9, 
   3.4018050212909236`*^9}, {3.401805266266924*^9, 3.4018052786189237`*^9}, {
   3.4018056198839235`*^9, 3.4018056199799232`*^9}, {3.4018058362039237`*^9, 
   3.4018058675949235`*^9}, {3.501732780104068*^9, 3.5017327802930784`*^9}, {
   3.501732812206904*^9, 3.50173281230991*^9}, 3.5017330060359907`*^9, {
   3.586838359776084*^9, 3.586838424152156*^9}}],

Cell[TextData[{
 "Hydrogen density ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Rho]", "'"}], "/", "\[Rho]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["M", "hydrogen"], "/", 
       SubscriptBox["M", "air"]}], "\[TildeTilde]", 
      RowBox[{"1", "/", "15"}]}]}], ")"}], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.5017330166335964`*^9, 3.5017330251690845`*^9}, {
  3.586838198575845*^9, 3.5868382595774918`*^9}, {3.5868384503774357`*^9, 
  3.586838455193058*^9}, {3.586838726122085*^9, 3.5868387307436666`*^9}, {
  3.5868387657060456`*^9, 3.5868387657060456`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]", "'"}], "=", 
  RowBox[{"\[Rho]", "/", "15"}]}]], "Input",
 CellChangeTimes->{{3.501733011867324*^9, 3.50173302770723*^9}, {
  3.501734385515892*^9, 3.5017343857239037`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.07333333333333333`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575531566788772*^9, 3.5755317267089586`*^9, 3.575536118611924*^9, 
   3.586581182083626*^9, {3.588569483072545*^9, 3.5885695163557553`*^9}, 
   3.588579693650623*^9}]
}, Open  ]],

Cell["Volume of balloon", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"V", "=", 
  RowBox[{
   RowBox[{
    FractionBox["4", "3"], "\[Pi]", " ", 
    SuperscriptBox["a", "3"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.401679164944377*^9, 3.401679165352377*^9}, {
   3.4018005895939236`*^9, 3.4018005901869235`*^9}, 3.5755317379493093`*^9}],

Cell[BoxData[
 TemplateBox[{"291.833270900734`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "3"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755315668177743`*^9, {3.575531726769967*^9, 3.575531738616389*^9}, 
   3.5755361186839347`*^9, 3.58658118211563*^9, {3.5885694831075506`*^9, 
   3.58856951637877*^9}, 3.5885796936835885`*^9}]
}, Open  ]],

Cell["Mass of displaced air", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"M", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Rho]", "-", 
      RowBox[{"\[Rho]", "'"}]}], ")"}], " ", "V"}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4016791679603767`*^9, 3.4016791683353767`*^9}, {
  3.401680021163377*^9, 3.4016800219243765`*^9}, {3.4018021301869235`*^9, 
  3.4018021357229233`*^9}, {3.4018050259629235`*^9, 3.401805030938924*^9}, {
  3.4018064283149233`*^9, 3.401806433754924*^9}, {3.501669037421635*^9, 
  3.501669037589645*^9}}],

Cell[BoxData[
 TemplateBox[{"299.615491458087`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kg\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilograms\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575531566848778*^9, 3.575531726864977*^9, 3.575536118890959*^9, 
   3.586581182167637*^9, {3.5885694831595592`*^9, 3.588569516427781*^9}, 
   3.5885796937356377`*^9}]
}, Open  ]],

Cell["Skin area", "Text",
 CellChangeTimes->{{3.50166909240378*^9, 3.501669092826804*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"4", "\[Pi]", " ", 
     SuperscriptBox["a", "2"]}], "//", "N"}], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"212.7684973029557`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "2"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575531566874782*^9, 3.5755317269239845`*^9, 3.5755361190199738`*^9, 
   3.5865811821996427`*^9, {3.5885694831945395`*^9, 3.5885695164507694`*^9}, 
   3.588579693769599*^9}]
}, Open  ]],

Cell["Skin mass", "Text",
 CellChangeTimes->{{3.5016690945039005`*^9, 3.501669108394695*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["M", "1"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Rho]", "1"], "A", " ", "d"}], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"127.66109838177343`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kg\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilograms\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755315670127974`*^9, 3.575531727100006*^9, 3.5755361202611217`*^9, 
   3.586581182289649*^9, {3.5885694832915764`*^9, 3.588569516547807*^9}, 
   3.588579693862611*^9}]
}, Open  ]],

Cell["Payload", "Text",
 CellChangeTimes->{{3.5016691541953144`*^9, 3.5016691554663873`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"M", "-", 
   SubscriptBox["M", "1"]}], "//", "ToSI"}]], "Input",
 CellChangeTimes->{{3.401680008923377*^9, 3.4016800391813765`*^9}, {
  3.401804134042924*^9, 3.401804150346924*^9}, {3.4018064385389233`*^9, 
  3.4018064394499235`*^9}, {3.501669084403322*^9, 3.5016690853563766`*^9}}],

Cell[BoxData[
 TemplateBox[{"171.95439307631355`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kg\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilograms\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755315670528026`*^9, 3.57553172721602*^9, 3.5755361204531465`*^9, 
   3.586581182306635*^9, {3.588569483334582*^9, 3.5885695165628185`*^9}, 
   3.588579693907672*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 3.4 Ship stability (p. 48)\
\>", "Title",
 CellChangeTimes->{{3.501736630222282*^9, 3.5017366486343355`*^9}, {
  3.58683891637495*^9, 3.5868389189822903`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 3.4 Elliptical rowboat (p. 52)\
\>", "Section",
 CellChangeTimes->{{3.501736675378865*^9, 3.5017366915047874`*^9}, {
  3.5868389426152506`*^9, 3.5868389475098815`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(7\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755318456332283`*^9, 3.575531914301467*^9, 
  3.57553613716715*^9, 3.5865811826807013`*^9, 3.58856948348656*^9, 
  3.588569516714816*^9, 3.5885796940656366`*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"1", "Meter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"half", " ", "length", " ", "of", " ", "boat"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b", "=", 
   RowBox[{"0.5", "Meter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"half", " ", "width", " ", "of", " ", "boat"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"75", "+", "50"}], ")"}], " ", "Kilogram"}], "//", "N"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "total", " ", "weight", " ", "of", " ", "boat", " ", "and", " ", 
    "passenger"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{
    RowBox[{"1000", "  ", 
     RowBox[{"Kilogram", "/", 
      SuperscriptBox["Meter", "3"]}]}], "//", "N"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"water", " ", "density"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5868389566710215`*^9, 3.5868390288470984`*^9}}],

Cell["Boat area", "Text",
 CellChangeTimes->{{3.586839048550558*^9, 3.586839050165757*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["A", "0"], "=", 
  RowBox[{"4", " ", "a", " ", "b"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"2.`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "2"]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.57553184588326*^9, 3.575531914526496*^9, 
  3.5755361373511744`*^9, 3.586581182748707*^9, 3.588569483557607*^9, 
  3.5885695168068333`*^9, 3.5885796941366453`*^9}]
}, Open  ]],

Cell["Second order moment", "Text",
 CellChangeTimes->{{3.586839092678116*^9, 3.586839095237455*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["I", "0"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["\[Pi]", "4"], "a", " ", 
    SuperscriptBox["b", "3"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{3.575531906313512*^9}],

Cell[BoxData[
 TemplateBox[{"0.09817477042468103`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "4"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters to the fourth"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "4"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755318459882703`*^9, {3.575531907872696*^9, 3.5755319146165066`*^9}, 
   3.5755361374551864`*^9, 3.586581182801718*^9, 3.588569483615617*^9, 
   3.588569516851841*^9, 3.588579694192653*^9}]
}, Open  ]],

Cell["Displacement", "Text",
 CellChangeTimes->{{3.5868391183023252`*^9, 3.5868391199655223`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["V", "0"], "=", 
  FractionBox["M", 
   SubscriptBox["\[Rho]", "0"]]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.125`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "3"]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575531846106285*^9, 3.575531914736521*^9, 
  3.5755361375702*^9, 3.586581182862722*^9, 3.5885694836786222`*^9, 
  3.588569516913852*^9, 3.5885796942566614`*^9}]
}, Open  ]],

Cell["Draft", "Text",
 CellChangeTimes->{{3.586839129981799*^9, 3.586839130853899*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"d", "=", 
  RowBox[{
   FractionBox[
    SubscriptBox["V", "0"], 
    SubscriptBox["A", "0"]], "//", 
   RowBox[{"ToUnit", "[", "Centimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5017367765656524`*^9, 3.5017367831970315`*^9}}],

Cell[BoxData[
 TemplateBox[{"6.25`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"cm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: centimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Centimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575531846197297*^9, 3.5755319148195314`*^9, 
  3.5755361376612115`*^9, 3.586581182903686*^9, 3.5885694837246323`*^9, 
  3.5885695169558516`*^9, 3.588579694301705*^9}]
}, Open  ]],

Cell["Center of boyancy", "Text",
 CellChangeTimes->{{3.586839154341921*^9, 3.5868391626138425`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["z", "B"], "=", 
  RowBox[{"-", 
   FractionBox["d", "2"]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "3.125`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"cm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: centimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Centimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755318462293005`*^9, 3.575531914848533*^9, 
  3.575536137691214*^9, 3.58658118291873*^9, 3.5885694837425904`*^9, 
  3.5885695169758124`*^9, 3.5885796943197117`*^9}]
}, Open  ]],

Cell["Metacenter", "Text",
 CellChangeTimes->{{3.5868391853738337`*^9, 3.5868391879580364`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["z", "M"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["z", "B"], "+", 
    FractionBox[
     SubscriptBox["I", "0"], 
     SubscriptBox["V", "0"]]}], "//", 
   RowBox[{"ToUnit", "[", "Centimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.501736822231264*^9, 3.501736832589857*^9}}],

Cell[BoxData[
 TemplateBox[{"75.41481633974483`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"cm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: centimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Centimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755318463713164`*^9, 3.5755319149845505`*^9, 
  3.575536137829231*^9, 3.586581182986735*^9, 3.5885694838176003`*^9, 
  3.5885695170468645`*^9, 3.5885796943916783`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{1037, 773},
WindowMargins->{{308, Automatic}, {Automatic, 32}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 262, 6, 65, "Title"],
Cell[832, 30, 369, 9, 62, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1238, 44, 192, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[1455, 52, 282, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[1762, 61, 59, 1, 41, "Input"],
Cell[1824, 64, 585, 12, 21, "Message"]
}, Open  ]],
Cell[2424, 79, 3144, 72, 164, "Input"],
Cell[5571, 153, 98, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[5694, 158, 329, 10, 41, "Input"],
Cell[6026, 170, 937, 20, 41, "Output"]
}, Open  ]],
Cell[6978, 193, 100, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[7103, 198, 261, 7, 41, "Input"],
Cell[7367, 207, 724, 15, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8128, 227, 173, 4, 41, "Input"],
Cell[8304, 233, 740, 15, 41, "Output"]
}, Open  ]],
Cell[9059, 251, 108, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[9192, 256, 215, 5, 41, "Input"],
Cell[9410, 263, 778, 16, 41, "Output"]
}, Open  ]],
Cell[10203, 282, 162, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[10390, 288, 476, 14, 43, "Input"],
Cell[10869, 304, 930, 20, 41, "Output"]
}, Open  ]],
Cell[11814, 327, 151, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[11990, 333, 293, 9, 41, "Input"],
Cell[12286, 344, 749, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13072, 365, 232, 5, 41, "Input"],
Cell[13307, 372, 773, 16, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14129, 394, 259, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[14413, 403, 59, 1, 41, "Input"],
Cell[14475, 406, 390, 10, 21, "Message"]
}, Open  ]],
Cell[14880, 419, 26, 0, 30, "Text"],
Cell[14909, 421, 1867, 43, 127, "Input"],
Cell[16779, 466, 672, 18, 33, "Text"],
Cell[CellGroupData[{
Cell[17476, 488, 217, 5, 41, "Input"],
Cell[17696, 495, 810, 19, 41, "Output"]
}, Open  ]],
Cell[18521, 517, 33, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[18579, 521, 302, 7, 56, "Input"],
Cell[18884, 530, 720, 18, 41, "Output"]
}, Open  ]],
Cell[19619, 551, 37, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[19681, 555, 505, 11, 41, "Input"],
Cell[20189, 568, 603, 14, 41, "Output"]
}, Open  ]],
Cell[20807, 585, 90, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[20922, 590, 172, 5, 41, "Input"],
Cell[21097, 597, 703, 18, 41, "Output"]
}, Open  ]],
Cell[21815, 618, 93, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[21933, 623, 156, 5, 41, "Input"],
Cell[22092, 630, 607, 14, 41, "Output"]
}, Open  ]],
Cell[22714, 647, 93, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[22832, 652, 317, 6, 41, "Input"],
Cell[23152, 660, 606, 14, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[23819, 681, 177, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[24021, 689, 186, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[24232, 697, 59, 1, 41, "Input"],
Cell[24294, 700, 379, 9, 21, "Message"]
}, Open  ]],
Cell[24688, 712, 26, 0, 30, "Text"],
Cell[24717, 714, 1077, 32, 104, "Input"],
Cell[25797, 748, 91, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[25913, 753, 104, 3, 41, "Input"],
Cell[26020, 758, 676, 17, 41, "Output"]
}, Open  ]],
Cell[26711, 778, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[26837, 783, 222, 7, 54, "Input"],
Cell[27062, 792, 729, 18, 41, "Output"]
}, Open  ]],
Cell[27806, 813, 98, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[27929, 818, 120, 4, 58, "Input"],
Cell[28052, 824, 674, 17, 41, "Output"]
}, Open  ]],
Cell[28741, 844, 87, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[28853, 849, 256, 7, 58, "Input"],
Cell[29112, 858, 592, 13, 41, "Output"]
}, Open  ]],
Cell[29719, 874, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[29845, 879, 111, 4, 56, "Input"],
Cell[29959, 885, 607, 13, 41, "Output"]
}, Open  ]],
Cell[30581, 901, 96, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[30702, 906, 325, 10, 58, "Input"],
Cell[31030, 918, 607, 13, 73, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
