(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     10094,        355]
NotebookOptionsPosition[      8253,        287]
NotebookOutlinePosition[      8702,        304]
CellTagsIndexPosition[      8659,        301]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Chapter 6 Stress (p.97)", "Title",
 CellChangeTimes->{
  3.4309105403616*^9, {3.5018298473522167`*^9, 3.501829850270384*^9}, {
   3.5872766520518527`*^9, 3.587276655019183*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{"<<", "Lautrup`"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.4093893906808243`*^9, 3.4093893926158247`*^9}, 
   3.479209186094*^9, 3.589038612357942*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 6.2 Stress fields (p. 99)\
\>", "Title",
 CellChangeTimes->{{3.501830042679389*^9, 3.501830047374657*^9}, {
  3.587276662411212*^9, 3.587276667106783*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 6.1 Classic gallows (p. 100)\
\>", "Section",
 CellChangeTimes->{{3.4093895237668247`*^9, 3.4093895246148243`*^9}, {
  3.5018300547290783`*^9, 3.501830059918375*^9}, {3.5872766832117805`*^9, 
  3.5872766911707573`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(16\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755421776771755`*^9, 3.5865812773705645`*^9, 
  3.587276503369156*^9, 3.5890773427014236`*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"70.", " ", "Kilogram"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"mass", " ", "of", " ", "body"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"A", "=", 
   RowBox[{"100.", " ", 
    SuperscriptBox["Centimeter", "2"]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"cross", "-", 
    RowBox[{"section", " ", "of", " ", "horizontal", " ", "beam"}]}], " ", 
   "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5054090778503685`*^9, 3.505409079352454*^9}, {
  3.5872767083879337`*^9, 3.587276742108176*^9}}],

Cell["Stress", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Sigma]", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"M", " ", 
     SubscriptBox["g", "0"]}], "A"], "//", 
   RowBox[{"ToUnit", "[", "Pascal", "]"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"68646.55`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"Pa\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pascals"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Pascals\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755421779172063`*^9, 3.586581277487602*^9, 
  3.587276503493182*^9, 3.5890773428164797`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Sigma]", "//", 
  RowBox[{"ToUnit", "[", "Bar", "]"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.6864655000000001`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"bars\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: bars"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Bars\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.57554217795621*^9, 3.5865812775035925`*^9, 
  3.5872765035141973`*^9, 3.589077342834467*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 6.2 Water mains (p. 100)\
\>", "Section",
 CellChangeTimes->{{3.501830089015039*^9, 3.5018301113183146`*^9}, {
  3.587276787242852*^9, 3.58727679233846*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(4\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.575542178225245*^9, 3.5865812776635866`*^9, 
  3.587276503681223*^9, 3.58907734299148*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"0.6", " ", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"pipe", " ", "radius"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalDelta]p", "=", 
   RowBox[{
    SuperscriptBox["10", "4"], "Pascal"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"pressure", " ", "drop"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L", "=", 
   RowBox[{"10", " ", "Meter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"pipe", " ", "length"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5872767976281195`*^9, 3.587276835707905*^9}}],

Cell["Stress", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Sigma]", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"\[CapitalDelta]p", " ", "a"}], 
    RowBox[{"2", "L"}]], "//", 
   RowBox[{"ToUnit", "[", "Pascal", "]"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"3.`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"Pa\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pascals"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Pascals\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575542178420266*^9, 3.5865812777666407`*^9, 
  3.587276503782203*^9, 3.589077343096511*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 6.3 Steel rod (p. 100)", "Section",
 CellChangeTimes->{{3.5018301556388493`*^9, 3.5018301607341413`*^9}, {
  3.587277092074123*^9, 3.587277102850587*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(4\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755421787563066`*^9, 3.5755422502908897`*^9, 
  3.5865812779276586`*^9, 3.5872765039492173`*^9, 3.5890773432585363`*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Sigma]", "=", 
   RowBox[{"415.", " ", "Mega", " ", "Pascal"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"1", " ", "Centimeter"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.501830519418657*^9, 3.501830519985689*^9}, 
   3.5755422325917654`*^9}],

Cell["Max load", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"M", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"\[Sigma]", " ", "\[Pi]", " ", 
     SuperscriptBox["a", "2"]}], 
    SubscriptBox["g", "0"]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.501830195439126*^9, 3.5018302044986444`*^9}, {
  3.57554223545311*^9, 3.5755422378714004`*^9}}],

Cell[BoxData[
 TemplateBox[{"13294.661798267136`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kg\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilograms\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755421790273385`*^9, {3.5755422385064735`*^9, 3.575542250597927*^9}, 
   3.5865812780696716`*^9, 3.5872765040892315`*^9, 3.589077343408551*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{840, 783},
WindowMargins->{{79, Automatic}, {Automatic, 32}},
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 183, 3, 65, "Title"],
Cell[753, 27, 303, 7, 62, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1093, 39, 173, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[1291, 47, 237, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[1553, 56, 59, 1, 41, "Input"],
Cell[1615, 59, 313, 8, 21, "Message"]
}, Open  ]],
Cell[1943, 70, 26, 0, 30, "Text"],
Cell[1972, 72, 698, 20, 82, "Input"],
Cell[2673, 94, 22, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[2720, 98, 191, 6, 56, "Input"],
Cell[2914, 106, 515, 12, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3466, 123, 92, 2, 41, "Input"],
Cell[3561, 127, 520, 12, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[4130, 145, 175, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[4330, 153, 59, 1, 41, "Input"],
Cell[4392, 156, 307, 8, 21, "Message"]
}, Open  ]],
Cell[4714, 167, 26, 0, 30, "Text"],
Cell[4743, 169, 639, 18, 82, "Input"],
Cell[5385, 189, 22, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[5432, 193, 201, 6, 57, "Input"],
Cell[5636, 201, 507, 12, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6192, 219, 168, 2, 75, "Section"],
Cell[CellGroupData[{
Cell[6385, 225, 59, 1, 41, "Input"],
Cell[6447, 228, 338, 8, 21, "Message"]
}, Open  ]],
Cell[6800, 239, 26, 0, 30, "Text"],
Cell[6829, 241, 433, 12, 82, "Input"],
Cell[7265, 255, 24, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[7314, 259, 313, 8, 61, "Input"],
Cell[7630, 269, 583, 13, 73, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
