(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[      4675,        167]
NotebookOptionsPosition[      3608,        126]
NotebookOutlinePosition[      4058,        143]
CellTagsIndexPosition[      4015,        140]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Chapter 8 Hooke\[CloseCurlyQuote]s law (p. 125)", "Title",
 CellChangeTimes->{{3.5018326748159385`*^9, 3.50183268166133*^9}, {
  3.5872836503375597`*^9, 3.5872836608329353`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{"<<", "Lautrup`"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5890386328935328`*^9, 3.5890386402824116`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 8.1 Young\[CloseCurlyQuote]s modulus and Poisson\[CloseCurlyQuote]s \
ratio\
\>", "Title",
 CellChangeTimes->{{3.5018327034875784`*^9, 3.5018327152382507`*^9}}],

Cell[CellGroupData[{

Cell["Example 8.1 Rope pulling context", "Section",
 CellChangeTimes->{{3.5018327414717507`*^9, 3.501832754583501*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(6\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{{3.5755423090069337`*^9, 3.5755423352980895`*^9}, 
   3.586581309439621*^9, 3.587283725545044*^9, 3.587283774343131*^9, 
   3.5890774114501457`*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"F", "=", 
   RowBox[{"7000", " ", "Newton"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"2.5", " ", "Centimeter"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"EE", "=", 
   RowBox[{"360", " ", "Megapascal"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5755423140635414`*^9, 3.575542331950688*^9}, 
   3.5872837705937004`*^9}],

Cell["Stress", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Sigma]", "=", 
  RowBox[{
   FractionBox["F", 
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["a", "2"]}]], "//", 
   RowBox[{"ToUnit", "[", "Megapascal", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5755423196012053`*^9, 3.5755423201762757`*^9}}],

Cell[BoxData[
 TemplateBox[{"3.5650707252584555`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"MPa\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: megapascals"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Megapascals\""}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.5755423091599536`*^9, 3.575542335507115*^9}, 
   3.5865813095176296`*^9, 3.587283725625017*^9, 3.5872837744501457`*^9, 
   3.58907741153014*^9}]
}, Open  ]],

Cell["Strain", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"u", "=", 
  RowBox[{
   FractionBox["\[Sigma]", "EE"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData["0.009902974236829043`"], "Output",
 CellChangeTimes->{{3.5755423092829676`*^9, 3.575542335628129*^9}, 
   3.586581309585622*^9, 3.5872837256880445`*^9, 3.5872837745161533`*^9, 
   3.5890774115931377`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{816, 740},
WindowMargins->{{126, Automatic}, {Automatic, 71}},
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 184, 2, 65, "Title"],
Cell[754, 26, 258, 6, 62, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1049, 37, 178, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[1252, 45, 119, 1, 75, "Section"],
Cell[CellGroupData[{
Cell[1396, 50, 59, 1, 41, "Input"],
Cell[1458, 53, 363, 9, 21, "Message"]
}, Open  ]],
Cell[1836, 65, 26, 0, 30, "Text"],
Cell[1865, 67, 404, 11, 82, "Input"],
Cell[2272, 80, 22, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[2319, 84, 271, 7, 58, "Input"],
Cell[2593, 93, 586, 13, 41, "Output"]
}, Open  ]],
Cell[3194, 109, 22, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[3241, 113, 106, 3, 54, "Input"],
Cell[3350, 118, 218, 3, 73, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
