(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    261727,       6570]
NotebookOptionsPosition[    243634,       5966]
NotebookOutlinePosition[    244105,       5984]
CellTagsIndexPosition[    244062,       5981]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 14 Compressible flow (p. 229)\
\>", "Title",
 CellChangeTimes->{{3.420447284371626*^9, 3.420447293186626*^9}, {
  3.5020217133679047`*^9, 3.5020217155020266`*^9}, {3.587448172002575*^9, 
  3.5874481772741995`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{"<<", "Lautrup`"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.3901205251263547`*^9, {3.420290463587*^9, 3.4202904667609997`*^9}, 
   3.4792680098129997`*^9, 3.5890386869183664`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 14.1 Small-amplitude sound waves (p. 229)\
\>", "Title",
 CellChangeTimes->{{3.5020217426225777`*^9, 3.502021753941225*^9}, {
  3.587448181593775*^9, 3.5874481857222924`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Wave equation for sound (p. 230)\
\>", "Section",
 CellChangeTimes->{{3.5020218330077477`*^9, 3.502021836917971*^9}, {
  3.587448210514417*^9, 3.5874482133537674`*^9}}],

Cell[CellGroupData[{

Cell["Speed of sound in water", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(4\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.57554593006441*^9, 3.5865827880272784`*^9, 
  3.5874483457263656`*^9, 3.5890778553572083`*^9}]
}, Open  ]],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.587448274298443*^9, 3.587448275785627*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["K", "0"], "=", 
   RowBox[{"23", " ", "Kilobar"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{
    SuperscriptBox["10", "3"], 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5755459377443304`*^9, 3.5755459381433787`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"c", "=", 
  RowBox[{
   RowBox[{
    SqrtBox[
     FractionBox[
      SubscriptBox["K", "0"], 
      SubscriptBox["\[Rho]", "0"]]], "//", "ToSI"}], "//", "N"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"1516.57508881031`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575545930362446*^9, 3.586582788180255*^9, 
  3.587448345906377*^9, 3.5890778555091853`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"c", "//", 
  RowBox[{"ToUnit", "[", 
   RowBox[{"Kilometer", "/", "Hour"}], "]"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"5459.670319717116`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"km\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"h\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers per hour"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755459304864597`*^9, 3.5865827882572656`*^9, 
  3.587448345965437*^9, 3.5890778555861983`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 14.1 Sound speed in the atmosphere (p. 231)\
\>", "Section",
 CellChangeTimes->{{3.5020220132320557`*^9, 3.5020220263098035`*^9}, {
  3.5874483245457163`*^9, 3.5874483278341208`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5874483315826178`*^9, 3.587448334683014*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(2\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{{3.587448340252708*^9, 3.587448346134416*^9}, 
   3.587448431528182*^9, 3.587448541680998*^9, 3.589077855756261*^9}]
}, Open  ]],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.5874483555945897`*^9, 3.587448358217975*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Gamma]", "=", 
   FractionBox["7", "5"]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["T", "0"], "=", 
   RowBox[{
    RowBox[{"20.", " ", "Celsius"}], "//", 
    RowBox[{"ToUnit", "[", "Kelvin", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["R", "mol"], "=", " ", "MolarGasConstant"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["M", "mol"], "=", "AirMolarMass"}], ";"}]}], "Input",
 CellChangeTimes->{{3.420290678748*^9, 3.420290696832*^9}, 
   3.5020222776871815`*^9, {3.5874484014033947`*^9, 3.5874484101234393`*^9}, {
   3.5874484601797314`*^9, 3.587448506659601*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"R", "=", 
  RowBox[{
   FractionBox[
    SubscriptBox["R", "mol"], 
    SubscriptBox["M", "mol"]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.587448414550993*^9, 3.587448442275478*^9}}],

Cell[BoxData[
 TemplateBox[{"287.06689799230065`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"], "\[NegativeMediumSpace]", "\"/(\"", 
            "\[InvisibleSpace]", 
            SuperscriptBox["\"s\"", "2"], "\[InvisibleSpace]", "\"K\"", 
            "\[InvisibleSpace]", "\")\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared per second squared kelvin"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox[
        SuperscriptBox["\"Meters\"", "2"], 
        RowBox[{"\"Kelvins\"", " ", 
          SuperscriptBox["\"Seconds\"", "2"]}]]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.587448427864708*^9, 3.5874484426685486`*^9}, 
   3.5874485418240175`*^9, 3.589077855876274*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["c", "0"], "=", 
  RowBox[{
   RowBox[{
    SqrtBox[
     RowBox[{"\[Gamma]", " ", "R", " ", 
      SubscriptBox["T", "0"]}]], "//", "ToSI"}], "//", "N"}]}]], "Input",
 CellChangeTimes->{
  3.5020222776891813`*^9, {3.5874484518267136`*^9, 3.587448452682818*^9}}],

Cell[BoxData[
 TemplateBox[{"343.242080178145`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755459308355007`*^9, 3.5865827884332867`*^9, {3.5874483404247274`*^9, 
   3.587448346299466*^9}, {3.5874484319352193`*^9, 3.5874484543490157`*^9}, 
   3.587448541894024*^9, 3.5890778559482408`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["c", "0"], "//", 
  RowBox[{"ToUnit", "[", 
   RowBox[{"Kilometer", "/", "Hour"}], "]"}]}]], "Input",
 CellChangeTimes->{3.502022277693182*^9}],

Cell[BoxData[
 TemplateBox[{"1235.671488641322`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"km\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"h\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers per hour"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575545931056527*^9, 3.5865827884952946`*^9, {3.5874483405017233`*^9, 
   3.587448346358472*^9}, 3.5874484320942216`*^9, 3.587448541957013*^9, 
   3.5890778560102806`*^9}]
}, Open  ]],

Cell["\<\
Variation of speed of sound with altitude\
\>", "Text",
 CellChangeTimes->{{3.5874485569139233`*^9, 3.5874485615056176`*^9}, {
  3.587448630394096*^9, 3.5874486381931505`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"z", "=", 
   RowBox[{"10", " ", "Kilometer"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"aircraft", " ", "altitude"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["h", "2"], "=", 
   RowBox[{"30", " ", "Kilometer"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"homentropic", " ", "atmosphere", " ", "scale", " ", "height"}], 
   " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5020220466969695`*^9, 3.5020220752956057`*^9}, {
  3.5020222777011824`*^9, 3.5020223039296827`*^9}, {3.587448593538468*^9, 
  3.5874486153542643`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"c", "=", 
  RowBox[{
   SubscriptBox["c", "0"], 
   SqrtBox[
    RowBox[{"1", "-", 
     FractionBox["z", 
      SubscriptBox["h", "2"]]}]]}]}]], "Input",
 CellChangeTimes->{{3.5020220547634315`*^9, 3.502022064748002*^9}, 
   3.5020222777021823`*^9}],

Cell[BoxData[
 TemplateBox[{"280.2559848959758`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575545931214548*^9, 3.5865827885593023`*^9, {3.5874483405577064`*^9, 
   3.587448346416483*^9}, 3.5874484321552362`*^9, 3.587448542014013*^9, 
   3.5890778560733013`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"c", "//", 
  RowBox[{"ToUnit", "[", 
   RowBox[{"Kilometer", "/", "Hour"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5020221307487774`*^9, 3.5020221356090555`*^9}, 
   3.5020222777041826`*^9}],

Cell[BoxData[
 TemplateBox[{"1008.921545625513`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"km\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"h\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers per hour"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755459313465643`*^9, 3.586582788623351*^9, {3.587448340615714*^9, 
   3.5874483464754734`*^9}, 3.5874484322812843`*^9, 3.587448542076047*^9, 
   3.589077856139265*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Jeans instability", "Section",
 CellChangeTimes->{{3.477715883170868*^9, 3.4777158865220594`*^9}, {
  3.502022410374771*^9, 3.5020224138629704`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.477715896523631*^9, 3.4777158991177797`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(9\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.575545931618594*^9, 3.5755459747287693`*^9, 
  3.575546025236827*^9, 3.5755462786642356`*^9, 3.5755464464443674`*^9, 
  3.586582788796376*^9, 3.5874483466385126`*^9, 3.5874489349113708`*^9, 
  3.587449280640788*^9, 3.5874493226520205`*^9, 3.589077856314316*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.5874487241209*^9, 3.5874487257931137`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{"2", " ", 
    SuperscriptBox["10", 
     RowBox[{"3", " "}]], 
    RowBox[{"ProtonMass", "/", 
     SuperscriptBox["Centimeter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "corresponds", " ", "to", " ", "1000", " ", "hydrogen", " ", "molecules", 
    " ", "per", " ", 
    SuperscriptBox["cm", "3"]}], " ", "*)"}], " ", 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["M", "mol"], "=", 
   RowBox[{
    RowBox[{"AvogadroConstant", " ", "*", "2", "ProtonMass"}], "//", 
    "ToSI"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "this", " ", "is", " ", "slightly", " ", "wrong", " ", "in", " ", "book"}],
    " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["T", "0"], "=", 
   RowBox[{"3", " ", "Kelvin"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"G", "=", "GravitationalConstant"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["R", "mol"], "=", "MolarGasConstant"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Lightyear", "=", 
   RowBox[{"SpeedOfLight", "*", "Year"}]}], "  ", 
  RowBox[{"(*", 
   RowBox[{"Lightyear", "=", 
    RowBox[{
     RowBox[{"Quantity", "[", "\"\<Lightyears\>\"", "]"}], "  ", "too", " ", 
     "slow", " ", "to", " ", "load"}]}], " ", "*)"}], " ", ";"}]}], "Input",
 CellChangeTimes->{{3.477715906839221*^9, 3.4777159930181503`*^9}, {
   3.4777169634976587`*^9, 3.4777169802676177`*^9}, {3.4777174234029636`*^9, 
   3.4777174647503285`*^9}, 3.4777179049185047`*^9, {3.477717944768784*^9, 
   3.477718017939969*^9}, {3.477718088553008*^9, 3.477718225863862*^9}, {
   3.4777183490509076`*^9, 3.4777183561953163`*^9}, 3.4777184252772675`*^9, {
   3.477718547841278*^9, 3.477718548880337*^9}, 3.502022432896059*^9, {
   3.5020228385722623`*^9, 3.5020228663558517`*^9}, {3.5755462529591513`*^9, 
   3.575546265822696*^9}, {3.575546415858698*^9, 3.575546419935187*^9}, {
   3.5874487863387227`*^9, 3.5874487892980485`*^9}, {3.5874488927870874`*^9, 
   3.5874489229778976`*^9}, 3.587448957866268*^9, {3.5874489955060005`*^9, 
   3.5874490304743853`*^9}, {3.5874490908739567`*^9, 3.587449102946462*^9}, {
   3.587449136266659*^9, 3.587449159402523*^9}, {3.587449204411199*^9, 
   3.587449268826267*^9}, {3.587449607673854*^9, 3.5874496302976913`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.00201455293256068460733047896956636782`6.749280476867381"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"mol\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms per mole"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", "\"Moles\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.587449237418383*^9, 3.587449280789833*^9}, 
   3.5874493228130417`*^9, 3.5890778564873433`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Reslts", "Subsection",
 CellChangeTimes->{{3.5874495123528414`*^9, 3.587449515520277*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Rho]", "0"], "//", "ToSI"}]], "Input",
 CellChangeTimes->{{3.477718358808466*^9, 3.4777183623996716`*^9}}],

Cell[BoxData[
 TemplateBox[{
  "3.345243552996553344290418104986062561135`7.053283984852396*^-18"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.57554593183362*^9, 3.5755459750138016`*^9, 
  3.5755460255218625`*^9, 3.5755462840268793`*^9, 3.5755464468294125`*^9, 
  3.5865827890064*^9, 3.5874483468355055`*^9, 3.5874489351123905`*^9, 
  3.587449280833769*^9, 3.5874493228660483`*^9, 3.5890778565333147`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"R", "=", 
   FractionBox[
    SubscriptBox["R", "mol"], 
    SubscriptBox["M", "mol"]]}], ";"}]], "Input",
 CellChangeTimes->{{3.502022970761823*^9, 3.502022983345543*^9}, 
   3.587449190348465*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["c", "0"], "=", 
  RowBox[{
   SqrtBox[
    RowBox[{"R", " ", 
     SubscriptBox["T", "0"]}]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.47771600000355*^9, 3.4777160269860935`*^9}, {
  3.50202299535623*^9, 3.502022997714365*^9}}],

Cell[BoxData[
 TemplateBox[{"111.27263303873929217915720402562114957302`6.003880718053775"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575545932432694*^9, 3.5755459754018493`*^9, 
  3.5755460259109106`*^9, 3.575546284516938*^9, 3.575546447218461*^9, 
  3.586582789271405*^9, 3.587448347099546*^9, 3.5874489353213844`*^9, 
  3.587449281018835*^9, 3.5874493230651007`*^9, 3.589077856733384*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Lambda]", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["c", "0"], 
    SqrtBox[
     FractionBox["\[Pi]", 
      RowBox[{"G", " ", 
       SubscriptBox["\[Rho]", "0"]}]]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4777160367366505`*^9, 3.477716059716965*^9}}],

Cell[BoxData[
 TemplateBox[{"1.31996232250194609402099444762161511`3.91755089027608*^16"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575545932808739*^9, 3.5755459757548923`*^9, 
  3.575546026264952*^9, 3.5755462849059834`*^9, 3.5755464475825033`*^9, 
  3.5865827894734507`*^9, 3.5874483473056126`*^9, 3.58744893551143*^9, 
  3.587449281214859*^9, 3.5874493232700977`*^9, 3.5890778569374094`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Lambda]", "//", 
  RowBox[{"ToUnit", "[", "Lightyear", "]"}]}]], "Input",
 CellChangeTimes->{{3.4777186106008673`*^9, 3.4777186159351726`*^9}, {
  3.5755463045593433`*^9, 3.5755463128433385`*^9}}],

Cell[BoxData[
 TemplateBox[{"1.39615689307779370927573860272415201673`3.91755089027608"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"yr\"", "\[ThinSpace]", 
            StyleBox["\"c\"", Italic, StripOnInput -> False]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: year speed of light in vacuum"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       RowBox[{"\"SpeedOfLight\"", " ", "\"Years\""}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755459328467436`*^9, 3.575545975880905*^9, 3.5755460263939676`*^9, {
   3.5755462850760045`*^9, 3.575546314483533*^9}, 3.575546447795531*^9, 
   3.5865827895174665`*^9, 3.587448347345625*^9, 3.5874489355304527`*^9, 
   3.587449281237862*^9, 3.587449323295102*^9, 3.589077856975408*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Lambda]", "/", 
   SqrtBox["\[Pi]"]}], "//", 
  RowBox[{"ToUnit", "[", "Parsec", "]"}]}]], "Input",
 CellChangeTimes->{{3.4777160747218237`*^9, 3.4777160813332014`*^9}, {
  3.477716115173137*^9, 3.4777161351822815`*^9}, {3.4777174395068846`*^9, 
  3.477717440573946*^9}, {3.4777175267768764`*^9, 3.4777175342603045`*^9}, {
  3.477717575056638*^9, 3.47771758016893*^9}, {3.477717925344673*^9, 
  3.4777179256466904`*^9}, {3.4777182401616793`*^9, 3.477718254622507*^9}, {
  3.4777185672063856`*^9, 3.477718569919541*^9}}],

Cell[BoxData[
 TemplateBox[{"0.24134374812970726151748153823949990496`3.91755089027608"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"pc\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: parsecs"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Parsecs\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755459328937497`*^9, 3.575545975919912*^9, 
  3.575546026432971*^9, 3.575546285123015*^9, 3.575546447845536*^9, 
  3.586582789735494*^9, 3.587448347368559*^9, 3.5874489355484324`*^9, 
  3.587449281259865*^9, 3.5874493233181086`*^9, 3.589077857014412*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["M", "J"], "=", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"4", "\[Pi]"}], "3"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["\[Lambda]", "2"], ")"}], "3"], 
    SubscriptBox["\[Rho]", "0"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4777188090552187`*^9, 3.4777188432791758`*^9}}],

Cell[BoxData[
 TemplateBox[{"4.0281994781461979253941624686`3.4403237401763924*^30"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kg\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilograms\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575545933039767*^9, 3.575545976059929*^9, 
  3.575546026566987*^9, 3.5755462852660265`*^9, 3.5755464479965525`*^9, 
  3.586582790094536*^9, 3.587448347442607*^9, 3.587448935619475*^9, 
  3.5874492813298397`*^9, 3.5874493233951397`*^9, 3.5890778570854006`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["M", "sun"], "=", "SunMass"}]], "Input",
 CellChangeTimes->{{3.477718901373499*^9, 3.477718917985449*^9}}],

Cell[BoxData[
 TemplateBox[{"1.9891`*^30"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kg\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilograms\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755459330787716`*^9, 3.575545976095933*^9, 
  3.575546026602993*^9, 3.575546285303034*^9, 3.5755464480445576`*^9, 
  3.586582790173588*^9, 3.5874483474666104`*^9, 3.587448935639457*^9, 
  3.587449281348835*^9, 3.587449323416117*^9, 3.5890778571043873`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox[
  SubscriptBox["M", "J"], 
  SubscriptBox["M", "sun"]]], "Input",
 CellChangeTimes->{{3.477718921394644*^9, 3.4777189335033364`*^9}}],

Cell[BoxData["2.025136734274897`"], "Output",
 CellChangeTimes->{3.5755459331857834`*^9, 3.575545976200945*^9, 
  3.5755460267090063`*^9, 3.5755462854090443`*^9, 3.575546448162572*^9, 
  3.586582791325692*^9, 3.58744834751862*^9, 3.5874489356934977`*^9, 
  3.58744928140289*^9, 3.587449323473124*^9, 3.589077857157434*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 14.2 Steady compressible flow (p. 233)\
\>", "Title",
 CellChangeTimes->{{3.5020231789587317`*^9, 3.5020231876532288`*^9}, {
  3.587449583736843*^9, 3.5874495888154774`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 14.3  Mach numbers (p. 233)\
\>", "Section",
 CellChangeTimes->{{3.5020232092474637`*^9, 3.5020232136847177`*^9}, {
  3.5874496527605143`*^9, 3.5874496558798623`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(11\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755463466673956`*^9, 3.5755465138914595`*^9, 
  3.5865827917327614`*^9, 3.589077857332416*^9}]
}, Open  ]],

Cell["\<\
Velocities and sound speeds for waving, driving, and flying\
\>", "Text",
 CellChangeTimes->{{3.587449663295822*^9, 3.5874496771595736`*^9}, {
  3.5874497241604366`*^9, 3.5874497357919617`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"v", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"3", " ", 
      RowBox[{"Meter", "/", "Second"}]}], ",", 
     RowBox[{"120", " ", 
      RowBox[{"Kilometer", "/", "Hour"}]}], ",", 
     RowBox[{"900", " ", 
      RowBox[{"Kilometer", "/", "Hour"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"c", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"343.", ",", "343.", ",", "280."}], "}"}], 
    RowBox[{"Meter", "/", "Second"}]}]}], ";"}]}], "Input"],

Cell["Mach numbers", "Text",
 CellChangeTimes->{{3.5874497443920326`*^9, 3.5874497479434614`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ma", "=", 
  RowBox[{
   FractionBox["v", "c"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.008746355685131196`", ",", "0.09718172983479105`", ",", 
   "0.8928571428571427`"}], "}"}]], "Output",
 CellChangeTimes->{3.575546347398484*^9, 3.5755465145985456`*^9, 
  3.5865827925368595`*^9, 3.589077857679504*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Thread", "[", 
   RowBox[{"{", 
    RowBox[{"v", ",", "c", ",", "Ma"}], "}"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5020234797869377`*^9, 3.5020234859202886`*^9}, {
  3.502023538643304*^9, 3.5020235405364122`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      TemplateBox[{"3"},
       "QuantityUnit",
       DisplayFunction->(TooltipBox[
         StyleBox[
          RowBox[{#, "\[InvisibleSpace]", 
            StyleBox[
             RowBox[{" ", 
               
               RowBox[{
                "\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
                 "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
          ShowStringCharacters -> False], "Unit: meters per second"]& ),
       InterpretationFunction->(RowBox[{"Quantity", "[", 
          RowBox[{#, ",", 
            FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )], 
      TemplateBox[{"343.`"},
       "QuantityUnit",
       DisplayFunction->(TooltipBox[
         StyleBox[
          RowBox[{#, "\[InvisibleSpace]", 
            StyleBox[
             RowBox[{" ", 
               
               RowBox[{
                "\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
                 "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
          ShowStringCharacters -> False], "Unit: meters per second"]& ),
       InterpretationFunction->(RowBox[{"Quantity", "[", 
          RowBox[{#, ",", 
            FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )], 
      "0.008746355685131196`"},
     {
      TemplateBox[{"120"},
       "QuantityUnit",
       DisplayFunction->(TooltipBox[
         StyleBox[
          RowBox[{#, "\[InvisibleSpace]", 
            StyleBox[
             RowBox[{" ", 
               
               RowBox[{
                "\"km\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
                 "\"h\""}]}], "QuantityUnitTraditionalLabel"]}], 
          ShowStringCharacters -> False], "Unit: kilometers per hour"]& ),
       InterpretationFunction->(RowBox[{"Quantity", "[", 
          RowBox[{#, ",", 
            FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )], 
      TemplateBox[{"343.`"},
       "QuantityUnit",
       DisplayFunction->(TooltipBox[
         StyleBox[
          RowBox[{#, "\[InvisibleSpace]", 
            StyleBox[
             RowBox[{" ", 
               
               RowBox[{
                "\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
                 "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
          ShowStringCharacters -> False], "Unit: meters per second"]& ),
       InterpretationFunction->(RowBox[{"Quantity", "[", 
          RowBox[{#, ",", 
            FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )], 
      "0.09718172983479105`"},
     {
      TemplateBox[{"900"},
       "QuantityUnit",
       DisplayFunction->(TooltipBox[
         StyleBox[
          RowBox[{#, "\[InvisibleSpace]", 
            StyleBox[
             RowBox[{" ", 
               
               RowBox[{
                "\"km\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
                 "\"h\""}]}], "QuantityUnitTraditionalLabel"]}], 
          ShowStringCharacters -> False], "Unit: kilometers per hour"]& ),
       InterpretationFunction->(RowBox[{"Quantity", "[", 
          RowBox[{#, ",", 
            FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )], 
      TemplateBox[{"280.`"},
       "QuantityUnit",
       DisplayFunction->(TooltipBox[
         StyleBox[
          RowBox[{#, "\[InvisibleSpace]", 
            StyleBox[
             RowBox[{" ", 
               
               RowBox[{
                "\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
                 "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
          ShowStringCharacters -> False], "Unit: meters per second"]& ),
       InterpretationFunction->(RowBox[{"Quantity", "[", 
          RowBox[{#, ",", 
            FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )], 
      "0.8928571428571427`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.575546347472492*^9, 3.5755465146395507`*^9, 
  3.5865827926298265`*^9, 3.5890778577005033`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 14.4 Stagnation  temperature  rise (p. 235)\
\>", "Section",
 CellChangeTimes->{{3.502023714751377*^9, 3.5020237177015457`*^9}, {
  3.587450352247357*^9, 3.5874503958227744`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(3\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.5755463477235236`*^9, 3.575546514899582*^9, 3.5865827928338914`*^9, {
   3.5874501786845274`*^9, 3.5874502037286716`*^9}, 3.589077857910529*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"R", "=", 
   FractionBox["MolarGasConstant", "AirMolarMass"]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"specific", " ", "gas", " ", "constant", " ", "for", " ", "air"}], 
   " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Gamma]", "=", 
   FractionBox["7", "5"]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5874498400170417`*^9, 3.587449912305101*^9}}],

Cell[BoxData[
 TemplateBox[{"34.52621402800076`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"mol\"", "\[ThinSpace]", 
            StyleBox["\"R\"", Italic, StripOnInput -> False], 
            "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", "\"kg\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: mole molar gas constants per kilogram"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox[
        RowBox[{"\"MolarGasConstant\"", " ", "\"Moles\""}], 
        "\"Kilograms\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.5874501788125935`*^9, 3.587450203853675*^9}, 
   3.5890778580375443`*^9}]
}, Open  ]],

Cell["\<\
Specific heat at constant pressure\
\>", "Text",
 CellChangeTimes->{{3.58745001583206*^9, 3.5874500504634633`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["c", "p"], "=", 
   RowBox[{
    FractionBox["\[Gamma]", 
     RowBox[{"\[Gamma]", "-", "1"}]], "R"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5874500351955347`*^9, 3.5874500379918833`*^9}, {
  3.5874500870890503`*^9, 3.5874500988565273`*^9}}],

Cell["Stagnation point rise", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalDelta]T", "=", 
   FractionBox[
    SuperscriptBox["U", "2"], 
    RowBox[{"2", 
     SubscriptBox["c", "p"]}]]}], ";"}]], "Input",
 CellChangeTimes->{
  3.502023776044883*^9, 3.5020238067016363`*^9, {3.5874499182888684`*^9, 
   3.587449920235113*^9}, {3.5874501095838833`*^9, 3.587450115600629*^9}}],

Cell["\<\
Velocities for driving, flying, Concorde, Space Shuttle\
\>", "Text",
 CellChangeTimes->{{3.5874501236476393`*^9, 3.587450168703351*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Us", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"100", " ", 
      RowBox[{"Kilometer", "/", "Hour"}]}], ",", 
     RowBox[{"900", 
      RowBox[{"Kilometer", "/", "Hour"}]}], ",", 
     RowBox[{"2300", 
      RowBox[{"Kilometer", "/", "Hour"}]}], ",", 
     RowBox[{"3", " ", 
      RowBox[{"Kilometer", "/", "Second"}]}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{3.502023875504572*^9}],

Cell["Stagnation point rises", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]Ts", "=", 
  RowBox[{
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{"Function", "[", 
      RowBox[{"U", ",", 
       RowBox[{"\[CapitalDelta]T", "//", "Evaluate"}]}], "]"}], ",", "Us"}], 
    "]"}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.5020237601859756`*^9, 3.5020238161041737`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TemplateBox[{"0.3839846309932258`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
       ShowStringCharacters -> False], "Unit: kelvins"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )], ",", 
   TemplateBox[{"31.102755110451287`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
       ShowStringCharacters -> False], "Unit: kelvins"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )], ",", 
   TemplateBox[{"203.12786979541644`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
       ShowStringCharacters -> False], "Unit: kelvins"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )], ",", 
   TemplateBox[{"4478.796735904985`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
       ShowStringCharacters -> False], "Unit: kelvins"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5755463491096897`*^9, 3.575546516186734*^9, 3.586582794470153*^9, {
   3.587450179414627*^9, 3.5874502044397707`*^9}, 3.589077859013669*^9}]
}, Open  ]],

Cell["Make pretty", "Text",
 CellChangeTimes->{{3.5874502851439333`*^9, 3.587450286759177*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Thread", "[", 
   RowBox[{"{", 
    RowBox[{"Us", ",", "\[CapitalDelta]Ts"}], "}"}], "]"}], "//", 
  "TableForm"}]], "Input",
 CellChangeTimes->{{3.5874503247609053`*^9, 3.5874503258280134`*^9}}],

Cell[BoxData[
 TagBox[GridBox[{
    {
     TemplateBox[{"100"},
      "QuantityUnit",
      DisplayFunction->(TooltipBox[
        StyleBox[
         RowBox[{#, "\[InvisibleSpace]", 
           StyleBox[
            RowBox[{" ", 
              
              RowBox[{"\"km\"", "\[InvisibleSpace]", "\"/\"", 
                "\[InvisibleSpace]", "\"h\""}]}], 
            "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False],
         "Unit: kilometers per hour"]& ),
      InterpretationFunction->(RowBox[{"Quantity", "[", 
         RowBox[{#, ",", 
           FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )], 
     TemplateBox[{"0.3839846309932258`"},
      "QuantityUnit",
      DisplayFunction->(TooltipBox[
        StyleBox[
         RowBox[{#, "\[InvisibleSpace]", 
           StyleBox[
            RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
         ShowStringCharacters -> False], "Unit: kelvins"]& ),
      InterpretationFunction->(RowBox[{"Quantity", "[", 
         RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]},
    {
     TemplateBox[{"900"},
      "QuantityUnit",
      DisplayFunction->(TooltipBox[
        StyleBox[
         RowBox[{#, "\[InvisibleSpace]", 
           StyleBox[
            RowBox[{" ", 
              
              RowBox[{"\"km\"", "\[InvisibleSpace]", "\"/\"", 
                "\[InvisibleSpace]", "\"h\""}]}], 
            "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False],
         "Unit: kilometers per hour"]& ),
      InterpretationFunction->(RowBox[{"Quantity", "[", 
         RowBox[{#, ",", 
           FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )], 
     TemplateBox[{"31.102755110451287`"},
      "QuantityUnit",
      DisplayFunction->(TooltipBox[
        StyleBox[
         RowBox[{#, "\[InvisibleSpace]", 
           StyleBox[
            RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
         ShowStringCharacters -> False], "Unit: kelvins"]& ),
      InterpretationFunction->(RowBox[{"Quantity", "[", 
         RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]},
    {
     TemplateBox[{"2300"},
      "QuantityUnit",
      DisplayFunction->(TooltipBox[
        StyleBox[
         RowBox[{#, "\[InvisibleSpace]", 
           StyleBox[
            RowBox[{" ", 
              
              RowBox[{"\"km\"", "\[InvisibleSpace]", "\"/\"", 
                "\[InvisibleSpace]", "\"h\""}]}], 
            "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False],
         "Unit: kilometers per hour"]& ),
      InterpretationFunction->(RowBox[{"Quantity", "[", 
         RowBox[{#, ",", 
           FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )], 
     TemplateBox[{"203.12786979541644`"},
      "QuantityUnit",
      DisplayFunction->(TooltipBox[
        StyleBox[
         RowBox[{#, "\[InvisibleSpace]", 
           StyleBox[
            RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
         ShowStringCharacters -> False], "Unit: kelvins"]& ),
      InterpretationFunction->(RowBox[{"Quantity", "[", 
         RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]},
    {
     TemplateBox[{"3"},
      "QuantityUnit",
      DisplayFunction->(TooltipBox[
        StyleBox[
         RowBox[{#, "\[InvisibleSpace]", 
           StyleBox[
            RowBox[{" ", 
              
              RowBox[{"\"km\"", "\[InvisibleSpace]", "\"/\"", 
                "\[InvisibleSpace]", "\"s\""}]}], 
            "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False],
         "Unit: kilometers per second"]& ),
      InterpretationFunction->(RowBox[{"Quantity", "[", 
         RowBox[{#, ",", 
           FractionBox["\"Kilometers\"", "\"Seconds\""]}], "]"}]& )], 
     TemplateBox[{"4478.796735904985`"},
      "QuantityUnit",
      DisplayFunction->(TooltipBox[
        StyleBox[
         RowBox[{#, "\[InvisibleSpace]", 
           StyleBox[
            RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
         ShowStringCharacters -> False], "Unit: kelvins"]& ),
      InterpretationFunction->(RowBox[{"Quantity", "[", 
         RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.575546349324715*^9, 3.5755465163377523`*^9, 3.586582794557096*^9, {
   3.5874501794876137`*^9, 3.5874502044617877`*^9}, 3.5874503262270412`*^9, 
   3.5890778590776854`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Figure 14.1 Sonic transition in air (p. 236)\
\>", "Section",
 CellChangeTimes->{{3.502089041654758*^9, 3.502089046886057*^9}, {
  3.5874502470950994`*^9, 3.587450279135172*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(8\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.575546349590748*^9, 3.5755465166117873`*^9, 
  3.586582796289324*^9, 3.587450179660674*^9, 3.5890778596937723`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "=", 
   RowBox[{"7", "/", "5"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Tratio", "=", 
   FractionBox["1", 
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["Ma", "2"], "-", "1"}], ")"}], 
      FractionBox[
       RowBox[{"\[Gamma]", "-", "1"}], 
       RowBox[{"\[Gamma]", "+", "1"}]]}]}]]}], ";"}]], "Input",
 CellChangeTimes->{{3.3901208422713547`*^9, 3.390120872985355*^9}, {
  3.428373100342216*^9, 3.428373109382216*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]ratio", "=", 
   SuperscriptBox["Tratio", 
    RowBox[{" ", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]}]]}], ";"}]], "Input",
 CellChangeTimes->{3.3901208789363546`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pratio", "=", 
   SuperscriptBox["Tratio", 
    RowBox[{"\[Gamma]", "/", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}], ";"}]], "Input",
 CellChangeTimes->{3.3901208824243546`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vratio", "=", 
   RowBox[{"Ma", " ", 
    SqrtBox["Tratio"]}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.3901208854093547`*^9, {3.4283731192152157`*^9, 3.428373119647216*^9}, {
   3.4715092151619005`*^9, 3.4715092154769187`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Aratio", "=", 
   RowBox[{
    FractionBox["1", "Ma"], " ", 
    SuperscriptBox["Tratio", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{"\[Gamma]", "+", "1"}], ")"}]}], "/", "2"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.390120888216355*^9, {3.4283731237992163`*^9, 3.428373134290216*^9}, 
   3.4283731783002157`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]Uratio", "=", 
   FractionBox["1", "Aratio"]}], ";"}]], "Input",
 CellChangeTimes->{{3.428634334364613*^9, 3.428634342282613*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Ahigh", "=", 
   RowBox[{
    SuperscriptBox["Ma", 
     RowBox[{"2", "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{"\[Gamma]", "-", "1"}], 
       RowBox[{"\[Gamma]", "+", "1"}]], ")"}], 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"\[Gamma]", "+", "1"}], ")"}], "/", "2"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.428374458103216*^9, 3.428374624901216*^9}, {
   3.4283746970172157`*^9, 3.428374794790216*^9}, {3.428375020114216*^9, 
   3.428375075199216*^9}, {3.428375278363216*^9, 3.428375303691216*^9}, 
   3.4283754552742157`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Alow", "=", 
   RowBox[{
    FractionBox["1", "Ma"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["2", 
       RowBox[{"\[Gamma]", "+", "1"}]], ")"}], 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"\[Gamma]", "+", "1"}], ")"}], "/", "2"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.428374644593216*^9, 3.4283746903992157`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Aratio", ",", "Ahigh", ",", "Alow"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Ma", ",", "0", ",", "4.2"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "6"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.428374855810216*^9, 3.428374871871216*^9}, {
  3.4283749341092157`*^9, 3.428374949772216*^9}, {3.4283749978332157`*^9, 
  3.428374997960216*^9}, {3.428375423180216*^9, 3.4283754232732162`*^9}, {
  3.428375469632216*^9, 3.428375478747216*^9}, {3.428375786959216*^9, 
  3.428375840647216*^9}, {3.4678808313551416`*^9, 3.4678808601437883`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJw1mHk4lO37/8e+VkT2XaQn2bKkcF4pWSKF9ChKPZJosVS2LC0kSckWLdai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     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwdlGcg1v/fxe2QcVE2hQiRLcT1eX8lRSqyosio7IRIWdmUkb1nlF3ZZWRn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     "]]}, 
   {Hue[0.1421359549995791, 0.6, 0.6], LineBox[CompressedData["
1:eJwdkmk41XkDho996eBvzZKkFEpIQ1E8vwaTylbC1FCSJNpMNcmSpZpkmjay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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 4.2}, {0, 6}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.5755463498187723`*^9, 3.5755465167398024`*^9, 
  3.5865827967893867`*^9, 3.587450179778648*^9, 3.589077860704914*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figa", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "Aratio", ",", "Tratio", ",", "pratio", ",", "\[Rho]ratio", ",", 
      "vratio"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"Ma", ",", "0", ",", "3"}], "}"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<Ma\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "3.6"}], "}"}]}], ",", 
    RowBox[{"ColorFunction", "\[Rule]", 
     RowBox[{"Function", "[", 
      RowBox[{
       RowBox[{"{", "}"}], ",", "Black"}], "]"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "}"}], ",", 
       RowBox[{"Dashing", "[", "Large", "]"}], " ", ",", 
       RowBox[{"Dashing", "[", "Medium", "]"}], ",", 
       RowBox[{"Dashing", "[", "Small", "]"}], ",", 
       RowBox[{"Dashing", "[", "Tiny", "]"}]}], "}"}]}], ",", 
    RowBox[{"Epilog", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Text", "[", 
        RowBox[{"\"\<subsonic\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"0.7", ",", "2.8"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\<supersonic\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"1.3", ",", "2.8"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(A\), \(\\\ \)]\)/\!\(\*SubscriptBox[\(A\), \
\(1\)]\)\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"2.4", ",", "2.7"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(T\), \(\\\ \)]\)/\!\(\*SubscriptBox[\(T\), \
\(1\)]\)\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "0.8"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\<\[Rho]/\!\(\*SubscriptBox[\(\[Rho]\), \(1\)]\)\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "0.5"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(p\), \(\\\ \)]\)/\!\(\*SubscriptBox[\(p\), \
\(1\)]\)\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"1.5", ",", "0.4"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(v\), \(\\\ \)]\)/\!\(\*SubscriptBox[\(v\), \
\(1\)]\)\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"2.4", ",", "1.9"}], "}"}]}], "]"}]}], "}"}]}], ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.3901208986603546`*^9, 3.390120908036355*^9}, {
   3.427694900720812*^9, 3.427694953076812*^9}, {3.427695030812812*^9, 
   3.4276950321328115`*^9}, {3.4276951058338118`*^9, 
   3.4276951089148116`*^9}, {3.4276951484298115`*^9, 
   3.4276952947898116`*^9}, {3.4276953442308116`*^9, 
   3.4276953851578116`*^9}, {3.4276954369198112`*^9, 
   3.4276954655248117`*^9}, {3.427695512003812*^9, 3.4276955573998117`*^9}, {
   3.4276957282178116`*^9, 3.4276958180708113`*^9}, {3.4276959993398113`*^9, 
   3.4276961529128113`*^9}, {3.4276962002618113`*^9, 
   3.4276962022788115`*^9}, {3.4276962464908113`*^9, 3.4276962542178116`*^9}, 
   3.4276962998878117`*^9, {3.4276963640348115`*^9, 3.427696384154812*^9}, {
   3.42829220862265*^9, 3.42829222656065*^9}, {3.42829242294065*^9, 
   3.42829246269565*^9}, {3.4282925066536503`*^9, 3.4282925107676497`*^9}, {
   3.4283729665502157`*^9, 3.428372993664216*^9}, {3.428373138008216*^9, 
   3.428373144466216*^9}, {3.428374183209216*^9, 3.428374214650216*^9}, {
   3.428374245945216*^9, 3.428374336309216*^9}, {3.4283748144692163`*^9, 
   3.428374819992216*^9}, 3.428374853850216*^9, {3.4283758467072163`*^9, 
   3.428375850710216*^9}, {3.4283759103982162`*^9, 3.4283759358662157`*^9}, {
   3.428375994008216*^9, 3.428375994437216*^9}, {3.428376035183216*^9, 
   3.428376042603216*^9}, {3.4286341580716133`*^9, 3.428634163057613*^9}, {
   3.428634294896613*^9, 3.428634305032613*^9}, {3.428634353507613*^9, 
   3.4286343864726133`*^9}, {3.428634465414613*^9, 3.428634468174613*^9}, {
   3.428634508904613*^9, 3.428634519273613*^9}, {3.428810234980406*^9, 
   3.428810262334406*^9}, {3.467880802986519*^9, 3.4678808090228643`*^9}, 
   3.4678808891204453`*^9, {3.4678809219773245`*^9, 3.467880936706167*^9}, {
   3.4678809716671667`*^9, 3.4678809799216394`*^9}, {3.4711639449982677`*^9, 
   3.4711639517476535`*^9}, {3.471509223005349*^9, 3.471509223083354*^9}}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF23dYze//B/D2KaFQadAgaUpDKvI6Eg2riIRSqbQ0jBLaGtKiEKKSJA2E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   "], {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwl1Hl8z3UcB/DfZmNGzJhO7WA2KtOhy3TYKlRsS0o3yiq1VUrk6qa73N3o
IESuig7KkSIUhVS6CJVO0eF4vh/+eO71+by+j/32+33en99ye1ZXVCUlEoly
PyprJBIXWp9ovUsu5AmSqav7WI6hkva01v0u5/MQRWTrvpGvcAfd8LKJg/Qr
5ViupYRjdX/It3g4Xo8c3bdyCgO5iBTq6VfJcVxHafw+ufrv5FQGcTGp1Nev
lk9yPWdxHHn67+U0BtOdmmToP5FP0YezOZ6m+h/kqwzhEmrRQP+pfJobOIcT
aKbfLKczlEtJI1O/Rj7DjXSgDfn6LXIGd3IZtWmoXyufpYqOMRea63+Ur3EX
l5NOI/1n8jmq6cRJFOi3ypnczRXUIUv/uXyemziXkynUb5OzuIcrY/Y01q+T
47mZ8ziFFvrtcjb3clXMmoP16+UEbuH8OH/dn/JtHuFUWup+knO4jx7xvnW7
5XuMiNlziG6DnEhf2tr/L5fQOWZl/5d8h0fjOUfpfpZzuZ/T7PfJZfSMz2v/
j3w/5m1di5HWZ8pklsc5yjrUtz5UfiFfoMQ6hVuti+We+IxyadwFmUYZXeJe
yJ1xZvLduIuyZjzjMfszZBKd+ShmLdOTDnwni+2Plr/EjOTrslSmMsz69Pjy
8mHcKVmbXjEb+a9cxCja29dghXUGh1lvlC9yG+3s98oPKIu7af+3XMDj8Zxj
dDvkGwzn6pih7j+5mNE04HDdl/Il+lEeZ08r/a/yTR7gGjI5Qv+VfJnbqYjP
RZH+NzmPB+lNQ5rov5aT6M8FcYY04kjPNsnJDKBrzJOsuMNxL+MuxSzjDOK9
xt+P14zfJZsccsmjKc3IpzkFFNIi7m7csZhPnE28P//o9gPZmI91
      "],
      VertexColors->Automatic]}, 
    {Hue[0.9060679774997897, 0.6, 0.6], Dashing[Large], 
     LineBox[CompressedData["
1:eJwV0ddaDgAAgOG/S3FLDakU/RqKhpBCKk1RaSCjgQYhRSEUDRElDdI2kqyu
odfB+zzf+bcjmBGaHhIIBBoJFbM84w8d/GSKf1ziO+/4y2N+Mc1Z1njNQzb5
wE1+MEk/xSwyTBdNrDPBE2r5yjh9tBHGHM+5RQPn+MIbemmlhCVGuE8zdYTz
kQFuc5lKSllmlG5aqCeCTwzSyRWqKGOFV/RwnQvsZJ4X3OEq1ZQTyWdecpdr
nOcMu4gimhh2E0sce9hLPEH2kUAiSSSznxRSOcBB0vg/NINMDpHFYY5wlGyO
kUMuxznBSfI4RT4FFHKaBYa4RyM1VLDKGA+4wUW+8ZZHtLPBe57ymxm2KGIb
flplDg==
      "],
      VertexColors->Automatic]}, 
    {Hue[0.1421359549995791, 0.6, 0.6], Dashing[Medium], 
     LineBox[CompressedData["
1:eJwV0Gkz0AEQB+A/Ih2UCskVHXKG6FChFEru+5Y7hHIkhM/gi+YKoXi8eOa3
M7s7s7NpgzNN0yFBEGzyWzEWGgSN/KKMnwyxTgEL9LNGLSuMkME3uvnAMsO8
5AefqSeNr7TznmcsMkAduczRSw2v2XLXuGyinEIe8Z0ePvKKdKbpoJLn5LFt
/4ts5g1PyOQeM3RSxQses2N+QrbwliKyuM+u3qRspYJisnnAnt4f9jngkL8c
ccwJ//jP6fnT7YQQShgXCCeCi0Ryictc4SpRRHON68Rwg5vcIpY44rlNAndI
JIlkUkhlijbe8ZQcHjJLF9WUkM88fXyilCUGaWCVUTa4yxk4fzQA
      "],
      VertexColors->Automatic]}, 
    {Hue[0.37820393249936934`, 0.6, 0.6], Dashing[Small], 
     LineBox[CompressedData["
1:eJwNz9leTmEUwOH9NVCUKOS0W3IJLiDXoFEhadKENFAUGoQG0kCRKKWUhEoD
FQ2EwnPw/P5rrXef7KQTJ48nh4IgKOZUWBD8ZYEj4UEwqof0tyZqn8bqDz2q
s5qgO9pGtHldZ4g3/9FXxJm3dZ5mIu2rOsUgB+w/dY7H7Ldv6kdGSOEfi4zx
hHb2+W5DPzBMC3vc1vQdL+gmlcB9Sd/wlA7ustf9m04zRA9phNyXdZx+Omkl
nTBvX3SCAR5yjwzCvX3VtzzjEffJ5DRZZHOGs5wjh/PkcoE88imgkCIuUkwJ
pZRRziUuc4UKrlJJFdXUcI3r1FLHDW5STwO3uM0dGonwDys6yXO6eECU+3d9
z0t6iXHb0k+85qD9l37msHlXj2mT/gf+x2QB
      "],
      VertexColors->Automatic]}, 
    {Hue[0.6142719099991583, 0.6, 0.6], Dashing[Tiny], LineBox[CompressedData["
1:eJwNw9k2QlEAANBj+QoylUQikpIpKlOU4TYgJa56zf+/tfdaOxn/R7OFEELb
pcUQxn7a9dW2Ty6bcMVV11x3w6QpN027ZcZtd8y6a84998174KEFjyx6bMmy
J1Y89cxzL7y06pXX1qzb8MZb77z3waY/Duz55rMtf/2yb+SLsUPf7fjnyA8n
fjv10TnvRhXb
      "],
      VertexColors->Automatic]}},
   VertexColors->CompressedData["
1:eJztwUENACAMBLAjKJklJJAQnvj/DSFta791RpI7AwAAAAAAAHwNwmUCFQ==

    "]],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["\"Ma\"", TraditionalForm], 
    FormBox["\"\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  Epilog->{
    InsetBox[
     FormBox["\"subsonic\"", TraditionalForm], {0.7, 2.8}], 
    InsetBox[
     FormBox["\"supersonic\"", TraditionalForm], {1.3, 2.8}], 
    InsetBox[
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(A\\), \\(\\\\ \\)]\\)/\\!\\(\\*SubscriptBox[\
\\(A\\), \\(1\\)]\\)\"", TraditionalForm], {2.4, 2.7}], 
    InsetBox[
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(T\\), \\(\\\\ \\)]\\)/\\!\\(\\*SubscriptBox[\
\\(T\\), \\(1\\)]\\)\"", TraditionalForm], {2, 0.8}], 
    InsetBox[
     FormBox[
     "\"\[Rho]/\\!\\(\\*SubscriptBox[\\(\[Rho]\\), \\(1\\)]\\)\"", 
      TraditionalForm], {2, 0.5}], 
    InsetBox[
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(p\\), \\(\\\\ \\)]\\)/\\!\\(\\*SubscriptBox[\
\\(p\\), \\(1\\)]\\)\"", TraditionalForm], {1.5, 0.4}], 
    InsetBox[
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(v\\), \\(\\\\ \\)]\\)/\\!\\(\\*SubscriptBox[\
\\(v\\), \\(1\\)]\\)\"", TraditionalForm], {2.4, 1.9}]},
  ImageSize->400,
  Method->{},
  PlotRange->{{0, 3}, {0, 3.6}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.575546349899782*^9, 3.5755465168208103`*^9, 
  3.5865827968473916`*^9, 3.587450179823666*^9, 3.589077860904908*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figa", ",", 
    RowBox[{"Graphics", "[", 
     RowBox[{"{", 
      RowBox[{"Dashed", ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"1", ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "3"}], "}"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
   "]"}], "//", "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", "\"\<compressible-local2sonic\>\"", "]"}]}]], "Input",
 CellChangeTimes->{{3.42829221505965*^9, 3.42829227587265*^9}, {
   3.4282923565506496`*^9, 3.4282924067386503`*^9}, 3.4309108210556*^9, {
   3.47926806821*^9, 3.479268073425*^9}, {3.5014912982119823`*^9, 
   3.501491343770588*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755463501068087`*^9, 3.575546517035837*^9, 
  3.5865827973594575`*^9, 3.587450179960729*^9, 3.589077861289916*^9}],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxF23dYze//B/D2KaFQadAgaUpDKvI6Eg2riIRSqbQ0jBLaGtKiEKKSJA2E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    "], {{}, {}, 
     {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwl1Hl8z3UcB/DfZmNGzJhO7WA2KtOhy3TYKlRsS0o3yiq1VUrk6qa73N3o
IESuig7KkSIUhVS6CJVO0eF4vh/+eO71+by+j/32+33en99ye1ZXVCUlEoly
PyprJBIXWp9ovUsu5AmSqav7WI6hkva01v0u5/MQRWTrvpGvcAfd8LKJg/Qr
5ViupYRjdX/It3g4Xo8c3bdyCgO5iBTq6VfJcVxHafw+ufrv5FQGcTGp1Nev
lk9yPWdxHHn67+U0BtOdmmToP5FP0YezOZ6m+h/kqwzhEmrRQP+pfJobOIcT
aKbfLKczlEtJI1O/Rj7DjXSgDfn6LXIGd3IZtWmoXyufpYqOMRea63+Ur3EX
l5NOI/1n8jmq6cRJFOi3ypnczRXUIUv/uXyemziXkynUb5OzuIcrY/Y01q+T
47mZ8ziFFvrtcjb3clXMmoP16+UEbuH8OH/dn/JtHuFUWup+knO4jx7xvnW7
5XuMiNlziG6DnEhf2tr/L5fQOWZl/5d8h0fjOUfpfpZzuZ/T7PfJZfSMz2v/
j3w/5m1di5HWZ8pklsc5yjrUtz5UfiFfoMQ6hVuti+We+IxyadwFmUYZXeJe
yJ1xZvLduIuyZjzjMfszZBKd+ShmLdOTDnwni+2Plr/EjOTrslSmMsz69Pjy
8mHcKVmbXjEb+a9cxCja29dghXUGh1lvlC9yG+3s98oPKIu7af+3XMDj8Zxj
dDvkGwzn6pih7j+5mNE04HDdl/Il+lEeZ08r/a/yTR7gGjI5Qv+VfJnbqYjP
RZH+NzmPB+lNQ5rov5aT6M8FcYY04kjPNsnJDKBrzJOsuMNxL+MuxSzjDOK9
xt+P14zfJZsccsmjKc3IpzkFFNIi7m7csZhPnE28P//o9gPZmI91
       "],
       VertexColors->Automatic]}, 
     {Hue[0.9060679774997897, 0.6, 0.6], Dashing[Large], 
      LineBox[CompressedData["
1:eJwV0ddaDgAAgOG/S3FLDakU/RqKhpBCKk1RaSCjgQYhRSEUDRElDdI2kqyu
odfB+zzf+bcjmBGaHhIIBBoJFbM84w8d/GSKf1ziO+/4y2N+Mc1Z1njNQzb5
wE1+MEk/xSwyTBdNrDPBE2r5yjh9tBHGHM+5RQPn+MIbemmlhCVGuE8zdYTz
kQFuc5lKSllmlG5aqCeCTwzSyRWqKGOFV/RwnQvsZJ4X3OEq1ZQTyWdecpdr
nOcMu4gimhh2E0sce9hLPEH2kUAiSSSznxRSOcBB0vg/NINMDpHFYY5wlGyO
kUMuxznBSfI4RT4FFHKaBYa4RyM1VLDKGA+4wUW+8ZZHtLPBe57ymxm2KGIb
flplDg==
       "],
       VertexColors->Automatic]}, 
     {Hue[0.1421359549995791, 0.6, 0.6], Dashing[Medium], 
      LineBox[CompressedData["
1:eJwV0Gkz0AEQB+A/Ih2UCskVHXKG6FChFEru+5Y7hHIkhM/gi+YKoXi8eOa3
M7s7s7NpgzNN0yFBEGzyWzEWGgSN/KKMnwyxTgEL9LNGLSuMkME3uvnAMsO8
5AefqSeNr7TznmcsMkAduczRSw2v2XLXuGyinEIe8Z0ePvKKdKbpoJLn5LFt
/4ts5g1PyOQeM3RSxQses2N+QrbwliKyuM+u3qRspYJisnnAnt4f9jngkL8c
ccwJ//jP6fnT7YQQShgXCCeCi0Ryictc4SpRRHON68Rwg5vcIpY44rlNAndI
JIlkUkhlijbe8ZQcHjJLF9WUkM88fXyilCUGaWCVUTa4yxk4fzQA
       "],
       VertexColors->Automatic]}, 
     {Hue[0.37820393249936934`, 0.6, 0.6], Dashing[Small], 
      LineBox[CompressedData["
1:eJwNz9leTmEUwOH9NVCUKOS0W3IJLiDXoFEhadKENFAUGoQG0kCRKKWUhEoD
FQ2EwnPw/P5rrXef7KQTJ48nh4IgKOZUWBD8ZYEj4UEwqof0tyZqn8bqDz2q
s5qgO9pGtHldZ4g3/9FXxJm3dZ5mIu2rOsUgB+w/dY7H7Ldv6kdGSOEfi4zx
hHb2+W5DPzBMC3vc1vQdL+gmlcB9Sd/wlA7ustf9m04zRA9phNyXdZx+Omkl
nTBvX3SCAR5yjwzCvX3VtzzjEffJ5DRZZHOGs5wjh/PkcoE88imgkCIuUkwJ
pZRRziUuc4UKrlJJFdXUcI3r1FLHDW5STwO3uM0dGonwDys6yXO6eECU+3d9
z0t6iXHb0k+85qD9l37msHlXj2mT/gf+x2QB
       "],
       VertexColors->Automatic]}, 
     {Hue[0.6142719099991583, 0.6, 0.6], Dashing[Tiny], 
      LineBox[CompressedData["
1:eJwNw9k2QlEAANBj+QoylUQikpIpKlOU4TYgJa56zf+/tfdaOxn/R7OFEELb
pcUQxn7a9dW2Ty6bcMVV11x3w6QpN027ZcZtd8y6a84998174KEFjyx6bMmy
J1Y89cxzL7y06pXX1qzb8MZb77z3waY/Duz55rMtf/2yb+SLsUPf7fjnyA8n
fjv10TnvRhXb
       "],
       VertexColors->Automatic]}},
    VertexColors->CompressedData["
1:eJztwUENACAMBLAjKJklJJAQnvj/DSFta791RpI7AwAAAAAAAHwNwmUCFQ==

     "]], 
   {Dashing[{Small, Small}], LineBox[{{1, 0}, {1, 3}}]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["\"Ma\"", TraditionalForm], 
    FormBox["\"\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  Epilog->{
    InsetBox[
     FormBox["\"subsonic\"", TraditionalForm], {0.7, 2.8}], 
    InsetBox[
     FormBox["\"supersonic\"", TraditionalForm], {1.3, 2.8}], 
    InsetBox[
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(A\\), \\(\\\\ \\)]\\)/\\!\\(\\*SubscriptBox[\
\\(A\\), \\(1\\)]\\)\"", TraditionalForm], {2.4, 2.7}], 
    InsetBox[
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(T\\), \\(\\\\ \\)]\\)/\\!\\(\\*SubscriptBox[\
\\(T\\), \\(1\\)]\\)\"", TraditionalForm], {2, 0.8}], 
    InsetBox[
     FormBox[
     "\"\[Rho]/\\!\\(\\*SubscriptBox[\\(\[Rho]\\), \\(1\\)]\\)\"", 
      TraditionalForm], {2, 0.5}], 
    InsetBox[
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(p\\), \\(\\\\ \\)]\\)/\\!\\(\\*SubscriptBox[\
\\(p\\), \\(1\\)]\\)\"", TraditionalForm], {1.5, 0.4}], 
    InsetBox[
     FormBox[
     "\"\\!\\(\\*SubscriptBox[\\(v\\), \\(\\\\ \\)]\\)/\\!\\(\\*SubscriptBox[\
\\(v\\), \\(1\\)]\\)\"", TraditionalForm], {2.4, 1.9}]},
  ImageSize->400,
  Method->{},
  PlotRange->{{0, 3}, {0, 3.6}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.5755463501348133`*^9, 3.575546517046838*^9, 
  3.5865827973614554`*^9, 3.5874501799637146`*^9, 3.589077861297917*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 14.3  Application: The Laval nozzle (p. 237)\
\>", "Title",
 CellChangeTimes->{{3.4283723065052156`*^9, 3.428372309926216*^9}, {
  3.428993647884076*^9, 3.428993649516076*^9}, {3.5020905898483095`*^9, 
  3.5020906081913586`*^9}, {3.5874508971827283`*^9, 3.5874509040385885`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Figure 14.2 Simple model of Laval nozzle (p. 237)\
\>", "Section",
 CellChangeTimes->{{3.502090805232629*^9, 3.5020908120230174`*^9}, {
  3.587451022437481*^9, 3.5874510326218104`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(11\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.5755465812735443`*^9, 3.575546728190172*^9, 3.575546841799806*^9, 
   3.5865827980995417`*^9, 3.587520302601589*^9, {3.587522421596837*^9, 
   3.5875224509793606`*^9}, 3.5875225474799395`*^9, 3.587522974858221*^9, 
   3.587523032966193*^9, 3.587523264101924*^9, 3.58752460514883*^9, 
   3.58907786208206*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Set parameters", "Subsection",
 CellChangeTimes->{{3.428310126353825*^9, 3.4283101282098255`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Gamma]", "=", 
   RowBox[{"7", "/", "5"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"k", "=", "0.1"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"x0", "=", "3"}], ";"}]}], "Input",
 CellChangeTimes->{{3.3901209228623548`*^9, 3.390120929694355*^9}, 
   3.5875197087323303`*^9, {3.58751974647686*^9, 3.5875197563490467`*^9}}],

Cell["\<\
Nozzle cross section along its axis\
\>", "Text",
 CellChangeTimes->{{3.5875197692015867`*^9, 3.5875198139099517`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"A", "[", "x_", "]"}], ":=", " ", 
   RowBox[{"1", "+", 
    RowBox[{"k", " ", 
     SuperscriptBox["x", "2"]}]}]}], ";"}]], "Input",
 CellChangeTimes->{3.428310108306825*^9, 3.5875202914862556`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions", "Subsection",
 CellChangeTimes->{{3.4283230722358255`*^9, 3.4283230745948253`*^9}, 
   3.502091179680046*^9}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["T", "1"], "/", "T"}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " ratio as a function of Mach number ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]],
  FormatType->"TraditionalForm"],
 " from eq (14.29)"
}], "Text",
 CellChangeTimes->{{3.587519976666479*^9, 3.587520008324278*^9}, {
  3.5875200940705686`*^9, 3.5875201222309456`*^9}, {3.5875201706797605`*^9, 
  3.587520171621872*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "m_", "]"}], "=", 
   RowBox[{"1", "+", 
    RowBox[{
     FractionBox[
      RowBox[{"\[Gamma]", "-", "1"}], 
      RowBox[{"\[Gamma]", "+", "1"}]], 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m", "2"], "-", "1"}], ")"}]}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4283121004998255`*^9, 3.428312111538825*^9}}],

Cell[TextData[{
 "Area ratio ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "/", 
    SubscriptBox["A", "1"]}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " from eq (14.30)"
}], "Text",
 CellChangeTimes->{{3.587520088887946*^9, 3.5875200907731705`*^9}, {
  3.5875201302309055`*^9, 3.5875201655571437`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "m_", "]"}], "=", 
   RowBox[{
    FractionBox["1", "m"], " ", 
    SuperscriptBox[
     RowBox[{"t", "[", "m", "]"}], 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"\[Gamma]", "+", "1"}], ")"}], "/", "2"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.3901209442663546`*^9, 3.390120957135355*^9}, {
  3.428312118391825*^9, 3.4283121246178255`*^9}}],

Cell[TextData[{
 "Pressure ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "/", 
    SubscriptBox["p", "1"]}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " "
}], "Text",
 CellChangeTimes->{{3.587522675576311*^9, 3.587522683590272*^9}, {
   3.5875227254532957`*^9, 3.5875227350634513`*^9}, {3.587522808870305*^9, 
   3.587522811189582*^9}, 3.5875229466618366`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"p", "[", "m_", "]"}], ":=", 
  SuperscriptBox[
   RowBox[{"t", "[", "m", "]"}], 
   RowBox[{
    RowBox[{"-", "\[Gamma]"}], "/", 
    RowBox[{"(", 
     RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]], "Input",
 CellChangeTimes->{3.428312155542825*^9}],

Cell[TextData[{
 "Subsonic and supersonic Mach number for given area ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.587520347398967*^9, 3.5875203538457355`*^9}, {
   3.587520407446168*^9, 3.587520459754445*^9}, 3.5875228386298733`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sub", "[", "a_", "]"}], ":=", 
  RowBox[{"m", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "m", "]"}], "\[Equal]", "a"}], ",", 
     RowBox[{"{", 
      RowBox[{"m", ",", "0.5"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"super", "[", "a_", "]"}], ":=", 
  RowBox[{"m", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "m", "]"}], "\[Equal]", "a"}], ",", 
     RowBox[{"{", 
      RowBox[{"m", ",", "1.5"}], "}"}]}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.4289943131960764`*^9, 3.4289943337340765`*^9}, {
  3.5875229974219275`*^9, 3.587523004829816*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[" Mach number as function of x", "Subsection",
 CellChangeTimes->{{3.428310133530825*^9, 3.4283101347168255`*^9}, 
   3.428993739035076*^9, {3.587520325130291*^9, 3.587520328661714*^9}, {
   3.5875224682304325`*^9, 3.587522470981762*^9}, 3.5875231813209934`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"sub", "[", 
    RowBox[{
     RowBox[{"x_", "?", "NumericQ"}], ",", "A1_"}], "]"}], ":=", 
   RowBox[{"sub", "[", " ", 
    FractionBox[
     RowBox[{"A", "[", "x", "]"}], "A1"], "]"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"subsonic", " ", "region", " ", Cell["m < 1"]}], " ", 
   "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"super", "[", 
    RowBox[{"x_", ",", "A1_"}], "]"}], ":=", 
   RowBox[{"super", "[", " ", 
    FractionBox[
     RowBox[{"A", "[", "x", "]"}], "A1"], "]"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"supersonic", " ", "region", " ", Cell["m > 1; not used "]}], 
   "*)"}]}], "\n", 
 RowBox[{
  RowBox[{"crit", "[", "x_", "]"}], ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"x", "<", "0"}], ",", 
    RowBox[{"sub", "[", 
     RowBox[{"A", "[", "x", "]"}], "]"}], ",", 
    RowBox[{"super", "[", 
     RowBox[{"A", "[", "x", "]"}], "]"}]}], "]"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"critical", " ", "case"}], ";", " ", 
    RowBox[{
     RowBox[{"all", " ", "m"}], "-", "values"}]}], " ", "*)"}]}]}], "Input",
 CellChangeTimes->{{3.4283118947308254`*^9, 3.428311977514825*^9}, {
  3.587520484258385*^9, 3.587520573661112*^9}, {3.587522386951681*^9, 
  3.5875224405831137`*^9}, {3.587522500245273*^9, 3.587522503610677*^9}, {
  3.587522593386449*^9, 3.5875226489721184`*^9}, {3.5875230090953293`*^9, 
  3.5875230285046577`*^9}, {3.5875230914462085`*^9, 3.5875230952066603`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot Mach number", "Subsection",
 CellChangeTimes->{{3.58752319112717*^9, 3.587523197397921*^9}}],

Cell["Select curves", "Text",
 CellChangeTimes->{{3.5875224894469776`*^9, 3.5875224920222864`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"curves", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"crit", "[", "x", "]"}], ",", 
     RowBox[{"sub", "[", 
      RowBox[{"x", ",", "0.999"}], "]"}], ",", 
     RowBox[{"sub", "[", 
      RowBox[{"x", ",", "0.9"}], "]"}], ",", 
     RowBox[{"sub", "[", 
      RowBox[{"x", ",", "0.6"}], "]"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.428049952965*^9, 3.428050022329*^9}, {
  3.4282236573269997`*^9, 3.428223679837*^9}, {3.4282237864560003`*^9, 
  3.428223820453*^9}, {3.4283119206078253`*^9, 3.4283119269618254`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{"curves", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"-", "x0"}], ",", "x0"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "x0"}], "-", "0.1"}], ",", "x0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "2"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"x", ",", "Ma"}], "}"}]}], ",", 
    RowBox[{"AxesOrigin", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "x0"}], "-", "0.1"}], ",", "0"}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
    RowBox[{"Epilog", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Text", "[", 
        RowBox[{"\"\<0.6\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0.25"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\<0.9\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0.55"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\<0.999\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0.8"}], "}"}]}], "]"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\<1\>\"", ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1.1"}], "}"}]}], "]"}]}], "}"}]}]}], "]"}], "//",
   "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<compressible-lavalmach\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "225"}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.390121267357355*^9, {3.428049860477*^9, 3.42804986146*^9}, 
   3.428050072826*^9, {3.428050104106*^9, 3.428050148343*^9}, 
   3.428050256795*^9, {3.428050305447*^9, 3.428050332101*^9}, {3.428050401*^9,
    3.428050401117*^9}, {3.428223690784*^9, 3.4282237046949997`*^9}, 
   3.428223836392*^9, {3.428310242974825*^9, 3.428310254274825*^9}, {
   3.4283102871798253`*^9, 3.4283103021388254`*^9}, {3.428310779545825*^9, 
   3.428310816198825*^9}, 3.4283109576178255`*^9, {3.428310993161825*^9, 
   3.428311048668825*^9}, {3.428311127448825*^9, 3.4283111282948256`*^9}, {
   3.428312192394825*^9, 3.4283121925548253`*^9}, {3.428993678278076*^9, 
   3.4289937292130766`*^9}, 3.4289988355290766`*^9, 3.4309108219816*^9, {
   3.4678819796668215`*^9, 3.4678819864042063`*^9}, {3.479268126816*^9, 
   3.479268129408*^9}, {3.50140694883125*^9, 3.5014069697104445`*^9}, {
   3.501491389668213*^9, 3.501491410036378*^9}, {3.5014914510947266`*^9, 
   3.501491452610813*^9}, 3.587522544166545*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5755465823916793`*^9, 3.5755467293893175`*^9, 3.575546842962943*^9, 
   3.5865827987565985`*^9, 3.587520303817733*^9, {3.587522422403934*^9, 
   3.5875224520164876`*^9}, 3.587522548523066*^9, 3.5875229759333496`*^9, 
   3.5875230340483227`*^9, 3.5875232651560526`*^9, 3.5875246062089577`*^9, 
   3.5890778634502316`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwBYQOe/CFib1JlAgAAADUAAAACAAAAY6+Q7///B8CCGA9nm7/UPxT7GiY7
/AfAuI3NTyHD1D/ERqVcdvgHwMzxgvSnxtQ/Jt65yezwB8Bd3W5yt83UP+oM
46PZ4QfA5rCSRN/b1D9wajVYs8MHwHuRVWRS+NQ/fiXawGaHB8BTnBylxzHV
P5qbI5LNDgfAQqOT1fam1T+f34hvUQkGwH4UgY/Dr9Y/FtBTTykVBcAN/XpL
2LTXP8hQU0TLJQTAh3qqRG/C2D9IKSV0IyIDwBJJnhnj9tk/Oq5cps8vAsCR
ZmF/fybbP/qKZhMyKQHAnfXUO3yB3D/196SVXicAwD4pp0Mb6d0/xCKSNL5t
/r8vydHILUrfPzsFf7MrZPy/Lxyl6Xtu4D+UQDc3QX36v8BwunMpNOE/Y5xY
5eqf+L/yAmyBRf/hP8+nHgkBmva/F/FrFVfm4j8eDLAxv7b0v8pYrhgvyOM/
CiDmz+mq8r/6PF/+UsjkP2xUhZioqPC/DScPD1zP5T9gw9/LHpLtv3nMvDgL
z+Y/Ij3+UcWB6b9Xmk1Bpe/nP6los+G7tuW//YH13oIH6T9q87Fci5rhv4uK
E+3/Qeo/Ln4FWQYj27+HXu/h1YHrPxN51AuWm9O/4k4EdMO17D/XZGwqr+PG
v00OJs+VDe4/d3oqjsmVrr88iYB1d1fvP6qPOECkMK0/hGz6BlVR8D+OaYHW
YILHP5aGeLgGCfE/DfRMO6xR0z8wzLE47LfxP+B0xjR2hNs/H3H93eF48j/u
uU3Cd8jhPzAxus5cOfM/poehYGSJ5T8zi8f2Pu/zPyX2qxN4m+k/YOQwaai2
9D/esh+9O2jtP2DWsFp6cvU/VZfgiOuQ8D9dQsVk1Sv2P56ljL1MlvI/ltxz
t/z19j8EW23tBXn0P2MiS4o9s/c/zmCpp1KE9j8rxsuqwoD4P7UNGl33bPg/
WuK72bZA+T8mmiHoB0z6P6QnG8ax/Pk/+naE/atT/D/blYYwGsj6P+v6Gw6o
OP4/VJwl5kKF+z+gZ4fUGyMAQI9BmOhHUfw/kEHMjBklAUCre4BOaBj9Pw5v
q0LDFQJAi3ArnvTQ/T++RLi9thoDQNxvvhOal/4//W1fNlYOBEBNg9Qrkk//
PzrlCT/q+QRAAAAAAAAAAECjBLbU
     "]]}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJw1lnk0lWsbxnemV4oMJ6d0ZFYyJyLD/ZKKZhQqBwlJikqadYjYRMiUMiVT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     "]]}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJw113k0Vd0bB3BDuoVUSj9DITKVOXorw3OF0oSIVDI3CamECJF5JpnqDcks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     "]]}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJw113k0Vd/7B3DzjUrmdCtDMiQalMj03ERpkKESfYQ0ElGUTImMRSGEBhpM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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["x", TraditionalForm], 
    FormBox["Ma", TraditionalForm]},
  AxesOrigin->{-3.1, 0},
  Epilog->{
    InsetBox[
     FormBox["\"0.6\"", TraditionalForm], {0, 0.25}], 
    InsetBox[
     FormBox["\"0.9\"", TraditionalForm], {0, 0.55}], 
    InsetBox[
     FormBox["\"0.999\"", TraditionalForm], {0, 0.8}], 
    InsetBox[
     FormBox["\"1\"", TraditionalForm], {0, 1.1}]},
  Method->{},
  PlotRange->{{-3.1, 3}, {0, 2}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.5755465824196815`*^9, 3.575546729401317*^9, 3.5755468429689455`*^9, 
   3.586582798758583*^9, 3.5875203038587394`*^9, {3.5875224236850877`*^9, 
   3.587522452051491*^9}, 3.587522548539068*^9, 3.5875229759683537`*^9, 
   3.5875230340813265`*^9, 3.587523265197056*^9, 3.587524606242964*^9, 
   3.589077863452236*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot pressure ratio", "Subsection",
 CellChangeTimes->{{3.5875232144239664`*^9, 3.5875232198936214`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"p", " ", "/@", " ", "curves"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{"x", ",", " ", 
       RowBox[{"-", "x0"}], ",", " ", "x0"}], "}"}], ",", " ", 
     RowBox[{"PlotRange", " ", "->", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "x0"}], " ", "-", " ", "0.1"}], ",", " ", "x0"}], 
         "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{"0", ",", " ", "2"}], "}"}]}], "}"}]}], ",", 
     RowBox[{"AxesOrigin", " ", "->", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "x0"}], " ", "-", " ", "0.1"}], ",", " ", "0"}], 
       "}"}]}], ",", " ", 
     RowBox[{"AxesLabel", " ", "->", " ", 
      RowBox[{"{", 
       RowBox[{"x", ",", " ", 
        RowBox[{"HoldForm", "[", 
         RowBox[{"p", "\"\</\>\"", 
          SubscriptBox["p", "1"]}], "]"}]}], "}"}]}], ",", 
     RowBox[{"PlotStyle", " ", "->", " ", "Black"}], ",", 
     RowBox[{"Epilog", " ", "->", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Text", "[", 
         RowBox[{"\"\<0.6\>\"", ",", " ", 
          RowBox[{"{", 
           RowBox[{"0", ",", " ", "1.8"}], "}"}]}], "]"}], ",", " ", 
        RowBox[{"Text", "[", 
         RowBox[{"\"\<0.9\>\"", ",", " ", 
          RowBox[{"{", 
           RowBox[{"0", ",", " ", "1.5"}], "}"}]}], "]"}], ",", "      ", 
        RowBox[{"Text", "[", 
         RowBox[{"\"\<0.999\>\"", ",", " ", 
          RowBox[{"{", 
           RowBox[{"0", ",", " ", "1.2"}], "}"}]}], "]"}], ",", " ", 
        RowBox[{"Text", "[", 
         RowBox[{"\"\<1\>\"", ",", " ", 
          RowBox[{"{", 
           RowBox[{"0", ",", " ", "0.85"}], "}"}]}], "]"}]}], "}"}]}]}], 
    "]"}], "//", "\[IndentingNewLine]", 
   RowBox[{"Figure", "[", 
    RowBox[{"\"\<compressible-lavalpressure\>\"", ",", " ", 
     RowBox[{"ImageSize", " ", "->", " ", "225"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{
  3.3901212782373548`*^9, 3.3901213219313545`*^9, {3.42804986466*^9, 
   3.4280498657790003`*^9}, {3.428050134561*^9, 3.428050141057*^9}, {
   3.428050197506*^9, 3.4280501982060003`*^9}, 3.428050251351*^9, {
   3.428050356988*^9, 3.428050392856*^9}, {3.428223753637*^9, 
   3.428223763987*^9}, 3.428223858863*^9, {3.428223889566*^9, 
   3.4282239062349997`*^9}, {3.4283102752358255`*^9, 3.428310277101825*^9}, {
   3.428310310611825*^9, 3.428310335313825*^9}, {3.4283108342858253`*^9, 
   3.4283108782308254`*^9}, {3.428310908911825*^9, 3.428310945026825*^9}, {
   3.428311061993825*^9, 3.428311074486825*^9}, {3.428311119302825*^9, 
   3.428311120102825*^9}, {3.4283112200018253`*^9, 3.4283112201008253`*^9}, {
   3.4283121949238253`*^9, 3.428312195082825*^9}, 3.4309108224816*^9, {
   3.4678820302517147`*^9, 3.4678820374751277`*^9}, {3.479268148568*^9, 
   3.479268150226*^9}, {3.5014067838858156`*^9, 3.5014067990226817`*^9}, {
   3.5014068666955523`*^9, 3.5014069232477865`*^9}, 3.5014074216122913`*^9, 
   3.5014075552079325`*^9, {3.501407589385888*^9, 3.5014076580958176`*^9}, {
   3.5014077082006836`*^9, 3.5014077309979877`*^9}, {3.5014914726929617`*^9, 
   3.501491503861745*^9}, {3.5875232544687696`*^9, 3.5875233277475615`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755465851730127`*^9, 3.5755467322316628`*^9, 
  3.575546845486246*^9, 3.5865828001827807`*^9, 3.5875203066050687`*^9, 
  3.5875224239111156`*^9, 3.587522455059854*^9, 3.5875225509513597`*^9, 
  3.5875229789497128`*^9, 3.58752303662763*^9, 3.587523268329432*^9, 
  3.5875246093163333`*^9, 3.589077865695472*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwtlPs3lHkcgF3GfDcre06TLrTINCNq27RuJX2+dOGQjZRLaU3lEtEopSSs
3XFrlcmSYjddlCklUlYX3jekeRWjVIrC5BK5TYwZvTO07Tn7w3Oe8/wDz8Jd
/M3BWhoaGh5f+c+7bwpHv3xBZFg2Q0QvVsNslSHXQY1I9d6cn9y+dt2GywEx
k4gUnRtPOmuuBm7HXWpIhsjE9/Uhdlw1fNTrvvRGisimA5k/R7HVEBVu41de
i0g5v1Hw6ns1JHDayJBURHZZLlZHz1JDXj47q1EfkXva7NJGaBUUdKbxvWYw
yU1xFSWedSqY0+y9ycJAhzT3ljWnZqhA7Om97gmHQQa6lz9WearA1YJzkG2m
TXrF/1CcoquCVSUBVyQ/apGOb2vOtdfQQB8PYa2w0CTls3w5rvtomFAW5vxi
rkHePbXX5CmLhjqzLOvqyGniUUj8J9/Kz+Cgk1i+JEhN3AoOmmHg8RmysZ2d
UzxNTFhNlDOkk7D7L7/ejwWTxDflPGI3fxIkRcYzNPMUxJm8If8Ng0ow1iu1
IirkxGF3x21nQ5Wga9onGbg2RhSa1KpLJQqI2JJ8RiSSEb/yzNuCrRTAq+18
Ypw1TCS7t+u+zJkAM8dpr0dJA0TMfCr3XY8cru4pl96v6CVSnY8nJtjLIXy8
LOCPPClRaiPdUBQzDssSdPyZ7HbihjjXintrDFixL749nd9C9K4XubgqPkFr
UITlze56In+ed/RL7ic4fNTyJJVTQhiuYY407pLBtUx7XLj8BmTGBq9dvnYU
ToYlveL99hgstnft3Hx+GGbKNzpc92qBVWX592wXDEHXwXrrGEE7+CyaW+8c
/xFG7nrUHBZL4e937Cj2SD9cSb3AC0rvhbDuaq21v38AjqKYdSR/ADoqS0r7
dfqg45ZJlcOeYZgXdP1Ya0kPbM3tShsUymD/kqj6ur3dcO5yRtXG02Pw5+1/
UlQL3gNje+SwTqwcPoiaBmfKu6CJd7Vos0ABiff9jzSldMKdmYY7lBGTYD7u
SCTsewfcvIX9aCMNufPDfTmJ7UDHCKaKvdVgsuOlGS/1DQzSRvoi+2koPt/3
m1TSCudDU18YsTXwjVyOWzfrFQidnmbM52jii8qoEOfEF6DPL3auNdDCtnpz
5TtNW6B6XWVVhaE2ZqZHb/MjnsFUZKCtjz4DC8KLU7RPNoMmen52yRQDe/TU
mHwnawJb1wM1bhpM7FfYn5T18CnQb6/3G4wxsUtn+fSpzgZYFyEWeiiY+MQW
p+i41gY4oeq2T6aZOCzX+lKopAFMDY3S5VoIu/Q/snIlGsDFN92ihYXw5aER
mUVBA2Q/Cw4X2iA8bVC3gh/YAMvqjYd0Y7/26j4RLaXgyNZV2c7xCM8+5PB6
ZRsFD3u2rj6ahPAzpzlFcc8p2KJ9ImMgHWH5oSETVEtBHFYtFechfEUwO8Sx
kALxvdbI5AcIj2RUCLihFMxyGzeoIhFeeefC+sxACgLe6FfL6xD2MAk9Q/tS
MKpcrx/UiHCH/wLZa1cK5tjcLnHuQPi40zHP25YU8OokPkffI/yAIUizYlNw
1XtwuqwP4ULzi+5lRhSsPmC2aeEowtZLa/mVehSkaK5R+o0jvN9nrgR0KGgW
+hcIlQgX7F9UQU2JwdD0kItYhbBRPmuRj0IM//8LV5nqz+sdEcO/az5pZw==

     "]], LineBox[CompressedData["
1:eJw11nk8VG0bB3AZnJIWa5YnRNZEQhu5TlLpeVSWSCWEChGVSJ6SIhRCJVSW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     "]], LineBox[CompressedData["
1:eJw113k0Vd/7B3CZbqmPD1JxK0SGSpQQiedmKIouylRClDIrs0bzLITQQJE5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     "]], LineBox[CompressedData["
1:eJw113k0Vd/7B3DzLUkhGSqUEImIKMNzK4rKXKJkVhFJRJIxQ9eskDTgI1OE
KKLYV2Qs85AhkiFTkcznXr7bWr/fH2ed9frjrL33OWfv53nvtnY2tGNiYGBY
xNf63SYvenptjUQ9lXpQxy6KDtsIQXEVGok6X7TTQRq76mSamfsSicqannL7
XyQdxPtL6qZmSNTWrVdM/LEnOIZSuwdJ1Evb7M8+i6DDTQdFk8JKEpWjsnui
MYwOPmI91CshJKr7WsKI+AM6JD4VffiVk0R1My1eSPajQ9LAA2eDjWzUtcpI
XUNnOmxvNtKT5GWl8ikSxoOX6VCrb6TRIMZCLa+9lB9yhg5akmJuonuYqZPV
ByVlDtPhaK5ZepMsE5XlgEzngAgdVkKv8MhLMlLtq8I0T26gw/ziyzhzCQbq
lxkW4aQ/NKja81Ch3GkVJV8bSLvWQQMVVt/C/bY0lNxunlv9ngaxZCWlY94r
iKFlf+LKMxrYPDMZmUhaQleVAqMm/GjQlCG0kTFxAX0NN/zWbkUDIY58OVQ0
h8q350yGadCAXWS0afzVLDrRoH72914aOJ4LSsjMnEEvD+ocCWSjgWXlQIPQ
w9+o+a8K5cgkAXvUVg0++4+jiYr+TJsmArKuFQ5+KBpBaapR9i+LCWi3ThI+
kz6CentseU9gM1wOu9wTN4JajbzmBosIMDaw6V5yG0GXVwV1RLCZjvK0KB4a
QR2mc45Jbwm4uMmVmpc3jOwl7SMy3hDAniuflJo5hDgHmQN+ZBPg8O+NWVji
IBou541fTSbgyeG/KldDB5G6Hct4Dnat58EdJ+4OIrabiR6XsMUY8rpXTAdR
6y5jrZIkAn5w5Bg7CAyitPojHz1eEHBePF1fO+EH+uA1zMnwjAD/AgUKp/QA
SpzQtYPHBPQrF56q3jSAmkY43efjCVBFcmw+k/2oXWZnTTb2QoPM/d+v+tGz
ztdSAtiOI/u8v0j2oz0vHxILsQSY8O9yCZX4jo4zqQ+VPyRAxofVlE20Fy2m
zn7wiyRAsLT58RhLL9IX2+t7Apt14Wln/WgPWrBi82HD7nOSPxf1qge1bDkl
GxFBQOhlCz1++R509W2i9fNwAn6plmhKHetGLjn6onWhBCQTjvK6Fl2oxS+5
XDeEgHAlZZeDx7pQY2qmpAC2hytzPrdoF4otCGkbCiZAd/LJga7RTjRuKDns
iU3vqd5neaMTJUeHE5lB+HuUigjf8u5Ag4e4+XgCCeDxbN8U/7QNTWnNPOfz
J2BpbZlr+m4bUlHsUx31w+8rWJhf62IbErWc3PcOOzPWYe8Kfxt63NM7ZoSt
ns+gZh7figy4v7HH+RJg/0v6hnhMC+LYeeG3mA+ezw1DN9+bLeipkp3DsjcB
CvMed7/ptSBzTonDX7FXmSqDQzlbkE7d3tLb2LFCpkl/wprReP9j9/p7BPhk
s6c4bW5CVe/OdwR4EaBdcv618JdGJGV6XesS9raa5JKW0EbkJ/5ikwJ21s/D
rYobGtEdvvyA0bv4/xWwYV5l+opaTzQq6mFLPfhgF7lUj4zDCytkPQnosnWU
yhuqRv4/788f8CCg/Lu+UlF6NUp6UR3AjZ1urKhRZl+NNn0bu7DoToCb1url
hunPKPfJWncF9lbp6JhfRBWKGO9Wv4it/a9wSXhbJZrhnolLuE3AQccEVomu
T+iM1VK6Pzb/yD1umcRPaPZsMsN17NFOTWlVkU/ok9VKhDp2QGmXhYl0BeJq
WOMadyPgNvNixG5KOVri2117BltjHmXIbilHv1hnrx7B5v71oEItrgx9j2Iw
kMDOrRecN035iHhUlleZsUdj1M0elpSi9ykaDVRXAt7eJ7knQymK3z79Lh87
4HZzVO7nEjQqIjCRjC1sal1Z3/IeRZqwywZgm4gESzFPFKEWQxlpLWxxbj0N
rptFiPtaqs1R7DlmfnPhhXeo5kTLsDR2zK+sGBWmd2jslEsVN3Z97tclV8FC
dPrRzeXBWwQkJMdzByQXoC3DF307sa88tJCOFi9A941k9Bqwmd3/WuTIv0E7
+NK/vMP2uCsVWReXi7buvgaR2Lp/Zjrz0l+j7JaQ40HY4tbFwvHFOcgwuTP8
HnaHtsYb6+5XqO0N1+J17Jwy9hWtiSy0lsvKZ4sdKNdyXJbIRK3qyj5m2IcE
zNuJnRloRujqHl1s9oi9u34eSEfxIg4Wp7B/Mkza1aqnoad8JUNk7Jgxj8VY
y1SU1XvykwL2NTN1spfLfyhFP2ynLDa5mYViFZCCBtglyiWx+51FFc20ktBL
uRtdwtgRF/Kr9zk8R/qqQno7sFVBzWQ+7Cl6UCvDyYc9KV4/XvH6CZJvTdnG
g53IecErsukxYlLwtN6Crb0wxHHpbxzSySha3IS99P3mCwmeWCRVdal9A3bG
Z7rsnMJDZMXosMSKbfw6tIJqHI1c3/60YV5fn5RsuuDfcEThqtnOuD7/Za26
QgUK8t/Pz7XmQkB1rfXU2TtBqP9fpyEdWzTh3pbRD/5o6ChLH4HN1CRtt7rq
hZzzknNWsMucLLMPBruhLQM5VcvYf/n3DX0WdEAshyX3rrs5R2jgnKExUljj
bVjCZrE6I/LghDpI2rqWrDtL5j8PjlJrEHU6Nr3uJ3eez7d13AQORj/H9ee/
d9vnpejeAdsRGbn18faoHLZ3rvEF13IdtfX5HNL1jxFKvg/8W/qiadgdPldv
BJmFwA/l7v2r2B55Omem+MOA454GJwNer+CPQ/uMOiLBe3GnMhO2+YYIpu9a
MdCRaZXFgp0nwuCyVeIRGDpyWZOwGY64DpxgjYMpmpQ1O7aBwaiOx1A8nJ3L
ydqM/Z+96cdXFQnwpT5OmQtbIxGecPk8A3H9OGkB7LiCApKm2Qs4pp0Zswvb
dYRbRXVXMnAXCarvwa6iB2U5EylgzsXrJI09RUGWlIn/YDM5YVoOm3f7Ml9q
dyo0W7uUKK3vFxnH4M7iNPB5LCiqgR1ZmqY2k54OnQPDn05jF58cmNsYnwG7
KctZBtgbLAxt1NyyIHx1TscS+1WUMvnlwdfQeah1yg+bpSrpI0khF0SdeUND
sS2W2I5cV8qDoeSdXrHYofkT/AJJ+XD0t6VJFra6QAnXj7QCCNLbtba+f68X
TjXFSBTCjbTvR4bW97eOSOSJrELYZypXO4096x/CnpHzFiz3ewxsxOdB+sR5
5huFRdBbmb94DLstkFIhrFgMHVHHGvWxGYTLfFuKiqEPSS5ZYF802ktTKH0P
zrP9mr7YWz7OzhHUUijTHa+pwL4TETVKaSyHTzl65sb4vEuTqHypoo/gMBPX
FQfs1ooF698tCEQWoqg+2NuJXOdMByq835/olIl9U3b71KYbFXBK5UX6GrZY
wvBwq1slMPjUUMrweaxqt1qZ/a4ShF82FndiG8rzpwYuVMK04lelGWy/L2es
Dt+pglUezwN78fnfw1D4/YnXZzAKpP+Owo609++0DKiBHsnSIndcTx40ML4b
Lq8BQnvr/EPsgAMBj64RNcDovuCZh+3xN0D/plsthBxQuzCObekZWO97pQ50
qArmlncIkKOElL843QBL3vYxl3D9as+MSO/jbgJXPxvRYFz/Gtk5gyz1mkBT
z4wvG7vWMdJmOKwJfIHQa8Y2Tzi11Z2pGZ6TRyN34PpqZKb11durGeKvBh17
h606rH0qwqkFJAykHeZxPd48d1Ylx6ANHK5rambier+1R97G41Yb8MkaeHVj
b6Pyhx1/1AYal+TX2HH/IBg+3POtvQ2iHNlnbmDv23vvLqtJO7wsFnY9GkDA
8XPZJebmHcCq3jU6cJ8A97cblbmud8GVlPqPNrif8UyctugL64KGOSWPF9j3
/DpCMnK6QNtp0Ksb+/7ZlC61P13AG+xoo/+AgOhhZQ8Hl2/gsmch4xiFgOxt
14oqPbqBBZgm5cJwf+hWreAe2AvGUp2uWlG4/6plFcx82QuOO4eXQ7G/7dRc
667qBblXwZVfsZurKuvUWPvgjC3r7nPRBFC3VZizBvfBCbGTjLYxBCS9/fAg
NuQ7sHdcORL9CL/Pufy+gtAB2PUk8bFAAgF/SnQ+edQOwmBtt8pYCv7fQ1Is
bSkjUKPw7E4C7o/FFrJ57jwdh9pXoUUczbg+FAiXqVz7DR/Cxny2TuD+9fGP
B5PRM6C70YpflJUGL9LCy87Gz4JEQqDSG1EasFxy+s3qOQda7Pz8jWQaNFpm
ZRgGLsCabJ9OkCUN3m0WvLzouAS9uQW/B71pIJ64e4x0dgUMyHyWQok0WHEP
pGcb0YA62PCiu5gGkys7ODOVV2HVT7cyopUGyVdD2neIMpD5tL09ZXFeiT72
JVxAjJG8QS1v+AeJDpzO2ccreZnIFJekXf5CdCjXeF9WJMhMDhT+sCdckQ50
J4vDxpws5LqLXYNZ2nRgJLU+2U9nIee3sKzym9HhsNatT6cZ2Mj98leWC27g
vNSXM8Y7y0b+85kjocGHDhqOtdE6C2xk+yOedVt96RBBDCkHrbCR5caqh85j
iwjuoMwxkchmwjvaBrBPXaBItvGQyK+dVZ7P4jwX22LnEK1IIv9QZh3muU8H
mWqhKXZPEtmBYmWlGUKHO+ePxh73JpG7fp4rCcGuGD6vetefRLYZ7Fipwz7H
HBE+TiGR41H8UV2cF73IhHRtIomcPPL69XkKzoOlXU5BH0nki7KsVy1xvuQ+
/Y+3jEoi32q/ejEF26ybs3yuikSu98hX+ok9vajJafuVROZZ3phvE47zpeLb
3OP9JPLpn54JdjivWlY1Gd/9SSJnOWX2pmFnGU2uvhklkRkfjW8YxVa9tUdv
9zSJrLu5dPcVnHeDGdUXTf6RyI/izm1Ox26ONk2KXsTjC/EPjWALitw+VUuQ
yEwa7KliOD//X74m/3++/h/IHPzv
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["x", TraditionalForm], 
    FormBox[
     TagBox[
      RowBox[{"p", " ", "\"/\"", " ", 
        SubscriptBox["p", "1"]}], HoldForm], TraditionalForm]},
  AxesOrigin->{-3.1, 0},
  Epilog->{
    InsetBox[
     FormBox["\"0.6\"", TraditionalForm], {0, 1.8}], 
    InsetBox[
     FormBox["\"0.9\"", TraditionalForm], {0, 1.5}], 
    InsetBox[
     FormBox["\"0.999\"", TraditionalForm], {0, 1.2}], 
    InsetBox[
     FormBox["\"1\"", TraditionalForm], {0, 0.85}]},
  Method->{},
  PlotRange->{{-3.1, 3}, {0, 2}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.5755465851850142`*^9, 3.5755467322456617`*^9, 3.5755468455212493`*^9, 
   3.586582800189763*^9, 3.587520306637072*^9, {3.587522425277278*^9, 
   3.5875224550738535`*^9}, 3.587522550963357*^9, 3.5875229789727144`*^9, 
   3.5875230366456327`*^9, 3.587523268359436*^9, 3.5875246093223314`*^9, 
   3.5890778656994724`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Entry and exit Mach numbers and pressure ratio\
\>", "Subsection",
 CellChangeTimes->{{3.428051170715*^9, 3.428051172769*^9}, {
  3.4283122566408253`*^9, 3.4283122590848255`*^9}, {3.428312299787825*^9, 
  3.4283123162898254`*^9}, {3.5875233950326366`*^9, 3.587523405589901*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"crit", "[", "x0", "]"}]], "Input",
 CellChangeTimes->{{3.428312327601825*^9, 3.428312331481825*^9}}],

Cell[BoxData["2.1388894888841503`"], "Output",
 CellChangeTimes->{
  3.575546585459046*^9, 3.575546732685712*^9, 3.5755468457162733`*^9, 
   3.5865828003028126`*^9, 3.587520306736083*^9, {3.5875224254142942`*^9, 
   3.5875224553128815`*^9}, 3.587522551101376*^9, 3.58752297901972*^9, 
   3.587523036732645*^9, 3.5875232684124427`*^9, 3.58752460939334*^9, 
   3.589077866334585*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"crit", "[", 
  RowBox[{"-", "x0"}], "]"}]], "Input",
 CellChangeTimes->{{3.4283122625548253`*^9, 3.428312288694825*^9}, {
  3.428372336170216*^9, 3.428372344335216*^9}}],

Cell[BoxData["0.32419475140189485`"], "Output",
 CellChangeTimes->{
  3.575546585672074*^9, 3.575546732735718*^9, 3.575546845757278*^9, 
   3.586582800349819*^9, 3.5875203067670865`*^9, {3.5875224254632993`*^9, 
   3.587522455360887*^9}, 3.587522551138379*^9, 3.5875229790497217`*^9, 
   3.587523036923667*^9, 3.5875232684494467`*^9, 3.5875246094353466`*^9, 
   3.589077866357585*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox[
  RowBox[{"p", "[", 
   RowBox[{"crit", "[", 
    RowBox[{"-", "x0"}], "]"}], "]"}], 
  RowBox[{"p", "[", 
   RowBox[{"crit", "[", "x0", "]"}], "]"}]]], "Input",
 CellChangeTimes->{{3.428051182225*^9, 3.4280512062609997`*^9}, {
  3.428309740996825*^9, 3.4283097449238253`*^9}}],

Cell[BoxData["9.035373179564248`"], "Output",
 CellChangeTimes->{
  3.5755465857490807`*^9, 3.575546732776722*^9, 3.575546845798283*^9, 
   3.5865828003908176`*^9, 3.5875203067990923`*^9, {3.587522425507306*^9, 
   3.587522455395892*^9}, 3.5875225511843843`*^9, 3.5875229790827274`*^9, 
   3.5875230369606733`*^9, 3.587523268487452*^9, 3.5875246094783506`*^9, 
   3.589077866393592*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Table 14.1 Left: German V2 Rocket Engine (A4/V2) (p. 239) \
\>", "Section",
 CellChangeTimes->{{3.5020912154080896`*^9, 3.502091219519325*^9}, {
  3.5875234710647573`*^9, 3.5875234940705185`*^9}, {3.587523684598379*^9, 
  3.587523770309663*^9}, {3.587523899655182*^9, 3.5875239210937543`*^9}, {
  3.587523963334823*^9, 3.587523974309139*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.428379525827216*^9, 3.428379528378216*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(16\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.5755465860451164`*^9, 3.575546689421523*^9, 3.5755467330597563`*^9, 
   3.5755468460623145`*^9, 3.5865828006238556`*^9, 3.587523424494169*^9, 
   3.5875236324521227`*^9, {3.5875246097653837`*^9, 3.5875246214037805`*^9}, 
   3.587524802923564*^9, 3.5875249108405085`*^9, 3.5875257419892364`*^9, 
   3.58907786664062*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.5875239871126757`*^9, 3.5875239887258687`*^9}}],

Cell["General constants", "Text",
 CellChangeTimes->{{3.587524036936654*^9, 3.5875240411261563`*^9}, {
  3.587524079752792*^9, 3.587524080741911*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["R", "mol"], "=", "MolarGasConstant"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Gamma]", "=", "1.2"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "adiabatic", " ", "index", " ", "for", " ", "burning", " ", "gases"}], " ",
    "*)"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.428559452370986*^9, 3.4285594616959863`*^9}, {
  3.587523995421672*^9, 3.5875240005222874`*^9}, {3.5875240492111273`*^9, 
  3.587524069883608*^9}}],

Cell["\<\
Diameters of V2 engine (from Czech data sheet)\
\>", "Text",
 CellChangeTimes->{{3.428551817927986*^9, 3.4285518219499865`*^9}, {
  3.428900664500208*^9, 3.428900674789208*^9}, {3.58752409999022*^9, 
  3.587524100215247*^9}, {3.587524184136317*^9, 3.587524193862483*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["D", "entry"], "=", 
   RowBox[{"0.94", " ", "Meter"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["D", "throat"], "=", 
   RowBox[{"0.405", " ", "Meter"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["D", "exit"], "=", 
   RowBox[{"0.735", " ", "Meter"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.428551728816986*^9, 3.4285517654239864`*^9}, {
  3.428551857888986*^9, 3.428551869007986*^9}, {3.4285519603529863`*^9, 
  3.428551980288986*^9}, {3.428552908273986*^9, 3.428552926368986*^9}, {
  3.428553079985986*^9, 3.4285530830889864`*^9}, {3.428566143216986*^9, 
  3.4285661655209866`*^9}, {3.4285679137289863`*^9, 3.428567933328986*^9}, {
  3.4285697151699862`*^9, 3.4285697179529862`*^9}, {3.428902274572208*^9, 
  3.428902279175208*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basic results", "Subsection",
 CellChangeTimes->{{3.5875241459907403`*^9, 3.587524146901848*^9}, {
  3.5875251397509747`*^9, 3.587525141589196*^9}}],

Cell["Nozzle areas", "Text",
 CellChangeTimes->{{3.587524115990139*^9, 3.5875241289976997`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["A", "entry"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["\[Pi]", "4"], 
    SuperscriptBox[
     SubscriptBox["D", "entry"], "2"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4285517710309863`*^9, 3.428551780832986*^9}, {
  3.428551831617986*^9, 3.4285518723539863`*^9}, {3.428551982480986*^9, 
  3.428552011856986*^9}}],

Cell[BoxData[
 TemplateBox[{"0.6939778171779852`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "2"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465881953773`*^9, 3.5755466896435485`*^9, 3.5755467331747713`*^9, 
   3.575546846148325*^9, 3.586582801046876*^9, 3.5875234246861916`*^9, 
   3.5875236326031404`*^9, {3.5875246098353925`*^9, 3.5875246859225245`*^9}, 
   3.5875248030575767`*^9, 3.587524910980526*^9, 3.587525742109251*^9, 
   3.5890778668636694`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["A", "throat"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["\[Pi]", "4"], 
    SuperscriptBox[
     SubscriptBox["D", "throat"], "2"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.428551848278986*^9, 3.4285518803529863`*^9}, {
  3.428551987488986*^9, 3.4285520187209864`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.12882493375126647`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "2"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465882563815`*^9, 3.5755466900195913`*^9, 3.575546733233777*^9, 
   3.575546846203333*^9, 3.5865828010768747`*^9, 3.5875234247311974`*^9, 
   3.587523632658148*^9, {3.587524609889399*^9, 3.5875246860395365`*^9}, 
   3.587524803108585*^9, 3.587524911030532*^9, 3.5875257421582565`*^9, 
   3.5890778673187103`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["A", "exit"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["\[Pi]", "4"], 
    SuperscriptBox[
     SubscriptBox["D", "exit"], "2"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.428551902546986*^9, 3.428551914000986*^9}, {
  3.4285519930579863`*^9, 3.428551994959986*^9}, {3.428552025072986*^9, 
  3.4285520269119864`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.42429172282138644`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "2"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465883053885`*^9, 3.5755466902296185`*^9, 3.5755467332827845`*^9, 
   3.5755468462553396`*^9, 3.5865828011188817`*^9, 3.587523424778203*^9, 
   3.587523632706153*^9, {3.5875246099424057`*^9, 3.5875246860955424`*^9}, 
   3.5875248031585884`*^9, 3.587524911080538*^9, 3.5875257422082634`*^9, 
   3.5890778673427095`*^9}]
}, Open  ]],

Cell["\<\
Functions (see Fig. 14.1 above)\
\>", "Text",
 CellChangeTimes->{{3.4283230722358255`*^9, 3.4283230745948253`*^9}, {
  3.5875242182824135`*^9, 3.5875242369186487`*^9}, {3.587524279381744*^9, 
  3.5875242871266727`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "m_", "]"}], "=", 
   RowBox[{"1", "+", 
    RowBox[{
     FractionBox[
      RowBox[{"\[Gamma]", "-", "1"}], 
      RowBox[{"\[Gamma]", "+", "1"}]], 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m", "2"], "-", "1"}], ")"}]}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4283121004998255`*^9, 3.428312111538825*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "m_", "]"}], "=", 
   RowBox[{
    FractionBox["1", "m"], " ", 
    SuperscriptBox[
     RowBox[{"t", "[", "m", "]"}], 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"\[Gamma]", "+", "1"}], ")"}], "/", "2"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.3901209442663546`*^9, 3.390120957135355*^9}, {
  3.428312118391825*^9, 3.4283121246178255`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sub", "[", "a_", "]"}], ":=", 
  RowBox[{"m", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "m", "]"}], "\[Equal]", "a"}], ",", 
     RowBox[{"{", 
      RowBox[{"m", ",", "0.5"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"super", "[", "a_", "]"}], ":=", 
  RowBox[{"m", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "m", "]"}], "\[Equal]", "a"}], ",", 
     RowBox[{"{", 
      RowBox[{"m", ",", "1.5"}], "}"}]}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.5875235249752264`*^9, 3.5875235308889356`*^9}}],

Cell["Mach numbers", "Text",
 CellChangeTimes->{{3.428379586441216*^9, 3.4283795970902157`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox[
  SubscriptBox["A", "entry"], 
  SubscriptBox["A", "throat"]]], "Input",
 CellChangeTimes->{{3.428380207893216*^9, 3.4283802140572157`*^9}}],

Cell[BoxData["5.38698369151044`"], "Output",
 CellChangeTimes->{
  3.5755465886024227`*^9, 3.5755466906366663`*^9, 3.5755467335358143`*^9, 
   3.575546846372353*^9, 3.586582805526476*^9, 3.5875234248922167`*^9, 
   3.587523632820166*^9, {3.5875246100604196`*^9, 3.587524686215557*^9}, 
   3.5875248032736025`*^9, 3.5875249111965523`*^9, 3.587525742318276*^9, 
   3.5890778674046917`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["Ma", "entry"], "=", 
  RowBox[{"sub", "[", 
   FractionBox[
    SubscriptBox["A", "entry"], 
    SubscriptBox["A", "throat"]], "]"}]}]], "Input",
 CellChangeTimes->{{3.428380058718216*^9, 3.428380065106216*^9}, {
  3.4283802756482162`*^9, 3.428380288525216*^9}, {3.587523534478367*^9, 
  3.5875235351454463`*^9}}],

Cell[BoxData["0.11064118140192954`"], "Output",
 CellChangeTimes->{
  3.575546588784446*^9, 3.5755466911567307`*^9, 3.575546733711836*^9, 
   3.575546846555375*^9, 3.586582805774514*^9, 3.5875234250692377`*^9, 
   3.5875236330001864`*^9, {3.587524610153431*^9, 3.5875246863115683`*^9}, 
   3.5875248033626127`*^9, 3.587524911288563*^9, 3.587525742408287*^9, 
   3.5890778674506936`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox[
  SubscriptBox["A", "exit"], 
  SubscriptBox["A", "throat"]]], "Input",
 CellChangeTimes->{{3.428380253680216*^9, 3.428380261921216*^9}, {
   3.428592859641*^9, 3.428592866599*^9}, 3.428592956901*^9}],

Cell[BoxData["3.2935528120713298`"], "Output",
 CellChangeTimes->{
  3.5755465886934357`*^9, 3.57554669074968*^9, 3.575546733621825*^9, 
   3.5755468464633636`*^9, 3.58658280565545*^9, 3.587523424979227*^9, 
   3.587523632909177*^9, {3.5875246102434425`*^9, 3.58752468640458*^9}, 
   3.5875248034516234`*^9, 3.587524911377573*^9, 3.5875257424972973`*^9, 
   3.5890778674947414`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["Ma", "exit"], "=", 
  RowBox[{"super", "[", 
   FractionBox[
    SubscriptBox["A", "exit"], 
    SubscriptBox["A", "throat"]], "]"}]}]], "Input",
 CellChangeTimes->{{3.4283802937552156`*^9, 3.428380309022216*^9}, {
  3.5875235421732893`*^9, 3.587523543016389*^9}}],

Cell[BoxData["2.4706202211386015`"], "Output",
 CellChangeTimes->{
  3.575546588969467*^9, 3.575546691394759*^9, 3.575546733900859*^9, 
   3.575546847093439*^9, 3.586582807142671*^9, 3.5875234252462606`*^9, 
   3.58752363317721*^9, {3.587524610338453*^9, 3.5875246864985924`*^9}, 
   3.587524803542636*^9, 3.5875249114695845`*^9, 3.5875257425883083`*^9, 
   3.589077867541763*^9}]
}, Open  ]],

Cell["\<\
Exit-to-entry ratios for temperature, pressure, density, and  sound velocity\
\>", "Text",
 CellChangeTimes->{{3.428387031055216*^9, 3.4283870348912163`*^9}, {
  3.587524344566565*^9, 3.5875243826461344`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tratio", "=", 
  FractionBox[
   RowBox[{"1", "+", 
    RowBox[{
     FractionBox["1", "2"], 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      SubscriptBox["Ma", "exit"], "2"]}]}], 
   RowBox[{"1", "+", 
    RowBox[{
     FractionBox["1", "2"], 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      SubscriptBox["Ma", "entry"], "2"]}]}]]}]], "Input",
 CellChangeTimes->{{3.428387048216216*^9, 3.428387101801216*^9}, {
  3.4284691610673866`*^9, 3.4284691680913863`*^9}}],

Cell[BoxData["1.6084274758762003`"], "Output",
 CellChangeTimes->{
  3.575546589024474*^9, 3.5755466917567997`*^9, 3.575546733938862*^9, 
   3.575546847227454*^9, 3.5865828072366886`*^9, 3.5875234252852635`*^9, 
   3.587523633217213*^9, {3.587524610381461*^9, 3.5875246865435963`*^9}, 
   3.5875248035806417`*^9, 3.587524911509591*^9, 3.587525742626313*^9, 
   3.5890778675617075`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pratio", "=", 
  SuperscriptBox["Tratio", 
   RowBox[{"\[Gamma]", "/", 
    RowBox[{"(", 
     RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]], "Input",
 CellChangeTimes->{{3.428387126890216*^9, 3.428387148669216*^9}}],

Cell[BoxData["17.314457944152107`"], "Output",
 CellChangeTimes->{
  3.575546589067481*^9, 3.57554669181381*^9, 3.5755467339768667`*^9, 
   3.5755468473244686`*^9, 3.586582807300697*^9, 3.5875234253252697`*^9, 
   3.587523633256219*^9, {3.5875246105084734`*^9, 3.5875246865876036`*^9}, 
   3.587524803621644*^9, 3.5875249115495944`*^9, 3.587525742664316*^9, 
   3.589077867590711*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Rho]ratio", "=", 
  SuperscriptBox["Tratio", 
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]], "Input",
 CellChangeTimes->{{3.428387126890216*^9, 3.428387148669216*^9}, {
  3.428387571429216*^9, 3.4283875750612164`*^9}}],

Cell[BoxData["10.764835967950594`"], "Output",
 CellChangeTimes->{
  3.5755465891224866`*^9, 3.575546693268984*^9, 3.5755467340168705`*^9, 
   3.57554684742048*^9, 3.5865828073416796`*^9, 3.5875234253642735`*^9, 
   3.587523633295223*^9, {3.5875246105514793`*^9, 3.587524686629608*^9}, 
   3.587524803660651*^9, 3.587524911590601*^9, 3.5875257427013226`*^9, 
   3.5890778676107173`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cratio", "=", 
  SqrtBox["Tratio"]}]], "Input",
 CellChangeTimes->{{3.4283876254432163`*^9, 3.4283876344182158`*^9}}],

Cell[BoxData["1.2682379413486258`"], "Output",
 CellChangeTimes->{
  3.5755465891784935`*^9, 3.575546693551017*^9, 3.575546734056877*^9, 
   3.575546847468485*^9, 3.5865828073906794`*^9, 3.587523425403279*^9, 
   3.587523633335229*^9, {3.587524610593484*^9, 3.587524686672614*^9}, 
   3.5875248036976533`*^9, 3.5875249116296043`*^9, 3.5875257427393255`*^9, 
   3.589077867629757*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Mass flux", "Subsection",
 CellChangeTimes->{{3.4285593589709864`*^9, 3.4285593672939863`*^9}}],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.5875257787446465`*^9, 3.587525781894024*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Mgas", "=", 
    RowBox[{"27.3", " ", 
     RowBox[{"Gram", "/", "Mole"}]}]}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"see", " ", "problem", " ", "14.6"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["T", "entry"], "=", 
   RowBox[{"3000", "Kelvin"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Q", "=", 
   RowBox[{"125", " ", 
    RowBox[{"Kilogram", "/", "Second"}]}]}], ";"}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQ3S/zyr+0+6XjGTXDQBD9gfF/Iog+cSEpCURLxLGl
g+h9P/+B6Qs16g5lQPrKJV8XED3jxCkPEP089ByYrsp7tqICbI79KhDd+3HX
FhA9de7h7SC65G3AYRC94YzPKRD9+L+6diWQZmrdD6ZrZu71B9GtrmVBINq+
Xyy0CkinHegKA9HuqaaPZgHpvGmhYPrYecYns4F0hqjBUxA922P+RxCt0rUe
TFfNaPkOoh2fOP8C0eXfA3bK9Lx0ZLCsBdMuR1IvguhdTQvBNJdAZZkskJ51
qQlMN9/5nbf290vH6vyjxSC6qvFjDYh+dli3FkSLL+pvXLnrlWOTgEoLiAYA
Zom4Nw==
  "]],

Cell["Specific gas constant", "Text",
 CellChangeTimes->{{3.587525785174418*^9, 3.5875257887098417`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"R", "=", 
   RowBox[{
    SubscriptBox["R", "mol"], "/", "Mgas"}]}], ";"}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQ3S/zyr+0+6XjGTXDQBD9gfF/Iog+cSEpCURLxLGl
g+h9P/+B6Qs16g5lQPrKJV8XED3jxCkPEP089ByYrsp7tqICbI79KhDd+3HX
FhA9de7h7SC65G3AYRC94YzPKRD9+L+6diWQZmrdD6ZrZu71B9GtrmVBINq+
Xyy0CkinHegKA9HuqaaPZgHpvGmhYPrYecYns4F0hqjBUxA922P+RxCt0rUe
TFfNaPkOoh2fOP8C0eXfA3bK9Lx0ZLCsBdMuR1IvguhdTQvBNJdAZZkskJ51
qQlMN9/5nbf290vH6vyjxSC6qvFjDYh+dli3FkSLL+pvXLnrleOreWdbQDQA
caG51g==
  "]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["c", "entry"], "=", 
  RowBox[{
   SqrtBox[
    RowBox[{"\[Gamma]", " ", "R", " ", 
     SubscriptBox["T", "entry"]}]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.428559413894986*^9, 3.4285594440679865`*^9}, {
  3.428559508181986*^9, 3.4285595104829865`*^9}, {3.4285596377799864`*^9, 
  3.4285596445779862`*^9}, {3.587524642590323*^9, 3.5875246445505595`*^9}, {
  3.587525679038682*^9, 3.587525687607712*^9}}],

Cell[BoxData[
 TemplateBox[{"1047.0972183209087`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546589812569*^9, 3.5755466940690784`*^9, 3.575546734492929*^9, 
   3.5755468484276004`*^9, 3.586582807628716*^9, 3.5875234259383445`*^9, 
   3.587523633806284*^9, {3.587524610881518*^9, 3.5875246870946627`*^9}, 
   3.587524804134708*^9, 3.587524912059655*^9, 3.587525743263389*^9, 
   3.58907786789275*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["U", "entry"], "=", 
  RowBox[{
   SubscriptBox["Ma", "entry"], 
   SubscriptBox["c", "entry"]}]}]], "Input",
 CellChangeTimes->{{3.428559843763986*^9, 3.428559853759986*^9}}],

Cell[BoxData[
 TemplateBox[{"115.85207327769947`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465898665752`*^9, 3.575546694696154*^9, 3.5755467376523075`*^9, 
   3.5755468489176574`*^9, 3.5865828077838306`*^9, 3.5875234259793477`*^9, 
   3.58752363384629*^9, {3.5875246109285245`*^9, 3.587524687155672*^9}, 
   3.5875248042367196`*^9, 3.5875249121736717`*^9, 3.587525743305393*^9, 
   3.5890778679587584`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Rho]", "entry"], "=", 
  FractionBox["Q", 
   RowBox[{
    SubscriptBox["A", "entry"], 
    SubscriptBox["U", "entry"]}]]}]], "Input",
 CellChangeTimes->{{3.4289995844850764`*^9, 3.428999603028076*^9}}],

Cell[BoxData[
 TemplateBox[{"1.554750174230843`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546590081602*^9, 3.5755466950441947`*^9, 3.5755467384114*^9, 
   3.575546849742758*^9, 3.586582807902754*^9, 3.5875234262103753`*^9, 
   3.5875236340583134`*^9, {3.5875246111165485`*^9, 3.5875246873596945`*^9}, 
   3.587524804434744*^9, 3.5875249123806934`*^9, 3.587525743507419*^9, 
   3.5890778680658274`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["p", "entry"], "=", 
  RowBox[{
   RowBox[{"R", " ", 
    SubscriptBox["T", "entry"], " ", 
    SubscriptBox["\[Rho]", "entry"]}], "//", 
   RowBox[{"ToUnit", "[", "Bar", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.4290004420330763`*^9, 3.429000511759076*^9}, {
  3.587524667343294*^9, 3.5875246683104086`*^9}, {3.587525693272391*^9, 
  3.5875256975919075`*^9}}],

Cell[BoxData[
 TemplateBox[{"14.205397141330483`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"bars\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: bars"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Bars\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465904986506`*^9, 3.575546695385236*^9, 3.5755467387634425`*^9, 
   3.575546850223814*^9, 3.5865828081187935`*^9, 3.5875234265684175`*^9, 
   3.5875236344073563`*^9, {3.5875246114485884`*^9, 3.587524687700735*^9}, 
   3.587524804762781*^9, 3.587524912712734*^9, 3.587525743804453*^9, 
   3.5890778682248335`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"fuel", "=", 
   RowBox[{"8610.", " ", "Kilogram"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"burningtime", "=", 
  FractionBox["fuel", "Q"]}]}], "Input",
 CellChangeTimes->{{3.428568400752986*^9, 3.428568417983986*^9}, {
  3.4873950700049825`*^9, 3.4873950878790045`*^9}}],

Cell[BoxData[
 TemplateBox[{"68.88`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"s\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Seconds\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546591931824*^9, 3.5755466955212517`*^9, 3.5755467391954937`*^9, 
   3.575546850477844*^9, 3.586582808254835*^9, 3.5875234267574406`*^9, 
   3.5875236345443735`*^9, {3.587524611657613*^9, 3.587524687844753*^9}, 
   3.587524804891797*^9, 3.5875249128407507`*^9, 3.587525743945471*^9, 
   3.5890778682978*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Performance", "Subsection",
 CellChangeTimes->{{3.587525259896391*^9, 3.5875252632057886`*^9}}],

Cell["Exit values", "Text",
 CellChangeTimes->{{3.428564480531986*^9, 3.428564481933986*^9}, {
  3.5875245298457956`*^9, 3.5875245315099964`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["T", "exit"], "=", 
  RowBox[{
   SubscriptBox["T", "entry"], "/", "Tratio"}]}]], "Input",
 CellChangeTimes->{{3.4285644869519863`*^9, 3.4285645090889864`*^9}}],

Cell[BoxData[
 TemplateBox[{"1865.1757974761856`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kelvins"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465924758883`*^9, 3.5755466955612574`*^9, 3.575546739267501*^9, 
   3.5755468505188494`*^9, 3.5865828083467956`*^9, 3.5875234269944696`*^9, 
   3.587523634650388*^9, {3.587524611705618*^9, 3.5875246878997607`*^9}, 
   3.5875248049428024`*^9, 3.587524912891757*^9, {3.587525726710405*^9, 
   3.587525743986477*^9}, 3.589077868321804*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["p", "exit"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["p", "entry"], "/", "pratio"}], "//", 
   RowBox[{"ToUnit", "[", "Bar", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.428564513079986*^9, 3.428564524112986*^9}, {
  3.428568457473986*^9, 3.4285684620969863`*^9}, {3.4285933074700003`*^9, 
  3.4285933079960003`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.8204355681910506`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"bars\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: bars"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Bars\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465925238943`*^9, 3.575546695601263*^9, 3.575546739669551*^9, 
   3.5755468505578537`*^9, 3.5865828084208117`*^9, 3.58752342739052*^9, 
   3.587523634691389*^9, {3.5875246117746277`*^9, 3.5875246879527683`*^9}, 
   3.5875248049858084`*^9, 3.58752491292976*^9, {3.5875257268034143`*^9, 
   3.587525744036482*^9}, 3.5890778683448086`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Rho]", "exit"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Rho]", "entry"], "/", "\[Rho]ratio"}], "//", 
   "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4293247539726105`*^9, 3.429324770764611*^9}}],

Cell[BoxData[
 TemplateBox[{"0.14442859871341224`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465925809026`*^9, 3.5755466956402664`*^9, 3.5755467400195932`*^9, 
   3.5755468505988593`*^9, 3.586582808517853*^9, 3.587523427525535*^9, 
   3.587523634732398*^9, {3.587524611827632*^9, 3.587524688004774*^9}, 
   3.587524805040815*^9, 3.587524912967766*^9, {3.587525726841419*^9, 
   3.587525744150494*^9}, 3.5890778683738103`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["c", "exit"], "=", 
  RowBox[{
   SqrtBox[
    RowBox[{"\[Gamma]", " ", "R", " ", 
     SubscriptBox["T", "exit"]}]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4285645313209863`*^9, 3.428564549778986*^9}, {
  3.587524680558879*^9, 3.587524681623008*^9}, {3.587525710142415*^9, 
  3.5875257173842835`*^9}}],

Cell[BoxData[
 TemplateBox[{"825.6315192774006`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546593854055*^9, 3.575546695937304*^9, 3.5755467405556555`*^9, 
   3.5755468509078984`*^9, 3.586582808991892*^9, 3.5875234280105925`*^9, 
   3.5875236350224295`*^9, {3.587524612021656*^9, 3.5875246883048077`*^9}, 
   3.587524805349852*^9, 3.5875249132728004`*^9, {3.587525726879424*^9, 
   3.5875257443705206`*^9}, 3.5890778685088696`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["U", "exit"], "=", 
  RowBox[{
   SubscriptBox["Ma", "exit"], 
   SubscriptBox["c", "exit"]}]}]], "Input",
 CellChangeTimes->{{3.428564571130986*^9, 3.4285645785439863`*^9}}],

Cell[BoxData[
 TemplateBox[{"2039.821926736131`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465945671406`*^9, 3.575546696092321*^9, 3.575546740660672*^9, 
   3.575546850948901*^9, 3.5865828090369205`*^9, 3.587523428266621*^9, 
   3.5875236350614347`*^9, {3.5875246120746617`*^9, 3.5875246883698187`*^9}, 
   3.5875248054618654`*^9, 3.5875249133668118`*^9, {3.587525726923429*^9, 
   3.587525744416527*^9}, 3.589077868565875*^9}]
}, Open  ]],

Cell["Thrust at ground level", "Text",
 CellChangeTimes->{{3.4290031464470763`*^9, 3.4290031476850767`*^9}, {
  3.5875248738800774`*^9, 3.587524877302486*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["p", "back"], "=", 
   RowBox[{"1", " ", "Bar"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4285662598199863`*^9, 3.428566269071986*^9}, {
  3.428566482992986*^9, 3.428566483151986*^9}, {3.4285666325779862`*^9, 
  3.428566638959986*^9}, {3.4285678182589865`*^9, 3.428567824943986*^9}, {
  3.428568442689986*^9, 3.428568451855986*^9}, {3.428903835372208*^9, 
  3.428903841846208*^9}, {3.4290000092690763`*^9, 3.4290000164770765`*^9}, {
  3.429000192523076*^9, 3.4290001928380766`*^9}, {3.4290031652390766`*^9, 
  3.4290031680710764`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"thrust", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Q", " ", 
     SubscriptBox["U", "exit"]}], "+", 
    RowBox[{
     SubscriptBox["A", "exit"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["p", "exit"], "-", 
       SubscriptBox["p", "back"]}], ")"}]}]}], "//", 
   RowBox[{"ToUnit", "[", "Kilonewton", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.428564584778986*^9, 3.428564619007986*^9}, {
  3.4285662255209866`*^9, 3.428566252992986*^9}, {3.5020914268911858`*^9, 
  3.5020914276922317`*^9}, {3.575546716734799*^9, 3.5755467175839005`*^9}}],

Cell[BoxData[
 TemplateBox[{"247.35897062905016`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kN\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilonewtons"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilonewtons\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465950832043`*^9, {3.575546696377355*^9, 3.5755467421278477`*^9}, 
   3.5755468512369375`*^9, 3.586582809252924*^9, 3.5875234288296924`*^9, 
   3.5875236353314667`*^9, {3.587524612324694*^9, 3.5875246886608505`*^9}, 
   3.587524805759903*^9, 3.587524913643845*^9, {3.587525727045442*^9, 
   3.5875257447095613`*^9}, 3.5890778692259636`*^9}]
}, Open  ]],

Cell["Take-off acceleration", "Text",
 CellChangeTimes->{{3.429329106941611*^9, 3.4293291105016108`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"mass", "=", 
   RowBox[{"12500", " ", "Kilogram"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.429329163329611*^9, 3.429329179134611*^9}, {
  3.4293292564516106`*^9, 3.429329277190611*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["thrust", 
   RowBox[{"mass", " ", 
    SubscriptBox["g", "0"]}]], "//", "ToSI"}]], "Input",
 CellChangeTimes->{{3.429329208242611*^9, 3.4293292232516108`*^9}, {
  3.429329298961611*^9, 3.429329299966611*^9}}],

Cell[BoxData["2.0178876222077893`"], "Output",
 CellChangeTimes->{
  3.5755465961883345`*^9, 3.5755466966093817`*^9, 3.575546743779043*^9, 
   3.5755468515999804`*^9, 3.5865828097180014`*^9, 3.5875234294167614`*^9, 
   3.5875236355524926`*^9, {3.587524612929764*^9, 3.587524688885877*^9}, 
   3.587524805993931*^9, 3.5875249138778734`*^9, {3.5875257272074633`*^9, 
   3.5875257449555902`*^9}, 3.5890778699670258`*^9}]
}, Open  ]],

Cell["Thrust outside atmosphere", "Text",
 CellChangeTimes->{{3.4290031464470763`*^9, 3.4290031476850767`*^9}, {
  3.587524880149827*^9, 3.587524887845751*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["p", "back"], "=", 
   RowBox[{"0", " ", "Bar"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4285662598199863`*^9, 3.428566269071986*^9}, {
  3.428566482992986*^9, 3.428566483151986*^9}, {3.4285666325779862`*^9, 
  3.428566638959986*^9}, {3.4285678182589865`*^9, 3.428567824943986*^9}, {
  3.428568442689986*^9, 3.428568451855986*^9}, {3.428903835372208*^9, 
  3.428903841846208*^9}, {3.4290000092690763`*^9, 3.4290000164770765`*^9}, {
  3.429000192523076*^9, 3.4290001928380766`*^9}, {3.4290031652390766`*^9, 
  3.4290031680710764`*^9}, {3.5875249059809275`*^9, 3.5875249063779745`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"thrust", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Q", " ", 
     SubscriptBox["U", "exit"]}], "+", 
    RowBox[{
     SubscriptBox["A", "exit"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["p", "exit"], "-", 
       SubscriptBox["p", "back"]}], ")"}]}]}], "//", 
   RowBox[{"ToUnit", "[", "Kilonewton", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.428564584778986*^9, 3.428564619007986*^9}, {
  3.4285662255209866`*^9, 3.428566252992986*^9}, {3.5020914268911858`*^9, 
  3.5020914276922317`*^9}, {3.575546716734799*^9, 3.5755467175839005`*^9}}],

Cell[BoxData[
 TemplateBox[{"289.78814291118886`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kN\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilonewtons"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilonewtons\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755465950832043`*^9, {3.575546696377355*^9, 3.5755467421278477`*^9}, 
   3.5755468512369375`*^9, 3.586582809252924*^9, 3.5875234288296924`*^9, 
   3.5875236353314667`*^9, {3.587524612324694*^9, 3.5875246886608505`*^9}, 
   3.587524805759903*^9, 3.587524914158907*^9, {3.5875257273324776`*^9, 
   3.5875257452366247`*^9}, 3.5890778701490345`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Table 14.1 Right: Space Shuttle Main Engine (SSME) (p. 239) \
\>", "Section",
 CellChangeTimes->{{3.5875238143359475`*^9, 3.5875238285506516`*^9}, {
  3.5875238686944675`*^9, 3.5875238831422014`*^9}, {3.5875239514794006`*^9, 
  3.5875239536696625`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.428379525827216*^9, 3.428379528378216*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(31\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{{3.5875255301548204`*^9, 3.58752554566168*^9}, 
   3.587525610018401*^9, 3.587525879646752*^9, 3.5890778705660872`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basic parameters", "Subsection",
 CellChangeTimes->{{3.5875253158151026`*^9, 3.5875253219898415`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["R", "mol"], "=", "MolarGasConstant"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Gamma]", "=", "1.2"}], ";"}]}], "Input",
 CellChangeTimes->{{3.428559452370986*^9, 3.4285594616959863`*^9}, {
  3.5875252809879227`*^9, 3.5875252890948944`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fuel mixture", "Subsection",
 CellChangeTimes->{{3.4285554368509865`*^9, 3.4285554380299864`*^9}, {
  3.428558074686986*^9, 3.428558078877986*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Qh2", "=", 
    RowBox[{"70.3", " ", 
     RowBox[{"Kilogram", "/", "Second"}]}]}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"hydrogen", " ", "mass", " ", "flow"}], " ", "*)"}], ";"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Qo2", "=", 
   RowBox[{"424", " ", 
    RowBox[{"Kilogram", "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"oxygen", " ", "mass", " ", "flow"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4285576022749863`*^9, 3.4285576830089865`*^9}, {
  3.428557893104986*^9, 3.428557912815986*^9}, {3.428569466453986*^9, 
  3.428569473391986*^9}, {3.428569890816986*^9, 3.428569901679986*^9}, {
  3.428907139727208*^9, 3.4289071527522078`*^9}, {3.428907633429208*^9, 
  3.428907684471208*^9}, {3.4289078709822083`*^9, 3.428907878230208*^9}, {
  3.428911432668208*^9, 3.428911440505208*^9}, {3.5875250019474416`*^9, 
  3.587525051148345*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"Qh2", "+", "Qo2"}]}]], "Input",
 CellChangeTimes->{{3.428907730360208*^9, 3.428907739352208*^9}, 
   3.428907796084208*^9, {3.428907883045208*^9, 3.428907888213208*^9}}],

Cell[BoxData[
 TemplateBox[{"494.3`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546599274705*^9, 3.5755466974694853`*^9, 3.57554674782953*^9, 
   3.5755468531331635`*^9, 3.5865828119512925`*^9, 3.5875234316280265`*^9, 
   3.587523636336587*^9, 3.587524613605845*^9, {3.5875255304708567`*^9, 
   3.5875255459207096`*^9}, 3.5875256102654314`*^9, 3.5875258799357877`*^9, 
   3.589077870844187*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "//", 
  RowBox[{"ToUnit", "[", 
   RowBox[{"Pound", "/", "Second"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4289238321662083`*^9, 3.428923838598208*^9}}],

Cell[BoxData[
 TemplateBox[{"1089.74496197985`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"lb\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pounds per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Pounds\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546599653753*^9, 3.575546697848532*^9, 3.5755467502878246`*^9, 
   3.5755468535692177`*^9, 3.5865828120793686`*^9, 3.587523431795045*^9, 
   3.5875236364415994`*^9, 3.5875246137298613`*^9, {3.587525530583871*^9, 
   3.587525546033725*^9}, 3.587525610374443*^9, 3.5875258800488005`*^9, 
   3.5890778709511695`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox["Qo2", "Qh2"]], "Input",
 CellChangeTimes->{{3.4289128968572083`*^9, 3.428912923229208*^9}, {
  3.4289129538512077`*^9, 3.4289129566662083`*^9}}],

Cell[BoxData["6.031294452347084`"], "Output",
 CellChangeTimes->{
  3.5755465998427725`*^9, 3.5755466980515566`*^9, 3.5755467514739685`*^9, 
   3.5755468539492626`*^9, 3.586582812782384*^9, 3.587523432153088*^9, 
   3.587523636579616*^9, 3.5875246138868804`*^9, {3.5875255307268877`*^9, 
   3.5875255461817417`*^9}, 3.5875256105184603`*^9, 3.5875258801928186`*^9, 
   3.589077871213169*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox["Qh2", "Q"]], "Input",
 CellChangeTimes->{{3.4291633126210003`*^9, 3.429163320106*^9}}],

Cell[BoxData["0.1422213230831479`"], "Output",
 CellChangeTimes->{
  3.575546601271944*^9, 3.5755466987566423`*^9, 3.5755467517489986`*^9, 
   3.5755468543413095`*^9, 3.58658281288739*^9, 3.587523432329111*^9, 
   3.587523636716632*^9, 3.5875246140438995`*^9, {3.5875255308759046`*^9, 
   3.5875255463297606`*^9}, 3.5875256106594777`*^9, 3.587525880334834*^9, 
   3.589077871313199*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox["Qo2", "Q"]], "Input",
 CellChangeTimes->{{3.429163323006*^9, 3.429163325915*^9}}],

Cell[BoxData["0.8577786769168522`"], "Output",
 CellChangeTimes->{
  3.575546601774006*^9, 3.575546699315709*^9, 3.575546753845251*^9, 
   3.5755468546833487`*^9, 3.5865828138165503`*^9, 3.587523432609146*^9, 
   3.5875236369116554`*^9, 3.587524614267926*^9, {3.587525531032925*^9, 
   3.587525546478778*^9}, 3.587525610802494*^9, 3.587525880476851*^9, 
   3.5890778714312263`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basic results", "Subsection",
 CellChangeTimes->{{3.587525369126498*^9, 3.5875253709977217`*^9}}],

Cell["Nozzle sizes", "Text",
 CellChangeTimes->{{3.428551817927986*^9, 3.4285518219499865`*^9}, {
  3.4289002570752077`*^9, 3.428900261522208*^9}, {3.428900390143208*^9, 
  3.428900394155208*^9}, {3.505447512352451*^9, 3.5054475176397533`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["D", "throat"], "=", 
   RowBox[{"10.3", "Inch"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["D", "exit"], "=", 
   RowBox[{"90.7", " ", "Inch"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["D", "entry"], "=", 
   RowBox[{"2", 
    SubscriptBox["D", "throat"]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.428913356753208*^9, 3.4289133850562077`*^9}, 
   3.4289181009762077`*^9, 3.428923503304208*^9, {3.428923658701208*^9, 
   3.428923665039208*^9}, {3.4289912505660763`*^9, 3.4289912576210766`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["A", "entry"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["\[Pi]", "4"], 
    SuperscriptBox[
     SubscriptBox["D", "entry"], "2"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.428918111723208*^9, 3.4289181188512077`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.21502638582981418`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "2"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466031791725`*^9, 3.5755467018880167`*^9, 3.575546756973627*^9, 
   3.5755468547733617`*^9, 3.5865828138635125`*^9, 3.5875234327531605`*^9, 
   3.5875236369876647`*^9, 3.5875246143589373`*^9, {3.5875255310969324`*^9, 
   3.5875255465477867`*^9}, 3.5875256108665037`*^9, 3.58752588054386*^9, 
   3.589077871604252*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["A", "throat"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["\[Pi]", "4"], 
    SuperscriptBox[
     SubscriptBox["D", "throat"], "2"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.428913389323208*^9, 3.4289134194172077`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.053756596457453544`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "2"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466033001885`*^9, 3.5755467026041026`*^9, 3.575546757262662*^9, 
   3.5755468548253684`*^9, 3.586582813889516*^9, 3.5875234328731747`*^9, 
   3.5875236370406713`*^9, 3.5875246144259453`*^9, {3.5875255311569395`*^9, 
   3.5875255466067934`*^9}, 3.5875256109225082`*^9, 3.587525880599866*^9, 
   3.5890778718642974`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["A", "exit"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["\[Pi]", "4"], 
    SuperscriptBox[
     SubscriptBox["D", "exit"], "2"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.428913421531208*^9, 3.428913435672208*^9}}],

Cell[BoxData[
 TemplateBox[{"4.168424009720774`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       SuperscriptBox["\"Meters\"", "2"]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466033561945`*^9, 3.575546702707115*^9, 3.575546757848731*^9, 
   3.575546854878372*^9, 3.5865828139165416`*^9, 3.587523433017194*^9, 
   3.5875236370926776`*^9, 3.587524614494953*^9, {3.587525531284954*^9, 
   3.5875255466707993`*^9}, 3.5875256109785166`*^9, 3.587525880657873*^9, 
   3.58907787191029*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["D", "exit"], "//", "ToSI"}]], "Input",
 CellChangeTimes->{{3.429164334355*^9, 3.429164338539*^9}}],

Cell[BoxData[
 TemplateBox[{"2.30378`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546603443204*^9, 3.575546703460207*^9, 3.575546758209776*^9, 
   3.5755468549143767`*^9, 3.586582813968533*^9, 3.5875234333292303`*^9, 
   3.5875236371316833`*^9, 3.587524614546958*^9, {3.5875255313379593`*^9, 
   3.587525546800815*^9}, 3.5875256110195208`*^9, 3.5875258806998777`*^9, 
   3.589077872123284*^9}]
}, Open  ]],

Cell["Functions", "Text",
 CellChangeTimes->{{3.4283230722358255`*^9, 3.4283230745948253`*^9}, 
   3.5875253806618814`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "m_", "]"}], "=", 
   RowBox[{"1", "+", 
    RowBox[{
     FractionBox[
      RowBox[{"\[Gamma]", "-", "1"}], 
      RowBox[{"\[Gamma]", "+", "1"}]], 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m", "2"], "-", "1"}], ")"}]}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4283121004998255`*^9, 3.428312111538825*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "m_", "]"}], "=", 
   RowBox[{
    FractionBox["1", "m"], " ", 
    SuperscriptBox[
     RowBox[{"t", "[", "m", "]"}], 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"\[Gamma]", "+", "1"}], ")"}], "/", "2"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.3901209442663546`*^9, 3.390120957135355*^9}, {
  3.428312118391825*^9, 3.4283121246178255`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sub", "[", "a_", "]"}], ":=", 
  RowBox[{"m", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "m", "]"}], "\[Equal]", "a"}], ",", 
     RowBox[{"{", 
      RowBox[{"m", ",", "0.5"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"super", "[", "a_", "]"}], ":=", 
  RowBox[{"m", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "m", "]"}], "\[Equal]", "a"}], ",", 
     RowBox[{"{", 
      RowBox[{"m", ",", "1.5"}], "}"}]}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.587523576153366*^9, 3.5875235825691357`*^9}}],

Cell["Mach numbers", "Text",
 CellChangeTimes->{{3.428379586441216*^9, 3.4283795970902157`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox[
  SubscriptBox["A", "entry"], 
  SubscriptBox["A", "throat"]]], "Input",
 CellChangeTimes->{{3.428380207893216*^9, 3.4283802140572157`*^9}}],

Cell[BoxData["3.9999999999999996`"], "Output",
 CellChangeTimes->{
  3.5755466037892485`*^9, 3.57554670365923*^9, 3.5755467586848307`*^9, 
   3.575546855149405*^9, 3.586582814023503*^9, 3.587523433795285*^9, 
   3.587523637242696*^9, 3.5875246146769757`*^9, {3.5875255314719763`*^9, 
   3.587525546945834*^9}, 3.5875256111965413`*^9, 3.5875258808738995`*^9, 
   3.5890778723573275`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox[
  SubscriptBox["A", "exit"], 
  SubscriptBox["A", "throat"]]], "Input",
 CellChangeTimes->{{3.428380253680216*^9, 3.428380261921216*^9}, {
   3.428592859641*^9, 3.428592866599*^9}, 3.428592956901*^9}],

Cell[BoxData["77.54255820529737`"], "Output",
 CellChangeTimes->{
  3.575546605469451*^9, 3.575546703795247*^9, 3.5755467593199096`*^9, 
   3.5755468552374153`*^9, 3.586582814065548*^9, 3.5875234339223022`*^9, 
   3.5875236373297057`*^9, 3.5875246147779875`*^9, {3.58752553156699*^9, 
   3.587525547056848*^9}, 3.587525611285552*^9, 3.5875258809649096`*^9, 
   3.5890778724283223`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["Ma", "entry"], "=", 
  RowBox[{"sub", "[", 
   FractionBox[
    SubscriptBox["A", "entry"], 
    SubscriptBox["A", "throat"]], "]"}]}]], "Input",
 CellChangeTimes->{{3.428380058718216*^9, 3.428380065106216*^9}, {
  3.4283802756482162`*^9, 3.428380288525216*^9}, {3.587523590393076*^9, 
  3.58752359109616*^9}}],

Cell[BoxData["0.14984332496240302`"], "Output",
 CellChangeTimes->{
  3.575546607466688*^9, 3.575546704067278*^9, 3.5755467599329815`*^9, 
   3.575546855325426*^9, 3.586582814108511*^9, 3.5875234351164455`*^9, 
   3.587523637418717*^9, 3.587524614880999*^9, {3.5875255316700006`*^9, 
   3.5875255471708612`*^9}, 3.5875256113765645`*^9, 3.587525881056923*^9, 
   3.5890778726373897`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["Ma", "exit"], "=", 
  RowBox[{"super", "[", 
   FractionBox[
    SubscriptBox["A", "exit"], 
    SubscriptBox["A", "throat"]], "]"}]}]], "Input",
 CellChangeTimes->{{3.4283802937552156`*^9, 3.428380309022216*^9}, {
  3.5875235950376325`*^9, 3.587523595788722*^9}}],

Cell[BoxData["4.7070233894209155`"], "Output",
 CellChangeTimes->{
  3.575546609683954*^9, 3.575546705275425*^9, 3.5755467600359936`*^9, 
   3.5755468554144382`*^9, 3.5865828141535397`*^9, 3.587523436459605*^9, 
   3.587523637505728*^9, 3.5875246153550572`*^9, {3.5875255317670135`*^9, 
   3.587525547289874*^9}, 3.5875256114685755`*^9, 3.587525881148934*^9, 
   3.5890778727553988`*^9}]
}, Open  ]],

Cell["Exit-to-entry ratios", "Text",
 CellChangeTimes->{{3.428387031055216*^9, 3.4283870348912163`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tratio", "=", 
  FractionBox[
   RowBox[{"1", "+", 
    RowBox[{
     FractionBox["1", "2"], 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      SubscriptBox["Ma", "exit"], "2"]}]}], 
   RowBox[{"1", "+", 
    RowBox[{
     FractionBox["1", "2"], 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      SubscriptBox["Ma", "entry"], "2"]}]}]]}]], "Input",
 CellChangeTimes->{{3.428387048216216*^9, 3.428387101801216*^9}, {
  3.4284691610673866`*^9, 3.4284691680913863`*^9}}],

Cell[BoxData["3.208403084340317`"], "Output",
 CellChangeTimes->{
  3.575546610759083*^9, 3.5755467060095105`*^9, 3.575546760117004*^9, 
   3.5755468554534426`*^9, 3.586582814177579*^9, 3.5875234366366262`*^9, 
   3.587523637544733*^9, 3.5875246163611765`*^9, {3.587525531816017*^9, 
   3.58752554733988*^9}, 3.587525611509581*^9, 3.5875258811899385`*^9, 
   3.5890778736495094`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pratio", "=", 
  SuperscriptBox["Tratio", 
   RowBox[{"\[Gamma]", "/", 
    RowBox[{"(", 
     RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]], "Input",
 CellChangeTimes->{{3.428387126890216*^9, 3.428387148669216*^9}}],

Cell[BoxData["1090.7709197542035`"], "Output",
 CellChangeTimes->{
  3.5755466118372135`*^9, 3.57554670689962*^9, 3.57554676049405*^9, 
   3.5755468554934464`*^9, 3.5865828141985245`*^9, 3.587523436717637*^9, 
   3.5875236375817375`*^9, 3.5875246164181833`*^9, {3.5875255318630247`*^9, 
   3.587525547389886*^9}, 3.5875256115505843`*^9, 3.5875258812309427`*^9, 
   3.589077873684507*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Rho]ratio", "=", 
  SuperscriptBox["Tratio", 
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]], "Input",
 CellChangeTimes->{{3.428387126890216*^9, 3.428387148669216*^9}, {
  3.428387571429216*^9, 3.4283875750612164`*^9}}],

Cell[BoxData["339.97315520549006`"], "Output",
 CellChangeTimes->{
  3.575546612668312*^9, 3.575546708030755*^9, 3.5755467607650805`*^9, 
   3.575546855532451*^9, 3.586582814219558*^9, 3.587523436778646*^9, 
   3.587523637619743*^9, 3.587524616609206*^9, {3.5875255319100294`*^9, 
   3.5875255474378915`*^9}, 3.58752561159159*^9, 3.587525881271947*^9, 
   3.5890778737694917`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cratio", "=", 
  SqrtBox["Tratio"]}]], "Input",
 CellChangeTimes->{{3.4283876254432163`*^9, 3.4283876344182158`*^9}}],

Cell[BoxData["1.791201575574429`"], "Output",
 CellChangeTimes->{
  3.5755466134034004`*^9, 3.575546708608824*^9, 3.575546760824087*^9, 
   3.5755468555724573`*^9, 3.586582814238567*^9, 3.5875234368526535`*^9, 
   3.587523637656746*^9, 3.587524616762225*^9, {3.5875255319490356`*^9, 
   3.5875255474859*^9}, 3.587525611632594*^9, 3.5875258813139515`*^9, 
   3.5890778738054967`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Mass flux", "Subsection",
 CellChangeTimes->{{3.4285593589709864`*^9, 3.4285593672939863`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["T", "entry"], "=", 
   RowBox[{"3600", "Kelvin"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mgas", "=", 
   RowBox[{"14.3", " ", 
    RowBox[{"Gram", "/", "Mole"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.428559487315986*^9, 3.428559497535986*^9}, {
   3.4285596319689865`*^9, 3.428559635087986*^9}, {3.428559672198986*^9, 
   3.428559679967986*^9}, {3.428561409233986*^9, 3.428561442431986*^9}, {
   3.4285614783369865`*^9, 3.428561478447986*^9}, {3.428566343200986*^9, 
   3.4285663539689865`*^9}, {3.428566437841986*^9, 3.428566462112986*^9}, {
   3.428566554528986*^9, 3.4285666103999863`*^9}, {3.428567385249986*^9, 
   3.4285673899849863`*^9}, {3.428567677924986*^9, 3.4285676996959863`*^9}, {
   3.428569768704986*^9, 3.428569780335986*^9}, {3.428636433668613*^9, 
   3.428636434674613*^9}, {3.428638496056613*^9, 3.428638505502613*^9}, {
   3.428638604977613*^9, 3.428638605485613*^9}, {3.428638652143613*^9, 
   3.428638674121613*^9}, {3.4289074927402077`*^9, 3.428907493437208*^9}, {
   3.428908386327208*^9, 3.4289083867002077`*^9}, 3.428911305247208*^9, {
   3.48739683895516*^9, 3.487396856677174*^9}}],

Cell["Specific gas constant", "Text",
 CellChangeTimes->{{3.587525785174418*^9, 3.5875257887098417`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"R", "=", 
   RowBox[{
    SubscriptBox["R", "mol"], "/", "Mgas"}]}], ";"}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQ3S/zyr+0+6XjGTXDQBD9gfF/Iog+cSEpCURLxLGl
g+h9P/+B6Qs16g5lQPrKJV8XED3jxCkPEP089ByYrsp7tqICbI79KhDd+3HX
FhA9de7h7SC65G3AYRC94YzPKRD9+L+6diWQZmrdD6ZrZu71B9GtrmVBINq+
Xyy0CkinHegKA9HuqaaPZgHpvGmhYPrYecYns4F0hqjBUxA922P+RxCt0rUe
TFfNaPkOoh2fOP8C0eXfA3bK9Lx0ZLCsBdMuR1IvguhdTQvBNJdAZZkskJ51
qQlMN9/5nbf290vH6vyjxSC6qvFjDYh+dli3FkSLL+pvXLnrleOreWdbQDQA
caG51g==
  "]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["c", "entry"], "=", 
  RowBox[{
   SqrtBox[
    RowBox[{"\[Gamma]", " ", "R", " ", 
     SubscriptBox["T", "entry"]}]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.428559413894986*^9, 3.4285594440679865`*^9}, {
  3.428559508181986*^9, 3.4285595104829865`*^9}, {3.4285596377799864`*^9, 
  3.4285596445779862`*^9}, {3.5875254324941006`*^9, 3.587525433621235*^9}, {
  3.587525830126814*^9, 3.5875258395469413`*^9}}],

Cell[BoxData[
 TemplateBox[{"1584.860106596957`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466179159417`*^9, 3.575546710812087*^9, 3.5755467612531395`*^9, {
   3.5755468367511997`*^9, 3.5755468559305005`*^9}, 3.586582814422594*^9, 
   3.587523437563738*^9, 3.5875236380667963`*^9, 3.587524617552322*^9, {
   3.587525532326079*^9, 3.5875255478549433`*^9}, 3.5875256119896393`*^9, 
   3.587525881705999*^9, 3.5890778745225873`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["U", "entry"], "=", 
  RowBox[{
   SubscriptBox["Ma", "entry"], 
   SubscriptBox["c", "entry"]}]}]], "Input",
 CellChangeTimes->{{3.428559843763986*^9, 3.428559853759986*^9}}],

Cell[BoxData[
 TemplateBox[{"237.4807079727565`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466188640547`*^9, 3.575546711171131*^9, 3.5755467613111477`*^9, {
   3.5755468367982025`*^9, 3.575546855970503*^9}, 3.586582814444553*^9, 
   3.58752343950497*^9, 3.5875236381057997`*^9, 3.5875246176483316`*^9, {
   3.587525532420091*^9, 3.587525547967955*^9}, 3.5875256120306416`*^9, 
   3.587525881807012*^9, 3.5890778746626673`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Rho]", "entry"], "=", 
  FractionBox["Q", 
   RowBox[{
    SubscriptBox["A", "entry"], 
    SubscriptBox["U", "entry"]}]]}]], "Input",
 CellChangeTimes->{{3.4289995844850764`*^9, 3.428999603028076*^9}}],

Cell[BoxData[
 TemplateBox[{"9.679892185071528`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466205862627`*^9, 3.575546711731199*^9, 3.575546761675192*^9, {
   3.5755468368442097`*^9, 3.575546856315545*^9}, 3.586582814541609*^9, 
   3.5875234397389975`*^9, 3.587523638297823*^9, 3.587524617864356*^9, {
   3.5875255328671446`*^9, 3.5875255481799803`*^9}, 3.5875256122316675`*^9, 
   3.587525882004035*^9, 3.5890778748216877`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["p", "entry"], "=", 
  RowBox[{
   RowBox[{"R", " ", 
    SubscriptBox["T", "entry"], " ", 
    SubscriptBox["\[Rho]", "entry"]}], "//", 
   RowBox[{"ToUnit", "[", "Bar", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.4290004420330763`*^9, 3.429000511759076*^9}, {
  3.587525446349766*^9, 3.5875254474528947`*^9}, {3.587525843914466*^9, 
  3.587525850331236*^9}}],

Cell[BoxData[
 TemplateBox[{"202.61478890734844`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"bars\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: bars"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Bars\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466225324984`*^9, 3.575546712826331*^9, 3.5755467636564283`*^9, {
   3.575546837019231*^9, 3.5755468566515856`*^9}, 3.5865828147136273`*^9, 
   3.587523440054036*^9, 3.5875236386248612`*^9, 3.587524618207399*^9, {
   3.5875255333542013`*^9, 3.587525548524022*^9}, 3.5875256126197147`*^9, 
   3.5875258822810698`*^9, 3.589077875010693*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"PSI", "=", 
   RowBox[{"Quantity", "[", "\"\<PSI\>\"", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.575546824488727*^9, 3.575546830654467*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["p", "entry"], "//", 
  RowBox[{"ToUnit", "[", "PSI", "]"}]}]], "Input",
 CellChangeTimes->{{3.428923554206208*^9, 3.428923560691208*^9}}],

Cell[BoxData[
 TemplateBox[{"2938.6790613805706`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"lbf\"", "\[InvisibleSpace]", "\"/\"", 
            "\[InvisibleSpace]", 
            SuperscriptBox["\"in\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: pounds\[Hyphen]force per inch squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"PoundsForce\"", 
        SuperscriptBox["\"Inches\"", "2"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466239476643`*^9, 3.575546713017351*^9, 3.575546763988469*^9, {
   3.5755468370832396`*^9, 3.575546856891618*^9}, 3.5865828193821898`*^9, 
   3.5875234464258003`*^9, 3.5875236386718674`*^9, 3.5875246182584057`*^9, {
   3.587525533464216*^9, 3.5875255485820303`*^9}, 3.587525612670719*^9, 
   3.5875258823310757`*^9, 3.5890778798212957`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Performance", "Subsection",
 CellChangeTimes->{{3.587525464598954*^9, 3.587525466390168*^9}}],

Cell["Thrust", "Text",
 CellChangeTimes->{{3.428564480531986*^9, 3.428564481933986*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]], "Input",
 CellChangeTimes->{{3.428920584040208*^9, 3.4289205911272078`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["T", "exit"], "=", 
  RowBox[{
   SubscriptBox["T", "entry"], "/", "Tratio"}]}]], "Input",
 CellChangeTimes->{{3.4285644869519863`*^9, 3.4285645090889864`*^9}}],

Cell[BoxData[
 TemplateBox[{"1122.0535279906078`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kelvins"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466256368666`*^9, 3.575546713085362*^9, 3.5755467653356304`*^9, 
   3.5755468577727194`*^9, 3.5865828199082966`*^9, 3.5875234464718056`*^9, 
   3.587523638718872*^9, 3.5875246183184123`*^9, {3.587525533513221*^9, 
   3.587525548639035*^9}, 3.587525612720724*^9, 3.58752588238108*^9, 
   3.58907787984929*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["p", "exit"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["p", "entry"], "/", "pratio"}], "//", 
   RowBox[{"ToUnit", "[", "Bar", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.428564513079986*^9, 3.428564524112986*^9}, {
  3.428568457473986*^9, 3.4285684620969863`*^9}, {3.4285933074700003`*^9, 
  3.4285933079960003`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.18575375015773804`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"bars\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: bars"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Bars\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546625678874*^9, 3.575546713144367*^9, 3.5755467653746333`*^9, 
   3.5755468578757324`*^9, 3.5865828204343796`*^9, 3.58752344650781*^9, 
   3.5875236387578773`*^9, 3.587524618369416*^9, {3.587525533554226*^9, 
   3.5875255486870413`*^9}, 3.58752561276173*^9, 3.5875258824240866`*^9, 
   3.5890778798722954`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["p", "exit"], "//", 
  RowBox[{"ToUnit", "[", "PSI", "]"}]}]], "Input",
 CellChangeTimes->{{3.428911352440208*^9, 3.428911360552208*^9}}],

Cell[BoxData[
 TemplateBox[{"2.694130369778082`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"lbf\"", "\[InvisibleSpace]", "\"/\"", 
            "\[InvisibleSpace]", 
            SuperscriptBox["\"in\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: pounds\[Hyphen]force per inch squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"PoundsForce\"", 
        SuperscriptBox["\"Inches\"", "2"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546625716878*^9, 3.5755467132043743`*^9, 3.57554676541464*^9, 
   3.5755468582277756`*^9, 3.586582820518382*^9, 3.5875234465438147`*^9, 
   3.5875236388538914`*^9, 3.587524618409422*^9, {3.587525533594232*^9, 
   3.587525548736047*^9}, 3.5875256128047347`*^9, 3.58752588246509*^9, 
   3.5890778798973074`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Rho]", "exit"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Rho]", "entry"], "/", "\[Rho]ratio"}], "//", 
   "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4293247539726105`*^9, 3.429324770764611*^9}}],

Cell[BoxData[
 TemplateBox[{"0.028472519188230346`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466257548823`*^9, 3.5755467132623806`*^9, 3.5755467654536457`*^9, 
   3.5755468591728888`*^9, 3.5865828205593596`*^9, 3.5875234465798197`*^9, 
   3.587523638892894*^9, 3.587524618459427*^9, {3.5875255336362357`*^9, 
   3.5875255487830534`*^9}, 3.5875256128457394`*^9, 3.5875258825080953`*^9, 
   3.5890778799213243`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["c", "exit"], "=", 
  RowBox[{
   SqrtBox[
    RowBox[{"\[Gamma]", " ", "R", " ", 
     SubscriptBox["T", "exit"]}]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4285645313209863`*^9, 3.428564549778986*^9}, {
  3.5875255398719854`*^9, 3.587525540871105*^9}, {3.5875258618866215`*^9, 
  3.5875258708917017`*^9}}],

Cell[BoxData[
 TemplateBox[{"884.8027649198`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466260409155`*^9, 3.5755467135754194`*^9, 3.5755467657536807`*^9, 
   3.5755468597109547`*^9, 3.586582820814473*^9, 3.587523446867853*^9, 
   3.587523639181928*^9, 3.587524618754464*^9, {3.58752553375725*^9, 
   3.587525549088091*^9}, 3.5875256131457777`*^9, 3.5875258827131195`*^9, 
   3.589077880036283*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["U", "exit"], "=", 
  RowBox[{
   SubscriptBox["Ma", "exit"], 
   SubscriptBox["c", "exit"]}]}]], "Input",
 CellChangeTimes->{{3.428564571130986*^9, 3.4285645785439863`*^9}}],

Cell[BoxData[
 TemplateBox[{"4164.787309501794`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466260799203`*^9, 3.575546713622425*^9, 3.5755467657906866`*^9, 
   3.5755468598759727`*^9, 3.5865828210864286`*^9, 3.5875234469048595`*^9, 
   3.587523639220933*^9, 3.587524618849474*^9, {3.587525533798255*^9, 
   3.587525549198103*^9}, 3.587525613189781*^9, 3.5875258827581267`*^9, 
   3.5890778800572867`*^9}]
}, Open  ]],

Cell["Thrust at ground level", "Text",
 CellChangeTimes->{{3.4290029746750765`*^9, 3.4290029873990765`*^9}, {
  3.587525486886628*^9, 3.58752549474257*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["p", "back"], "=", 
   RowBox[{"1", "Bar"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4285662598199863`*^9, 3.428566269071986*^9}, {
  3.428566482992986*^9, 3.428566483151986*^9}, {3.4285666325779862`*^9, 
  3.428566638959986*^9}, {3.4285678182589865`*^9, 3.428567824943986*^9}, {
  3.428568442689986*^9, 3.428568451855986*^9}, {3.4289116074612083`*^9, 
  3.4289116077842083`*^9}, {3.4289137049942083`*^9, 3.4289137155562077`*^9}, {
  3.428923140514208*^9, 3.428923152400208*^9}, {3.428923374415208*^9, 
  3.428923374504208*^9}, {3.428923419906208*^9, 3.428923420260208*^9}, {
  3.428923576234208*^9, 3.428923594322208*^9}, {3.4289912868830767`*^9, 
  3.4289913328570766`*^9}, {3.4290027455980763`*^9, 3.4290027480640764`*^9}, {
  3.4290030846650763`*^9, 3.4290030878790765`*^9}, {3.4290031212320766`*^9, 
  3.4290031221170764`*^9}, {3.429164173084*^9, 3.429164173227*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"thrust", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Q", " ", 
     SubscriptBox["U", "exit"]}], "+", 
    RowBox[{
     SubscriptBox["A", "exit"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["p", "exit"], "-", 
       SubscriptBox["p", "back"]}], ")"}]}]}], "//", 
   RowBox[{"ToUnit", "[", "Kilonewton", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.428564584778986*^9, 3.428564619007986*^9}, {
  3.4285662255209866`*^9, 3.428566252992986*^9}, {3.487397083519149*^9, 
  3.4873970843821983`*^9}, {3.5875255835652275`*^9, 3.5875255842033043`*^9}}],

Cell[BoxData[
 TemplateBox[{"1719.2420053199783`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kN\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilonewtons"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilonewtons\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546626345952*^9, 3.575546713917459*^9, 3.5755467660647182`*^9, 
   3.5755468609321012`*^9, 3.586582821262446*^9, 3.5875234471728897`*^9, 
   3.587523639490965*^9, 3.587524619124508*^9, {3.587525533923272*^9, 
   3.5875255494861383`*^9}, {3.587525585058405*^9, 3.5875256134678164`*^9}, 
   3.5875258830891666`*^9, 3.589077880203326*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["thrust", 
   SubscriptBox["g", "0"]], "//", 
  RowBox[{"ToUnit", "[", "Pound", "]"}]}]], "Input",
 CellChangeTimes->{{3.428923159506208*^9, 3.428923170253208*^9}}],

Cell[BoxData[
 TemplateBox[{"386500.97814861115`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"lb\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pounds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Pounds\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755466265139723`*^9, 3.5755467141874914`*^9, 3.575546766202734*^9, 
   3.5755468616911917`*^9, 3.5865828218555775`*^9, 3.587523447314908*^9, 
   3.587523639624981*^9, 3.587524619447546*^9, {3.5875255340192833`*^9, 
   3.5875255496301565`*^9}, 3.5875256136658373`*^9, 3.5875258832251825`*^9, 
   3.5890778802843304`*^9}]
}, Open  ]],

Cell["Thrust outside atmosphere", "Text",
 CellChangeTimes->{{3.4290029746750765`*^9, 3.4290029873990765`*^9}, {
  3.5875255049978004`*^9, 3.587525509637356*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["p", "back"], "=", 
   RowBox[{"0", "Bar"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4285662598199863`*^9, 3.428566269071986*^9}, {
  3.428566482992986*^9, 3.428566483151986*^9}, {3.4285666325779862`*^9, 
  3.428566638959986*^9}, {3.4285678182589865`*^9, 3.428567824943986*^9}, {
  3.428568442689986*^9, 3.428568451855986*^9}, {3.4289116074612083`*^9, 
  3.4289116077842083`*^9}, {3.4289137049942083`*^9, 3.4289137155562077`*^9}, {
  3.428923140514208*^9, 3.428923152400208*^9}, {3.428923374415208*^9, 
  3.428923374504208*^9}, {3.428923419906208*^9, 3.428923420260208*^9}, {
  3.428923576234208*^9, 3.428923594322208*^9}, {3.4289912868830767`*^9, 
  3.4289913328570766`*^9}, {3.4290027455980763`*^9, 3.4290027480640764`*^9}, {
  3.4290030846650763`*^9, 3.4290030878790765`*^9}, {3.4290031212320766`*^9, 
  3.4290031221170764`*^9}, {3.429164173084*^9, 3.429164173227*^9}, {
  3.487396932115489*^9, 3.4873969332225523`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"thrust", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Q", " ", 
     SubscriptBox["U", "exit"]}], "+", 
    RowBox[{
     SubscriptBox["A", "exit"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["p", "exit"], "-", 
       SubscriptBox["p", "back"]}], ")"}]}]}], "//", 
   RowBox[{"ToUnit", "[", "Kilonewton", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.428564584778986*^9, 3.428564619007986*^9}, {
  3.4285662255209866`*^9, 3.428566252992986*^9}, {3.487396955681837*^9, 
  3.487396956474882*^9}, {3.587525563292795*^9, 3.5875255705566673`*^9}}],

Cell[BoxData[
 TemplateBox[{"2136.084406292056`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kN\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilonewtons"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilonewtons\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546626917021*^9, 3.575546714468527*^9, 3.575546766605783*^9, 
   3.575546862579298*^9, 3.5865828220335073`*^9, 3.5875234476359453`*^9, 
   3.587523639908015*^9, 3.587524619722581*^9, {3.5875255341482987`*^9, 
   3.5875255711547384`*^9}, 3.5875256139418726`*^9, 3.587525883512216*^9, 
   3.5890778804363737`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["thrust", 
   SubscriptBox["g", "0"]], "//", 
  RowBox[{"ToUnit", "[", "Pound", "]"}]}]], "Input",
 CellChangeTimes->{{3.428923159506208*^9, 3.428923170253208*^9}}],

Cell[BoxData[
 TemplateBox[{"480210.8777502897`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"lb\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pounds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Pounds\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575546627047036*^9, 3.575546714605543*^9, 3.5755467667467985`*^9, 
   3.575546863068359*^9, 3.5865828221145544`*^9, 3.587523447769962*^9, 
   3.587523640107039*^9, 3.5875246198575945`*^9, {3.587525534219306*^9, 
   3.587525550058205*^9}, 3.5875256140798893`*^9, 3.5875258836572328`*^9, 
   3.5890778805133433`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Acceleration at lift-off (p. 239)\
\>", "Section",
 CellChangeTimes->{{3.5020918483442917`*^9, 3.5020918530305595`*^9}, {
  3.5875259189204655`*^9, 3.587525922006835*^9}}],

Cell[CellGroupData[{

Cell["V2", "Subsubsection",
 CellChangeTimes->{{3.502091987439247*^9, 3.5020919881032853`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5020919245676513`*^9, 3.5020919276338267`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(33\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755466273140707`*^9, 3.5755467148835754`*^9, 
  3.575546767020831*^9, 3.575546863681432*^9, 3.5865828222956033`*^9, 
  3.5875234480479946`*^9, 3.587523640400075*^9, 3.5875246201996365`*^9, 
  3.587525894890582*^9, 3.58907788070041*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"mass", "=", 
   RowBox[{"12500", " ", "Kilogram"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"thrust", " ", "=", " ", 
   RowBox[{"247", " ", "Kilo", " ", "Newton"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.429329163329611*^9, 3.429329179134611*^9}, {
  3.4293292564516106`*^9, 3.429329277190611*^9}, {3.4293294032056108`*^9, 
  3.4293294145956106`*^9}, {3.5020918790980506`*^9, 3.502091905985588*^9}, {
  3.502091976243607*^9, 3.502091977449676*^9}, {3.502092229595098*^9, 
  3.5020922414347754`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["thrust", 
    RowBox[{"mass", " ", 
     SubscriptBox["g", "0"]}]], "//", "ToSI"}], "//", "N"}]], "Input",
 CellChangeTimes->{{3.429329208242611*^9, 3.4293292232516108`*^9}, {
  3.429329298961611*^9, 3.429329299966611*^9}, {3.429329488889611*^9, 
  3.4293294895316105`*^9}, {3.5020923065865016`*^9, 3.502092311578787*^9}}],

Cell[BoxData["2.014959236844386`"], "Output",
 CellChangeTimes->{3.5755466275450964`*^9, 3.5755467151156025`*^9, 
  3.575546767270862*^9, 3.5755468640534735`*^9, 3.5865828224205933`*^9, 
  3.587523448276023*^9, 3.5875236406311016`*^9, 3.5875246204336653`*^9, 
  3.5875258951816196`*^9, 3.589077880875389*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SSME", "Subsubsection",
 CellChangeTimes->{{3.429329106941611*^9, 3.4293291105016108`*^9}, {
  3.5020919156641417`*^9, 3.5020919163661823`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5020919245676513`*^9, 3.5020919276338267`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(3\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755466279621468`*^9, 3.57554671550365*^9, 
  3.57554676768091*^9, 3.5755468643335085`*^9, 3.586582822588619*^9, 
  3.5875234485540557`*^9, 3.587523640910136*^9, 3.5875246207187014`*^9, 
  3.5875258954466505`*^9, 3.589077881062454*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"mass", "=", 
   RowBox[{"2029203", " ", "Kilogram"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"thrust", " ", "=", " ", 
   RowBox[{"1719", " ", "Kilo", " ", "Newton"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.429329163329611*^9, 3.429329179134611*^9}, {
  3.4293292564516106`*^9, 3.429329277190611*^9}, {3.4293294032056108`*^9, 
  3.4293294145956106`*^9}, {3.5020918790980506`*^9, 3.502091905985588*^9}, {
  3.502091976243607*^9, 3.502091977449676*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"totalthrust", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"3", "*", "thrust"}], "+", 
    RowBox[{"2", "*", "12.5", " ", "Meganewton"}]}], "//", 
   RowBox[{"ToUnit", "[", "Meganewton", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.429329429165611*^9, 3.429329481851611*^9}, {
  3.487397121573325*^9, 3.4873971321209283`*^9}, {3.5755468965433726`*^9, 
  3.5755469003658314`*^9}}],

Cell[BoxData[
 TemplateBox[{"30.157`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"MN\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meganewtons"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meganewtons\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755466281601706`*^9, 3.5755467155566564`*^9, 
  3.5755467677459183`*^9, 3.575546864399515*^9, 3.575546900908896*^9, 
  3.58658282265564*^9, 3.5875234487190757`*^9, 3.587523641196173*^9, 
  3.5875246208307114`*^9, 3.587525895561664*^9, 3.589077881133437*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["totalthrust", 
   RowBox[{"mass", " ", 
    SubscriptBox["g", "0"]}]], "//", "ToSI"}]], "Input",
 CellChangeTimes->{{3.429329208242611*^9, 3.4293292232516108`*^9}, {
  3.429329298961611*^9, 3.429329299966611*^9}, {3.429329488889611*^9, 
  3.4293294895316105`*^9}}],

Cell[BoxData["1.5154512305952328`"], "Output",
 CellChangeTimes->{3.575546628382198*^9, 3.5755467157786827`*^9, 
  3.575546767976948*^9, 3.5755468646325436`*^9, 3.5865828227726145`*^9, 
  3.587523448939101*^9, 3.587523641427198*^9, 3.5875246210557394`*^9, 
  3.587525895790691*^9, 3.589077881254469*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{1077, 862},
WindowMargins->{{46, Automatic}, {Automatic, 54}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 232, 5, 65, "Title"],
Cell[802, 29, 332, 8, 62, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171, 42, 193, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[1389, 50, 178, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[1592, 58, 45, 0, 36, "Subsection"],
Cell[CellGroupData[{
Cell[1662, 62, 59, 1, 41, "Input"],
Cell[1724, 65, 311, 8, 21, "Message"]
}, Open  ]],
Cell[2050, 76, 92, 1, 30, "Text"],
Cell[2145, 79, 398, 12, 64, "Input"],
Cell[CellGroupData[{
Cell[2568, 95, 196, 7, 66, "Input"],
Cell[2767, 104, 670, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3474, 125, 118, 3, 41, "Input"],
Cell[3595, 130, 680, 16, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[4336, 153, 201, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[4562, 161, 127, 2, 41, "Input"],
Cell[4692, 165, 331, 8, 21, "Message"]
}, Open  ]],
Cell[5038, 176, 94, 1, 30, "Text"],
Cell[5135, 179, 814, 23, 138, "Input"],
Cell[CellGroupData[{
Cell[5974, 206, 216, 6, 58, "Input"],
Cell[6193, 214, 945, 22, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7175, 241, 301, 9, 45, "Input"],
Cell[7479, 252, 780, 18, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8296, 275, 182, 5, 41, "Input"],
Cell[8481, 282, 758, 18, 41, "Output"]
}, Open  ]],
Cell[9254, 303, 186, 4, 30, "Text"],
Cell[9443, 309, 607, 16, 62, "Input"],
Cell[CellGroupData[{
Cell[10075, 329, 274, 9, 66, "Input"],
Cell[10352, 340, 753, 18, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11142, 363, 216, 5, 41, "Input"],
Cell[11361, 370, 756, 18, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[12166, 394, 159, 2, 75, "Section"],
Cell[CellGroupData[{
Cell[12350, 400, 127, 2, 41, "Input"],
Cell[12480, 404, 478, 10, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12995, 419, 98, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[13118, 424, 2369, 55, 144, "Input"],
Cell[15490, 481, 721, 16, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16260, 503, 96, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[16381, 508, 148, 3, 41, "Input"],
Cell[16532, 513, 950, 20, 41, "Output"]
}, Open  ]],
Cell[17497, 536, 233, 7, 58, "Input"],
Cell[CellGroupData[{
Cell[17755, 547, 276, 8, 45, "Input"],
Cell[18034, 557, 879, 18, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18950, 580, 296, 9, 66, "Input"],
Cell[19249, 591, 730, 14, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20016, 610, 222, 4, 41, "Input"],
Cell[20241, 616, 930, 19, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21208, 640, 556, 10, 49, "Input"],
Cell[21767, 652, 725, 14, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22529, 671, 354, 11, 57, "Input"],
Cell[22886, 684, 729, 14, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23652, 703, 145, 3, 41, "Input"],
Cell[23800, 708, 685, 14, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24522, 727, 161, 4, 58, "Input"],
Cell[24686, 733, 322, 4, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[25081, 745, 192, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[25298, 753, 185, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[25508, 761, 59, 1, 41, "Input"],
Cell[25570, 764, 313, 8, 21, "Message"]
}, Open  ]],
Cell[25898, 775, 204, 4, 30, "Text"],
Cell[26105, 781, 516, 17, 62, "Input"],
Cell[26624, 800, 98, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[26747, 805, 99, 3, 54, "Input"],
Cell[26849, 810, 262, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[27148, 821, 276, 7, 41, "Input"],
Cell[27427, 830, 4438, 113, 82, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[31914, 949, 197, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[32136, 957, 59, 1, 41, "Input"],
Cell[32198, 960, 364, 9, 21, "Message"]
}, Open  ]],
Cell[32577, 972, 26, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[32628, 976, 400, 10, 92, "Input"],
Cell[33031, 988, 811, 19, 41, "Output"]
}, Open  ]],
Cell[33857, 1010, 125, 3, 30, "Text"],
Cell[33985, 1015, 295, 8, 56, "Input"],
Cell[34283, 1025, 37, 0, 30, "Text"],
Cell[34323, 1027, 344, 9, 63, "Input"],
Cell[34670, 1038, 147, 3, 30, "Text"],
Cell[34820, 1043, 434, 13, 41, "Input"],
Cell[35257, 1058, 38, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[35320, 1062, 340, 9, 41, "Input"],
Cell[35663, 1073, 1861, 45, 41, "Output"]
}, Open  ]],
Cell[37539, 1121, 95, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[37659, 1126, 230, 6, 41, "Input"],
Cell[37892, 1134, 4801, 122, 96, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[42742, 1262, 188, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[42955, 1270, 59, 1, 41, "Input"],
Cell[43017, 1273, 332, 8, 21, "Message"]
}, Open  ]],
Cell[43364, 1284, 98, 3, 41, "Input"],
Cell[43465, 1289, 438, 13, 66, "Input"],
Cell[43906, 1304, 259, 8, 41, "Input"],
Cell[44168, 1314, 238, 7, 41, "Input"],
Cell[44409, 1323, 265, 7, 45, "Input"],
Cell[44677, 1332, 476, 15, 56, "Input"],
Cell[45156, 1349, 171, 4, 56, "Input"],
Cell[45330, 1355, 773, 22, 59, "Input"],
Cell[46106, 1379, 472, 15, 59, "Input"],
Cell[CellGroupData[{
Cell[46603, 1398, 676, 14, 41, "Input"],
Cell[47282, 1414, 20187, 340, 268, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67506, 1759, 4459, 94, 142, "Input"],
Cell[71968, 1855, 18965, 340, 287, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90970, 2200, 748, 19, 62, "Input"],
Cell[91721, 2221, 303, 6, 21, "Message"],
Cell[92027, 2229, 19055, 342, 287, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[111143, 2578, 297, 5, 65, "Title"],
Cell[CellGroupData[{
Cell[111465, 2587, 195, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[111685, 2595, 59, 1, 41, "Input"],
Cell[111747, 2598, 529, 12, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112313, 2615, 104, 1, 36, "Subsection"],
Cell[112420, 2618, 348, 9, 82, "Input"],
Cell[112771, 2629, 129, 3, 30, "Text"],
Cell[112903, 2634, 245, 7, 41, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[113185, 2646, 127, 2, 36, "Subsection"],
Cell[113315, 2650, 472, 14, 30, "Text"],
Cell[113790, 2666, 383, 12, 58, "Input"],
Cell[114176, 2680, 315, 10, 30, "Text"],
Cell[114494, 2692, 499, 15, 56, "Input"],
Cell[114996, 2709, 371, 11, 30, "Text"],
Cell[115370, 2722, 283, 9, 41, "Input"],
Cell[115656, 2733, 307, 7, 30, "Text"],
Cell[115966, 2742, 698, 20, 62, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[116701, 2767, 267, 3, 36, "Subsection"],
Cell[116971, 2772, 1484, 40, 114, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118492, 2817, 103, 1, 36, "Subsection"],
Cell[118598, 2820, 99, 1, 30, "Text"],
Cell[118700, 2823, 569, 14, 41, "Input"],
Cell[CellGroupData[{
Cell[119294, 2841, 2619, 62, 102, "Input"],
Cell[121916, 2905, 511, 10, 21, "Message"],
Cell[122430, 2917, 16350, 284, 263, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[138829, 3207, 111, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[138965, 3212, 3269, 72, 102, "Input"],
Cell[142237, 3286, 499, 9, 21, "Message"],
Cell[142739, 3297, 16508, 287, 265, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[159296, 3590, 288, 5, 36, "Subsection"],
Cell[CellGroupData[{
Cell[159609, 3599, 124, 2, 41, "Input"],
Cell[159736, 3603, 381, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[160154, 3614, 193, 4, 41, "Input"],
Cell[160350, 3620, 384, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[160771, 3631, 305, 8, 58, "Input"],
Cell[161079, 3641, 386, 6, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[161526, 3654, 352, 6, 75, "Section"],
Cell[CellGroupData[{
Cell[161903, 3664, 125, 2, 41, "Input"],
Cell[162031, 3668, 540, 12, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[162608, 3685, 102, 1, 36, "Subsection"],
Cell[162713, 3688, 150, 2, 30, "Text"],
Cell[162866, 3692, 567, 15, 82, "Input"],
Cell[163436, 3709, 281, 5, 30, "Text"],
Cell[163720, 3716, 847, 19, 82, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[164604, 3740, 154, 2, 36, "Subsection"],
Cell[164761, 3744, 96, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[164882, 3749, 371, 10, 54, "Input"],
Cell[165256, 3761, 852, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[166145, 3786, 324, 9, 54, "Input"],
Cell[166472, 3797, 849, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[167358, 3822, 369, 10, 54, "Input"],
Cell[167730, 3834, 855, 20, 41, "Output"]
}, Open  ]],
Cell[168600, 3857, 229, 5, 30, "Text"],
Cell[168832, 3864, 383, 12, 58, "Input"],
Cell[169218, 3878, 499, 15, 56, "Input"],
Cell[169720, 3895, 647, 19, 62, "Input"],
Cell[170370, 3916, 96, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[170491, 3921, 168, 4, 59, "Input"],
Cell[170662, 3927, 387, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[171086, 3938, 353, 9, 59, "Input"],
Cell[171442, 3949, 386, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[171865, 3960, 228, 5, 58, "Input"],
Cell[172096, 3967, 382, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[172515, 3978, 304, 8, 58, "Input"],
Cell[172822, 3988, 380, 6, 41, "Output"]
}, Open  ]],
Cell[173217, 3997, 219, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[173461, 4005, 585, 18, 73, "Input"],
Cell[174049, 4025, 385, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[174471, 4036, 238, 6, 41, "Input"],
Cell[174712, 4044, 384, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[175133, 4055, 287, 7, 41, "Input"],
Cell[175423, 4064, 386, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[175846, 4075, 141, 3, 45, "Input"],
Cell[175990, 4080, 383, 6, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[176422, 4092, 101, 1, 36, "Subsection"],
Cell[176526, 4095, 94, 1, 30, "Text"],
Cell[176623, 4098, 911, 25, 82, "Input"],
Cell[177537, 4125, 105, 1, 30, "Text"],
Cell[177645, 4128, 538, 13, 41, "Input"],
Cell[CellGroupData[{
Cell[178208, 4145, 455, 10, 50, "Input"],
Cell[178666, 4157, 893, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[179596, 4182, 214, 6, 41, "Input"],
Cell[179813, 4190, 903, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[180753, 4215, 244, 7, 60, "Input"],
Cell[181000, 4224, 956, 21, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[181993, 4250, 400, 10, 41, "Input"],
Cell[182396, 4262, 747, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[183180, 4283, 310, 7, 79, "Input"],
Cell[183493, 4292, 731, 16, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[184273, 4314, 101, 1, 36, "Subsection"],
Cell[184377, 4317, 146, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[184548, 4323, 199, 5, 41, "Input"],
Cell[184750, 4330, 774, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[185561, 4351, 361, 9, 41, "Input"],
Cell[185925, 4362, 771, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[186733, 4383, 245, 7, 41, "Input"],
Cell[186981, 4392, 980, 21, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[187998, 4418, 352, 9, 45, "Input"],
Cell[188353, 4429, 925, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[189315, 4454, 213, 6, 41, "Input"],
Cell[189531, 4462, 923, 20, 41, "Output"]
}, Open  ]],
Cell[190469, 4485, 159, 2, 30, "Text"],
Cell[190631, 4489, 587, 11, 41, "Input"],
Cell[CellGroupData[{
Cell[191243, 4504, 577, 15, 41, "Input"],
Cell[191823, 4521, 787, 16, 41, "Output"]
}, Open  ]],
Cell[192625, 4540, 105, 1, 30, "Text"],
Cell[192733, 4543, 222, 5, 41, "Input"],
Cell[CellGroupData[{
Cell[192980, 4552, 247, 6, 58, "Input"],
Cell[193230, 4560, 417, 6, 41, "Output"]
}, Open  ]],
Cell[193662, 4569, 160, 2, 30, "Text"],
Cell[193825, 4573, 637, 11, 41, "Input"],
Cell[CellGroupData[{
Cell[194487, 4588, 577, 15, 41, "Input"],
Cell[195067, 4605, 789, 16, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[195917, 4628, 263, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[196205, 4637, 125, 2, 41, "Input"],
Cell[196333, 4641, 335, 8, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196705, 4654, 108, 1, 36, "Subsection"],
Cell[196816, 4657, 290, 7, 62, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[197143, 4669, 153, 2, 36, "Subsection"],
Cell[197299, 4673, 938, 21, 62, "Input"],
Cell[CellGroupData[{
Cell[198262, 4698, 214, 4, 41, "Input"],
Cell[198479, 4704, 895, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[199411, 4729, 184, 4, 41, "Input"],
Cell[199598, 4735, 899, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[200534, 4760, 172, 3, 58, "Input"],
Cell[200709, 4765, 390, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[201136, 4776, 114, 2, 58, "Input"],
Cell[201253, 4780, 386, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[201676, 4791, 109, 2, 58, "Input"],
Cell[201788, 4795, 381, 6, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[202218, 4807, 103, 1, 36, "Subsection"],
Cell[202324, 4810, 244, 3, 30, "Text"],
Cell[202571, 4815, 590, 16, 82, "Input"],
Cell[CellGroupData[{
Cell[203186, 4835, 271, 8, 54, "Input"],
Cell[203460, 4845, 854, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[204351, 4870, 273, 8, 54, "Input"],
Cell[204627, 4880, 856, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[205520, 4905, 267, 8, 54, "Input"],
Cell[205790, 4915, 842, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206669, 4940, 138, 3, 41, "Input"],
Cell[206810, 4945, 735, 16, 41, "Output"]
}, Open  ]],
Cell[207560, 4964, 123, 2, 30, "Text"],
Cell[207686, 4968, 383, 12, 58, "Input"],
Cell[208072, 4982, 499, 15, 56, "Input"],
Cell[208574, 4999, 645, 19, 62, "Input"],
Cell[209222, 5020, 96, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[209343, 5025, 168, 4, 59, "Input"],
Cell[209514, 5031, 386, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[209937, 5042, 228, 5, 58, "Input"],
Cell[210168, 5049, 385, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[210590, 5060, 350, 9, 59, "Input"],
Cell[210943, 5071, 386, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[211366, 5082, 304, 8, 58, "Input"],
Cell[211673, 5092, 387, 6, 41, "Output"]
}, Open  ]],
Cell[212075, 5101, 104, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[212204, 5106, 585, 18, 73, "Input"],
Cell[212792, 5126, 383, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[213212, 5137, 238, 6, 41, "Input"],
Cell[213453, 5145, 387, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[213877, 5156, 287, 7, 41, "Input"],
Cell[214167, 5165, 380, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[214584, 5176, 141, 3, 45, "Input"],
Cell[214728, 5181, 382, 6, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[215159, 5193, 101, 1, 36, "Subsection"],
Cell[215263, 5196, 1185, 21, 62, "Input"],
Cell[216451, 5219, 105, 1, 30, "Text"],
Cell[216559, 5222, 538, 13, 41, "Input"],
Cell[CellGroupData[{
Cell[217122, 5239, 457, 10, 50, "Input"],
Cell[217582, 5251, 925, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[218544, 5276, 214, 6, 41, "Input"],
Cell[218761, 5284, 922, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[219720, 5309, 244, 7, 60, "Input"],
Cell[219967, 5318, 982, 21, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[220986, 5344, 398, 10, 41, "Input"],
Cell[221387, 5356, 773, 16, 41, "Output"]
}, Open  ]],
Cell[222175, 5375, 181, 4, 41, "Input"],
Cell[CellGroupData[{
Cell[222381, 5383, 177, 4, 41, "Input"],
Cell[222561, 5389, 1027, 23, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[223637, 5418, 99, 1, 36, "Subsection"],
Cell[223739, 5421, 88, 1, 30, "Text"],
Cell[223830, 5424, 181, 4, 41, "Input"],
Cell[CellGroupData[{
Cell[224036, 5432, 199, 5, 41, "Input"],
Cell[224238, 5439, 746, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[225021, 5460, 361, 9, 41, "Input"],
Cell[225385, 5471, 746, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[226168, 5492, 176, 4, 41, "Input"],
Cell[226347, 5498, 994, 23, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[227378, 5526, 245, 7, 41, "Input"],
Cell[227626, 5535, 969, 21, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[228632, 5561, 356, 9, 45, "Input"],
Cell[228991, 5572, 893, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[229921, 5597, 213, 6, 41, "Input"],
Cell[230137, 5605, 899, 20, 41, "Output"]
}, Open  ]],
Cell[231051, 5628, 156, 2, 30, "Text"],
Cell[231210, 5632, 924, 15, 41, "Input"],
Cell[CellGroupData[{
Cell[232159, 5651, 577, 15, 41, "Input"],
Cell[232739, 5668, 781, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[233557, 5689, 202, 5, 58, "Input"],
Cell[233762, 5696, 753, 16, 41, "Output"]
}, Open  ]],
Cell[234530, 5715, 162, 2, 30, "Text"],
Cell[234695, 5719, 975, 16, 41, "Input"],
Cell[CellGroupData[{
Cell[235695, 5739, 573, 15, 41, "Input"],
Cell[236271, 5756, 756, 16, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[237064, 5777, 202, 5, 58, "Input"],
Cell[237269, 5784, 746, 16, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[238076, 5807, 181, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[238282, 5815, 95, 1, 27, "Subsubsection"],
Cell[CellGroupData[{
Cell[238402, 5820, 129, 2, 41, "Input"],
Cell[238534, 5824, 454, 10, 21, "Message"]
}, Open  ]],
Cell[239003, 5837, 669, 15, 82, "Input"],
Cell[CellGroupData[{
Cell[239697, 5856, 373, 8, 58, "Input"],
Cell[240073, 5866, 308, 4, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[240430, 5876, 150, 2, 27, "Subsubsection"],
Cell[CellGroupData[{
Cell[240605, 5882, 129, 2, 41, "Input"],
Cell[240737, 5886, 452, 10, 21, "Message"]
}, Open  ]],
Cell[241204, 5899, 621, 14, 82, "Input"],
Cell[CellGroupData[{
Cell[241850, 5917, 397, 9, 41, "Input"],
Cell[242250, 5928, 686, 14, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[242973, 5947, 303, 7, 58, "Input"],
Cell[243279, 5956, 303, 4, 73, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
