(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    603887,      12369]
NotebookOptionsPosition[    587450,      11822]
NotebookOutlinePosition[    587900,      11839]
CellTagsIndexPosition[    587857,      11836]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 16. Channels and pipes (p. 261)\
\>", "Title",
 InitializationCell->True,
 CellChangeTimes->{{3.502163345367333*^9, 3.5021633468961353`*^9}, {
  3.587606439560202*^9, 3.587606442835594*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{"<<", "Lautrup`"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.390121786133355*^9, 3.3901217875333548`*^9}, {
   3.390122038390355*^9, 3.3901220414453545`*^9}, {3.421900824546653*^9, 
   3.421900853121653*^9}, {3.479265462341*^9, 3.479265470243*^9}, 
   3.5890387031263714`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 16.2 Pressure-driven channel flow (p. 262)\
\>", "Title",
 CellChangeTimes->{{3.4221975154820004`*^9, 3.422197519163*^9}, {
  3.422198589342*^9, 3.422198590568*^9}, {3.422198671344*^9, 
  3.42219867235*^9}, {3.5021633502657413`*^9, 3.50216337779979*^9}, {
  3.5876064453809004`*^9, 3.5876064552670865`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.422197522209*^9, 3.422197525587*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(7\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755476613191357`*^9, 3.586582931441271*^9, 
  3.5875544942767773`*^9, 3.5876064037199025`*^9, 3.5876359483162823`*^9, 
  3.5890779555638347`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 16.1 Reynolds number (p. 264)\
\>", "Section",
 CellChangeTimes->{
  3.4226718667596254`*^9, {3.5021634016210318`*^9, 3.502163408828244*^9}, {
   3.587606505047059*^9, 3.587606517651572*^9}}],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Eta]", "=", 
   RowBox[{"2", " ", 
    SuperscriptBox["10", 
     RowBox[{"-", "2"}]], "Pascal", " ", "Second"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"oil", " ", "viscosity"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]", "=", 
   RowBox[{"800", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], 
  RowBox[{"(*", " ", 
   RowBox[{"oil", " ", "density"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"d", "=", 
   RowBox[{"1", " ", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"plate", " ", "distance"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"G", "=", " ", 
   RowBox[{
    SuperscriptBox["10", "3"], " ", 
    RowBox[{"Pascal", "/", "Meter"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"pressure", " ", "gradient"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.587606533451468*^9, 3.5876065832904468`*^9}}],

Cell["Average velocity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"U", "=", " ", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"G", " ", 
      SuperscriptBox["d", "2"]}], 
     RowBox[{"12", "\[Eta]"}]], " ", "//", "ToSI"}], "//", "N"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.4166666666666667`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575547661891204*^9, 3.586582931716281*^9, 
  3.5875544948108425`*^9, 3.58760640505206*^9, 3.587635948853347*^9, 
  3.5890779558539*^9}]
}, Open  ]],

Cell["Reynolds number", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reynolds", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"\[Rho]", " ", "U", " ", "d"}], "\[Eta]"], "//", 
   "ToSI"}]}]], "Input"],

Cell[BoxData["166.66666666666669`"], "Output",
 CellChangeTimes->{3.575547662297251*^9, 3.586582931918355*^9, 
  3.5875544951898875`*^9, 3.5876064054391084`*^9, 3.587635949243395*^9, 
  3.589077956056926*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Figure 16.1 Entrance flow (p. 265)\
\>", "Section",
 CellChangeTimes->{{3.5021645545236626`*^9, 3.5021645572380676`*^9}, {
  3.5876066258435545`*^9, 3.5876066455389166`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.587606691818468*^9, 3.587606694532796*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(7\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5876067018016663`*^9, 3.587635949626439*^9, 
  3.5890779562289066`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Read Comsol data", "Subsection",
 CellChangeTimes->{{3.502164580138908*^9, 3.502164581823711*^9}, {
  3.58760680583015*^9, 3.5876068071233044`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"ReadList", "[", 
    RowBox[{"\"\<data/pipes.planar2d.data\>\"", ",", "Word", ",", 
     RowBox[{"RecordLists", "\[Rule]", "True"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.422197557093*^9, 3.4221975858450003`*^9}, {
   3.422198318259*^9, 3.422198331632*^9}, {3.422198600071*^9, 
   3.422198601049*^9}, {3.422198675572*^9, 3.422198679362*^9}, {
   3.4221990361140003`*^9, 3.422199041567*^9}, {3.422199661207*^9, 
   3.422199678507*^9}, {3.422237156232394*^9, 3.422237156684394*^9}, {
   3.4224247365747733`*^9, 3.4224247397107735`*^9}, {3.422516853466*^9, 
   3.4225168539779997`*^9}, 3.4309107773775997`*^9, {3.432107223237*^9, 
   3.4321072257790003`*^9}, {3.432107275525*^9, 3.43210730434*^9}, {
   3.432107356116*^9, 3.432107360468*^9}, 3.432107398901*^9, {
   3.485048298041482*^9, 3.485048299964592*^9}, {3.5053649292587795`*^9, 
   3.5053649305068502`*^9}, {3.505368322714874*^9, 3.5053683232989073`*^9}, {
   3.505447700878234*^9, 3.5054477072595987`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"x", "=", 
   RowBox[{
    RowBox[{"data", "[", 
     RowBox[{"[", "2", "]"}], "]"}], "//", "ToExpression"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"coordinates", " ", "along", " ", "the", " ", "flow"}], " ", 
   "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"y", "=", " ", 
   RowBox[{
    RowBox[{"data", "[", 
     RowBox[{"[", "3", "]"}], "]"}], "//", "ToExpression"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"coordinates", " ", "across", " ", "the", " ", "flow"}], " ", 
   "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"u", "=", 
  RowBox[{
   RowBox[{"Drop", "[", 
    RowBox[{"data", ",", "4"}], "]"}], "//", "ToExpression", " ", 
   RowBox[{"(*", " ", 
    RowBox[{"velocity", " ", "values"}], " ", "*)"}]}]}]}], "Input",
 CellChangeTimes->{{3.4221975986029997`*^9, 3.422197598986*^9}, {
   3.422197632599*^9, 3.4221976985290003`*^9}, {3.422197940203*^9, 
   3.422197962167*^9}, {3.422199050526*^9, 3.4221990977679996`*^9}, {
   3.422199219829*^9, 3.422199227591*^9}, {3.422199495073*^9, 
   3.422199512376*^9}, {3.4850492188111467`*^9, 3.485049219555189*^9}, 
   3.4850492583024054`*^9, {3.587606758681493*^9, 3.5876067648052273`*^9}, {
   3.587606817896598*^9, 3.587606881049175*^9}, {3.587607053928917*^9, 
   3.58760710613418*^9}}],

Cell[BoxData[
 InterpretationBox[
  TagBox[
   PanelBox[GridBox[{
      {
       StyleBox[
        StyleBox[
         DynamicBox[ToBoxes[
           FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
           StandardForm],
          ImageSizeCache->{299., {3., 9.}}],
         StripOnInput->False,
         DynamicUpdating->True], "Panel",
        StripOnInput->False,
        Background->None]},
      {
       ItemBox[
        TagBox[
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
            "0.2394174137928864`", ",", "0.10763286958146291`", ",", 
             "0.07607102467124288`", ",", "0.06203447148290821`", ",", 
             "0.05414785101048249`", ",", "0.049639762250851864`", ",", 
             "0.04606887326987368`", ",", "0.0436269809614362`", ",", 
             RowBox[{"\[LeftSkeleton]", "84", "\[RightSkeleton]"}], ",", 
             "0.02978119571108893`", ",", "0.029781002691945835`", ",", 
             "0.02978080967280221`", ",", "0.029782583406191228`", ",", 
             "0.029781475824281783`", ",", "0.029762407279412633`", ",", 
             "0.029670432478167065`", ",", "0.025956577604714964`"}], "}"}], 
           ",", 
           RowBox[{"\[LeftSkeleton]", "98", "\[RightSkeleton]"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"\[LeftSkeleton]", "20", "\[RightSkeleton]"}], ",", 
             RowBox[{"\[LeftSkeleton]", "98", "\[RightSkeleton]"}], ",", 
             RowBox[{"\[LeftSkeleton]", "21", "\[RightSkeleton]"}]}], "}"}]}],
           "}"}],
         Short[#, 5]& ],
        Background->GrayLevel[1],
        BaseStyle->{Deployed -> False},
        Frame->True,
        FrameStyle->GrayLevel[0, 0.2],
        StripOnInput->False]},
      {
       RowBox[{
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
             StandardForm],
            ImageSizeCache->{54., {1., 9.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 4, 31951037751521048055, 5/2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
             StandardForm],
            ImageSizeCache->{60., {1., 9.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 4, 31951037751521048055, 5 2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
             StandardForm],
            ImageSizeCache->{92., {3., 9.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 4, 31951037751521048055, Infinity],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
             StandardForm],
            ImageSizeCache->{78., {1., 9.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>FrontEndExecute[{
            FrontEnd`SetOptions[
            FrontEnd`$FrontEnd, 
             FrontEnd`PreferencesSettings -> {"Page" -> "Evaluation"}], 
            FrontEnd`FrontEndToken["PreferencesDialog"]}],
         Evaluator->None,
         Method->"Preemptive"]}]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[1.2]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    DefaultBaseStyle->{},
    FrameMargins->5],
   Deploy,
   DefaultBaseStyle->"Deploy"],
  Out[4]]], "Output",
 CellChangeTimes->{3.5876359498814707`*^9, 3.5890779563699408`*^9}]
}, Open  ]],

Cell["Coordinate end points", "Text",
 CellChangeTimes->{{3.5876087846455793`*^9, 3.5876087874439135`*^9}, {
  3.587608890710306*^9, 3.587608892883565*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", "[", 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}]}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5876088009355345`*^9, 3.587608801031545*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.05`", ",", "9.95`"}], "}"}]], "Output",
 CellChangeTimes->{3.587608802536726*^9, 3.587635949927476*^9, 
  3.589077956432932*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"y", "[", 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"-", "1"}]}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.587607295944957*^9, {3.587608631911254*^9, 3.587608672839164*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.495`"}], ",", "0.495`"}], "}"}]], "Output",
 CellChangeTimes->{
  3.5876072976091566`*^9, {3.587608643151602*^9, 3.587608673339223*^9}, 
   3.5876359499634824`*^9, 3.5890779564529343`*^9}]
}, Open  ]],

Cell["Construct smooth function", "Text",
 CellChangeTimes->{{3.587608813557049*^9, 3.5876088347705917`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vx", "=", 
  RowBox[{"ListInterpolation", "[", 
   RowBox[{
    RowBox[{"Transpose", "[", "u", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"2", "y"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422199287601*^9, 3.4221992909969997`*^9}, {
   3.422199348712*^9, 3.422199370057*^9}, {3.422199471094*^9, 
   3.4221995040620003`*^9}, 3.485049281053707*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"InterpolatingFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0.05`", ",", "9.95`"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.99`"}], ",", "0.99`"}], "}"}]}], "}"}], 
    ",", "\<\"<>\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 CellChangeTimes->{3.575547662714303*^9, 3.5755476957392645`*^9, 
  3.5865829320883827`*^9, 3.5875544955139275`*^9, 3.587606405712142*^9, 
  3.5876067021637096`*^9, 3.5876359500104856`*^9, 3.5890779564779773`*^9}]
}, Open  ]],

Cell["Get 99% of final velocity", "Text",
 CellChangeTimes->{{3.5876089070602674`*^9, 3.5876089297469883`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindRoot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"vx", "[", 
     RowBox[{"xx", ",", "0"}], "]"}], "==", 
    RowBox[{"0.99", " ", 
     FractionBox["3", "2"]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"xx", ",", "3", ",", "3.1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4850490455252357`*^9, 3.485049048086382*^9}, {
  3.485049294681486*^9, 3.485049295652542*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"xx", "\[Rule]", "5.048201584426423`"}], "}"}]], "Output",
 CellChangeTimes->{3.575547662789312*^9, 3.575547695812275*^9, 
  3.586582932124372*^9, 3.587554495586933*^9, 3.587606405831154*^9, 
  3.587606702230719*^9, 3.58763595004849*^9, 3.589077956497983*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot central velocity", "Subsection",
 CellChangeTimes->{{3.4222373106523943`*^9, 3.422237313991394*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figa", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"vx", "[", 
     RowBox[{"s", ",", "0"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", 
      RowBox[{"First", "[", "x", "]"}], ",", "10"}], "}"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"HoldForm", "[", 
        FractionBox["x", "d"], "]"}], ",", 
       RowBox[{"HoldForm", "[", 
        RowBox[{
         SubscriptBox["v", "x"], "\"\</\>\"", "U"}], "]"}]}], "}"}]}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1.51"}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422199519625*^9, 3.422199611606*^9}, {
   3.4221997855109997`*^9, 3.422199800777*^9}, {3.422237174036394*^9, 
   3.422237175068394*^9}, {3.422240316316394*^9, 3.4222403210213943`*^9}, {
   3.422243526427394*^9, 3.4222435293503942`*^9}, 3.422518140236*^9, {
   3.432108292134*^9, 3.432108292484*^9}, {3.479265602568*^9, 
   3.479265607814*^9}, {3.4850495887163043`*^9, 3.4850495991739025`*^9}, {
   3.4850504128794436`*^9, 3.4850504132254634`*^9}, {3.485050507081832*^9, 
   3.48505051387422*^9}, {3.501492484393828*^9, 3.501492533600642*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwd13k8lF8XAHBL9nXMjJ0ZM2SLSiWKntOGVJYWpJJCqWQNFUq0qGyJSPWL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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["x", "d"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox[
      RowBox[{
        SubscriptBox["v", "x"], " ", "\"/\"", " ", "U"}], HoldForm], 
     TraditionalForm]},
  AxesOrigin->{0, 1.},
  Method->{},
  PlotRange->{{0.05, 10}, {1, 1.51}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.575547662852317*^9, 3.5755476958792825`*^9, 
  3.586582932162345*^9, 3.587554495655944*^9, 3.5876064060351787`*^9, 
  3.5876067023017263`*^9, 3.5876359501204987`*^9, 3.589077956535945*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figa", ",", 
    RowBox[{"Graphics", "[", 
     RowBox[{"{", 
      RowBox[{"Black", ",", "Dashed", ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "1.5"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"10.0", ",", "1.5"}], "}"}]}], "}"}], "]"}]}], "}"}], 
     "]"}]}], "]"}], "//", "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<pipes-planarentrance\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422198019755*^9, 3.422198139676*^9}, {3.422198191025*^9,
    3.422198205168*^9}, {3.4221982403640003`*^9, 3.422198245216*^9}, 
   3.422199802226*^9, 3.4222371969853945`*^9, {3.4222385901323943`*^9, 
   3.422238590200394*^9}, {3.4222436482113943`*^9, 3.422243667104394*^9}, {
   3.422516897627*^9, 3.422516899099*^9}, 3.4309107790716*^9, {
   3.4792655120559998`*^9, 3.479265514263*^9}, {3.479265591495*^9, 
   3.4792655923859997`*^9}, {3.485048528459661*^9, 3.485048529127699*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.575547663061346*^9, 3.575547696084306*^9, 
  3.5865829322983923`*^9, 3.5875544958849716`*^9, 3.5876064062992115`*^9, 
  3.587606702501751*^9, 3.58763595036353*^9, 3.5890779566760054`*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJwd13k8lF8XAHBL9nXMjJ0ZM2SLSiWKntOGVJYWpJJCqWQNFUq0qGyJSPWL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      "]]}}, 
   {GrayLevel[0], Dashing[{Small, Small}], LineBox[{{0, 1.5}, {10., 1.5}}]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["x", "d"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox[
      RowBox[{
        SubscriptBox["v", "x"], " ", "\"/\"", " ", "U"}], HoldForm], 
     TraditionalForm]},
  AxesOrigin->{0, 1.},
  Method->{},
  PlotRange->{{0.05, 10}, {1, 1.51}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.5755476630663457`*^9, 3.5755476960883055`*^9, 
  3.5865829323013945`*^9, 3.58755449588897*^9, 3.5876064063042126`*^9, 
  3.5876067025077524`*^9, 3.587635950369529*^9, 3.589077956678963*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot profiles", "Subsection",
 CellChangeTimes->{{3.422237319588394*^9, 3.422237326347394*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figc", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"vx", "[", 
       RowBox[{"s", ",", "y"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"s", ",", 
        RowBox[{"{", 
         RowBox[{
         "0.1", ",", "0.3", ",", "0.6", ",", "1", ",", "1.5", ",", "3"}], 
         "}"}]}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", 
      RowBox[{"-", "0.99"}], ",", "0.99"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1.6"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"HoldForm", "[", 
        FractionBox["y", "a"], "]"}], ",", 
       RowBox[{"HoldForm", "[", 
        RowBox[{
         SubscriptBox["v", "x"], "\"\</\>\"", "U"}], "]"}]}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4222373363083944`*^9, 3.422237537524394*^9}, {
  3.4222375811973944`*^9, 3.422237623104394*^9}, {3.4222377617423944`*^9, 
  3.4222377815043945`*^9}, {3.422240355206394*^9, 3.4222404556133943`*^9}, {
  3.4222428714783945`*^9, 3.422242875483394*^9}, {3.422242989897394*^9, 
  3.422243004656394*^9}, {3.4222430516103945`*^9, 3.422243275121394*^9}, {
  3.422243341464394*^9, 3.4222433421463943`*^9}, {3.422243517640394*^9, 
  3.4222435211923943`*^9}, {3.422516929947*^9, 3.422516982045*^9}, {
  3.4850491853542333`*^9, 3.485049198793002*^9}, {3.4850493881418324`*^9, 
  3.485049401965623*^9}, {3.501492566002496*^9, 3.501492602864604*^9}, {
  3.502164621681781*^9, 3.5021646835358896`*^9}, {3.5876089769276514`*^9, 
  3.5876089852876515`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwt23c8ld8fAHAJEUWkoezs0SKyPih7Zic7lSSjYYVKVkUhpUJmZuZXItVJ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     "]], LineBox[CompressedData["
1:eJwt2nc8Vu/7AHCrQaQhnxJCyyptilxGsiIzKlJSkhJSUaKQUUbICGVWRmRH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     "]], LineBox[CompressedData["
1:eJwt2nc81d8fB3AZZSVJRfYIGRUNJXkbJSQiZBS+KIWIIjIqikLZDXuVFSE7
HCsrKyvbvTbhuveGSPodj8fvnzyeD5/u+YzzeZ/X+1xCVk76N2hpaGgi8D9b
P7sTjtzgzCGhnIavFa5O6fBn53+EAxkkZF5GVuGcyQBhr3AzoVQSmrMzOX/T
MgvuGi/rHY4moe7JgG6RyznAsvuLkoY/CT2UCzzyb28+qPqqc3mZk5CHlmqF
91wx5Npc/zbORkISQWN80WXV8PJQkOxnp0VUeYen/BZDK6j2sMsH7ltA5nTs
ffXCAzB6frROufsn+rwr/3c+3xhUkV5Xab6dQ1ICJv0QPQVxX9yXA1Vm0XnT
O/lO/XNAuHHHPUhxFsXp/uoXnpkDUXarP0Hys+gEb+/L1pU5yLS5uO2lzCzy
tDeJYeL8CUU7+XeGcM8ioeOH9Hl1fkKbRa1oBHkG/dgAw49VP+EfHZtBTNIM
OsfqNfgkYR5Uc2h/xMTOILXhQyJns+fB33jVJPbNDIpSeN5GKpsH1myCRdzL
GbTG6fZdoXceuIw+2yd4zKBfKZRdNqwLcDTd2C9Fbwal0uvnTLstwH/aKflZ
22ZQb2KVbY/aIkgneO1T2ZhGhm9vJsroLcIq2fBh78o0+viYSrIzX4SXbxhV
t81PI/MMK+ky90UoHbvz3ah3Gg16XqTP+bgIuz1OLW3LmkZWMkvetBwkqH7f
KmNsMI1WvzUuS3WRIPj3h7AFnWmUSpJVmhwhwdWLj5d9NaeRu44kS8QcCRaW
5CqylaZRHK2P8BTNEnArvtWmOzSNTMS+Dg7sXwLnTiv7nL9TaHG1TrPg3BII
/VtNp0+bQsYWauMK0UvQufLc6HPSFJodqrpyKWEJfBe56a1ip5CnrauHaeoS
jA2fsUBhU6iwR2fSJmcJkssf7X3oNYWOueqdU6hZAuGHO3xJelMovZBRe3pm
CUSW95kNbEyi38fAT0eODN3zaYwvVifRRa4/gSUnyfB04lSRPGUSfXyyocF3
hgwTXaa7o6YnEZ9a8USvGhlS8+MbdDsnUQaf5TNmQzKI3hU7/jVtEnly8Va6
uJHh4M8TO3P1J9HiNQYX1UIyzDDMnCq7NIk+UEsiqkvI8FEw2qZOYxLJrKXT
K5aT4bjR5pc+pUl00JHSIVxLBtXqerttUpNIrm3/RnEHGSzfXG3Up5tEvAMc
E//myBCt5uG3/HkCnT+9AOs8FDA3l/r0L2cCkT/GhxP4KSDsMTzAlDmB2lM8
RWuEKJCRoyLHnziButM0mB6IU6CEm5moHjyBaMaEl9LkKNBDilZ6azOByqfc
1sU0KMAeW76msHcCyWhYp/g6UeCaxZXtA7smUKaXW+SGMwXShOc4PJgn0KRM
uq7zfQooZeyXLv43jpSmZSwuelDgTpGL+bG5ccQ5IOdT60uB5g6JWik0jowm
U950RFLgKUNUMO/tcbRwwsZuspACHY1S775Y498raHcMFFOAJ7jmvan5OHpk
3hbZUkqBfI6lyrdXxlG4KqvW+woKEAUukjmVxpGF0PU+qa94fIVtRjv3jKMK
t0aesC4KrDreEdgsH0N7GgJ/zy5QwMjwYObv4jEkv2dZ2oxEgaIzw8ep+WPo
doBacNMSBe4zXtKaSR9DaT+Sm+OoFFhKknbtjBxDOu/plUXXKDDX/bP5g8MY
umQtNJBORwXNLymGSbZjqELdMmmengrpSWaEGKsxRGUfDJHZTgVbx2/LocZj
yFIzmviBkQoTO7IEPc+Nob8/+vQe7KTCiIKdmy7vGJpOXaoP30eFs0LC27T2
j6H7DxusMvZTIXbHQNA5Dnx871+eCi4qmHVrJp1mHENyG6Zjgweo0HfnUIvI
LyJS7Y9IWuGnQmfijNDvb0S0mzT1L0GMCrIBiR8p9USkY3/pV4A4FULvGMsv
VBORzDfTZQcJKugqNGqPFRPRP1ZF7sOSVGjpSnvQkkJEHJ49zcEyVKjfbtua
6ElEwsWi5kXHqMAc3ptU4EZEhFFbY7vjVNDhU3drdCaiPA3t+zwnqPDj2EGB
JVsiCvmkv9v1JBVmLMfvggERCbxvHmM4TQXpef3zV3SJyC4peD4Z2/lBDbet
FhHVBqRwnVWgwvrLpJpXykQUM32o79YZKrB8seAckSaizvqdq3FnqXBZvX2G
LE5ED6Z+7hVVokLkd6UKBhEiEonuvJqGzTvDd1OGm4hehe85/x6oYHnvpYIK
JxG1VEcvCShT4f3mBpvhLiJiM32K3mAf3jtU7MVARLHRfQ0+KlRwSbwYHEpD
RDdl7m5bxC6S+mKZ+oeAjm7WmpuqUgFUo5layAR044LrdzE1fD5OxmY8IwS0
smo3HH4On896w5Ej/QSUbVW+OoXd90yeXq0bH3/9qdmp81T4L3Zftl0zATGI
GnJ3YX8Q93/i85WA/B5rP+ZVp8Jc/rJheBUBybx6o2WN7dLUvVlahD3CIDKL
XWRwrqs1j4C48vYrSVzA92v0cxrxIwGZGJn22mD7rYRfZkohoO2ndd/2Yjc+
oT3IF4/H27HUxqJBBdadLmtH3xHQs8b4WCXsSBG9ZOMQApK8QLc/Drsvp8rN
IZCAJiPTOhqxeRWOXnz8jIA0V0+eoWBbfk0QiHxMQIlFEte5NfH9vLzrV5on
AZ3g+aWohD036NP4xY2A9O6wEy2xD9suxrY7E5Dy8i/dJ9hF3q3nV20JCDVQ
075grzOePcBiTUATyRtxPdgQ+XGR35yA6Hsy3Rewnwrw1sqZEJDDB1clOi18
PZlBb9QNCMjGont5HzbryT/2proEVNO0aSOGfbnaTtlRi4CG9k7XyWJHag9w
+p4noL1rE3JnsPt/aM5GKRNQibXDF1XsDyzx8UpnCKjMWeW7BrYLUK5MnyCg
5hNizdrYSvfUmUKP4vGyaIt0sJnToitPSREQXVtl6pZ7BxbvEQ8SkMg71Q9b
xyezqR0KFCQgYHRr3vo8R9U3I3I8BNS1T0VIDfuM28+Iwb0EZFYVVLZ1PoyZ
oPmUnYCu0Ch9kMPuHo7YlGYhoJOd8EscO3H3zOceBgJ6LOtRwIPtcF7xtg8N
AeVwd67sxL5LETyrszKKFPzlvm3i+8Vk2pSW8HMUJbr4qixiKxw64K3aO4p+
loReacLuDK2Zjvg2ihqYBHYVYtv/ttOfrBpFDo2KHxOwYxu+iD/PGkXbvOub
nbE3b1zvbH08iojnXIxYsKuTE8VNpEaR+vLaDkM8H0yZNcMzBUdR/cSucWVs
ijN548/eUTQr5HdCGltYRbUznmYUDbxdOr4N++nouNdEzwjKpiNrp+L5yXsh
ePrEtxH0am49LAi7MOe4fkDVCIpUV5Z3wZ7yfiYumTWCOpznX5/F1uAV73R6
PIKuCHSbtOD3geDXrljtOoI+nK7VzcH2+PkgjcN+BMUvfXsVgp1Z1uhVaDiC
Lmi8G9LFZjWxE/8jOYK+SMjcbMDvX2oVR7i2IB4/n3IlBfusxJeNuL0jKJAm
954PtuMqS6cyzQgi3+GeOozdEZXt5d8zjOyW38Q8w+93qNrb8VffhtFlRQfF
a9iXyb5ab6qH0ZXYveJy2O3axlzp2cMoN2i6fxDXizY6uoKmZ8NoxjqgcKt+
fHM2/sl6chgtDOSei8b1JlBATX+v9DD6Y9/lewNbq1WmlE94GMl9aGY+it0s
QRdwmG0Y8a2f/F2D61fjaLbw5akhRHdtuyYB17v6S3SmEa+HkKoi76F1XC/9
/yxUxQQPoZ8V71XKsNUz+sRTfYfQmtCbJA/sr/Q5ywWOQ4hZ+ujJFUUq1H0x
DutVH0I3/PLSpnD9rTmU08i9OoguHUi5morrdQWDiXzi1UG0Rz7cNRzX/2D6
nZad2oNIKVrxxzHsa3TVz+lVBxGxT/FSN14vNmgODdySHkRF4e13ObAVNn57
yW4bRLkJcnf9ZHE9oLytqckcQOlXIwXkD1Mhd7Tv0uTffnRCf5amGq9fj0eC
3fZT+1G95vZjSti6w8oJmjP9OE+rPy07SIXFgfSl7M5+JFmgHZ8riteXXvcI
t7R+JCV23jZCmAoZLVz9O/T60anV23Sn8fqYXGpsI5nahyo9WmvP7KUCG0Hy
jsnfHpSkrPTTn0IBrdBX/heediEFtwUyXyfOH3craF56dCE6lz7GiO8UmNed
f9jp2IXiozpkGbFf7tJyum7ShZZtWcso7Ti/vGIwvne4C9HX5qQ2tlLg6kuv
Qwl9nUgjyXfuQRMFbALtWlakOxHnu9C/E1UUOGH3Tl1RuBNN0HS7XMPertVY
9WR/Jzorc2q9C+F8xSRWyErbiaz4+KC2EueP58Q44R/f0SNL54TkcgrcDTB2
0nnyHS28o8uyKaGA91N1jg89HUho+sultk8UcMssfp78rQNdV47y0sF27JD4
F1/dgYxsezfbcnBe5GWZf5PdgbZLNu35no3zU0H71xfPOpBI178DA1kU2Bw3
9rhzogNZK0h70qdTYMO/t+q2ZDvqsi5iEErC96OzJNtqXzua76ktL0ukwBB/
TLQZbTs6y6PPZIhdXmRx/9JAGyrOG9YPSsDnNzUjLhvYhrR2HpLfjMOff/7P
q9+zrWjc/XLVWjQF/tELXg/IaEG37WVcOKIowEaNbO2KaEF7nzYYZOO8yE9k
UhL0aUEBN7z8NLCVKqj8pfot6PmZo18fRVDAx7WBMPfnG7qhEtlPDsPnO+lo
o6PzDdUJLJ6beEWBtfoK+73UJpQ6doG67QUFGAvlhv4bbkLFu3uufHhOAa6U
NO2chib0piqe5iL2qUdhMhdim1CdxV+lqAAKuMvfILmfb0KMlEe6Mv4UWElj
vTf0phG1/rhWZ+9HAepzs4cpig3IbNP7vrAPBd4yf9ptKN6AdgRwzXz3poBi
MG3Gdo4GdCVV8v0T7KchGT9uT9ejnH9v/xC9KMD5evX40fB6FL7Qw/XBkwLH
UyJI5dNf0Z4Lr9gUcZ6+X/HNpje8Dv1o+vU0HudtLhDYCPCuQ4pB+lxXscur
XCJO36pDiCX71y5s+jrumtizdeim99XnvvcoENlsK2A9U4tYAr147F0oUPiD
boB0thapPxpY0ryLr4d8Rpdptho9WhrhVXKgwMy/PaMSPdVI18punQ57cOe8
o0Z1NUp/MLu/2Z4CVYdiXwW8q0YRMkMcV7ED/9toZdCqRgWSF5ju2+Hn9b1c
mzarCk0Et/0tvUWBV5aekSv9legIwfm22w0K3POSKjiRX4kC8oSunMM2fjvY
dT+wEl0KFffmwBbuOLOHqlCJBN1OhH+ywXkdNsJIMRVoesykat6aAiP83iEz
18vRsvJX6XtWFDgy7BM4QCxFp3y1NXMtKGBw1iCvqqgUpWXeKvLFdo871Pch
qBSxePA7GWLXXO8+eO9EKSJzG9b+McfHjxyqZnlRgnbME1YuYruPdq8qHi1G
le3bA9evUSAWMvlFGIrRNtfM3O/Y1QmPzjMNFCGVhm2iGdjMlpIRPX5FKOyv
6YLx1vGER4cdfxQikpUjc7kZvl9EyRsJPgUoJrbNPtSUAhMqNMHPrhSg3N/3
LzpgMyX35NtLFCBtAT83DWx9q8c0pzo/I5krLTnbto4f64npOPgZ7d3RlPbA
BB8/8bhzW2seemHC9cXZmAIpLdmPGk/moFVmS9t4IwqIVdp90p7KRuY7PF2e
Yad/Eh/Faxiy+1Fd6YCdE56k1P/rI2K0/XJAEbvMOOrvbH4WCr1fOjpsSIHO
CS9PliMZ6GCXnKsMtmHv6ayQkXSkeDHDlQu7r2FlYM+rdJQ3dKGYDnsk864C
70IaKnF6sjlggOvVXZs16awPaE3PLzEYm/bvxQc64qmobzJDd+MKBZ4tMqZ1
9qYgUSl+l5/YjISvvUb+KaiA+9uPAWy2WjhpMZmM4vZduVOGfeDFsWWnlCRk
EVRS54Ud93BJ9Jd+EkrZF0LviC3okG3gTpuE+CM+PLfAFtMVL3j8XyLK9+M1
V8O2kTw5JQzxqNPR/jobtt3O3PNS8XFovMJnehv23aVD74/9jUV0q95Fy/oU
8Criszn3JQatXyMJj2BHqjKM3ZB/h2Icbx3Lx44++EjF8fVbRH36fiwNO5Fx
LdFt+Q1yXpLujcP+2DZvEfD5NbLfaHoRiF1r2j2cfiQSCVjsELLFthtQvT8Q
FoGY2aa5LLA5TPOYWX+FozX1Nr2r2FYmr+SdSsLQQXlbfw1s5v6N1sQDYejR
COWJCna+sb1Np1co4lo6XamATZcbKmzv+wpdkMv5LYOt/+mObpv6S7SuX7gi
jp2co+UlyxKMxCulTghjq2bT965GvECHDQ7c248d/pFIa2b8HIk6ObhwYBOz
Ko9U8gagjgJi3k7sx5nuL56+f4p8E2eX6LE7MgyLpm/7oflDTydosAUy5Ma1
DvsipzMy+/7qUcApfRd7DuUx+tBaEbCG7a3b7yHs+AiFE06cXsHm1Ynomlnx
QnL3oySp2OXal2Q+PXqIAp4PmS1hm13cEeDK6I5C+VhaF7DXNasJZ8Jckc4x
IZ+f2O80PBVoD9xDqInfaRb71IUTkY3Jd9EUgS5xGvtT0Mgg31UHVOXewT6F
vfhcPyTzli3SDXtaPYEt7V+vKv/QCoVLCuaPY/dud50P2G+GAs+ljI1hX9l2
/UqSgh46ObrzypaVSXtm59zOotfrFnRbfnvp7Yr5mgh8fP2OSsR24O8liZZo
wP3PZSJbvy/lpNu9dsoIKJdrI7bM/3JnD4nZEkSdPqttjUcIeSdkvWYDzGxB
x7bOJyn8oGPv9G14L6NlPYltFZVXptnrCMutS51b58/REu9nPuYMq4s+vjPY
91onDr01uQ+5ob+c57C72yQ7vne4Ae9nvaR57OMdd91YNDzgrNFrdhJ21Pci
3vPIEzge1iAy9krnRo3PSR+I4e/N+YUtZZd0fMjoMfSrfR9exQ62nx7Y2+YL
Luw+fzaxFR2Yc/KZnkJr8ZE5Wvy8FxxknuiefwZ14vTsO7B1HO+JvygPACvH
lt3s2JuOr9cP/n4OXYWtFE7sHKfS1ppjgRDb/GfPAexdzv/ubWQGw19Pyf0H
sauchS+8m3oJ7hJ6a5LYd13OHzgpHAK7a5Z4ZLFnSs8TiltCwWpoSlgJW/NL
kD9ZLQKmgmpPm2GnSj7vEiFFwMMy3w7rrfHfPRU0io6EvIIfnx223hd377Ky
pSgwP5tq+Qj7wEmnRb+4t8C+5mSYjn3/vf2ZIo13wD3y7Mpn7DbOW89nqO/A
mUyNr8T2o1oKX9KKAdvu0/y92PO5+kb7VuOgcYKzmhHXD3VB3RSN5HjIioqU
3oedGHJx6eGlBAhKPM0oii0pL/yVpJcIdIfNPytjj+7lsTBITwK2a/c/emNr
dLFG8BmmgvQA247tuF4uMweVFmWmAq3/wdtc2MmqTARdmveQLDy+Twp7I49B
xjfrPUSfLDDQw84L3ayf2pYG0vSGhknYB3SW1j/lZED+niplXVyfG545CmnS
Z4JENu/hm4ZbeWL+wphJJoQXjNh4Y7dJz0ZyMmSBruWnnI/Yvixjhz3MPsLw
Xc0CNrw+/GzsslJl/AQJSeEvCdjm7ZAmYvwJHHb6n1nH7uzJ+kmf/gm+vq0/
xHmVAtfIKmXjIrkwyqHeoIGtusmt9qE2F6SLRrcXYu/iajaQossHUh9rwju8
fmVclHQ/4VsAHX7FdyPwejemY3lALLMAVqtJxwqxefVfl+/rLIAnn+blf2CH
GtNu+y1UCHSLirS8eH11uzHwoqy6EEouOSemY6s+CoyBbcXwEDbo2/B6PJQ/
V6n5uBTk36cGaeL1v9f5j0lEVinEdQPZDbvjKOvyUG8phDrxpqRi1+XISN2V
KQO9oZifNJYUyM68++b1QBl4KlXvqsD2SV5xGj9eDnGCEu/VcP4QCKcV8p6t
hOkXScyBOL8c0NtTXs+JgNfQaq0Km5Nd9Cq7MgKuCvvjv7EZX51/mfIawU1r
4xDbmxT4fa5uY0KoCq4zvevTtMX1QXnHbZb2KkDUgBre2/j5nnqpYnyoBog/
x2b+4DwWTS8/80apBvwrJOXO3MHPq4Pw6seVGnA1/a/uIbbBLfw6+9SAVJpn
7Tr27zeDboZdNdBt9UJq0xHn59VDH/W9a8HVmCOIw5kCLYX1+y59r4Okrk+v
nF0pMCVHM6/i0QCNtXvlmB7h+u9yMHrPywY41FYjYojNk6+lMZnYAL3bBAyT
sC/LRqU+b2yA/vk/bgqPcf07Inm9fX8j/NsxW+z0BNcTaYO2a0WNwPjM02gK
52VZsfTcB9Qm8LwzGE6H83jLjVZzzR3N0O2Z7HcV++Z7CisPTzNU3lv5nIUd
LXr2dqVqM4S+PN5lEIjzrUinEEN4M5y5GcWSGYSfp8BGePjRb3Dm3yU9S5z3
vbgvu2bfaYHdHeraG7g/WCkaS5DyawE64o55C9w/OBu4Nme8bYHw/ju9tdg3
Q6IFPtS2wGD04aBg3H/oMUw2xXO3gt2unPtCb3CeIHvwh35thfbSK/GXcL+S
+IpVk22wFQqPZXF/xuaRTrgXvNQKl80u/+aKoQC7bV3jc5428Ga3ok5irw+x
3Xvi3AbRscX2z3D/096Y0uDC1w6zge0VI7hf0rp5krIk1w4W1ew12ri/qqNr
4nXSaIdpLm6PL9hlSovO9vfagf759T0xybjeFZzitWlqh2tLTX7Wqbg+d9ws
yVTogJSIz/5quF/bXf6HNXaqAzxb8jzScnH+q6MVS13qgCeSj7Qk8vDzb2WC
j+sd8DzthmkGtt3Ifudytu+Q/FxM6GM+zsc0x3uGTn6Ho9l7cj4XUMDlvEMc
X8B3uNhXzJiD+82utiGZBPFOyNP68jcR969RxAqd5FtdUCvGfbCkA+eJyOwC
upEeuDh/+9bPJQq8odNM//W8D6KZTw+27sH9ulSP02x4H1Stiv24wkmFKv3/
5Edi+0B6Rry4H3suyaO+IbcPwpn37pvE/bqScuZEdF8fWA96Utb2U2HCi0VI
RaIfHg69dxPgpYLcSuu7lw398J/467tiolRo4DP9z/d7PzC3SgomY187PyXx
YLAfrgnuEOY7SAX/yM1iS1I/fG4UY9wtRoV+uSO9x/YPgNwCuY0kToXHTqEc
AzcHoFdn+18XKSq0zugHiW0fhDN8get5slTQR3vDyOyDMHJhOHafHBV6o/pe
l/MMQuV1b+2H2ARV82R92UFQ/m9Xv9IxKvyKvV3qc20Qmv671Vp+nAq8eo+n
e/MHQe1xzZMQeSokiastJFUMwuHfylxT2GKbDFSHxkHwCBNqPXOKCkezgjZp
Rwbhmy8lYgL7HMObvUeYhiB84HiKlAIVHEqz1QIshkDv6IVjoYpUIIfc1dS3
GwLta8qzfdhuN4/p8rkOwS5Ola+CZ/H17Sk1/Rw4BAFPVMnZ2JF36pxHC4fA
re2DWKUSFcqFBhPkWYeB34Jrd4kyFVR+x72n3T8M25uDE6nY9W2WWa1Cw8B6
9obpYRUqdHhOFVnLDwONheqlJOyJHnJriNUwqAXlHPFWpQJLIOPGdMkwGF80
mmY9R4VbB14wbNQMww3Z05ynsOszmXaxtw5DauegvxW27zdm4VPEYciwtjxb
iP2bdafGc6YRMA20Jemdp4JR3Eu92D0j4M+pqfUAu0CGzSyXbwSu7tW0icZ2
0tnl2Cc7Av/s2yRGsKdCdkdKmI7AnwtHLK6r4/snGB6naDMCyglsjzyxk3M5
0i47joBh0+TqW+zr3/eUufuOwIKB3OZ37G6OfYTGzBGY8VVLUryA51vK69mh
ghH4nEFoMcQOPbafulQ5ArpC4zaO2BcNuLZzd43A3+q28TjsmqgD0rf/4PFo
lf9bwRYUiznhzTAKKTPbj2/tz/sU8UDYrlHoj0v0F8A+/YNXv1RkFOqO0RWq
Y7+2jTNrlRmF9XtpUSbYv1b5bhDlR4FBteefPXYul4A7k/YoxKg/fRiCzZaR
8ITPaBQyhgghidgOpwWDZC1HIS78ZVghdnNTYuR5u1EoeWP4oQNb3FQo3uT+
KLic4D/7Ezs+RqQuI3YU+IS5k3Q1qbAhldpa8WEUoqJ9125im5aL/vieOwrP
7L/6eWPvHT44t1Y3Cp9f2/5Ow3a584G6s30Uxo/uatza7+/4K/ZXqH8UnFIE
I9uwg/kl2LUWRsE3bPf80tb3Bznp3Oaro7Chffo6jRYVUhMid5fSEKDI0iyK
Dbtr2mBfLQMBPs7zBGzth9Me3cvTykyAHnaBg1v75bLuPQI/dhFAaoHRamv/
37IqSpTISYD0JO+jCtivGI0O/eQmwDkdIT8V7IrL+w4v8xPgN8db7QvY23/x
z06KEKDV09nx4tb3CW/EU3slCFCSTd24hB2tcNSiQYYArBsBP7f2/yeGTx0o
kSMAV9kbmS0ffqLSky5PAOkzwUVb/99dVCv0nSIB3nW2eWx9fk2D/sVAFQJ8
6mq02xqf1d5su6c6ASrDxr1OYxux2VTbXyTAfsvnKUe29v/zHLyuXSYAY7hg
pwj2nIGr/CVDAmw4MTPswT7225ty1pQA0xeILVv7/14x/tmHLQhQdp37yQx2
vVLILQEbAixZHOLp2HpezxJHaBwJ8HQ8a+otdqpExjuyCwGGqllYHmIvfssz
GHtAgFB1/j/G2E84aptrnxBw335ClB27ufDbswJ/ApQHSVyewfOD06Rb+X0Q
AZoepstUYqcnTBb5RxHg+zGpcaut7wNUF50fRBOg0ZQnRxZbcWpF+lYCAZhi
77Fv4vnfIc2YoplBAAtx+Sch2KulkiGsFQTgfcBwLRK/b8rXj2n9rSaAkL+s
1Nb+fSCNIsNiPQHMRTyv78Dmv3DJs72DAK41Kt+d8Pt9a87wZFUPAbSEPjII
Yue/NCfnDhBAt5HNrxXXD/VuJ9vwCQL0p79REtzav3/gLuw3SwAzyHL6qkaF
gQNPhu8t4uvtC2u3xb7zX/gVw98EKD11JjoJ16/IxQLgYiWCgbDxxgSufyNh
FeuMu4lQJ5ya6IgtcaK+cG0vERidd1svAxW+eP6QGhQggo/4sso6rqdjjOv7
448R4YTujRtEXI9lRWBJxIwIp4VZjhzE9ZzCZ938yJII9rWx71xw/S/gCkgd
vEEE0jPj+PKTVDjF1m4ccZcI9S8GXNVP4PuzZl5N60+ESI1hE1m83ui2+4QT
c4lQtIfpgiVer9ibkx3OFhEhJPP2IzdJKnTW1au/+0KEtfgXV18cwvOtjO3P
5XoiJFrxzqfi9c78fbx11SAR2FP5mz+L4Ov3rDyesH0MzvsLusny4PntNsb2
h2UM93OTqhwHqLB0d/us0e4xcLpYUUviosL9mzpxbLxjMG9XMpW4D89HvREG
H9kxGG5y6hncTYUg8c3ea9fG4OXlwbaSHVTI6DrrcSB/DFrZzJl3kClgnKXc
c6B4DPrMdKKjSRTY7qcmy1M+Brddek2lFnE/LKc5y1M/BirqUY7nfuJ+IMzA
hG9gDHJDBKcNJinwSsf+tCDtOMQ0He7d7MN5pent2kH9cSD5hdVJVFJAMCnG
UOzqOLQs/0z5rxz3O+7xeWLXxsEy+Z/tmzLc/0qk3ha/OQ5j1gJha0UUmPD/
1C/xcBwIYf3Mbz9RwOhcfalU8jiYV+YfuYbzmgKieshS8HjWA4IaOP9aLu5e
jl0dh8S7g8GTXhTw5zt6l/HvOEwplZO9H+K85Olwc3THBARYr99OwPna4dSk
/ku+CbiV5eqUYk+BhLxeqVmNCdAJHOy/h/sp+tTS4cSECXh4/EVKpTjuH7t+
GLO+nwDSjOB/KaI4n9KudD3InIAzK1z3fYUoEG8p16xTOAGLOX5RR3goIM+X
VbTRPAFJSUXvz7PhfPc6NsR4ZQIUaxrtjKhkaH/+SIX90iTUpbUxni4lA/ue
KRcr/UkQLc7S8Cgkg36cdmrB1UngLk4UKcgjQ3ce9w4Tq0lw15jI4c4kw8DA
55akB5OQ/Mfudmo0GSakZ64eS56EYp/+k6UPyfC747Kj0eokWJFVXMZOkuG0
WXFC+gYe77rx+AM5Mjyc5Pu+vm0KBMeKmRgPk+Hv2tyxBNYpmDV28OA6SAZa
kWdr00JTwL2m5PWHgwysrqVPPbSnwB0JNXxdWAJBbuGY2KQpmEQ/VqwSlkDn
sa1OxIcpCJPnOZEWvQSe0x+3BWZNAUP1t96pqCXoLZS/9aBwCgwb2tSMg5fg
pf6lE/pNU+DG1xS2w2MJ1oPc23eQp+DzQxOX53pL0L3ZRndPeRqq9sZk0m2S
gPYmZ/Ht89PQNrbw7T8qCY60mthZak3Dz2+3dYtmSBAYM9FxyWAaLooPOqp2
kUDl1HqcxK1pYBfVy6BLJ0GO88FTIyHTwJ/4QapVlwQvJjwdNEen4V1FTqrB
m0U4nDKklDUxDbOp3eV2gYvQ/d/Z3TvnpuGR9vqQi/ciCI5uFnb8mgYz/6oW
Y+tFKOp/8u8q8wz4sZqmSB9ehIm2gPCbJ2bgUh8rc1jVAiiXRpT4Bc3Azex/
mpTBeVh5mUVfKT8LAX2yRIWVOfByCee84jEHqSuKtSx001DJF1oUVfATdEMa
ZzdejwF9mrVAIvMCRIQxd8Y/GwBr+8V/zZqLMJa1VGCQ3g7H7/odvKa7CAXb
3u1/eb4dtrtyXVwwWATHZyg9fawNMn1UX++yXIRB/XmjF7xtQAl7LW3gtgij
XeJFba9a4HGJkslw8iJoPCGWTN9pgliGkHzSn0VY1T5EMtpfC47Mon2Pt5Fg
jJM7kDGvBpR3lf7dvYME0tvjMhO0amCca1zjOAcJ/vUbdPp4V4OktPyouwQJ
TKnG0tvrEJToj7LQGpJA5vZDDwftUgi8el82wpQEhpGL/pFeJXDtGtNVUUsS
5JaJ6QZ+LIZ/N44lq9uTAMabvoyyFIG6R8CpoCck6GGSLj3ZmA9cPrzmvAEk
uObKfvn7Sh7M+eb5ZQeTQDfXf03hYB68DB5sa39DAo+//Ja/aT+BedjdX5Zx
eF6wSYmlm2fD0dcMByjJJLAOl7i9WpoFtDHR4IfnxfVN5kdTnJnw/7+PhGf/
//vI/wELB4Jg
     "]], LineBox[CompressedData["
1:eJwt2nc01e8fAHAziUpGyUjIlsqu5C0iK0oqVJqUvb6KSCgjq0JlRVYqISNR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     "]], LineBox[CompressedData["
1:eJwt2nc41t//B3A7s0hKyEqEFJVoeUllhTISKRlRySrjY2WVmVFIg0rI3smI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     "]], LineBox[CompressedData["
1:eJwt2Hc8Vu//B3CzbGVEIpISilSilHeUJKNCKIkKkYwKUcIH2XsTZWXvJCsX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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["y", "a"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox[
      RowBox[{
        SubscriptBox["v", "x"], " ", "\"/\"", " ", "U"}], HoldForm], 
     TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-1, 1}, {0, 1.6}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5755476637794285`*^9, 3.5755476967823906`*^9, 
  3.586582932805415*^9, 3.5875544966180587`*^9, 3.5876064070452995`*^9, 
  3.5876067032108355`*^9, 3.587608986530802*^9, 3.587635951004605*^9, 
  3.5890779570260477`*^9}]
}, Open  ]],

Cell[TextData[{
 "Initial curve (",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "0"}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ")"
}], "Text",
 CellChangeTimes->{{3.58760905051548*^9, 3.587609053138793*^9}, {
  3.5876091166914206`*^9, 3.5876091317482257`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fige", "=", 
  RowBox[{"Graphics", "[", 
   RowBox[{"{", 
    RowBox[{"Black", ",", "Dashed", ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.4222430239153943`*^9, 3.4222430365083942`*^9}, {
   3.422243373587394*^9, 3.4222433742633944`*^9}, {3.485049419605632*^9, 
   3.4850494666183205`*^9}, 3.587609168041581*^9}],

Cell[BoxData[
 GraphicsBox[
  {GrayLevel[0], Dashing[{Small, Small}], 
   LineBox[{{-1, 0}, {-1, 1}, {1, 1}, {1, 0}}]}]], "Output",
 CellChangeTimes->{3.587609168402624*^9, 3.5876359510556126`*^9, 
  3.5890779570490103`*^9}]
}, Open  ]],

Cell[TextData[{
 "Final curve (",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "\[Infinity]"}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ")"
}], "Text",
 CellChangeTimes->{{3.5876090172514877`*^9, 3.587609035923728*^9}, {
  3.587609077987775*^9, 3.587609137650934*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figd", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{
     FractionBox["3", "2"], 
     RowBox[{"(", 
      RowBox[{"1", "-", " ", 
       SuperscriptBox["y", "2"]}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", 
      RowBox[{"-", "0.5"}], ",", "0.5"}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Black", ",", "Dashed"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422240461255394*^9, 3.4222405135603943`*^9}, {
   3.422240558397394*^9, 3.422240588430394*^9}, {3.422242901828394*^9, 
   3.4222429067053943`*^9}, {3.4222429823233943`*^9, 
   3.4222430124153943`*^9}, {3.4222433656723943`*^9, 3.4222433663103943`*^9}, 
   3.4850494081479764`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJw12Hk0Vd37AHCRSJI0eDUYS5IhQyTpMWVIhCbK+FaUKUmkhJAQJXMylblk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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 1.12},
  Method->{},
  PlotRange->{{-0.5, 0.5}, {1.1250000306122443`, 1.4999999911146265`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.575547663974455*^9, 3.575547696868399*^9, 
  3.5865829328324633`*^9, 3.587554496766076*^9, 3.5876064072053185`*^9, 
  3.5876067033678555`*^9, 3.587635951113618*^9, 3.5890779570670547`*^9}]
}, Open  ]],

Cell["Combine figures", "Text",
 CellChangeTimes->{{3.5876090592515287`*^9, 3.5876090669634542`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figc", ",", "figd", ",", "fige"}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<pipes-planarprofiles\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422240518309394*^9, 3.422240524962394*^9}, {
   3.422242393140394*^9, 3.422242475177394*^9}, {3.422242524512394*^9, 
   3.422242541878394*^9}, {3.4222425878863945`*^9, 3.422242684548394*^9}, {
   3.422242851517394*^9, 3.4222428897703943`*^9}, {3.422243031724394*^9, 
   3.4222430419193945`*^9}, {3.422243696430394*^9, 3.4222436996523943`*^9}, 
   3.4309107796085997`*^9, {3.479265553541*^9, 3.47926555602*^9}, {
   3.4850484767507033`*^9, 3.485048478698815*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755476641854787`*^9, 3.5755476970864263`*^9, 
  3.586582932977447*^9, 3.5875544970041037`*^9, 3.5876064074543486`*^9, 
  3.5876067035838823`*^9, 3.5876359513506465`*^9, 3.589077957211032*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJwt23c8ld8fAHAJEUWkoezs0SKyPih7Zic7lSSjYYVKVkUhpUJmZuZXItVJ
hOy9977XuveSEup37uv1+6vX+3Xv8zznec7nfD7n83TxO3mYXKKloaGp2U5D
Q/23K/nopb0Fy8iJbG8mR6mGjV2OY1w5yyhKRE9HTew7CATEXuDPWEateUdX
H9rVgqfVz3PSr5fRnjPNNn3V9cC8p1JFO2wZiUt+YPX0awb1YM0DAXbLSI4c
12uv3QlFzraNk7uX0ZcL3e+/lQ9Cb8cTTSLTMtpZIOdyXX4I/qp+rSLRLSPD
06dPK34YAn1uwfKtP0toQ/9JoWrJMMx2z2Xsn11CudkF8mtJo8CjdStIDy2h
4/u5mTkyxyFK7PHx/zyW0LOjDNOytNOgpJoaZei6hBJ/qR+Okp8GosUHAsF5
CXFq/SRbuk+DdshkKp/1Ero4Vuoj2TcNdKOK7E/OLiEFbi+/ubcz4B+3QHE5
uIQGIuyqaMTnQCyP1ohu7xIy/ykgbm01B31VB/KSdy+hbV8tm0+HzcGp5bMX
e7YvIY3xrbNRY3NA1knqPLO0iF67vQt7+pQALv8MSnmqF5GuBwcy7yXC/n3O
bB8/LyKzfrawn1tEqJX0dzMvX0TqldFXDh6ZB0HrzCOR+YuI8Fd4D7PHPIyU
/o1bj19Eo9cWigv/zYO5a+HtLrdFVL6no7qRZRHo7n9v97iyiGpbopcTJReh
JH5QitkJO/rwQyb9RWD7vmNGzXIRJUbdLS5/uAhNvA4WhWqLyP1cyaMImiVQ
72aTe7RvEWnxDyzkDS0BHav2nvt7FpF3eHj0xOoS1GnfnfdhWUQZAsevtbAs
g/6nxeRLtIvo+fCBzXrFZbBIb2BUW1pAPu9e7XR4sQxXvUIGfn1fQOte+TPO
vCSQzKssXfq6gPhUawStxUiwNEV5Ml25gPpXXl49dpIEN6wcz3QWL6Dav425
9zVJEAAq7wqSFlCfRoT6t2skiN71O/DS7QVEuKwgbFlMAhOto1Y2ngtoX6tS
W30FCTjvXz5hem0BNS78O83/jQSvV7tmVB0XUJGDwOOwDhJkDBYbHTZYQHfa
DfuOUEhQnnONv/PIAppm3aj6LEkG/8m0jR+8+PqJ8WVpJ8mgzD3Q/ZVrAb1T
sXX2UCDDt2jtRwVsC8il/7PYVy0yNPkIrTzcnMfPX49NzZEMoxqjNapd88j9
x7UR9JQMYUda5oxb55HYntPLK3FkkN7+mcWxYR41S3u27XlNhqCvr83uf51H
duHkH8wZZDisaDFV9W4ewdJ+2r9lZKg5qMHYkTWPVoWY/4VWkuHa75OSE2nz
KNmVw2odkeHj+z23aF/NI9My+am0ejJYHWvefiZ0Ho3G5zcz9JOBhvWTiOm9
edTtw7d3eYgMWYu5ehfvzKP6BFP1mjEyrOVGPHvghcen13bo1BwZ4oTOCtbY
zqNbEX8klNfI0MZVqa5xah4JBTftoGOlgO96zmXz4/PopWXSfN8eCvD2vXx0
SXIenfZoP56zlwLuz707QgXmEbH609ZBLgqwsJ1wqt09jxJdO6XhCAVKl/jC
epjm0a+bDLWpwhS40MyaO0M3jzILVPf8FKVA7qNFMsMGEQXLHBJ0l6KANn3O
fa1ZIqomJLS+k6UAaTL+reUEERlNK6bFy1Eg/lvYjyvDRHSSOWT25mkKzNx1
3hPRSURhJrZi25Qp8MTOVPZlCxHJlbwwL1GhgKyy+vnsH/h85fWm5qoUCPnD
m1qPiGj51BUhrzMUkOzf/b3vIxG1LdwRJZ6lQOeHrbm590TEWttvY65JAYHb
g8eY3hGRiH1VOa0OBRpMG8wOZhGRqu0Kq4YuBbxOVPiKpRGR4saXX356FPi6
/OKrzksiMnl2s6/SAI/f2/X7UiwRhZ/ilxs1xM/vr3LDsygium1vsXPJiAKG
oXta5SOIyHEpXWPSmAJBLNOdw8FEdCNC8cGPcxQoeFbeFxxIRDKdnBspJhQY
5oocFvEloqO3JKavmlJASezkrNd1Ilo3nF9uM6OAWxHDwj4XIhrsDLvnYU6B
BLkBUqUTEXFpqF6kscDj/5L/08GWiNS12doeYP/RuP+H3oqIhoOX/65jizWb
/cs1ISKv1z0iFy0pYGUmSmdsQEQPZb4EI+yIwQ3Gn1pEVLj2RovVigIfnFp3
vVYnIrcwrwIT7BlCGjsoE5H2Q+nlh9j7vLz3T8kRUXNGi9p7bI3fOocfniCi
0CDdkS7sW3e5+aWl8PgrX2/OYWcwkIU6RYiohLa8i4zdGVUj7itARKd/v36z
jE3L+fIoNzcR7VdXip3APpF4TebbfiLqCY2bbcB2EoTTV9iJiOZqYk82dkwu
uwrLLiLijtGPD8D+enxGvXgHEX1IT3LUxCaVV2hZ0BKRhWbEdQZsPtUo/Y1N
AgrloV/5hO/XqM7hXMovAmpsZxF3xQ4ylLHQoBBQKf1L3d3YBd07LhAXCOiH
fXJkDn5+IzaD9k9nCahhb52ZAfauqQJnmQkCunPRIGUJz4fSteCr/UMExH9v
4kY4thvF3D2ol4BsbS6M7MdO9BO7KdhBQC2EyNVkPJ+NNFs+9U0ExMmlX8eL
Lc6acZ/9GwG9EasjMOD5t37hE/bhEwHxLUjauuP4eMit99jmAwH92n+rqxnH
z5wEJS7zHQEduvGH8zaOr0xt1WyFeAIqTzredxnHZ08rR/5oDAHpHli0jdan
AL3lbHFIJAFJPDzuXYLj2fnSk8qW+wRktzfn7RiOf4HgoRYnNwJaUU5amdGg
gAlTUeeOKwQU0Eek6cXr5370g753jgRkvOZy9CteX2NvxCfWLAjIPlhXLkiN
AkmVvj8fqRHQzu+TPNlK+PNL130fKxFQXt+5Jm9FChxhc9p4LEdAF49o06gq
4PzgrLctSoqAlO0r1b/h9V+2i2fX04MEVKWqIhBxggIt9tVHnpHn0KGMmlZR
EQrs2Vme9WxhDv3OeiWbLUQBs9J34nGzcyi2rUBZEOejAcYXx54PzyH+x+F3
WPhxvBa7KMX/mEO9D269ycf569/23WYJqXPomyDzuxO7KKBeQNubkDiHXtRp
8jgzUyDM6tf5xPg5dGPvXc9YJpzv8sfsk6LmkDPt2PkZegocsPjvWrLfHBJL
Vqz2+EuGY9lWD9LPzSFGG+4dUstkcNRPL8nbNofCxzOZbrSRgf/fr2y6rBnk
s3fk0aYdGTrWIiz+S51BDjQCbca2ZAheOkjnlDiDpnf6br69QIaJYUV7FDOD
mIYfcZpakSHt011O/4AZdJvnTXz5OTII+O8IXj43g4gX2Az/O0sGwZ/7Lgxs
TqOwd3EDnhJk6FrIYnz4axr1bHuoNi1GhpAp+TI5yjSqr2jZshYlw1Sn9Z7n
s9OoPLoyWleIDBklb+qMOqZRa9z+fgVeMhzxFJb5njWN5r9/qbzEQQahedld
RSbTiDbOXVJgiwRz9HPyHw2mUcwuHd7ZDRK843vtXKM9jZwG1vvy/5BAxuJv
ZZ8K/nzn+yzV3yRQr6p13SYxjVx2e07cWSGBQ7xlvcn2aXRjiainSyTBkRKm
VZu/U2jpj7GfJIEEs02VvFfWp9D75tkItjkSuNPy+dxZnkJzTkHmI9MkCLw+
J5QxOIXEnkt1RY/j/cAZvwc//5tClrNnj17sJ4GdnUThv4IpZFLx6vyVPhII
+A0PMOVOIQte00vXe0mQU6B2gidlCn2SZ+G82433Cwd3jmtGTqHJV2eCy9tJ
0L38WuWl8xR6NyS8fq+BBGyJn9YVOKeQXUnRPofPJLCxN2UYYJ1Cl0WaL0R9
IkGWAJHdb+cUYnRr0vtUSQKVnP2SH/5NopYbygy8H0lwveyG3UniJNId9uhf
KyNBQ5totQSaRPK2OkmzRSQIoX8eefjqJCIkWp3yfEuCtnqJV5UXJ9H7SL2M
ygwSHIr89tbabhI58dhU7cAuYSd9eWk6iX48sKNPSyPBOK8eea/KJKoZTksc
ScbXV9hmsYtjEnXS/9XweU2CX+7Xef9+mkBflyvoOKNJYGEulPv7wwTq844h
BD4lQZnisMxKyQQK9Mpwmn1CgluMBrpz2RNIJOHMgU9RJCClSt7uiJtAKqU9
X90ek4DYNd+Q6TaBzie3Bi+EkWBEwdXb6PAEWgssKfIJIoEyv8A23f0TKIBm
5dSvQBIk7hh4fJZ9Ao28kA70wb7QpZN6mnECfbAc1PUPIEHfdbEmwdVxZPSc
sBLhT4KOlDn+343j6KNNU3WDNwlqGa40p9wZR1k/XxuNeJBgZ2xPaqn3OLrH
87vXB9uQW9O73msc/b5WKMaO3XtSiJd0ZRy9mnOU1nHH8eow6Qlm46jrmf74
VzcSMFfa7x2RHEeCzES2vqskMNZsnSOLjCN6dqnGIOy4dpXP9ILjiG5XykVh
7MNz3JelDo4jOVM9GV8XEkhzDn0IoB9HcXpSfsJX8PEeVhcOjYyhP6za7LXO
+Pg/dUeP9o8hbxJ5/C52X6gc3ZmuMXS3MztAAdsxcV++a8MYmpSY2yy+iPfH
P7r+VpSNoQySRWSOEz5e8Fya1dMxdJ33DF2ZAwn6e3UIz1XHUNd0sM1+WxJk
Mr95o6I4hpzii9T6bfDxQDGdlR1DnYxfjRKwd2a9/iIvMYaSera6ebEVveef
DXKOoatN2vvFL5BAQYwrUL1nFOU9r2iF8ySoSksROS8xipzuJtxWNCeB9U6d
2Fy+UXS+oPPFTzMSULzImxuco8hrdiOyEFtATb3jDc0ociAo7xTEDhmdDJjq
HkFsmjfqGE1JoH1YpMPj3gjqO7Z4ptMYx/Pz/ICw7mGkR+DnfKKP9/tnXk4+
aRxG26ZSA89hG5ODdeOrhpHwfTcpTuxWfasD2fnDqG7tqEuSHglatm8v/RE6
jP47FOdRrEuCRi+reZZTw6isYGVpTpsEj3jPmHBKDqPcHL1rxdi6zVIV3ALD
SCFe5Kw/doPo9nDp3cNI/k0F2y7s+tF8AeOZIUSb7aogp4Xjy2C79bMXQ+j+
EVmBtxokCNtY/JoQOYSW8qrZb2Nr5vSJZAQPIZWnV1w1sL/TFfwsdR9CYr7J
BrNnSVBTaRXTozmEHp5OGTiK/U2soP7gr0GUm+/yvVOdBJ/pz8ulWA6iP3Xn
Si6rkiCSbpdDh/4gir0daK+KbbO9KoJOfRDl3V2+xYW9SSM24CI5iArLH3W2
Ap6vzd8Bx7cNIhAbUATsMsrLb99yB9BDu6+vj6rg8ZL151eTB1BdMvNJVmxz
Es1ekecDqNVfindZmQSriy6XHt8dQFG7+hcLsU8S5BnNzAbQ9j0sarLYRaN9
BtNb/cgz2gj0lUhwbyTSe/9KP+IbTNA4hm00rJqsM9ePmMMPP9mLvTSQTcrv
6Ecbwqq4VOH+r8f3mXdWPxIKKw26iZ3TdKB/x7l+FF3u9T1HgQS+jU3bFDT7
keqFN2Ex2NoN98TdFPtRK4kjwA97rm7uTptQP6IcsKfRxRauLud59acPnbA5
aLF4mgRrVW5ajct9qNQ9qKwX+/tXPs+tqT7kwmAkWI3t/CWiyrG1D+2QmXF/
jS3zWYn4rKYPtZY7aoRhb/9EYq+t6EMiwo2nbmB3fMxQ/F3Qh+6ubKrYYadV
WDmLZ/Shsy/bz+ti7x4Tv35+qxv9vQRs9SdwPLpHC91e6UYTHXmW+djRWz+H
owndiM2ZmB+LzX6oyrC+uxudsR6MsMXeZ2F5TK6gG/277M+wepwE3E0PVvba
dyP/h4eij2BLlg/5tX3tQgyWivKvj5JgUVP9xEJZFzo2w/QkGLugO4u4I78L
Cfh+3nLFPrZy4wK86kIqir/FlbBlpJmUC7y6kJbdp8+j0nh9Z8jSRAp0ocD9
95oksHWjn4RphXSiLgu7vE5JXD88P9NE+XWi6y6/j3zGXjBa8O9w70QPORJq
M7GjWHU9bM93op8qy6H+2G1P6K1uSnciHpEwtiPYllEBYsl9HUiGn/HkPQkS
iF7PS59q7kBRLRPpbti/9Qe4xas7EJ38a/3z2K9Z5Nnf53egVa04nZPYI49X
/jQEd6Dl/86PE8XxfD1ybVqT7ED39p8Ivoot6/pKU0mgA73Skr1sjc2gW//1
/v4OdOT17jA97Cwm4fcstB1oUvmprzQ2MWI8SaC3HbGn9wv+EiNBpQvbAZem
djTMlDxOwI7Uhtj8qnb0vqq9fwhbijEpTP5dO+qpbPCtxvYMt/IwvN+O1jcP
uT3DVrsSTnjm3Y5UWW1/h2Oza5Vd7L/WjqRS0z4HYE8ITw/zOLajuJeo0gv7
P4a9Vs4W7UhjMIxyGTtkRr0jR68d7TdqdLTBNqv10l9WbUc8aZvypthCmSm1
MqfakY3H+3Vd7LXQVlV/iXbkFmK2qI5dd+nvR8SHr2cQw6aI/VJDSpZ+XzsK
kxV0OImtQP9YLJqmHYXsPeJzBDswRJM9s7sNBfXliDJie+d+iEhrbEOWVW9q
t2G7t4n+e1PVhp6dvX9rQxTvhw4zL8Tnt6EAZsefS9gqpa3fH4a2oZ6TP1R7
sE8NqCqF3WlDyREuHG3Y0jQlJcFebcjY+tXYD2we/bjkO7ZtKP/SNofP2H8n
rfyuy7Yh7vei2enYa0wNy1cl2tDqwxi6JOylo4qXL/O3IfkvRzVfYM+Yvxty
2t+G5FosXZ9ij9zhNrXf1YY0Ttm5R2D3pD75cWF7Gyq4EW90Hzvx6b+7RZRW
9PU3wwFf7MjBcbeckVZUPsSGrmMHiNScT2toRdKfDdQvYrvdzNRMKGtFB5W1
062wbVDEybi0VuT979GkAbaSpcGuMP9W5NHLwSiPvRnW8/WqeCuqyLJhYcZe
6CjPd9rXip7vro79J0KCIZ6E1xdoW1Fpay3dKvanMvtbBgMtKHehrXgIO3Bm
TuT4oxY06HzwQj72X42NJ78JzciV8OOMAfY/Oj7b8Jwm9KzdtTNBGOeTlbjm
zmdNqPNGVF40Ns84kwpfUBNa5t0dG4qt8nmFp8KkCRkHKj3ywA66XTdG3GhE
HxzbRc5ib067OxsaNiLpis1DJCESrNd+vsa58gPx2b+UMsNmfH9iyHH4B3ph
s21BF/tAepZ+Qd0PtJ6+WqaGLX83Rkor8Qfi1PridRTbV+7Ssq/GD9RmU3mb
GXsti+XmUHw9uvJB+8n3IyRYibjgn65UhzyfeAupYL/cWbjHXKQOrZnlCp3C
VoqkzWFgr0M2gz+OS2OHPM3pvTpbi6Qs89x5sPe++CVzLLYWZVqw3/8niPNV
+rPlT7PfEaVD/XQNdt+R2TD3ju9IU8ia/TN2QKYCD9/n70jE+gV3GXZNzrj+
g9jv6Jj2NckcbLOio7k6yt/RtVmdpafYtz43OvfE1qBznTI37LHXyIpGTIQq
VGzwYO2VAK4n/zhGRburUGdLes8z7MFdC+7aVVUogH+5LAr7q1jik/BXVYhO
mOB7H/uR42YzvW4VstmXzHgVm6f9kz5t3lf0ffZ9hDz2E4c7cWv9X1DzFdv6
IX4S3AyQKJUt+YLu/v4o2ott9XKw89ajL6htJ3dkO7ZAmyLHisIX5JDedKEW
uww2Y5YTPqMIqXnuIuwRnsCnc7afUBipWCgE++hw0KOB8QrkP9lbJYVtpmxW
/LWsAvmWPt4SwfZNEuvLfFyBMpaFlASwv9l2Cd2UrUAfmXd920f9/ohYFfPD
csSVn0r6x4e/P9r1S+nYB7QmXN3XgZ0IuTyC9B+QrwKjbDN2VfJdDaaBMuSn
PyZVh73TQfxZ94MyZDwcIlFJ/f7YXWn33veog+njtXTsr+Pil5KDShGTblb2
bewpNZrIUNNSdCA6XdQTmymtu+SaaCn6VJvzwRXbxOkejXzHf4j3Ry+9A/X7
E90JbUL/oTq/Lh89bMYzed/K/pSgPwkKoZrYUun3CImtJchPtiRLDdv7ooTc
Vb8SVBzTLy2PnVBNY2dkWIIOnmJ+fpI6HsGeEFnBEiTCacJxlHr9qXsd25qL
0ST7yXxhbOmzFuuzqcWowe7tvAB1PBkSfC3excg8e/YML/V8zj3XX/MVoz9e
w2oHqOeryYu797MITUXHzO6ljvfI/crLDUWIY35f1h7q+KYlmE7eKkITT17c
ZKaeT2PbsYM6RcjUJ+YeI3W8b3ss/nEXoew+uyx67KBGWp34p4Uo+8wNdRrs
usf5Ly6eLkT8C+VlW7y4n9S3mjo6WYAE9DrPbmCnN+XfrT9VgKxcQwrWsIW/
uBbqz+Sj4X1F+avY2YUio3gPjSheDtUU7ILYVJX+1XdI+5TxiWXs4yF27rYZ
71AzX/TjRezS24fejJu+Q5c+mW/MY3+0er5FKMlDZkw3eQjYyromUu5OeWhB
sbF9FvurIqstZU8eOiyQmzCDfUaqKdK7KheFbLIGTGPX8jz89MczF9Uxb92Y
wtZh01wI4sOfR3vdm8Ru2rb98Pa2HGTbeD1jAttoBemF381B+jdoRsexO6YC
7jAfzUHLl04fo9q853Te05FstC3gT/wYdl/d2gDHk2x0v/TMPqovVPy386Vy
NvJi5cwbxR7J9VQ4vJiFWiPdLah2TJRyTUnMQkmSxgeonooivjqin4X6lypI
I9hX7mb9yN7IRCGdeaNUEz2d1yXzMtGbAaFpqq878YsVW2ciETrRbdTjSaYj
VrI7MxGPcclJqm9qJERUVLxFoV9qAqheO2VVrnz1LbI2sh+i2k+Uc67qwFuU
TBtsQh3v5sGO/Zr1GcikT3KU6rvMT7UafDLQrTb7YOr90m7p+RiKZCCGeS4l
6vMJXWLM6uhJR/SidizU58c49r3HIiwd3Q4XW6H6cXsww6BsOjJlCiRRn/fu
ajhlP52GkvOt6KjzE1O6eWkyLg1duPVJmjp/ezMrnl85m4ZCbbM95rC5Hp78
6ZGeioTjD8hS5z/Jn3Rk1SQVJc0mVixg87nlm/nSpqIQ6xyzJWr82bqGbBan
oL5lZQYSNf6MRErvOaagxCzrFjJ2jurUJN2eFNR6/9+7FWwrZ3UR9vvJyDNI
Mu0ntp3WpzCusTfoftJ4/i9sZ/FTMwLwBhUP/kz6g+26q0hD4k0SYkzfPkuN
f0+S2NuTW4loWHvyzDa8PgLKuJ3PViYg49HZE9T1FKdOP3FJ7hXypC39uA+7
2rprOPtoHDL0r90ni+06oH5rIOYZsnym0EbND+zWxTtZVmNR+eSvECVsp/NP
5DzKY1Curd/AGeztRdEC14KfoEuHa1lMqOu38LpRi2YU0rAo8THHTivQDTjO
HIlapFQ6rLDV8+l6fj17iLYR7I3tsWPfjdNesIpAcJnZywl7PO/L0S+Hw5Fh
6f07l7CP5SXY8I+HorVkA3cX7Hu5vg9D3oYgurV1rWvYbTnmZbNXHyAut7O8
17F5c05M6koHo/D8yA/u2B7ZrGwFlHtIvXKXLDWfBhr1+wm430U6z+vivbAP
Gz7rnFsLQG8UP47ewP6kbyBVeNcfQcE0yy3sC3o7wm8z+iKpRR1ean7+o1M1
phhzG139PbPPG/uV9h0FWq6biNz6aYVqeS3ZuPo0T1R7s/G9D3bh45FBbks3
NDfJbuuLvRRh8jTX5Qqy5HtGoFoyrFZdzt8JcYjpX/DD7mG4vRC+/wI6/Fe5
hGrTbbamqQrn0I1klyWqVZc5CERvZcS5q4nVH/ulwcs1u3VBoNdy2Ue1G0/P
8pFybVA3UKalumLv9j3r8hbQxKvXST2eJ2pX9/JOB0j/EvOQ6rGnr/gvrjtD
rdgeIapTY4Xce2avgpzjj7fU8Tk9L/6o0+MOP50/MlPN3vTmgd2EF9DITllQ
7+9m85TYy/O3wLJdM5x6/10t4m3tbd6wdXoikfq8ZNo8vZm1/WDJvSye+jyf
t5cd1kB3QMqz2ucm9lrH5regU0HwQZlJmfr8JVxTZYYs7kFMb/gsdb6GXGt2
+MXehzJl8PHAjrw2O8DZEgxinkcXqPOt5LazoIQpBObcrTTdsBfdpO4baYSC
3IG6/qvYb64bmy3cCwPv3pMrl7EN3W+KPPwUDqyryqsXsf+6v/gj9DsCZFOt
R6j1tMCjovnbyUfAy/6vxBab1evfzc3cSDjAUspvSa1nXgJar2aiIG6j4Isp
tucNDa5TAk8h7w6DvjH2XIXG2IemaAj7k6ysg61T+TiMfOYZdEYavlXAzhCP
6BRcfgYcezmSTlGv/yqEz+J1HJhsGEafwC7xDfz4kfQcCCNvPMWxuU55LD1I
egleBZ/0ubBvvb2mWKb9CiIJ582o67dlr0vE3MorILqbXGbHfrDiIGCgmwBR
BnVFO7FHnGw97v5MAKfUzQUGanx2nP9UnJIISTL/FLZT15+aBdOUXhI8CMlL
/Ifzx0KRicW+X0lgt3uEYxNbk88oXTvtDfgY3Uz7jZ3yVI/kb5AMf7litaj1
9M9fLeX838nA8jhFglpPxeUEvi+fSwFKiIQYtZ72uqfMR2SlQCHjSVVq/gvJ
5GEX2EoBzX0fPKn5cZTzkL1ZdipstMjwUPNppMGr0MWtVODyl06g1lOF0P3v
wkzTgL0w4zg1/85+et7Bm5MGE9fjx6n5Om6VY738bxo8KmPIo9ZTNclYXhOz
dGB+SImi5veli2ya8znp4Dxt/Yia/xMSnriF/EuH542QQq0X2p0sz7jNMyBZ
MbWVWk9+7nxcUZabAfdl7x6gOk2dacyI5i0oVvT7U+uPkX84A8H8LbTUFq1R
69VmMb1UcN5bkLeji6Q6h/DA9NC2TAi426VAtQU/rX+pRSbEi/Nvp5ru/L0U
g3eZ8NieOEWtl8XRf2tntmWBJc+xMart6gMW71pmweZlyirVLDQbHAfzs8Dv
9HFB6vEVcn4KJbTZ0PJ8wZXqyx6/HPSssmHzzpEmqjmybodP5WfDtoU+Tep4
v46s5Aduz4GxaaZBqq/vu9G173wOvLj6IYJ6f1yGpD+FBTlAK7ZoRH0edaHu
/Dp0uUDLX32M+rxufV7QmjifC6tqnBLU58n/09X9TmEu7FOZVKE+7xZJQtxe
+jzIqZN0o87HHecrlfnWebAQTS6h1k/RxOlxzaI84NU/zkGdz2DmCWm/C+/A
ypGPj1ovj55xMGcvfgdDvK1N1P3VkP/InTyGfDj36lcsdf91ijhQP1ycDzvD
Sq9R6+Mk//ll7x0F0GqrHUStj9HneznZbAvgQYVZLrU+ztd3OqkzFsLH792G
1Pi1a4UsQatCUHyikviXul/qzpunyy6E4fKuf9T9ow1Z7eOkYBH8kItgoq6P
F6s+Nl16RfDfrreF1HrZ+iv/b83NIqDRWbpG3Z+q/z14JrO6CHas0IlQ968B
24ynX8wXgeNkuxB1f/ueLiw8nKMYHJQaFPZjizJTGl0uFkOUhFIWNzWf7hZ1
P/+4GGzOvP/LR+0H9tix6f5XDBx1Vm5HqPnmQIOZxPYSSB+r/CGBrX2I5tch
iRK4+2h1irofD+Y59YrFtAQoL504Zaj580ja8GJaCTjX+f5Hrc9Dx/0vF6n/
B8fefus0xM7RE/eVDS6FotPePNT8PGHowCWcWwqio7s07lDrn8mLT/s6SuGm
r7rPPexoK9ptv/nfg77r4c1H1P33pYGHH6vew5fQeu8U6v3ffZQA2z4AxTMv
p4l6vRLiF517FRCZJ17Oh/ulHq+N88/yKuD6Hum7wthtx1h+DvVUgIE2j44k
dk2BlISn1Eforf4+LYedn+sZ/2LgI9RJyaobYQelrXlMynyC/3YvhwZh88bS
8gcSvsD3So+NEexi+Sg1K7FvwL/x9ns+7h9f08nNxat8g5CZbqNS7OC2sSe9
pt9g58hI30dsMxdcToK+wXr55Zk67N/xg97mnd/gc2zF7Di2yi+xdyaB1ZB3
8Iw5B+53Rao7TWJjq+HDnKXgQWy2p4Hr7VnVEEPYQeIRpL6vatc811ENDy7+
CBfHDrXwHTcSqQFWt6pvathN72v3GbTXQMj1fz89sd/f9/wcOVMDvxbT/3hj
vzHgcm7aqIEACUHGQGyP6evFesLfIearmupDbHZOTn3dO9/hedjSqTTs87cu
3tcSqoWxV7Kn+rFnTtAsqPnVAePOdWHzI3g/dEPoNUdUHVx3PP3VBvtQia72
dEod/K2adHLGNj7+PCOivg7oNr9V38SuOCpu27q/HmzHq5ZisSMlzVpsyuqB
7c+ZzHbs48LZRT4rPyA1OdmS+v4j4KDx7fzrTZDRsnI8UBjHW9lEssSDJpDU
0RSKwPYyu92Q87IJZmhKhZ9hX376mjezugmsO1pMc7DP0U//eHOwGVgz71v0
YAuT/XiivzfDies7XOVEcD15wqKze7AZlH3iDc9iH5JMvhlJaoZrR1PlzmGz
XampjzjUAjT/BbNew/4ztPvmfa8WaDsrlPoGu7U+ve4GdyvQ3RdI3S5KAt3L
pyikE60wcPvdHCt2zfYfhz20W+GQksy+w9gfVZa8rt1shWKNN4dksGWG7iUS
IlrBc1fxMmAX+nHUXXnTCm6FAml62Bml8oedf7QC+dGD107U93UmjZoTI60w
oZ0+RH1f9mrZ1sthtRUmCcE/qe/THmn+Nlima4OK2gXPYGyt52oXC1jbINPh
9a5H2HRTj3yvc7XBBxZCcAz2txNdUZJCbVAZLdT1kvo+re1yea5CG0zKJezP
wl7nKWq+qoGPn7nOVoBddn19QtS4DZJDHCml2HhvvSvrUhveF4wEfcMW1ix2
So9uA2/uF/uGsfd82mBJnGmD41vCRzjEcL9bQyucQWqDYgbXqoPYKs1M8O5P
G+jzRfvyYbuO7Pf6tLsdXh1TB2lsmlmeRzUH2qEgUNJMFvvFslB6k0A7pIeX
RSlhf6OR6R461Q6rQZf99LCtmBSXplTb4b9vxGPU97NLe9R3LOq2A99ncs4F
6vtcLh2+n2btcJqSEnoJm0vQ+PSWXTscPF4Q64F9Q8MtiTu8HYTXPT+8wmY0
vFkmFNMO2/wCP2dgv7Hwb5VKaIezSt9/FmI3XInYplLYDo2Mr7dqsQUeZVyy
620H8cOO7tT3150tQ1LJIh2QqZK96iiO7693UjPreAdM3C4+6olNM0a0L1Ts
gJjRM8lB2JLk3zHIqAOYauMPJ2KHcHCujfp0QLkaXVw/toyVwRe++g7Qiz8l
7ySBr+9g1iva0QG3E64738J2uHqBdGyoAy6/FKwKw470vyqgRuoANYs2xnfY
U4mhYY4HOoGpO/zjb+zn458N01w64aWvJuWVJD4+xbzL50YnePrttCnAlrRf
PG8Q0AkPardmv2HXDB2+9PtpJyTuv3xxAXu1984dww+d4GA+16EqhfNdy+ns
P/Rd8H1L8/YC9t7K99tM07sgojjwl+lRvN+Lyy/dPtINLoLn3tw8QYL47TrZ
qxF9kDacdvvAaXw9iW4PQmwfbAuLKBLH/mriKDeS2AeFEYcpStjEVL/auqI+
GLmh+8IRW0U1d+p1Xx+8+O/V5zzsqQBmfjXRfmgYXjQxUsD9fEb8nOyJfuD+
yy90CXt3k2CRuFI/vFa0E7iDLX9ICfYa94PNyeyULOzHFW62sz79wHgoFtEq
kuDEWvOrqLp+OLaizPsNu47b2jG4vR86zmauDGDbaMyI+gz2Q632xtYKdljc
3w8Oy/2wFHnkg7ASCfpPHO05uX8AtJhDWp5Q///NI5p94PIAiHF6qlxVJkHz
nMljYYZBuLIxEGwEuH9FnDFktkEgNjmcvond87zvxadDg7C1d8Y4HntM3S7N
5PggLP9hyB7DXk28WhFkMwhFn3S4fFVxPT13b7anZBBu3fhh+FkN91MiZxZT
Pw/Cnw420Tls4b/0K271gyC5onCDQ53a3z7+SzsyCE60E9bXsM/Sx3MeZRqC
Bl2hh3xncD9XkX8m3H4I2Ndby1LOkoD81FPHxHUI3kurtnVge18+acR9ewg0
Lio70mvg++OosP7v0RD8sno4fw077nqN1+j7IXC6tCdfRRP3t/yDyXIsw8Aw
Fu5Eq00C5keMm7PlwzAdsDgwpkcCF66H9JvfhmFHVGU8tz4JanOZWNmahyHv
X/wba+zgxp0C8uPDoN34L6Qb+zfLLu0IphGIPyn0pMUA16ene+JErUcg9AGX
R68Rzg/PuSSvbowAj1/0fzRmJOATTpANpB8FV65rh85gB5UdghjWUZiNkrkS
in2697BJheAoVNedT2AyJ0HRAV5fJv1RYIuyYGazwOs/QbAmJ3EU5uokZyiW
JPhsvE/6J88Y3Mi/lRR7gQQMqzyEacEx4PEUutGIbRwvktEjOgZEW1trOhsc
n8PyXOUnxmCswvuGNzbLtQsMdzTH4O+1wcM2tiSwDk0ZoXEfg3XK9m5he5z/
RXNekW+MgcNQ6ZQD9lJjsdmEzxh8KT6sn4B9n726ofr+GIyMZ8jvccD5NXm6
LOz5GJinpQrTOpLgV4X4U5bPY3By/Y3mlhPu321P6m5VjUGoEasPXMT1gkaJ
fqkWjyc+9HwwNo+WwZ3WtjGQ+7pfb6cz7r+6PK7ETo0Bw8ehIMFLeD6XSnEv
Og6uOqanXlzB/V3M5z+Me8ZhpdqqjYgtKlv7fp1zHBQdNIxVXUhQeadXYpB3
HNJpozcXsScY/+x/c3IcBFhZJ0xdcf0QBJLghXFQ0Omu0L5OAgr3xYa7DuPg
9vjRjf+wSw+EZwxeGofAiG4+Xne8Xne3Wj3zHIcku1uW69iq63ZVtGHj8FS8
cavcE/dHrUGx40Xj8PJJ2QWHW7jfuPNFJplhAj6+iBI2CSCBtPfE7g3mCWDx
NE3pwyZ5MhAs9kxAVkz0T/tA3G9cNkzafXgCTL1mjnkF4f3HuRH6oOMT8HRD
fizpHs4PIn97bGwmQFKywVsoFO9vO5X9uEom4MS8ElvLE1x/8lS7uT5MQK3H
2tTFpzgeHpw5fujTBBDlmQz+YDud0CEcqp2ALcOINtEYvF5jzM5zD0zAcAiL
d+QzEjwxvHaaj3YSNAQ5TZ1eUn8/8nJdyGQSZnQc2W6k4fhNTTAXtpyEdNkv
Y/vTcf/j+6ZY2GYSJrdXa37BlhDNuCpyeRJqiVE9LG9xvIUV9ov6T8KQ+bsD
xVm4HzxbWyGRNgl3/P2MN97h/hat+B2nTMI7nVumfOU4/y/t+Zn4axKa7+mN
zWOHcR/zZNyahNjJYUp5Ba5nd9wuj+6YAjk1/TbLSpwf5KdNorin4AD5V2DO
FxIkF/dIELSnILhLgb3iO16/Y6uZZoZTcDVQ+GVGLe7vWTkEvppOwdcnT/bH
1uH5dDc6EG83BQIGF9hv/sD9kXg9ncatKbhJeO1s0Iz3KxkVwynJU2AkbDx+
uRv39529Vixvp2BjeLTtZg/e39GudfrkTkF2ONPv4F68Ph1ONBi+n4LrHCWb
Wf0kkOPOK9tsmAKnz8z6u0Zw/X2R+NRqbQoYn7UEOs3i/VvEXTU2g2lYl7Te
93kD7/c4Zm44mUzDeTv/kd1bOB8n6WeUWk7DSJd4kdNfEnQVH9xx3mka+IMa
r7BtI8PAwH9NqT7TwOR261k4AxmmJOcsT6ZNw6VZAu34HjL8bjN2t/g1DQ4v
o5/1iJGB76BAQmLqDDQKJefr2JLh4dQdN53RWeDry3F4WUMG6fQhlbypWRif
dx++VkuGLkflPbuIs2Aqs1NVrR4fP/r3fdvqLMgda1onN5KhrP/+P8udc0Ci
FyJWduLrt4THXpadg+3VV+s0psjwKGrOuV5hDp4/fihlPUOGY/o6cuKqcyCq
ui/RfY4Md37sHFrUnYMFyo2aFwtkYK+JErrtMAfawvwpI6tkUK14Vv7g8RzQ
/Iu6wUtPgRmf1UfT0XMgeWyv4uEdFIg8ZW6r9WIO2i+ud3IyUaD3v320zKlz
IGE1nP2XhQLuBa/0YsvmYPyhwiXnvRRISE8eSRmfg4cS/k8MBSig6kRTTDs7
BwKBvF/WBPH5+RwfOC/MgQ/jg9VXQhQ48UZAVPTXHOwypBxuF6VA/cu3XoUs
BJjg8yT9k6bAWlQe3Rc5AnBpDKvpK1AgQmzeqkaJAN6U6oAQRQpwfRfPb1Aj
wH0TB+kPShRQ2cwx79UjgPSlY41MQIFw1+xMkj0BHjRvODmfwd9nmPuz5kyA
qFKiuttZCrxLFTHaukqAVrUXh901KNDWl/mL6RYBCLang+21KHBA6622YAQB
Uul0/X/pUiB3YipRLIoAH0LLHzfrUUAp6Aj5aCwB2hpY+ZP0KeDwPv2VUiIB
5gufJIoaUoBiPLmonkoAe0u1S1PYIQsC6jqZBKhop3i/MqJAtmAa0byIAPSG
LgrLxhS8XsdVbN4TwMXyQ8vTcxRosuZ/5vSRAM/VD3mJmVCAFJOi6FFDgOPN
cm+1TCkQLDX29PYPArDfunOwCXvvD96pOy0ESGMmOuiYUeA0TXJkRB8BCg7Z
SUub4/MljIw9GSbgOrf04jm2nRyP7PMJAvAe+hr/EzvYPWk4dYEAX8IH/6Rb
4PPtHD6eTSaA/1zJse2WFMh8ezisYI0AaoVXkqyx5dVsBko3CDA09YU9F7th
KEG6koYInLGBTmRsG9/B4Cp6IpQiF7tjVhRY4jjUW7eTCPe9NfouY98rtJZo
YSXCWdSXGofNrvf6btdeItSlzUZWUH9vPtPfOXCQCB1icte6sU8FHxQd5yGC
tG3kburv0eu5zwfMChIhTeOdNvX36NYVL9sWRYmwPn9hnoS9YNZ3ZFWKCL+t
HOpnsINI+/3+nCDCtedBxR3YbJGWzTTyRODKveNbip0mEs+/Q5kIjcmHlx9j
y1T33N6lTgTdu8KzVti1dvsaOLSIYHvO7sQhbKs/5jxc+kRo4XmY1oHvd/75
8xt854hwiXSN6S719+jHu2uFLYgw39Ioz4fN1rz3kNQFIohUX6ctw883zcXM
46QDESa798mrYsvQxVWfvkQEAmtk3hc8H7XJnftVXYnwzy9W+ji2lSKHm6YH
ERYPL8fG4/kNuBG719SPCP7HkCbg+Re7ysWrGkQEpc81XCE4Xnrs08SkHhCh
YWLPy084nh5YiMtwRRBhmv7Aszkcb8cMSlR2RBHB9LH22A7s4TMKOqsxRDjK
6PGDHcfnI4VvpuMviBDgU01iwvE7JdLhUplChD17TchtOP5jeKxvZr8lQv9N
2YQUvD5UOCcCn+cSISGPPO+kQ4GXtJRYj1Ii7GLUEkSaFNAf3vNZsJ4IYXKZ
Jp2qFFjvfFXH1kwEBq6YRFm8XjMb+Du22ong92a99aEyBWjKj8/2DhJBivI8
bgde/6Wx5/ZELhPhfbjSX/MTFHB82H/Ib5UIKUrmW2ePUWD3PUfhy+tEuGkb
LSqE84fLdS9F1e3z0Kz1NaNcjALc2jGXVvfNg0iwVpIYH45XlYOe44fmQfp8
6eVkbgr4yqb6t/DNA8fluTf0hyjQIVD8NFt8Hm6G/d5M46RA2FZbhY3KPHx0
uiyjvZMCJ1etanTOzEPNrsU2Y5wvx4hjLae05yEgvqJRmw6v5z7SJJvJPBR+
SdnP8JcMSyVsu79fmodjabtzikhkSMh5eaDEdR4qIiT6pBfJoJ3CJ5jsMQ9O
yuFDrwlkSIs6Ju/nNw+NET4pihNksHAxdpKKmgempbT3rzvI8IU7uux56Tzk
CdgfyCgkw+S1k2OLFfNw3WN70b1cMjB+7GHSRPj7dvRnzr4lg4kFt+3aj3k4
YiDucPc1GWaf5G63HJ2HlP1rjXEPyMAybCBVODUPYf6Pq8MDyXBCgmyxgzgP
QxPixx18yBBQJ5f7YXUecnb8E6xyJQMbzXfjAzsXQDUA7NSMySBr6OLvyboA
p6avbNrqkME6kTmjfu8CHBuMuGOvToYMeZNfvrwL8K9YooNNhgzyXqNJ/TIL
EDIt1yu+jwyOE+vE1/YLYCh/nyKB9wNhx5L2rjgvwOmLklNRTSR4F6Sqoue6
AEbi+ZGd1SRY4wqP2bi1AE1tmR+YSvB+13SvvM2jBVgN743viSJBSY10KPf7
BWBMuOIJuH/qZe8ouP1xAXapPL26LkuCTYfbfc1oAeKv1u+Lwf2y5lalRFDD
ApyZ+pZquY8Eg7I6HSOjC1Bso87m0rUMdFkXeVN2LsKL6Ng9gZzLsK9juiGR
dRFYuPPR7NYSiGxd8X61dxEqt513ZJpZAl1Tt+ZYnkVoG3dzcH+/BNH/bt0J
PbGI83w05wPjJThsFdbrar0Ia1Pd9q0+iyD9gPHBFftF0Lbj59S8sAhQ8Eja
2XkRZoyO/LqisghOdE9Dbd0XIYFO0jp3+yJkF8WfNA5ehIFodV1OfN8yTDlP
ZXMXwfXQY/Xx0HnQlJFQPFG4CA22dPn/XZwHS/v8GenSRThdmZYlpDYP/u+L
lUW/LMLwUfHL03+I8NXx4zxXxyJ07tpF1+FCBP2PjVo0fxZBL688Xus4AS5e
W/rXoLMEC6Kcp/JvzYCM5wMhG6MlqD8kH9cHM8Bw+4DeotkSSKi41AXunIHc
IPUXrA5LIPrbn07mzTRQYl5ImnkvQYavg92nL1Nwr1zl/HDaEqzZzrJ2T0+A
yefOIPfsJRiwqw8+lz0BR765ZPzLXwLbb2oHNVwnoK4xdpm/YglCM1/YxCyM
w+7RmdDLrUuQF/9rsZI4Bon0T0uWN5bA88H+R1Gvh8F955G+e9uWQd0EcVnu
HgZV1oqtPTuWgT3a3ojl/hBMHpjUlmFfBrkzlRvyFwdBXFJu1Fd0GfhIStUv
uPqh3GSUmdZ8GQzyzfjAvAseWd46/sx6GbinbUQLP3SCjQ2T5RGHZciVFTrO
eLAT/l06maZ5bRkeM4sm3uprB02/cPnH95cBTWfdco5rgQNBh+0Ohy+D+X+3
zBu1moEYXPwgP3IZhGwexm1bb4SoyMGW1vhliN67NDhg9gPsYjxXHZKWIWG3
p2Lgvzo49oKei5K2DA/Kk7IWs2qBNuE1PMhehs872JokDL/D//9eGO79/++F
/wfgjBe5
      "]], LineBox[CompressedData["
1:eJwt2nc8Vu/7AHCrQaQhnxJCyyptilxGsiIzKlJSkhJSUaKQUUbICGVWRmRH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      "]], LineBox[CompressedData["
1:eJwt2nc81d8fB3AZZSVJRfYIGRUNJXkbJSQiZBS+KIWIIjIqikLZDXuVFSE7
HCsrKyvbvTbhuveGSPodj8fvnzyeD5/u+YzzeZ/X+1xCVk76N2hpaGgi8D9b
P7sTjtzgzCGhnIavFa5O6fBn53+EAxkkZF5GVuGcyQBhr3AzoVQSmrMzOX/T
MgvuGi/rHY4moe7JgG6RyznAsvuLkoY/CT2UCzzyb28+qPqqc3mZk5CHlmqF
91wx5Npc/zbORkISQWN80WXV8PJQkOxnp0VUeYen/BZDK6j2sMsH7ltA5nTs
ffXCAzB6frROufsn+rwr/3c+3xhUkV5Xab6dQ1ICJv0QPQVxX9yXA1Vm0XnT
O/lO/XNAuHHHPUhxFsXp/uoXnpkDUXarP0Hys+gEb+/L1pU5yLS5uO2lzCzy
tDeJYeL8CUU7+XeGcM8ioeOH9Hl1fkKbRa1oBHkG/dgAw49VP+EfHZtBTNIM
OsfqNfgkYR5Uc2h/xMTOILXhQyJns+fB33jVJPbNDIpSeN5GKpsH1myCRdzL
GbTG6fZdoXceuIw+2yd4zKBfKZRdNqwLcDTd2C9Fbwal0uvnTLstwH/aKflZ
22ZQb2KVbY/aIkgneO1T2ZhGhm9vJsroLcIq2fBh78o0+viYSrIzX4SXbxhV
t81PI/MMK+ky90UoHbvz3ah3Gg16XqTP+bgIuz1OLW3LmkZWMkvetBwkqH7f
KmNsMI1WvzUuS3WRIPj3h7AFnWmUSpJVmhwhwdWLj5d9NaeRu44kS8QcCRaW
5CqylaZRHK2P8BTNEnArvtWmOzSNTMS+Dg7sXwLnTiv7nL9TaHG1TrPg3BII
/VtNp0+bQsYWauMK0UvQufLc6HPSFJodqrpyKWEJfBe56a1ip5CnrauHaeoS
jA2fsUBhU6iwR2fSJmcJkssf7X3oNYWOueqdU6hZAuGHO3xJelMovZBRe3pm
CUSW95kNbEyi38fAT0eODN3zaYwvVifRRa4/gSUnyfB04lSRPGUSfXyyocF3
hgwTXaa7o6YnEZ9a8USvGhlS8+MbdDsnUQaf5TNmQzKI3hU7/jVtEnly8Va6
uJHh4M8TO3P1J9HiNQYX1UIyzDDMnCq7NIk+UEsiqkvI8FEw2qZOYxLJrKXT
K5aT4bjR5pc+pUl00JHSIVxLBtXqerttUpNIrm3/RnEHGSzfXG3Up5tEvAMc
E//myBCt5uG3/HkCnT+9AOs8FDA3l/r0L2cCkT/GhxP4KSDsMTzAlDmB2lM8
RWuEKJCRoyLHnziButM0mB6IU6CEm5moHjyBaMaEl9LkKNBDilZ6azOByqfc
1sU0KMAeW76msHcCyWhYp/g6UeCaxZXtA7smUKaXW+SGMwXShOc4PJgn0KRM
uq7zfQooZeyXLv43jpSmZSwuelDgTpGL+bG5ccQ5IOdT60uB5g6JWik0jowm
U950RFLgKUNUMO/tcbRwwsZuspACHY1S775Y498raHcMFFOAJ7jmvan5OHpk
3hbZUkqBfI6lyrdXxlG4KqvW+woKEAUukjmVxpGF0PU+qa94fIVtRjv3jKMK
t0aesC4KrDreEdgsH0N7GgJ/zy5QwMjwYObv4jEkv2dZ2oxEgaIzw8ep+WPo
doBacNMSBe4zXtKaSR9DaT+Sm+OoFFhKknbtjBxDOu/plUXXKDDX/bP5g8MY
umQtNJBORwXNLymGSbZjqELdMmmengrpSWaEGKsxRGUfDJHZTgVbx2/LocZj
yFIzmviBkQoTO7IEPc+Nob8/+vQe7KTCiIKdmy7vGJpOXaoP30eFs0LC27T2
j6H7DxusMvZTIXbHQNA5Dnx871+eCi4qmHVrJp1mHENyG6Zjgweo0HfnUIvI
LyJS7Y9IWuGnQmfijNDvb0S0mzT1L0GMCrIBiR8p9USkY3/pV4A4FULvGMsv
VBORzDfTZQcJKugqNGqPFRPRP1ZF7sOSVGjpSnvQkkJEHJ49zcEyVKjfbtua
6ElEwsWi5kXHqMAc3ptU4EZEhFFbY7vjVNDhU3drdCaiPA3t+zwnqPDj2EGB
JVsiCvmkv9v1JBVmLMfvggERCbxvHmM4TQXpef3zV3SJyC4peD4Z2/lBDbet
FhHVBqRwnVWgwvrLpJpXykQUM32o79YZKrB8seAckSaizvqdq3FnqXBZvX2G
LE5ED6Z+7hVVokLkd6UKBhEiEonuvJqGzTvDd1OGm4hehe85/x6oYHnvpYIK
JxG1VEcvCShT4f3mBpvhLiJiM32K3mAf3jtU7MVARLHRfQ0+KlRwSbwYHEpD
RDdl7m5bxC6S+mKZ+oeAjm7WmpuqUgFUo5layAR044LrdzE1fD5OxmY8IwS0
smo3HH4On896w5Ej/QSUbVW+OoXd90yeXq0bH3/9qdmp81T4L3Zftl0zATGI
GnJ3YX8Q93/i85WA/B5rP+ZVp8Jc/rJheBUBybx6o2WN7dLUvVlahD3CIDKL
XWRwrqs1j4C48vYrSVzA92v0cxrxIwGZGJn22mD7rYRfZkohoO2ndd/2Yjc+
oT3IF4/H27HUxqJBBdadLmtH3xHQs8b4WCXsSBG9ZOMQApK8QLc/Drsvp8rN
IZCAJiPTOhqxeRWOXnz8jIA0V0+eoWBbfk0QiHxMQIlFEte5NfH9vLzrV5on
AZ3g+aWohD036NP4xY2A9O6wEy2xD9suxrY7E5Dy8i/dJ9hF3q3nV20JCDVQ
075grzOePcBiTUATyRtxPdgQ+XGR35yA6Hsy3Rewnwrw1sqZEJDDB1clOi18
PZlBb9QNCMjGont5HzbryT/2proEVNO0aSOGfbnaTtlRi4CG9k7XyWJHag9w
+p4noL1rE3JnsPt/aM5GKRNQibXDF1XsDyzx8UpnCKjMWeW7BrYLUK5MnyCg
5hNizdrYSvfUmUKP4vGyaIt0sJnToitPSREQXVtl6pZ7BxbvEQ8SkMg71Q9b
xyezqR0KFCQgYHRr3vo8R9U3I3I8BNS1T0VIDfuM28+Iwb0EZFYVVLZ1PoyZ
oPmUnYCu0Ch9kMPuHo7YlGYhoJOd8EscO3H3zOceBgJ6LOtRwIPtcF7xtg8N
AeVwd67sxL5LETyrszKKFPzlvm3i+8Vk2pSW8HMUJbr4qixiKxw64K3aO4p+
loReacLuDK2Zjvg2ihqYBHYVYtv/ttOfrBpFDo2KHxOwYxu+iD/PGkXbvOub
nbE3b1zvbH08iojnXIxYsKuTE8VNpEaR+vLaDkM8H0yZNcMzBUdR/cSucWVs
ijN548/eUTQr5HdCGltYRbUznmYUDbxdOr4N++nouNdEzwjKpiNrp+L5yXsh
ePrEtxH0am49LAi7MOe4fkDVCIpUV5Z3wZ7yfiYumTWCOpznX5/F1uAV73R6
PIKuCHSbtOD3geDXrljtOoI+nK7VzcH2+PkgjcN+BMUvfXsVgp1Z1uhVaDiC
Lmi8G9LFZjWxE/8jOYK+SMjcbMDvX2oVR7i2IB4/n3IlBfusxJeNuL0jKJAm
954PtuMqS6cyzQgi3+GeOozdEZXt5d8zjOyW38Q8w+93qNrb8VffhtFlRQfF
a9iXyb5ab6qH0ZXYveJy2O3axlzp2cMoN2i6fxDXizY6uoKmZ8NoxjqgcKt+
fHM2/sl6chgtDOSei8b1JlBATX+v9DD6Y9/lewNbq1WmlE94GMl9aGY+it0s
QRdwmG0Y8a2f/F2D61fjaLbw5akhRHdtuyYB17v6S3SmEa+HkKoi76F1XC/9
/yxUxQQPoZ8V71XKsNUz+sRTfYfQmtCbJA/sr/Q5ywWOQ4hZ+ujJFUUq1H0x
DutVH0I3/PLSpnD9rTmU08i9OoguHUi5morrdQWDiXzi1UG0Rz7cNRzX/2D6
nZad2oNIKVrxxzHsa3TVz+lVBxGxT/FSN14vNmgODdySHkRF4e13ObAVNn57
yW4bRLkJcnf9ZHE9oLytqckcQOlXIwXkD1Mhd7Tv0uTffnRCf5amGq9fj0eC
3fZT+1G95vZjSti6w8oJmjP9OE+rPy07SIXFgfSl7M5+JFmgHZ8riteXXvcI
t7R+JCV23jZCmAoZLVz9O/T60anV23Sn8fqYXGpsI5nahyo9WmvP7KUCG0Hy
jsnfHpSkrPTTn0IBrdBX/heediEFtwUyXyfOH3craF56dCE6lz7GiO8UmNed
f9jp2IXiozpkGbFf7tJyum7ShZZtWcso7Ti/vGIwvne4C9HX5qQ2tlLg6kuv
Qwl9nUgjyXfuQRMFbALtWlakOxHnu9C/E1UUOGH3Tl1RuBNN0HS7XMPertVY
9WR/Jzorc2q9C+F8xSRWyErbiaz4+KC2EueP58Q44R/f0SNL54TkcgrcDTB2
0nnyHS28o8uyKaGA91N1jg89HUho+sultk8UcMssfp78rQNdV47y0sF27JD4
F1/dgYxsezfbcnBe5GWZf5PdgbZLNu35no3zU0H71xfPOpBI178DA1kU2Bw3
9rhzogNZK0h70qdTYMO/t+q2ZDvqsi5iEErC96OzJNtqXzua76ktL0ukwBB/
TLQZbTs6y6PPZIhdXmRx/9JAGyrOG9YPSsDnNzUjLhvYhrR2HpLfjMOff/7P
q9+zrWjc/XLVWjQF/tELXg/IaEG37WVcOKIowEaNbO2KaEF7nzYYZOO8yE9k
UhL0aUEBN7z8NLCVKqj8pfot6PmZo18fRVDAx7WBMPfnG7qhEtlPDsPnO+lo
o6PzDdUJLJ6beEWBtfoK+73UJpQ6doG67QUFGAvlhv4bbkLFu3uufHhOAa6U
NO2chib0piqe5iL2qUdhMhdim1CdxV+lqAAKuMvfILmfb0KMlEe6Mv4UWElj
vTf0phG1/rhWZ+9HAepzs4cpig3IbNP7vrAPBd4yf9ptKN6AdgRwzXz3poBi
MG3Gdo4GdCVV8v0T7KchGT9uT9ejnH9v/xC9KMD5evX40fB6FL7Qw/XBkwLH
UyJI5dNf0Z4Lr9gUcZ6+X/HNpje8Dv1o+vU0HudtLhDYCPCuQ4pB+lxXscur
XCJO36pDiCX71y5s+jrumtizdeim99XnvvcoENlsK2A9U4tYAr147F0oUPiD
boB0thapPxpY0ryLr4d8Rpdptho9WhrhVXKgwMy/PaMSPdVI18punQ57cOe8
o0Z1NUp/MLu/2Z4CVYdiXwW8q0YRMkMcV7ED/9toZdCqRgWSF5ju2+Hn9b1c
mzarCk0Et/0tvUWBV5aekSv9legIwfm22w0K3POSKjiRX4kC8oSunMM2fjvY
dT+wEl0KFffmwBbuOLOHqlCJBN1OhH+ywXkdNsJIMRVoesykat6aAiP83iEz
18vRsvJX6XtWFDgy7BM4QCxFp3y1NXMtKGBw1iCvqqgUpWXeKvLFdo871Pch
qBSxePA7GWLXXO8+eO9EKSJzG9b+McfHjxyqZnlRgnbME1YuYruPdq8qHi1G
le3bA9evUSAWMvlFGIrRNtfM3O/Y1QmPzjMNFCGVhm2iGdjMlpIRPX5FKOyv
6YLx1vGER4cdfxQikpUjc7kZvl9EyRsJPgUoJrbNPtSUAhMqNMHPrhSg3N/3
LzpgMyX35NtLFCBtAT83DWx9q8c0pzo/I5krLTnbto4f64npOPgZ7d3RlPbA
BB8/8bhzW2seemHC9cXZmAIpLdmPGk/moFVmS9t4IwqIVdp90p7KRuY7PF2e
Yad/Eh/Faxiy+1Fd6YCdE56k1P/rI2K0/XJAEbvMOOrvbH4WCr1fOjpsSIHO
CS9PliMZ6GCXnKsMtmHv6ayQkXSkeDHDlQu7r2FlYM+rdJQ3dKGYDnsk864C
70IaKnF6sjlggOvVXZs16awPaE3PLzEYm/bvxQc64qmobzJDd+MKBZ4tMqZ1
9qYgUSl+l5/YjISvvUb+KaiA+9uPAWy2WjhpMZmM4vZduVOGfeDFsWWnlCRk
EVRS54Ud93BJ9Jd+EkrZF0LviC3okG3gTpuE+CM+PLfAFtMVL3j8XyLK9+M1
V8O2kTw5JQzxqNPR/jobtt3O3PNS8XFovMJnehv23aVD74/9jUV0q95Fy/oU
8Criszn3JQatXyMJj2BHqjKM3ZB/h2Icbx3Lx44++EjF8fVbRH36fiwNO5Fx
LdFt+Q1yXpLujcP+2DZvEfD5NbLfaHoRiF1r2j2cfiQSCVjsELLFthtQvT8Q
FoGY2aa5LLA5TPOYWX+FozX1Nr2r2FYmr+SdSsLQQXlbfw1s5v6N1sQDYejR
COWJCna+sb1Np1co4lo6XamATZcbKmzv+wpdkMv5LYOt/+mObpv6S7SuX7gi
jp2co+UlyxKMxCulTghjq2bT965GvECHDQ7c248d/pFIa2b8HIk6ObhwYBOz
Ko9U8gagjgJi3k7sx5nuL56+f4p8E2eX6LE7MgyLpm/7oflDTydosAUy5Ma1
DvsipzMy+/7qUcApfRd7DuUx+tBaEbCG7a3b7yHs+AiFE06cXsHm1Ynomlnx
QnL3oySp2OXal2Q+PXqIAp4PmS1hm13cEeDK6I5C+VhaF7DXNasJZ8Jckc4x
IZ+f2O80PBVoD9xDqInfaRb71IUTkY3Jd9EUgS5xGvtT0Mgg31UHVOXewT6F
vfhcPyTzli3SDXtaPYEt7V+vKv/QCoVLCuaPY/dud50P2G+GAs+ljI1hX9l2
/UqSgh46ObrzypaVSXtm59zOotfrFnRbfnvp7Yr5mgh8fP2OSsR24O8liZZo
wP3PZSJbvy/lpNu9dsoIKJdrI7bM/3JnD4nZEkSdPqttjUcIeSdkvWYDzGxB
x7bOJyn8oGPv9G14L6NlPYltFZVXptnrCMutS51b58/REu9nPuYMq4s+vjPY
91onDr01uQ+5ob+c57C72yQ7vne4Ae9nvaR57OMdd91YNDzgrNFrdhJ21Pci
3vPIEzge1iAy9krnRo3PSR+I4e/N+YUtZZd0fMjoMfSrfR9exQ62nx7Y2+YL
Luw+fzaxFR2Yc/KZnkJr8ZE5Wvy8FxxknuiefwZ14vTsO7B1HO+JvygPACvH
lt3s2JuOr9cP/n4OXYWtFE7sHKfS1ppjgRDb/GfPAexdzv/ubWQGw19Pyf0H
sauchS+8m3oJ7hJ6a5LYd13OHzgpHAK7a5Z4ZLFnSs8TiltCwWpoSlgJW/NL
kD9ZLQKmgmpPm2GnSj7vEiFFwMMy3w7rrfHfPRU0io6EvIIfnx223hd377Ky
pSgwP5tq+Qj7wEmnRb+4t8C+5mSYjn3/vf2ZIo13wD3y7Mpn7DbOW89nqO/A
mUyNr8T2o1oKX9KKAdvu0/y92PO5+kb7VuOgcYKzmhHXD3VB3RSN5HjIioqU
3oedGHJx6eGlBAhKPM0oii0pL/yVpJcIdIfNPytjj+7lsTBITwK2a/c/emNr
dLFG8BmmgvQA247tuF4uMweVFmWmAq3/wdtc2MmqTARdmveQLDy+Twp7I49B
xjfrPUSfLDDQw84L3ayf2pYG0vSGhknYB3SW1j/lZED+niplXVyfG545CmnS
Z4JENu/hm4ZbeWL+wphJJoQXjNh4Y7dJz0ZyMmSBruWnnI/Yvixjhz3MPsLw
Xc0CNrw+/GzsslJl/AQJSeEvCdjm7ZAmYvwJHHb6n1nH7uzJ+kmf/gm+vq0/
xHmVAtfIKmXjIrkwyqHeoIGtusmt9qE2F6SLRrcXYu/iajaQossHUh9rwju8
fmVclHQ/4VsAHX7FdyPwejemY3lALLMAVqtJxwqxefVfl+/rLIAnn+blf2CH
GtNu+y1UCHSLirS8eH11uzHwoqy6EEouOSemY6s+CoyBbcXwEDbo2/B6PJQ/
V6n5uBTk36cGaeL1v9f5j0lEVinEdQPZDbvjKOvyUG8phDrxpqRi1+XISN2V
KQO9oZifNJYUyM68++b1QBl4KlXvqsD2SV5xGj9eDnGCEu/VcP4QCKcV8p6t
hOkXScyBOL8c0NtTXs+JgNfQaq0Km5Nd9Cq7MgKuCvvjv7EZX51/mfIawU1r
4xDbmxT4fa5uY0KoCq4zvevTtMX1QXnHbZb2KkDUgBre2/j5nnqpYnyoBog/
x2b+4DwWTS8/80apBvwrJOXO3MHPq4Pw6seVGnA1/a/uIbbBLfw6+9SAVJpn
7Tr27zeDboZdNdBt9UJq0xHn59VDH/W9a8HVmCOIw5kCLYX1+y59r4Okrk+v
nF0pMCVHM6/i0QCNtXvlmB7h+u9yMHrPywY41FYjYojNk6+lMZnYAL3bBAyT
sC/LRqU+b2yA/vk/bgqPcf07Inm9fX8j/NsxW+z0BNcTaYO2a0WNwPjM02gK
52VZsfTcB9Qm8LwzGE6H83jLjVZzzR3N0O2Z7HcV++Z7CisPTzNU3lv5nIUd
LXr2dqVqM4S+PN5lEIjzrUinEEN4M5y5GcWSGYSfp8BGePjRb3Dm3yU9S5z3
vbgvu2bfaYHdHeraG7g/WCkaS5DyawE64o55C9w/OBu4Nme8bYHw/ju9tdg3
Q6IFPtS2wGD04aBg3H/oMUw2xXO3gt2unPtCb3CeIHvwh35thfbSK/GXcL+S
+IpVk22wFQqPZXF/xuaRTrgXvNQKl80u/+aKoQC7bV3jc5428Ga3ok5irw+x
3Xvi3AbRscX2z3D/096Y0uDC1w6zge0VI7hf0rp5krIk1w4W1ew12ri/qqNr
4nXSaIdpLm6PL9hlSovO9vfagf759T0xybjeFZzitWlqh2tLTX7Wqbg+d9ws
yVTogJSIz/5quF/bXf6HNXaqAzxb8jzScnH+q6MVS13qgCeSj7Qk8vDzb2WC
j+sd8DzthmkGtt3Ifudytu+Q/FxM6GM+zsc0x3uGTn6Ho9l7cj4XUMDlvEMc
X8B3uNhXzJiD+82utiGZBPFOyNP68jcR969RxAqd5FtdUCvGfbCkA+eJyOwC
upEeuDh/+9bPJQq8odNM//W8D6KZTw+27sH9ulSP02x4H1Stiv24wkmFKv3/
5Edi+0B6Rry4H3suyaO+IbcPwpn37pvE/bqScuZEdF8fWA96Utb2U2HCi0VI
RaIfHg69dxPgpYLcSuu7lw398J/467tiolRo4DP9z/d7PzC3SgomY187PyXx
YLAfrgnuEOY7SAX/yM1iS1I/fG4UY9wtRoV+uSO9x/YPgNwCuY0kToXHTqEc
AzcHoFdn+18XKSq0zugHiW0fhDN8get5slTQR3vDyOyDMHJhOHafHBV6o/pe
l/MMQuV1b+2H2ARV82R92UFQ/m9Xv9IxKvyKvV3qc20Qmv671Vp+nAq8eo+n
e/MHQe1xzZMQeSokiastJFUMwuHfylxT2GKbDFSHxkHwCBNqPXOKCkezgjZp
Rwbhmy8lYgL7HMObvUeYhiB84HiKlAIVHEqz1QIshkDv6IVjoYpUIIfc1dS3
GwLta8qzfdhuN4/p8rkOwS5Ola+CZ/H17Sk1/Rw4BAFPVMnZ2JF36pxHC4fA
re2DWKUSFcqFBhPkWYeB34Jrd4kyFVR+x72n3T8M25uDE6nY9W2WWa1Cw8B6
9obpYRUqdHhOFVnLDwONheqlJOyJHnJriNUwqAXlHPFWpQJLIOPGdMkwGF80
mmY9R4VbB14wbNQMww3Z05ynsOszmXaxtw5DauegvxW27zdm4VPEYciwtjxb
iP2bdafGc6YRMA20Jemdp4JR3Eu92D0j4M+pqfUAu0CGzSyXbwSu7tW0icZ2
0tnl2Cc7Av/s2yRGsKdCdkdKmI7AnwtHLK6r4/snGB6naDMCyglsjzyxk3M5
0i47joBh0+TqW+zr3/eUufuOwIKB3OZ37G6OfYTGzBGY8VVLUryA51vK69mh
ghH4nEFoMcQOPbafulQ5ArpC4zaO2BcNuLZzd43A3+q28TjsmqgD0rf/4PFo
lf9bwRYUiznhzTAKKTPbj2/tz/sU8UDYrlHoj0v0F8A+/YNXv1RkFOqO0RWq
Y7+2jTNrlRmF9XtpUSbYv1b5bhDlR4FBteefPXYul4A7k/YoxKg/fRiCzZaR
8ITPaBQyhgghidgOpwWDZC1HIS78ZVghdnNTYuR5u1EoeWP4oQNb3FQo3uT+
KLic4D/7Ezs+RqQuI3YU+IS5k3Q1qbAhldpa8WEUoqJ9125im5aL/vieOwrP
7L/6eWPvHT44t1Y3Cp9f2/5Ow3a584G6s30Uxo/uatza7+/4K/ZXqH8UnFIE
I9uwg/kl2LUWRsE3bPf80tb3Bznp3Oaro7Chffo6jRYVUhMid5fSEKDI0iyK
Dbtr2mBfLQMBPs7zBGzth9Me3cvTykyAHnaBg1v75bLuPQI/dhFAaoHRamv/
37IqSpTISYD0JO+jCtivGI0O/eQmwDkdIT8V7IrL+w4v8xPgN8db7QvY23/x
z06KEKDV09nx4tb3CW/EU3slCFCSTd24hB2tcNSiQYYArBsBP7f2/yeGTx0o
kSMAV9kbmS0ffqLSky5PAOkzwUVb/99dVCv0nSIB3nW2eWx9fk2D/sVAFQJ8
6mq02xqf1d5su6c6ASrDxr1OYxux2VTbXyTAfsvnKUe29v/zHLyuXSYAY7hg
pwj2nIGr/CVDAmw4MTPswT7225ty1pQA0xeILVv7/14x/tmHLQhQdp37yQx2
vVLILQEbAixZHOLp2HpezxJHaBwJ8HQ8a+otdqpExjuyCwGGqllYHmIvfssz
GHtAgFB1/j/G2E84aptrnxBw335ClB27ufDbswJ/ApQHSVyewfOD06Rb+X0Q
AZoepstUYqcnTBb5RxHg+zGpcaut7wNUF50fRBOg0ZQnRxZbcWpF+lYCAZhi
77Fv4vnfIc2YoplBAAtx+Sch2KulkiGsFQTgfcBwLRK/b8rXj2n9rSaAkL+s
1Nb+fSCNIsNiPQHMRTyv78Dmv3DJs72DAK41Kt+d8Pt9a87wZFUPAbSEPjII
Yue/NCfnDhBAt5HNrxXXD/VuJ9vwCQL0p79REtzav3/gLuw3SwAzyHL6qkaF
gQNPhu8t4uvtC2u3xb7zX/gVw98EKD11JjoJ16/IxQLgYiWCgbDxxgSufyNh
FeuMu4lQJ5ya6IgtcaK+cG0vERidd1svAxW+eP6QGhQggo/4sso6rqdjjOv7
448R4YTujRtEXI9lRWBJxIwIp4VZjhzE9ZzCZ938yJII9rWx71xw/S/gCkgd
vEEE0jPj+PKTVDjF1m4ccZcI9S8GXNVP4PuzZl5N60+ESI1hE1m83ui2+4QT
c4lQtIfpgiVer9ibkx3OFhEhJPP2IzdJKnTW1au/+0KEtfgXV18cwvOtjO3P
5XoiJFrxzqfi9c78fbx11SAR2FP5mz+L4Ov3rDyesH0MzvsLusny4PntNsb2
h2UM93OTqhwHqLB0d/us0e4xcLpYUUviosL9mzpxbLxjMG9XMpW4D89HvREG
H9kxGG5y6hncTYUg8c3ea9fG4OXlwbaSHVTI6DrrcSB/DFrZzJl3kClgnKXc
c6B4DPrMdKKjSRTY7qcmy1M+Brddek2lFnE/LKc5y1M/BirqUY7nfuJ+IMzA
hG9gDHJDBKcNJinwSsf+tCDtOMQ0He7d7MN5pent2kH9cSD5hdVJVFJAMCnG
UOzqOLQs/0z5rxz3O+7xeWLXxsEy+Z/tmzLc/0qk3ha/OQ5j1gJha0UUmPD/
1C/xcBwIYf3Mbz9RwOhcfalU8jiYV+YfuYbzmgKieshS8HjWA4IaOP9aLu5e
jl0dh8S7g8GTXhTw5zt6l/HvOEwplZO9H+K85Olwc3THBARYr99OwPna4dSk
/ku+CbiV5eqUYk+BhLxeqVmNCdAJHOy/h/sp+tTS4cSECXh4/EVKpTjuH7t+
GLO+nwDSjOB/KaI4n9KudD3InIAzK1z3fYUoEG8p16xTOAGLOX5RR3goIM+X
VbTRPAFJSUXvz7PhfPc6NsR4ZQIUaxrtjKhkaH/+SIX90iTUpbUxni4lA/ue
KRcr/UkQLc7S8Cgkg36cdmrB1UngLk4UKcgjQ3ce9w4Tq0lw15jI4c4kw8DA
55akB5OQ/Mfudmo0GSakZ64eS56EYp/+k6UPyfC747Kj0eokWJFVXMZOkuG0
WXFC+gYe77rx+AM5Mjyc5Pu+vm0KBMeKmRgPk+Hv2tyxBNYpmDV28OA6SAZa
kWdr00JTwL2m5PWHgwysrqVPPbSnwB0JNXxdWAJBbuGY2KQpmEQ/VqwSlkDn
sa1OxIcpCJPnOZEWvQSe0x+3BWZNAUP1t96pqCXoLZS/9aBwCgwb2tSMg5fg
pf6lE/pNU+DG1xS2w2MJ1oPc23eQp+DzQxOX53pL0L3ZRndPeRqq9sZk0m2S
gPYmZ/Ht89PQNrbw7T8qCY60mthZak3Dz2+3dYtmSBAYM9FxyWAaLooPOqp2
kUDl1HqcxK1pYBfVy6BLJ0GO88FTIyHTwJ/4QapVlwQvJjwdNEen4V1FTqrB
m0U4nDKklDUxDbOp3eV2gYvQ/d/Z3TvnpuGR9vqQi/ciCI5uFnb8mgYz/6oW
Y+tFKOp/8u8q8wz4sZqmSB9ehIm2gPCbJ2bgUh8rc1jVAiiXRpT4Bc3Azex/
mpTBeVh5mUVfKT8LAX2yRIWVOfByCee84jEHqSuKtSx001DJF1oUVfATdEMa
ZzdejwF9mrVAIvMCRIQxd8Y/GwBr+8V/zZqLMJa1VGCQ3g7H7/odvKa7CAXb
3u1/eb4dtrtyXVwwWATHZyg9fawNMn1UX++yXIRB/XmjF7xtQAl7LW3gtgij
XeJFba9a4HGJkslw8iJoPCGWTN9pgliGkHzSn0VY1T5EMtpfC47Mon2Pt5Fg
jJM7kDGvBpR3lf7dvYME0tvjMhO0amCca1zjOAcJ/vUbdPp4V4OktPyouwQJ
TKnG0tvrEJToj7LQGpJA5vZDDwftUgi8el82wpQEhpGL/pFeJXDtGtNVUUsS
5JaJ6QZ+LIZ/N44lq9uTAMabvoyyFIG6R8CpoCck6GGSLj3ZmA9cPrzmvAEk
uObKfvn7Sh7M+eb5ZQeTQDfXf03hYB68DB5sa39DAo+//Ja/aT+BedjdX5Zx
eF6wSYmlm2fD0dcMByjJJLAOl7i9WpoFtDHR4IfnxfVN5kdTnJnw/7+PhGf/
//vI/wELB4Jg
      "]], LineBox[CompressedData["
1:eJwt2nc01e8fAHAziUpGyUjIlsqu5C0iK0oqVJqUvb6KSCgjq0JlRVYqISNR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      "]], LineBox[CompressedData["
1:eJwt2nc41t//B3A7s0hKyEqEFJVoeUllhTISKRlRySrjY2WVmVFIg0rI3smI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      "]], LineBox[CompressedData["
1:eJwt2Hc8Vu//B3CzbGVEIpISilSilHeUJKNCKIkKkYwKUcIH2XsTZWXvJCsX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      "]]}}, {{}, {}, 
    {GrayLevel[0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJw12Hk0Vd37AHCRSJI0eDUYS5IhQyTpMWVIhCbK+FaUKUmkhJAQJXMylblk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      "]]}}, 
   {GrayLevel[0], Dashing[{Small, Small}], 
    LineBox[{{-1, 0}, {-1, 1}, {1, 1}, {1, 0}}]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["y", "a"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox[
      RowBox[{
        SubscriptBox["v", "x"], " ", "\"/\"", " ", "U"}], HoldForm], 
     TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-1, 1}, {0, 1.6}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.57554766419048*^9, 3.575547697091427*^9, 
  3.5865829329794426`*^9, 3.5875544970091047`*^9, 3.587606407460352*^9, 
  3.587606703589883*^9, 3.587635951357649*^9, 3.5890779572140727`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 16.3 Gravity-driven planar flow (p. 265)\
\>", "Title",
 CellChangeTimes->{{3.502164762706029*^9, 3.5021647700068417`*^9}, {
  3.5876091801800375`*^9, 3.58760921185984*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 16.2 Liquid film (p. 266)
Example 16.3 Stability of open-surface flow (p. 267)\
\>", "Section",
 CellChangeTimes->{{3.50216578797783*^9, 3.502165793453439*^9}, {
  3.502165939766096*^9, 3.502165941450899*^9}, {3.5876092397191815`*^9, 
  3.587609246851036*^9}, {3.5876096567912254`*^9, 3.587609760499668*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(15\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.575547707443668*^9, 3.5755477533471785`*^9, 3.586582933157466*^9, 
   3.58755449730714*^9, 3.587606407742383*^9, {3.5876094847175784`*^9, 
   3.5876095058741145`*^9}, 3.587635951647684*^9, 3.5890779573980975`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Parameters for water and glycerol\
\>", "Subsection",
 CellChangeTimes->{{3.5876092986442537`*^9, 3.587609303123789*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0.1", ",", "10"}], "}"}], " ", "Millimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"thicknesses", " ", "of", " ", "film"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Theta]", "=", 
   RowBox[{"30", "\[Degree]"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"inclination", " ", "angle"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", 
   RowBox[{"AccelerationDueToGravity", "//", "ToSI"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Nu]", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1.0", "\[Times]", 
       SuperscriptBox["10", 
        RowBox[{"-", "6"}]]}], ",", 
      RowBox[{"1.12", "\[Times]", 
       SuperscriptBox["10", 
        RowBox[{"-", "3"}]]}]}], " ", "}"}], 
    RowBox[{
     SuperscriptBox["Meter", "2"], "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", "viscosities", " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1.0", " ", 
       SuperscriptBox["10", "3"]}], ",", 
      RowBox[{"1.26", " ", 
       SuperscriptBox["10", "3"]}]}], "}"}], 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", "densities", " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4224428483050003`*^9, 3.422442849873*^9}, {
  3.42244344876*^9, 3.4224434666879997`*^9}, {3.502165812298272*^9, 
  3.502165815355878*^9}, {3.587609291211358*^9, 3.5876093360757427`*^9}, {
  3.5876093661883564`*^9, 3.5876095010755386`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 16.2", "Subsection",
 CellChangeTimes->{{3.587609827221672*^9, 3.587609830515068*^9}}],

Cell["Average velocity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"U", "=", 
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["g", "0"], 
     SuperscriptBox["a", "2"], 
     RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
    RowBox[{"3", "\[Nu]"}]], "//", 
   RowBox[{"ToUnit", "[", 
    RowBox[{"Millimeter", "/", "Second"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.42244285258*^9, 3.422442854065*^9}, {
  3.575547739106468*^9, 3.575547740562643*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TemplateBox[{"16.344416666666667`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", 
            
            RowBox[{"\"mm\"", "\[InvisibleSpace]", "\"/\"", 
              "\[InvisibleSpace]", "\"s\""}]}], 
          "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
      "Unit: millimeters per second"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", 
         FractionBox["\"Millimeters\"", "\"Seconds\""]}], "]"}]& )], ",", 
   TemplateBox[{"145.93229166666666`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", 
            
            RowBox[{"\"mm\"", "\[InvisibleSpace]", "\"/\"", 
              "\[InvisibleSpace]", "\"s\""}]}], 
          "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
      "Unit: millimeters per second"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", 
         FractionBox["\"Millimeters\"", "\"Seconds\""]}], "]"}]& )]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.575547708104748*^9, {3.575547740939687*^9, 3.5755477539662514`*^9}, 
   3.5865829334174957`*^9, 3.5875544978192034`*^9, 3.5876064082504463`*^9, {
   3.5876094855386763`*^9, 3.5876095066572084`*^9}, 3.5876359524587793`*^9, 
   3.589077957925124*^9}]
}, Open  ]],

Cell["Reynolds number", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reynolds", "=", " ", 
  RowBox[{
   FractionBox[
    RowBox[{"U", " ", "a"}], "\[Nu]"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.6344416666666666`", ",", "1.302966889880952`"}], "}"}]], "Output",
 CellChangeTimes->{
  3.5755477082577667`*^9, 3.575547754175275*^9, 3.5865829335725513`*^9, 
   3.587554498614299*^9, 3.587606408515478*^9, {3.587609485946724*^9, 
   3.587609507077261*^9}, 3.5876359528448253`*^9, 3.589077958606207*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 16.3", "Subsection",
 CellChangeTimes->{{3.5876098363877735`*^9, 3.587609839587157*^9}}],

Cell["Condition for stability", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reynolds", " ", 
  RowBox[{"Tan", "[", "\[Theta]", "]"}]}]], "Input",
 CellChangeTimes->{{3.422442836273*^9, 3.4224428593120003`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9436453362247407`", ",", "0.7522682846179373`"}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.575547708297771*^9, 3.5755477542172804`*^9, 3.5865829335895176`*^9, 
   3.5875544986573024`*^9, 3.587606408549483*^9, {3.5876094860847397`*^9, 
   3.587609507196275*^9}, 3.5876359529498377`*^9, 3.5890779587162633`*^9}]
}, Open  ]],

Cell["Velocity constant", "Text",
 CellChangeTimes->{{3.422443265875*^9, 3.422443267789*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["U", "0"], "=", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", "3"], "\[Nu]", " ", 
      SubscriptBox["g", "0"], 
      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ")"}], 
    RowBox[{"1", "/", "3"}]], "//", 
   RowBox[{"ToUnit", "[", "Centimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.422443271141*^9, 3.422443307248*^9}, {
  3.4226721322566257`*^9, 3.4226721332046256`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TemplateBox[{"1.1779399243421362`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", 
            
            RowBox[{"\"cm\"", "\[InvisibleSpace]", "\"/\"", 
              "\[InvisibleSpace]", "\"s\""}]}], 
          "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
      "Unit: centimeters per second"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", 
         FractionBox["\"Centimeters\"", "\"Seconds\""]}], "]"}]& )], ",", 
   TemplateBox[{"12.232892218962956`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", 
            
            RowBox[{"\"cm\"", "\[InvisibleSpace]", "\"/\"", 
              "\[InvisibleSpace]", "\"s\""}]}], 
          "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
      "Unit: centimeters per second"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", 
         FractionBox["\"Centimeters\"", "\"Seconds\""]}], "]"}]& )]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.5755477085187974`*^9, 3.5755477543752995`*^9, 3.586582933697527*^9, 
   3.5875544988693275`*^9, 3.5876064087705083`*^9, {3.587609486408779*^9, 
   3.5876095074543047`*^9}, 3.587635953197868*^9, 3.5890779588852634`*^9}]
}, Open  ]],

Cell["Effective surface tension", "Text",
 CellChangeTimes->{{3.5876098476051207`*^9, 3.5876098510435305`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Alpha]", "=", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], 
    SubscriptBox["\[Rho]", "0"], "\[Nu]", " ", 
    SuperscriptBox["Reynolds", 
     RowBox[{"2", "/", "3"}]], 
    SubscriptBox["U", "0"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Cot", "[", "\[Theta]", "]"}], "-", 
      RowBox[{
       FractionBox["4", "5"], "Reynolds"}]}], ")"}]}], "//", 
   RowBox[{"ToUnit", "[", 
    RowBox[{"Joule", "/", 
     SuperscriptBox["Meter", "2"]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.422443473566*^9, 3.422443542962*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TemplateBox[{"3.469081796426664`*^-6"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", 
            
            RowBox[{"\"J\"", "\[InvisibleSpace]", "\"/\"", 
              "\[InvisibleSpace]", 
              SuperscriptBox["\"m\"", "2"]}]}], 
          "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
      "Unit: joules per meter squared"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", 
         FractionBox["\"Joules\"", 
          SuperscriptBox["\"Meters\"", "2"]]}], "]"}]& )], ",", 
   TemplateBox[{"0.07101595044100724`"},
    "QuantityUnit",
    DisplayFunction->(TooltipBox[
      StyleBox[
       RowBox[{#, "\[InvisibleSpace]", 
         StyleBox[
          RowBox[{" ", 
            
            RowBox[{"\"J\"", "\[InvisibleSpace]", "\"/\"", 
              "\[InvisibleSpace]", 
              SuperscriptBox["\"m\"", "2"]}]}], 
          "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
      "Unit: joules per meter squared"]& ),
    InterpretationFunction->(RowBox[{"Quantity", "[", 
       RowBox[{#, ",", 
         FractionBox["\"Joules\"", 
          SuperscriptBox["\"Meters\"", "2"]]}], "]"}]& )]}], "}"}]], "Output",
 CellChangeTimes->{
  3.575547709267887*^9, 3.5755477549563684`*^9, 3.586582933967601*^9, 
   3.5875544993803883`*^9, 3.587606409250564*^9, {3.5876094871558685`*^9, 
   3.587609508036375*^9}, 3.587635953771937*^9, 3.5890779592443266`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Stability of open-surface flow (p. 267)\
\>", "Section",
 CellChangeTimes->{{3.5021647820032625`*^9, 3.5021647855132685`*^9}, {
  3.5876099332203913`*^9, 3.587609937986963*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(8\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.5755477171838365`*^9, 3.5865829371499624`*^9, 3.587554503407874*^9, 
   3.5876064113428154`*^9, 3.587609958310401*^9, 3.5876105200258*^9, {
   3.587611741181322*^9, 3.587611776744586*^9}, 3.58761184634494*^9, 
   3.5876120376218877`*^9, 3.5876122507324595`*^9, 3.587612335714657*^9, 
   3.587635954036968*^9, 3.589077959533369*^9}]
}, Open  ]],

Cell["Velocity field", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["v", "x"], "=", 
   RowBox[{
    FractionBox["3", "2"], "U", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      FractionBox[
       SuperscriptBox["y", "2"], 
       SuperscriptBox["a", "2"]]}], ")"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4224297367987733`*^9, 3.422429748082773*^9}}],

Cell[TextData[{
 "Energy ( cos = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Cos", "[", "\[Theta]", "]"}], ")"}], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.587609971812025*^9, 3.587609997009044*^9}, {
  3.5876101548019776`*^9, 3.5876101626089144`*^9}, {3.5876104672344656`*^9, 
  3.5876104672344656`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"energy", "=", 
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "L", " ", "W", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{"-", "a"}], "0"], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], 
        SuperscriptBox[
         SubscriptBox["v", "x"], "2"]}], "+", 
       RowBox[{
        SubscriptBox["g", "0"], 
        RowBox[{"(", 
         RowBox[{"y", "+", "a"}], ")"}], " ", "cos"}]}], ")"}], " ", 
     RowBox[{"\[DifferentialD]", "y"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.4224321252349997`*^9, 3.422432130514*^9}, {
  3.4224324713789997`*^9, 3.42243247565*^9}, {3.422432540851*^9, 
  3.422432549731*^9}, {3.42243278733*^9, 3.4224327904820004`*^9}, {
  3.422433768867*^9, 3.422433815729*^9}}],

Cell[BoxData[
 RowBox[{"L", " ", "W", " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox[
     RowBox[{"3", " ", "a", " ", 
      SuperscriptBox["U", "2"]}], "5"], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     SuperscriptBox["a", "2"], " ", "cos", " ", 
     SubscriptBox["g", "0"]}]}], ")"}], " ", 
  SubscriptBox["\[Rho]", "0"]}]], "Output",
 CellChangeTimes->{
  3.5755477179009237`*^9, 3.586582937742075*^9, 3.587554503562891*^9, 
   3.5876064116828566`*^9, 3.5876099585474305`*^9, 3.587610520272828*^9, {
   3.5876117414493537`*^9, 3.5876117769966164`*^9}, 3.587611846602971*^9, 
   3.5876120378829203`*^9, 3.5876122509804907`*^9, 3.5876123359556837`*^9, 
   3.587635954223991*^9, 3.58907796006745*^9}]
}, Open  ]],

Cell[TextData[{
 "Solve for ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]],
  FormatType->"TraditionalForm"],
 " and ",
 Cell[BoxData[
  FormBox["U", TraditionalForm]],
  FormatType->"TraditionalForm"],
 " in terms of Q and W  (sin = Sin[\[Theta]])"
}], "Text",
 CellChangeTimes->{{3.5876100046449623`*^9, 3.5876100397471724`*^9}, {
  3.5876101922914762`*^9, 3.5876102070582495`*^9}, {3.587610469410726*^9, 
  3.587610477218663*^9}, {3.58761211947571*^9, 3.5876121242912874`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sol", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Q", "\[Equal]", " ", 
       RowBox[{"U", " ", "a", " ", "W"}]}], ",", " ", 
      RowBox[{"U", " ", "\[Equal]", 
       FractionBox[
        RowBox[{
         SubscriptBox["g", "0"], 
         SuperscriptBox["a", "2"], "sin"}], 
        RowBox[{"3", "\[Nu]"}]]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "U"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422435438564*^9, 3.4224355003529997`*^9}, {
   3.422435561219*^9, 3.422435562514*^9}, {3.587610346844019*^9, 
   3.5876103497713714`*^9}, {3.587612068949647*^9, 3.587612070386819*^9}, 
   3.5876122313271313`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Rule]", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "3"}], ")"}], 
          RowBox[{"1", "/", "3"}]], " ", 
         SuperscriptBox["Q", 
          RowBox[{"1", "/", "3"}]], " ", 
         SuperscriptBox["\[Nu]", 
          RowBox[{"1", "/", "3"}]]}], 
        RowBox[{
         SuperscriptBox["sin", 
          RowBox[{"1", "/", "3"}]], " ", 
         SuperscriptBox["W", 
          RowBox[{"1", "/", "3"}]], " ", 
         SubsuperscriptBox["g", "0", 
          RowBox[{"1", "/", "3"}]]}]]}]}], ",", 
     RowBox[{"U", "\[Rule]", 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"2", "/", "3"}]], " ", 
        SuperscriptBox["Q", 
         RowBox[{"2", "/", "3"}]], " ", 
        SuperscriptBox["sin", 
         RowBox[{"1", "/", "3"}]], " ", 
        SubsuperscriptBox["g", "0", 
         RowBox[{"1", "/", "3"}]]}], 
       RowBox[{
        SuperscriptBox["3", 
         RowBox[{"1", "/", "3"}]], " ", 
        SuperscriptBox["W", 
         RowBox[{"2", "/", "3"}]], " ", 
        SuperscriptBox["\[Nu]", 
         RowBox[{"1", "/", "3"}]]}]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Rule]", 
      FractionBox[
       RowBox[{
        SuperscriptBox["3", 
         RowBox[{"1", "/", "3"}]], " ", 
        SuperscriptBox["Q", 
         RowBox[{"1", "/", "3"}]], " ", 
        SuperscriptBox["\[Nu]", 
         RowBox[{"1", "/", "3"}]]}], 
       RowBox[{
        SuperscriptBox["sin", 
         RowBox[{"1", "/", "3"}]], " ", 
        SuperscriptBox["W", 
         RowBox[{"1", "/", "3"}]], " ", 
        SubsuperscriptBox["g", "0", 
         RowBox[{"1", "/", "3"}]]}]]}], ",", 
     RowBox[{"U", "\[Rule]", 
      FractionBox[
       RowBox[{
        SuperscriptBox["Q", 
         RowBox[{"2", "/", "3"}]], " ", 
        SuperscriptBox["sin", 
         RowBox[{"1", "/", "3"}]], " ", 
        SubsuperscriptBox["g", "0", 
         RowBox[{"1", "/", "3"}]]}], 
       RowBox[{
        SuperscriptBox["3", 
         RowBox[{"1", "/", "3"}]], " ", 
        SuperscriptBox["W", 
         RowBox[{"2", "/", "3"}]], " ", 
        SuperscriptBox["\[Nu]", 
         RowBox[{"1", "/", "3"}]]}]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Rule]", 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"2", "/", "3"}]], " ", 
        SuperscriptBox["3", 
         RowBox[{"1", "/", "3"}]], " ", 
        SuperscriptBox["Q", 
         RowBox[{"1", "/", "3"}]], " ", 
        SuperscriptBox["\[Nu]", 
         RowBox[{"1", "/", "3"}]]}], 
       RowBox[{
        SuperscriptBox["sin", 
         RowBox[{"1", "/", "3"}]], " ", 
        SuperscriptBox["W", 
         RowBox[{"1", "/", "3"}]], " ", 
        SubsuperscriptBox["g", "0", 
         RowBox[{"1", "/", "3"}]]}]]}], ",", 
     RowBox[{"U", "\[Rule]", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            FractionBox["1", "3"]}], ")"}], 
          RowBox[{"1", "/", "3"}]], " ", 
         SuperscriptBox["Q", 
          RowBox[{"2", "/", "3"}]], " ", 
         SuperscriptBox["sin", 
          RowBox[{"1", "/", "3"}]], " ", 
         SubsuperscriptBox["g", "0", 
          RowBox[{"1", "/", "3"}]]}], 
        RowBox[{
         SuperscriptBox["W", 
          RowBox[{"2", "/", "3"}]], " ", 
         SuperscriptBox["\[Nu]", 
          RowBox[{"1", "/", "3"}]]}]]}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5755477186680145`*^9, 3.586582937943102*^9, 
  3.5875545036599054`*^9, 3.587606411831874*^9, 3.587609958610439*^9, 
  3.587610520534861*^9, 3.5876117417283864`*^9, 3.587612037951927*^9, 
  3.5876120709888916`*^9, 3.5876122510494986`*^9, 3.587612336029693*^9, 
  3.587635954277997*^9, 3.5890779602464433`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"sol", "[", 
      RowBox[{"[", "2", "]"}], "]"}], "/.", 
     RowBox[{"W", "\[Rule]", 
      FractionBox["Q", 
       RowBox[{"R", " ", "\[Nu]"}]]}]}], "/.", 
    RowBox[{
     SubscriptBox["g", "0"], "\[Rule]", 
     FractionBox[
      RowBox[{"3", 
       SuperscriptBox[
        SubscriptBox["U", "0"], "3"]}], 
      RowBox[{"\[Nu]", " ", "sin"}]]}]}], "//", "PowerExpand"}], "//", 
  "Simplify"}]], "Input",
 CellChangeTimes->{{3.422435568171*^9, 3.4224355884*^9}, {3.422436143586*^9, 
   3.422436185138*^9}, {3.422441463954*^9, 3.422441465201*^9}, {
   3.422441535811*^9, 3.422441545685*^9}, 3.4224416075699997`*^9, {
   3.4224416981940002`*^9, 3.422441702068*^9}, 3.5876120859006824`*^9, {
   3.5876122375628786`*^9, 3.5876122438826385`*^9}, 3.5876123899001584`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", "\[Rule]", 
    FractionBox[
     RowBox[{
      SuperscriptBox["R", 
       RowBox[{"1", "/", "3"}]], " ", "\[Nu]"}], 
     SubscriptBox["U", "0"]]}], ",", 
   RowBox[{"U", "\[Rule]", 
    RowBox[{
     SuperscriptBox["R", 
      RowBox[{"2", "/", "3"}]], " ", 
     SubscriptBox["U", "0"]}]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5755477189950533`*^9, 3.586582938013069*^9, 3.587554503818924*^9, 
   3.5876064118738785`*^9, 3.5876099586634426`*^9, 3.587610520583868*^9, 
   3.5876117417803926`*^9, 3.587612038003934*^9, 3.5876120862157197`*^9, {
   3.587612245249801*^9, 3.5876122511035037`*^9}, 3.5876123360846987`*^9, 
   3.5876359543180046`*^9, 3.589077960278418*^9}]
}, Open  ]],

Cell["\<\
Express the energy in terms of Reynolds number and dimensional factors\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EE", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"energy", "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], "/.", 
       RowBox[{"W", "\[Rule]", 
        FractionBox["Q", 
         RowBox[{"R", " ", "\[Nu]"}]]}]}], "/.", 
      RowBox[{
       SubscriptBox["g", "0"], "\[Rule]", 
       FractionBox[
        RowBox[{"3", 
         SuperscriptBox[
          SubscriptBox["U", "0"], "3"]}], 
        RowBox[{"\[Nu]", " ", "sin"}]]}]}], "/.", 
     RowBox[{"cos", "\[Rule]", 
      RowBox[{"cot", " ", "sin"}]}]}], "//", "PowerExpand"}], "//", 
   "Simplify"}]}]], "Input",
 CellChangeTimes->{{3.422435606867*^9, 3.422435628432*^9}, {3.422435762979*^9,
    3.422435900898*^9}, 3.422441953682*^9, {3.4224420652980003`*^9, 
   3.4224420661289997`*^9}, 3.5876120807810683`*^9, 3.5876122568421926`*^9, 
   3.587612360030575*^9, {3.587612485789663*^9, 3.5876124927294955`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"3", " ", "L", " ", "Q", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"5", " ", "cot"}], "+", 
     RowBox[{"2", " ", "R"}]}], ")"}], " ", 
   SubscriptBox["U", "0"], " ", 
   SubscriptBox["\[Rho]", "0"]}], 
  RowBox[{"10", " ", 
   SuperscriptBox["R", 
    RowBox[{"1", "/", "3"}]]}]]], "Output",
 CellChangeTimes->{
  3.575547719068065*^9, 3.586582938080118*^9, 3.5875545038899317`*^9, 
   3.5876064119178843`*^9, 3.5876099587204514`*^9, 3.5876105208188934`*^9, 
   3.5876117419934163`*^9, 3.587612038073942*^9, 3.5876120814931526`*^9, {
   3.587612251193513*^9, 3.587612258440384*^9}, {3.5876123361467066`*^9, 
   3.5876123617867827`*^9}, 3.5876124937706203`*^9, 3.5876359543620076`*^9, 
   3.5890779604474945`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Alpha]", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", " ", 
       FractionBox["Q", 
        RowBox[{" ", 
         RowBox[{"L", " ", "\[Nu]", " ", 
          SuperscriptBox["W", "2"]}]}]]}], 
      RowBox[{
       SubscriptBox["\[PartialD]", "R"], "EE"}]}], "/.", 
     RowBox[{"W", "\[Rule]", 
      FractionBox["Q", 
       RowBox[{"R", " ", "\[Nu]"}]]}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Rho]", "0"], "\[Rule]", 
     FractionBox["\[Eta]", "\[Nu]"]}]}], "//", "Simplify"}]}]], "Input",
 CellChangeTimes->{{3.502165486001299*^9, 3.5021655216005616`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", "10"]}], " ", 
  SuperscriptBox["R", 
   RowBox[{"2", "/", "3"}]], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "5"}], " ", "cot"}], "+", 
    RowBox[{"4", " ", "R"}]}], ")"}], " ", "\[Eta]", " ", 
  SubscriptBox["U", "0"]}]], "Output",
 CellChangeTimes->{3.5755477192110815`*^9, 3.5865829381171002`*^9, 
  3.587554504048951*^9, 3.58760641195689*^9, 3.5876099587754583`*^9, 
  3.587610520967911*^9, 3.587611742138436*^9, 3.587612038113948*^9, 
  3.5876122512765245`*^9, 3.587612336202713*^9, 3.587612445714854*^9, 
  3.587612497156026*^9, 3.5876359544030128`*^9, 3.5890779604794436`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 16.4 Laminar pipe flow (p. 268)\
\>", "Title",
 CellChangeTimes->{{3.4228516921814127`*^9, 3.422851693831413*^9}, {
  3.5021799931894035`*^9, 3.5021800003468122`*^9}, {3.5876306496175165`*^9, 
  3.587630652478861*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.422851696297413*^9, 3.4228516991024127`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(20\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755480048253508`*^9, 3.5865829383901286`*^9, 
  3.587554504375989*^9, 3.58760641221692*^9, 3.5876359546670437`*^9, 
  3.589077960750478*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 16.4. Aortic flow (p. 270)\
\>", "Section",
 CellChangeTimes->{{3.5021800787732983`*^9, 3.502180080163378*^9}, {
  3.502180172547662*^9, 3.502180176674898*^9}, {3.5876306618739877`*^9, 
  3.5876306647033253`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.575548005243402*^9, 3.5755480432039537`*^9, 
  3.586582938770216*^9, 3.587554504761035*^9, 3.587606412454951*^9, 
  3.587635954909073*^9, 3.5890779611655903`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{
    FractionBox["25.", "2"], " ", "Millimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"aorta", " ", "diameter", " ", "25", " ", "mm"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Q", "=", 
   RowBox[{"70.", " ", "Milli", " ", 
    RowBox[{"Liter", "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"discharge", " ", "rate", " ", "from", " ", "heart"}], " ", "*)"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Eta]", "=", 
   RowBox[{"2.7", " ", 
    SuperscriptBox["10", 
     RowBox[{"-", "3"}]], " ", "Pascal", " ", "Second"}]}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{"blood", " ", "viscosity"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]", "=", 
   RowBox[{"1000", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"blood", " ", "density"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4407344643502073`*^9, 3.4407344644902077`*^9}, {
  3.587630684614715*^9, 3.5876307635121803`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.587630677824902*^9, 3.587630678751011*^9}}],

Cell["Average velocity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"U", "=", 
  RowBox[{
   FractionBox["Q", 
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["a", "2"]}]], "//", 
   RowBox[{"ToUnit", "[", 
    RowBox[{"Centimeter", "/", "Second"}], "]"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"14.260282901033824`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"cm\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: centimeters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Centimeters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755480057004547`*^9, 3.5755480437710238`*^9, 
  3.5865829391682506`*^9, 3.587554506426235*^9, 3.587606412940009*^9, 
  3.5876359554021325`*^9, 3.589077961899667*^9}]
}, Open  ]],

Cell["Reynolds number", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reynolds", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"\[Rho]", " ", "2", " ", "a", " ", "U"}], "\[Eta]"], "//", 
   "ToSI"}]}]], "Input"],

Cell[BoxData["1320.3965649105394`"], "Output",
 CellChangeTimes->{3.5755480061955137`*^9, 3.575548044231078*^9, 
  3.5865829394452877`*^9, 3.587554507015308*^9, 3.587606413410065*^9, 
  3.5876359558471856`*^9, 3.589077962283671*^9}]
}, Open  ]],

Cell["Pressure gradient", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"8", "\[Eta]", " ", "U"}], 
    SuperscriptBox["a", "2"]], "//", 
   RowBox[{"ToUnit", "[", 
    RowBox[{"Pascal", "/", "Meter"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5755480339698477`*^9, 3.5755480359850893`*^9}, {
  3.5755481257628636`*^9, 3.575548126799987*^9}}],

Cell[BoxData[
 TemplateBox[{"19.71341508238916`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"Pa\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"m\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pascals per meter"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Pascals\"", "\"Meters\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.575548006711576*^9, 3.5755480447781453`*^9}, 
   3.5755481275880804`*^9, 3.5865829396933236`*^9, 3.5875545074933643`*^9, 
   3.5876064138211145`*^9, 3.58763595621523*^9, 3.5890779625547047`*^9}]
}, Open  ]],

Cell["Blood pressure", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"p", "=", 
  RowBox[{
   RowBox[{
    FractionBox["120.", "760"], "Atmosphere"}], "//", 
   RowBox[{"ToUnit", "[", "Kilopascal", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5755480546573296`*^9, 3.575548083905839*^9}}],

Cell[BoxData[
 TemplateBox[{"15.998684210526315`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kPa\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilopascals"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilopascals\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575548006770585*^9, {3.57554804483215*^9, 3.5755480845029097`*^9}, 
   3.586582939817355*^9, 3.5875545075373697`*^9, 3.587606413911125*^9, 
   3.5876359563242445`*^9, 3.5890779625807085`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 16.5 Hypodermic syringe (p. 270)\
\>", "Section",
 CellChangeTimes->{
  3.4226745132596254`*^9, {3.5021801632761316`*^9, 3.502180166363308*^9}, {
   3.587630924080449*^9, 3.587630931631353*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(9\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.575548007051619*^9, 3.575548138568396*^9, {3.575548371208312*^9, 
   3.575548382501665*^9}, 3.586582940149378*^9, 3.587554507799401*^9, 
   3.5876064142221627`*^9, 3.587635956591275*^9, 3.5890779627727346`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"a", "=", 
    RowBox[{
     FractionBox["1", "2"], "0.5", " ", "Millimeter"}]}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"internal", " ", "radius", " ", "of", " ", "needle"}], " ", 
    "*)"}], ";"}], " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b", "=", 
   RowBox[{
    FractionBox["1", "2"], "1", " ", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"internal", " ", "radius", " ", "of", " ", "chamber"}], " ", 
   "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L", "=", 
   RowBox[{"5", " ", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"needle", " ", "length"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"5", " ", "Gram"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"mass", " ", "of", " ", "injection"}], " ", "*)"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Q", "=", " ", 
   RowBox[{"0.5", " ", 
    RowBox[{"Milliliter", "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"rate", " ", "of", " ", "discharge"}], " ", "*)"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Eta]", "=", " ", 
   RowBox[{
    SuperscriptBox["10", 
     RowBox[{"-", "3"}]], "Pascal", " ", "Second"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"viscosity", " ", "of", " ", "water"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]", "=", 
   RowBox[{
    SuperscriptBox["10", "3"], " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"density", " ", "of", " ", "water"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.502180226028721*^9, 3.5021802656769886`*^9}, 
   3.5021803273785176`*^9, {3.575548374593717*^9, 3.575548376946*^9}, {
   3.587630941448533*^9, 3.5876311085805845`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5876311156644363`*^9, 3.5876311179507093`*^9}}],

Cell["Kinematic viscosity", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Nu]", "=", 
   FractionBox["\[Eta]", "\[Rho]"]}], " ", ";"}]], "Input"],

Cell["Needle cross section", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"A", "=", 
   RowBox[{"\[Pi]", " ", 
    SuperscriptBox["a", "2"]}]}], ";"}]], "Input"],

Cell["Mass of injection", "Text",
 CellChangeTimes->{{3.440307320309432*^9, 3.440307320866432*^9}, {
  3.587631160303792*^9, 3.5876311653273935`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"M", "=", 
  RowBox[{
   RowBox[{"A", " ", "L", " ", "\[Rho]"}], "//", 
   RowBox[{"ToUnit", "[", "Milligram", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.440307323652432*^9, 3.440307372024432*^9}, {
  3.575548148272562*^9, 3.5755481485085897`*^9}}],

Cell[BoxData[
 TemplateBox[{"9.817477042468104`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"mg\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: milligrams"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Milligrams\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755480078167086`*^9, {3.5755481392744813`*^9, 3.575548149107661*^9}, {
   3.5755483718153844`*^9, 3.575548383170745*^9}, 3.5865829405654287`*^9, 
   3.5875545087295113`*^9, 3.587606414850238*^9, 3.5876359571513414`*^9, 
   3.5890779631108294`*^9}]
}, Open  ]],

Cell["Average needle flow velocity", "Text",
 CellChangeTimes->{{3.587631238207138*^9, 3.5876312389262233`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"U", "=", " ", 
  RowBox[{
   FractionBox["Q", "A"], "//", "ToSI", "  "}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"2.5464790894703255`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755480079587255`*^9, 3.5755481394054995`*^9, {3.5755483720804143`*^9, 
   3.5755483833047624`*^9}, 3.586582941126503*^9, 3.5875545091325607`*^9, 
   3.5876064150402594`*^9, 3.58763595729536*^9, 3.5890779631927977`*^9}]
}, Open  ]],

Cell["Reynolds number", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reynolds", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"U", " ", "2", "a"}], "\[Nu]"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData["1273.2395447351628`"], "Output",
 CellChangeTimes->{
  3.5755480083237715`*^9, 3.57554813975054*^9, {3.5755483724074545`*^9, 
   3.575548383665805*^9}, 3.5865829413505397`*^9, 3.5875545095106063`*^9, 
   3.5876064153883023`*^9, 3.587635957699408*^9, 3.5890779633848095`*^9}]
}, Open  ]],

Cell["Pressure gradient in needle", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"8", "\[Eta]", " ", "U"}], 
    SuperscriptBox["a", "2"]], "//", 
   RowBox[{"ToUnit", "[", 
    RowBox[{"Bar", "/", "Meter"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.575548414433507*^9, 3.5755484154216146`*^9}}],

Cell[BoxData[
 TemplateBox[{"3.259493234522017`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"bars\"", "\[InvisibleSpace]", "\"/\"", 
            "\[InvisibleSpace]", "\"m\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: bars per meter"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Bars\"", "\"Meters\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755480118841977`*^9, 3.5755481402085943`*^9, {3.575548372722492*^9, 
   3.575548384083856*^9}, 3.5755484162397137`*^9, 3.586582941710575*^9, 
   3.587554509888651*^9, 3.5876064158063507`*^9, 3.587635958055451*^9, 
   3.5890779636618447`*^9}]
}, Open  ]],

Cell["Pressure drop in needle", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]p", "=", 
  RowBox[{
   RowBox[{"L", " ", "G"}], "//", 
   RowBox[{"ToUnit", "[", "Pascal", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.502180479039192*^9, 3.502180479942244*^9}}],

Cell[BoxData[
 TemplateBox[{"16297.466172610086`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"Pa\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pascals"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Pascals\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575548014634527*^9, 3.5755481403566113`*^9, {3.5755483728665104`*^9, 
   3.5755483842248716`*^9}, 3.586582941789584*^9, 3.5875545100156665`*^9, 
   3.5876064159483676`*^9, 3.5876359581804667`*^9, 3.5890779639578824`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 16.6 Water pipe (p. 270)\
\>", "Section",
 CellChangeTimes->{
  3.4226744169016256`*^9, {3.502180586627346*^9, 3.502180592275669*^9}, {
   3.587631290383401*^9, 3.587631299695515*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(14\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.575548018617005*^9, 3.5755484427969017`*^9, 
  3.5865829419976125`*^9, 3.587554510269697*^9, 3.5876064162794075`*^9, 
  3.5876313923156295`*^9, 3.5876359584424973`*^9, 3.589077964187913*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"0.25", " ", "Inch"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"half", " ", "inch", " ", "tube"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Q", "=", 
   RowBox[{"0.1", " ", 
    RowBox[{"Liter", "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"volume", " ", "discharge", " ", "rate"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Eta]", "=", " ", 
   RowBox[{
    SuperscriptBox["10", 
     RowBox[{"-", "3"}]], " ", "Pascal", " ", "Second"}]}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{"viscosity", " ", "of", " ", "water"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]", "=", 
   RowBox[{"1000", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"density", " ", "of", " ", "water"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.587631310580824*^9, 3.587631348854416*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5876313568003674`*^9, 3.5876313577904863`*^9}}],

Cell["Average velocity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"U", "=", 
  RowBox[{
   FractionBox["Q", 
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["a", "2"]}]], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.789410096555994`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575548019204076*^9, 3.5755484433299637`*^9, 
  3.586582942510665*^9, 3.587554510687746*^9, 3.5876064166784573`*^9, 
  3.5876313927706842`*^9, 3.5876359588765497`*^9, 3.589077964540956*^9}]
}, Open  ]],

Cell["Reynolds number", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reynolds", " ", "=", " ", 
  RowBox[{
   FractionBox[
    RowBox[{"\[Rho]", " ", "2", " ", "a", " ", "U"}], "\[Eta]"], "//", 
   "ToSI"}]}]], "Input"],

Cell[BoxData["10025.508226261125`"], "Output",
 CellChangeTimes->{3.5755480196661296`*^9, 3.575548443794022*^9, 
  3.5865829428997526`*^9, 3.5875545111197977`*^9, 3.5876064171305113`*^9, 
  3.5876313932137375`*^9, 3.587635959309602*^9, 3.5890779648539953`*^9}]
}, Open  ]],

Cell["Pressure gradient", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"8", " ", "\[Eta]", " ", "U"}], 
    SuperscriptBox["a", "2"]], "//", 
   RowBox[{"ToUnit", "[", 
    RowBox[{"Pascal", "/", "Meter"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5876313825664597`*^9, 3.58763138349957*^9}}],

Cell[BoxData[
 TemplateBox[{"156.61927639526203`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"Pa\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"m\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pascals per meter"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Pascals\"", "\"Meters\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575548020138188*^9, 3.5755484441550627`*^9, 3.586582943143757*^9, 
   3.587554511427836*^9, 3.587606417495555*^9, {3.5876313841536503`*^9, 
   3.5876313935667796`*^9}, 3.587635959824664*^9, 3.5890779657681694`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Case: Ostwald viscometer (p. 271)\
\>", "Section",
 CellChangeTimes->{{3.502180705500145*^9, 3.5021807110504627`*^9}, {
  3.5876314116639504`*^9, 3.58763141947189*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.485051190870942*^9, 3.485051193416088*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(8\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755480204172206`*^9, 3.5755484516759653`*^9, 
  3.586582943486797*^9, 3.5875545117578745`*^9, 3.587606417758587*^9, 
  3.587635960577755*^9, 3.5890779664852343`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Nu]", "=", 
   RowBox[{"1.", " ", 
    SuperscriptBox["10", 
     RowBox[{"-", "6"}]], 
    RowBox[{
     SuperscriptBox["Meter", "2"], "/", "Second"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rey", "=", "2000"}], ";"}]}], "Input",
 CellChangeTimes->{{3.485051203038638*^9, 3.4850512650391846`*^9}, {
  3.485051299962182*^9, 3.4850513298398905`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a", "=", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"4", " ", 
      SuperscriptBox["\[Nu]", "2"], 
      RowBox[{"Rey", "/", 
       SubscriptBox["g", "0"]}]}], ")"}], 
    RowBox[{"1", "/", "3"}]], "//", 
   RowBox[{"ToUnit", "[", "Millimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.485051268440379*^9, 3.485051315371063*^9}}],

Cell[BoxData[
 TemplateBox[{"0.9343790744786106`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"mm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: millimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Millimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575548020714256*^9, 3.575548452084016*^9, 
  3.586582944372907*^9, 3.587554512078913*^9, 3.5876064180596237`*^9, 
  3.5876359608627896`*^9, 3.589077966641221*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Figure 16.2L Velocity profile (p. 272)\
\>", "Section",
 CellChangeTimes->{{3.422237300989394*^9, 3.422237302122394*^9}, {
  3.4228537250984125`*^9, 3.4228537271154127`*^9}, {3.4228539458154125`*^9, 
  3.4228539472004128`*^9}, {3.502180759676244*^9, 3.50218076153135*^9}, {
  3.587631537475045*^9, 3.5876315459350605`*^9}, {3.5876322772148037`*^9, 
  3.5876322802711697`*^9}}],

Cell[CellGroupData[{

Cell["Get data", "Subsection",
 CellChangeTimes->{{3.5876316096957135`*^9, 3.5876316139342194`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"ReadList", "[", 
    RowBox[{"\"\<data/pipes.vzpipe.data\>\"", ",", "Word", ",", 
     RowBox[{"RecordLists", "\[Rule]", "True"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.422197557093*^9, 3.4221975858450003`*^9}, {
   3.422198318259*^9, 3.422198331632*^9}, {3.422198600071*^9, 
   3.422198601049*^9}, {3.422198675572*^9, 3.422198679362*^9}, {
   3.4221990361140003`*^9, 3.422199041567*^9}, {3.422199661207*^9, 
   3.422199678507*^9}, {3.422237156232394*^9, 3.422237156684394*^9}, {
   3.4224247365747733`*^9, 3.4224247397107735`*^9}, {3.422516853466*^9, 
   3.4225168539779997`*^9}, {3.422851722028413*^9, 3.422851723281413*^9}, 
   3.4309107804536*^9, 3.433126797743538*^9, {3.4850483869135647`*^9, 
   3.485048388890678*^9}, {3.505364959779525*^9, 3.5053649611556034`*^9}, {
   3.505368802130295*^9, 3.5053688043464217`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rp", "=", 
   RowBox[{
    RowBox[{"data", "[", 
     RowBox[{"[", "2", "]"}], "]"}], "//", "ToExpression"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radial", " ", "coordinates"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"z", "=", 
   RowBox[{
    RowBox[{"data", "[", 
     RowBox[{"[", "3", "]"}], "]"}], "//", "ToExpression"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"longitudinal", " ", "coordinates"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"up", "=", 
   RowBox[{
    RowBox[{"Drop", "[", 
     RowBox[{"data", ",", "4"}], "]"}], "//", "ToExpression"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"longitudinal", " ", "velocities"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4221975986029997`*^9, 3.422197598986*^9}, {
  3.422197632599*^9, 3.4221976985290003`*^9}, {3.422197940203*^9, 
  3.422197962167*^9}, {3.422199050526*^9, 3.4221990977679996`*^9}, {
  3.422199219829*^9, 3.422199227591*^9}, {3.422199495073*^9, 
  3.422199512376*^9}, {3.422851816573413*^9, 3.422851830481413*^9}, {
  3.422852494045413*^9, 3.4228525034314127`*^9}, {3.4228525608204126`*^9, 
  3.422852576732413*^9}, {3.422852685593413*^9, 3.4228526889054127`*^9}, {
  3.5876316416055403`*^9, 3.587631668787801*^9}, {3.5876318326774654`*^9, 
  3.5876318768537674`*^9}, {3.5876319538130007`*^9, 3.587631974515486*^9}}],

Cell["\<\
Extend radial coordinates and longitudinal velocities to negative values\
\>", "Text",
 CellChangeTimes->{{3.5876318979683*^9, 3.5876319243024592`*^9}, {
  3.587631983423554*^9, 3.587631985390789*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"r", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Reverse", "[", "rp", "]"}]}], ",", "rp"}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"u", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Reverse", "[", "#", "]"}], ",", "#"}], "]"}], "&"}], ",", 
     "up"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.422852694831413*^9, 3.422852751788413*^9}, {
  3.422852943479413*^9, 3.422852960323413*^9}}],

Cell["\<\
Interpolate the longitudinal velocity\
\>", "Text",
 CellChangeTimes->{{3.58763193481472*^9, 3.5876319492634535`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vz", "=", 
  RowBox[{"ListInterpolation", "[", 
   RowBox[{"u", ",", 
    RowBox[{"{", 
     RowBox[{"z", ",", 
      RowBox[{"2", "r"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422199287601*^9, 3.4221992909969997`*^9}, {
   3.422199348712*^9, 3.422199370057*^9}, {3.422199471094*^9, 
   3.4221995040620003`*^9}, {3.4228518771674128`*^9, 3.4228518880514126`*^9}, 
   3.422851946842413*^9, 3.4228519844454126`*^9, {3.422852514739413*^9, 
   3.422852523467413*^9}, 3.4228530072964125`*^9, 3.4850514743821583`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"InterpolatingFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0.0125`", ",", "9.9875`"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.975`"}], ",", "0.975`"}], "}"}]}], "}"}], 
    ",", "\<\"<>\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 CellChangeTimes->{
  3.5755480210843024`*^9, 3.5755484580527325`*^9, 3.58658294554902*^9, 
   3.587554512373951*^9, 3.58760641827265*^9, 3.587631646561134*^9, 
   3.5876318807762375`*^9, {3.5876322222462077`*^9, 3.587632240135354*^9}, 
   3.587632344469879*^9, 3.5876359610738125`*^9, 3.5890779667362657`*^9}]
}, Open  ]],

Cell["\<\
Entry length at 99% of Poiseuille velocity\
\>", "Text",
 CellChangeTimes->{{3.5876315816813498`*^9, 3.587631598462363*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindRoot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"vz", "[", 
     RowBox[{"zz", ",", "0"}], "]"}], "==", 
    RowBox[{"0.99", " ", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"zz", ",", "3", ",", "1", ",", "9"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.422423209645773*^9, 3.4224233262697735`*^9}, {
  3.4224233651017733`*^9, 3.422423391069773*^9}, {3.422425986655773*^9, 
  3.422425987086773*^9}, {3.422517372683*^9, 3.4225173857869997`*^9}, {
  3.4228520010324125`*^9, 3.4228520043134127`*^9}, {3.4228521067534127`*^9, 
  3.4228521128884125`*^9}, {3.422853034081413*^9, 3.422853034152413*^9}, {
  3.4850515849084797`*^9, 3.4850515877596426`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"zz", "\[Rule]", "5.810427961658635`"}], "}"}]], "Output",
 CellChangeTimes->{
  3.575548021122306*^9, 3.575548458091736*^9, 3.586582946110129*^9, 
   3.587554512438957*^9, 3.587606418306651*^9, 3.5876316466091413`*^9, 
   3.587631880814242*^9, {3.587632222281212*^9, 3.587632240173359*^9}, 
   3.5876323445078773`*^9, 3.587635961287839*^9, 3.5890779667582664`*^9}]
}, Open  ]],

Cell["\<\
Plot data (ignore extrapolation errors)\
\>", "Text",
 CellChangeTimes->{{3.587632068303738*^9, 3.5876320697269087`*^9}, {
  3.5876321792620497`*^9, 3.5876321928466806`*^9}, {3.587692353390559*^9, 
  3.587692356171893*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figc", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"vz", "[", 
       RowBox[{"s", ",", "y"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"s", ",", 
        RowBox[{"{", 
         RowBox[{
         "0.1", ",", "0.3", ",", "0.6", ",", "1", ",", "1.5", ",", "2.5"}], 
         "}"}]}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "2.01"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"HoldForm", "[", 
        FractionBox["r", "a"], "]"}], ",", 
       RowBox[{"HoldForm", "[", 
        RowBox[{
         SubscriptBox["v", "z"], "\"\</\>\"", "U"}], "]"}]}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4222373363083944`*^9, 3.422237537524394*^9}, {
   3.4222375811973944`*^9, 3.422237623104394*^9}, {3.4222377617423944`*^9, 
   3.4222377815043945`*^9}, {3.422240355206394*^9, 3.4222404556133943`*^9}, {
   3.4222428714783945`*^9, 3.422242875483394*^9}, {3.422242989897394*^9, 
   3.422243004656394*^9}, {3.4222430516103945`*^9, 3.422243275121394*^9}, {
   3.422243341464394*^9, 3.4222433421463943`*^9}, {3.422243517640394*^9, 
   3.4222435211923943`*^9}, {3.422516929947*^9, 3.422516982045*^9}, {
   3.4331270384265385`*^9, 3.433127038545538*^9}, {3.433127098571538*^9, 
   3.4331271052385383`*^9}, {3.4403076988464317`*^9, 3.440307708369432*^9}, {
   3.440307791586432*^9, 3.440307818711432*^9}, {3.4403107354454317`*^9, 
   3.440310779609432*^9}, {3.440311407995432*^9, 3.440311413396432*^9}, {
   3.440311454965432*^9, 3.440311455019432*^9}, 3.440311494707432*^9, {
   3.4850514888009825`*^9, 3.4850515034658213`*^9}, 3.4850516228806515`*^9, {
   3.4850516895974674`*^9, 3.4850516898374815`*^9}, 3.485051720351227*^9, {
   3.4850518975353613`*^9, 3.48505190153759*^9}, {3.5014926910976505`*^9, 
   3.5014927408504963`*^9}, {3.5054477795127316`*^9, 3.505447868057796*^9}, {
   3.505447905264924*^9, 3.505447937824786*^9}, {3.5876320149713383`*^9, 
   3.5876320502105665`*^9}, {3.587632169034826*^9, 3.5876321746344967`*^9}, {
   3.5876322318833647`*^9, 3.587632234450673*^9}, {3.5876323143472595`*^9, 
   3.5876323387081866`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJw123c41W/YAHCFjIS0SypRkaxQiDuSGRllZ/WjQoVEqSgkOytJykxEKMr2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     "]], LineBox[CompressedData["
1:eJwt2nk4Vd/XAHBlKgqpSIZEJClEEtUyNpq/klKokCQkkiEkiUSGSJKZyJgU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     "]], LineBox[CompressedData["
1:eJwt2nc41t//B3AZGdkVkRARKiupiFdJkRBCIspoaYgUycoK2SEkKYnskM2J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     "]], LineBox[CompressedData["
1:eJwt2nk8VN//B/A2oYS0EQkJyZaULb0TSUqpbEW2tCGVCJEthUhIsq+RLVvK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     "]], LineBox[CompressedData["
1:eJwt2Xc41t/7AHCJSKgUhewtK5EQdxpIyaiQraKyKsqI7KRIooSyIntlE469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     "]], LineBox[CompressedData["
1:eJwt2Hk4lN/bAHBZUvZEWuxEJFEipW4klbKWLbJUKLTJkghJtiSpJHtZQmWX
KJ2sY9+GIWuEJGaSssw8w+98r/f9a67PdZ3nPOecOefc9/1Inb9m5sTKwsJy
Yg0Ly3+/yrBpenWVhlii/IQDtJcOX1uWCV+m01BagLW3EnKCgmJVufl/NORv
GbeTLdML9uwwPD/5i4a2sheu3fDsPmhsCOlv6aWh30G+P23CnoH297mmF/k0
JPvabhOHVyaYhbW+2WtPQ8FXmS4dZ0rBpz3kistnKnre0fbcYVMddHdzT1c5
ziLvkRpJTfceEPtyTUhDdAatRu0c/8o7AqVutRz8Q9Po4MOr92RXv4NDlOrv
r8E/kZ5a9oE30VOwFOsXjmAK5f05lvE6YgbS+Ya3xI9OomSJtZcqj/6GJANV
D3LEBDqTz+PLLvwHgt0dbymcGEcu9m1XBtLmwY8t18djdgy9ekLae1/uHzgG
tctSskdRsMNtx7tXFuBZ5UR8kus35Ps0BPSjFqFiZAfPW+lhlHqDB+pSlkBG
QFv3TMoAyjhNZqzcWYYixXDXse99yHtG0EbFiA4SnQ7lO7/2oeC6X9slTOkQ
5aO5zqutD92jF9G5z9DBqXY6k7O8D11TCq0dsabDZjvDUaXoPiSZ9GT6mhMd
7jwWtPLZ34fe8Uxz7fWnw/GlpKM8D3vR2TZ3mn42HUbqCiXV1SmoZ3DOsYNO
B//Ty3r7lSjo9KysmCgT9zeqffmANAXt0GQcc16lgwnRXniYj4Lc91RS/7Ex
4Msequ6xyR507lbDI1ZeBqSlKjpZxfUgy4rIF53iDDjvm55ze6Eb9Vmrxhvo
MGBC6dke9J6MhPgiN9XeZYBiQCZ16g0ZbQmL8ft6jwFX20tzBV+SUZCf+d7Z
+wxYukGRdn5ARvKDK8n8kQzgKhPZyOtARuUZ3J7asQxQ0UmYP7uejMRjqbVm
rxjgZ55asmDThaiU7+ugigEbAnLUldd0os1VRgkOdAaEvt/RILDYgaKF7Ejz
DAbQqa+t52c60PbnKln3mQwYs8/y/9DXgSadLP0yWQgo0kmv1i3oQNS2fU96
1hJgxpFsZGnfgchLwf8WBQmQ3yLVyuLXjnToJOcTigQIREr8rnNuR6mWt0eK
dxKwRBfb+MCsHYXWFW4T20VAw+BWa2HFdiTCP/Xt124CLqUJTSh8bUPBK4x3
t/cRkCu/jmmm0YaO3+JrP6dNgJI6TSlzvgV1bT/Gf9icAGflwMDyoRYUxDaz
IdmCgFQ5/q5WUguaC3pWS7ckQFBkt89CYgsK273Ts+gsAYuLV6uO6bWg8Ii6
wA0OBFSXU82nnzYjxYfmvdGuBFgdogbsVm9CbqVDRGQAAbH7AjqPSDahxaaH
r1EgAc27+GStuJrQA26Ccz6IgMPiyo1Bw41oiCt1xvweATKrVzZ2hTaiitku
C55wAqhfZl979jag1LTHh7QeE3BPb7aj/BYJRSQ7Nb59hd9/fMZY7QIJSb09
H1qSTgCL4a+2PEMSynHN2fwxgwB/858t6dIk9I7wR7VZBHg7TzQ8bKlHFfHb
5Jty8XqFD31xlKxHfl7Zl1KLCDjV0lqwvqEWKdTXV3hVERDZ0bI7pKgWRWry
vbGuxuvd3Zy3klSLDDa43NWqIeDoUOPb+Ru1aMlLY2ylFs+HWpc9tK0W3W35
dNe7gQAVgc9phddq0OiN1+mq7QQIn8mLsRapRm0Vrq39A3g/yD2oOMpWjUqD
CZ9HgwSYLLuMq9KqkId60IEjQ/j9KVL7uUhV6H7Wr3XZwwSsTD0dqfCuQm5R
6hTnUQIm7/gpi1K+IPPIjxHlkwRcNX4cX6mA0LlD/IMffhOQ/ddg5tkahM7v
nW8ynSNg7AW79tWvn5FMnNfQT2yLcZ8f4hGfUYOYy1WReQLA1049aKoS6b/w
OXbpH95/6Tu7dV9/QrRzUoKjy3i8C3X8DbIVaCOH6pQKKxOic19VH14uR81O
zfbl2G52gV6lreVIetflYW02JsjW7x945V2OXO9w/znFzoS4Z2+y7jR8QPns
N/2t1zLhttrjQ3vdy5BKAY+rHhcTLKau/M7RLkMXRN7frsTem2SQLilchjRE
e57v42bCDBv7er7P71FIVMeSNA8T7Mje3VP879H8Dz37P7xM0PWwdUspLkGn
o85zXdzABHE5TXHh8BKk7HqYuw2b8VW484FtCcoTZRXXEGRCiU67ui9HCTr7
MPn22o1MkNugy3rGshi9u/N2S4IQE7gK5F+sZxSi6b1Tm16IMGHyAvupoPZC
tNCFJv5i14iMriykF6KihMom481M8A9MuDh+qhDpes31sm5hAtWIVwWlFCA5
vnR2u61MyByYePgzJA/9i7jZNSXKhH7/TanJe/PQmpm4NapiTOATP1ZgOvYO
uXtFu9zC9nbI7iqHd2jvakIfuzgT9CdcRSLpb9CJvzGnhSSYMEWlpSlez0H9
sTs6eKWYIBojWTQsnoOS7rkHnsA2UTWtiW3NRtyp6y6FYH+4WTRBV8xG1RbT
E4vYEUueis0TWci+vSK+R5oJiqz0YjebDPR4YwPTTRavb7pinQRXBpokuZsn
Ysfq2VDIH9KRWu7sVCM2PfTT0sFN6ejZsmyt7HYmNHMHHuLpfIkUGjuMO7FZ
3uUbfQl8ibbOV0bSsdWMvtl7Kr9Eb081ssrIMSEpRid48EEaCn00kHsDu09x
XbT1wxQ0am0+wiHPBO0rlwuqppNR5tpETgXs7PymLoXjyahwXPvqSWxftYci
dNYk5LnPaPEh9tbDgi8Tbr1AkdudJLh2MCE46GYNKyUeVc2Y/ZTH/lXdPeG6
Nx4l+hX90cP+dOy54kFqHNL4XV/jj33OVLRk4MJTJLDvqP137Jn5f5petU/Q
i4at+XRs/7h2xLf9CdqmMHJ0gwIe/0Bws87kY7TOVO24FrZSgK3pgP5j5LW5
rtgE+6Okeq/n6xg08FbP+SJ2SXNXnXFnNHqsJZAWgd0tENoZHPsQjWnYyidi
z5trDpWejkK7WJ7/e4O951vq322UCPR2sc+5Gdt0+2kW4+fhaFTu2/p+7Ouu
a3mCrcIQvwxl/gd2/l93man+EGQRHJ20RpEJbZqSu7cl3UM3JkxseLGpAeQD
RueC0T1hVsfN2Ly1ofp3xe+iR4Kv3kljWwl6ZlxCgejJoIqOEvbzpP6PbTp3
0OObedv2YVPkdMhqtbdReO/WQ4ewhQpfTyfo30K31nln6mGfPsjHuqbRCzmu
/XLmJHZsnecWl5M3kV7nkoEpdofxgEpr63UkcVky3AJbvORejHOaO5pvVeex
wU5sMEF7Ml3Ql9WDvXbYm4fEqCs551HYyu6fjtg7v/doxp2zQfpNgroXseV4
rj9XdjFFq+fHR52wg7p3dG+rOYQKGzIanLGV+IdcOXbJgi3j9MJ/pswFpwos
HYdV+pzrf/5tLP/nmocFJJEC5P573u+JfpObmgPscaTvuIC9EuCbmLrbCapI
Fz0c/uvf9a07WdEVTjAqWc9hs1mMHOKUuwbNDI5+K+wayYiwZAEPONqotXQa
OyRO+rpGtCeUXzh/zghbn+eTVSe3D8g2e68/js0ZbK7jFuELkau+hDZ2wyJV
gYPTH6ZY3NU1sSOvhAumhgSAdvupMhXsrLmYO6J3g+Cxq2igPLaLi6Lz+5Vg
EF8X/l0Q+7i0z9l7h0LgLJdsECf2jqEaIxP/+/Doa94FBt4f02bnNKaXw2B8
OGb1G3YTb+7Osv0RwC48VdaF/aZhQSLEJxLENyuX1WC7H45ZJ/4vCrTuh0am
YxsuDxLTe6NBhxZvF4utXKIwV+bxCA7LJfgGYS89HZ3ZsRQDsoxLIWexy7n8
eqq9nkCq9v5ZDuy0XcLkfOmnMKaQyJzB5ynUJL8jqf0psOTsViNjn4n73uyt
EAfkS7Mbk7F/S52qVhyKh0bqx2Q57F69SSQS+QJmQ9+MrMX+7BJUya6RALym
WTOT+Pw/eFfyYTgmES4WL8ZnYMtpiuXH6qWAEnt7sTA2j23Z28C5FBA/s6+X
hu+b+QDTXPeUVLirYiTRiO0wVm0m458GG4Z7RH2xlXcsGjJ7XkL2/jlSG76/
Govt9QojMkDLaLnYBN9/WnVKNfLfMsCltIdbDLuAsqyTop4JPybP3Z+SwfFu
+Qk8+J4Jjkv5g/7YF6HhgNOh1xC+hl8kEd+3q00qqlvmckCF/2zMO0kmeAwy
8x8dy4VxN+0ZN+zJ2SbltSm50NXR7KyA3SLgpDRv8AZ6WQXKXuL7P8HihXxr
xlvo2suKwnB82De2RvyuVT4IKO3WVcDxRbnCck4lJx/sdRpbh7fh9Yh9V/tt
OR8mGMGxT7C5X77dl2ZYAEZLZfV0HJ8IE6vBVWoBJKv19yAcv4YK8hWQahFc
/xXjLovjXX/S2Y0iZ4ugaluxT+smfJ7COZhXg4tA0SSw3Au7zcGmQ5xcBIu2
t77VCOP/R4DTJ8CzGEKNGGGWOJ6mXrerPVRWAosyZbtscDxOsl2f92ykBHzj
O7mXBJgQf7zk+SxnKURyjok+xY6R5HJLtioFMb92SiM/Pn8dpRuYy6UQon3s
1U4+Jjiq8Np/0ioDImzlbAPOF2R+f1w+UF0OI3LKVy/i/MO/9UjI39ly8L1R
kzCI8xNKbjNf3pYKMFw40XIaO9JpQEbqRgUcedaoBWvwfuunG3JKfoQw718n
uFYJ+Fyn+Yp85xM0afiTrtMJOJtYZuCuiaADJib8cX5Vcgt6tjsheHfvBUcP
jQA+C5L9SAyCSHV1CSXsKoFeT7MpBL3tpMO9swQIHfEzTjr5BTbcmJCS+UWA
34cE32D6F+CkCZdGTBBwIv1rq5FVNezO8eCN+krA7/t5znVXqkG8Mbe6vo+A
+Ev3Vg/eqwb3l+buLNhTu3apKuRVQ+ov0cTrFALCPwQ9ZWOvgeZlGWs9Ms5X
W+VtPuTXwKn4jJkPLQQcW/KakuKsg5z3f8XIn3F+3W9wN160Dq7bUbx+VRIQ
VymxlX9PHdzUXShixZ4MbjxJ2NZB/m6ugV0fCQjlFcunFNWBbZFaj3cZAfXS
tV4P7OohdV6Wk5yP82FDQbZ/pSRcd+qyf0glINpx/dxyIwlSnx4O/5xCQJ8X
y8jKEAnijutz1iYT4JpCrVi3tgEiaaF/mxNxe1qTh6hFA7we2R/Q/hy3f3xv
7MhCA0z4fhrNeITbU/5Wxao3QZeEUsh9XF8UT//Kf27QBFs+sPu63iGAWBlL
TrJrAo+XF6yN/HF/8p2+WaFNcKChpl/wNm5/651qBaUJjHh8u6K9cPutzi9H
vZshN1VIwcydgEf2vUEqZS3wwn+Zl80az3dGCyZbWiBXTOpRDq6HAnxfMRPH
WuBa2ZUlI1wvXX1y5TYnXyuMpv2penaaACMSm+fwxVYY2+8jttEI74fdKi5R
gm2wPNVk26KLx7MSbjh1pR1Ssp7vVsP1HMlIjNdEsBM+Bb2sLFpkgJDy6CYP
0U7YsX7OeNsCAxx4MyWfynVC7Z7amOC/uL5sVlLrO9AJGTIV2wznGKB0QsvG
4UInvIs9eqxrmgExR2xyrpd0grrLB0rSEAPO7n9x9LFFFwzXm3YoVzNgVko4
oCuBDH1/9ktJhDFgv9/LG1UZZJgJoRXux/VxcPcup4I8MvQKXew3wfXzpjD9
U9HVZDDVNfztH8iAwzM+Ww2myfDFIVKy1ocBj973v6/S7IY+QfZNfJcYoHoy
jVbQ1w2tN34bRx1jgJfnTsdHmyjgeeZZlwAr7o+L9slCigIbwzUdE3D9/yG1
aLO4EgWCv6aJSzPpQDRpdrzVocCI2qq00hIdgqWOQ5M7Bc/nx2FFKh2i2y6K
cdRQgBrapNj2lQ5ZO1K+3r7WCxdEWPkK8unQOyhg5tzQB4b604FF5nRw8NX6
wXlqAP5+HiGZxC9DzU6NyBqOYbCd3up/M2cJ5NZ/mso4/g2y5q1s57IXQeyT
21GO8FF4qlfElxS1ACx01hyPsTFIjHT7sWvPP1g0rbSr2z4OcmU/ep2b5mFA
zX7b9pAJWGcsVGYu+geuWwpskB6cBMM6g4jLhr9hy9NxK3axKTBoHr8sOD8D
eTZpr+dv/YQKpazZqbwpsDwUeLyhchqO9th6NbmNQ6HdFekDxC9Q05wcYVke
gSWuI2zl5rNwh41DvbSxB8w6D6qLxVMhE+Xqh4WTgOsgd9WnZCrMu5w92axE
gqqM/pO26VTQDWAnMTrqQeWWr2NiHhVi1RgK0lvqgU+iLGpLHRXmwqxbQ7Jr
odF9z3fhP1TgJvGvSa+pAu11CjF8p2hwQXr722RKGSzdWNqaZ0oDw+91cye2
lkH+ACnT0JIGPoGPYtrPvQfxfOePUedpMNvA9sN5vAQI84xJLl8aaEkeNq+n
FUJZusQhziwa+L9vNohbeQ1XeWmkrDc0eB+nLCF2NwvkfD6b6RfSQE5JZIMl
WyY8NTh36f4nGnT66F8aX/sKPOYSnrCRaZAgegOaheJB0cZVPL2PBnGqDRo8
ws9gtFYzR3eYBkcL7SjFQrFgEt/3+e5PGnwZJPlclQ2HdazZJ6RoNNhjCQfl
JIMBufl0f/lLg7RTZHfLfbfAu0ff3oFOgyYZYwHC+BL8//dHYIn8v++P/wMl
Cq2h
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["r", "a"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox[
      RowBox[{
        SubscriptBox["v", "z"], " ", "\"/\"", " ", "U"}], HoldForm], 
     TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-1, 1}, {0, 2.01}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.5755480218203883`*^9, 3.5755484589038324`*^9, 3.586582946853181*^9, 
   3.5875545132620544`*^9, 3.587606419000736*^9, 3.5876316473012247`*^9, 
   3.587631881493323*^9, {3.5876320115129223`*^9, 3.587632051242691*^9}, 
   3.5876321759666567`*^9, {3.5876322152323675`*^9, 3.587632240845439*^9}, {
   3.587632319006817*^9, 3.587632345202963*^9}, 3.5876359620419292`*^9, 
   3.5890779670682755`*^9}]
}, Open  ]],

Cell["Initial velocity distribution", "Text",
 CellChangeTimes->{{3.58763209732022*^9, 3.5876321291990433`*^9}, {
  3.5876923671172056`*^9, 3.587692369452488*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figd", "=", 
  RowBox[{"Graphics", "[", 
   RowBox[{"{", 
    RowBox[{"Dashed", ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0.0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "1.0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1.0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0.0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.440310443762432*^9, 3.440310447175432*^9}, {
   3.440310484770432*^9, 3.4403105641294317`*^9}, 3.440311289007432*^9, {
   3.4850515080800858`*^9, 3.48505151778164*^9}, 3.5876321103077765`*^9}],

Cell[BoxData[
 GraphicsBox[
  {Dashing[{Small, Small}], 
   LineBox[{{-1, 0.}, {-1, 1.}, {1, 1.}, {1, 0.}}]}]], "Output",
 CellChangeTimes->{
  3.587632110641817*^9, {3.587632222973295*^9, 3.5876322408874454`*^9}, 
   3.5876323452459664`*^9, 3.587635962310963*^9, 3.5890779671372843`*^9}]
}, Open  ]],

Cell["Final velocity distribution", "Text",
 CellChangeTimes->{{3.5876320862868953`*^9, 3.587632093358743*^9}, {
  3.5876321320313845`*^9, 3.5876321332145257`*^9}, {3.5876923731489315`*^9, 
  3.5876923754842105`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figf", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"2", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SuperscriptBox["r", "2"]}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Black", ",", "Dashed"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4850516287339864`*^9, 3.485051661559864*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJw12Hk0ld33AHClUpKkKJUxhIyRopctCa9CKqVSaKDMSUmETJEISeYhkqlr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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-1, 1}, {0., 1.9999999526113406`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.5755480220394154`*^9, 3.5755484590978565`*^9, 3.586582946991238*^9, 
   3.5875545133500667`*^9, 3.587606419052742*^9, 3.587631647352228*^9, 
   3.5876318815443325`*^9, {3.5876322230153*^9, 3.5876322409324493`*^9}, 
   3.5876323452729692`*^9, 3.5876359625049887`*^9, 3.589077967153329*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figc", ",", "figd", ",", "figf", ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "2.05"}], "}"}]}], "}"}]}]}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<pipes-entryflow\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.440310570543432*^9, 3.4403106217614317`*^9}, {
   3.440310694045432*^9, 3.4403106949784317`*^9}, 3.4403107403854322`*^9, {
   3.440310797252432*^9, 3.4403107980304317`*^9}, 3.440311389035432*^9, {
   3.440311499045432*^9, 3.440311501745432*^9}, {3.4792656630959997`*^9, 
   3.479265665639*^9}, {3.4850484407336435`*^9, 3.485048441681698*^9}, {
   3.4850515338195577`*^9, 3.485051536428707*^9}, {3.485051666488146*^9, 
   3.485051667901227*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.575548022247441*^9, 3.575548459339886*^9, 3.5865829471812224`*^9, 
   3.5875545136231003`*^9, 3.587606419277769*^9, 3.587631647698273*^9, 
   3.5876318817803574`*^9, 3.5876321383071375`*^9, {3.587632223240327*^9, 
   3.5876322411384754`*^9}, 3.5876323454719934`*^9, 3.5876359628330245`*^9, 
   3.5890779672933035`*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJw123c41W/YAHCFjIS0SypRkaxQiDuSGRllZ/WjQoVEqSgkOytJykxEKMr2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      "]], LineBox[CompressedData["
1:eJwt2nk4Vd/XAHBlKgqpSIZEJClEEtUyNpq/klKokCQkkiEkiUSGSJKZyJgU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      "]], LineBox[CompressedData["
1:eJwt2nc41t//B3AZGdkVkRARKiupiFdJkRBCIspoaYgUycoK2SEkKYnskM2J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      "]], LineBox[CompressedData["
1:eJwt2nk8VN//B/A2oYS0EQkJyZaULb0TSUqpbEW2tCGVCJEthUhIsq+RLVvK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      "]], LineBox[CompressedData["
1:eJwt2Xc41t/7AHCJSKgUhewtK5EQdxpIyaiQraKyKsqI7KRIooSyIntlE469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      "]], LineBox[CompressedData["
1:eJwt2Hk4lN/bAHBZUvZEWuxEJFEipW4klbKWLbJUKLTJkghJtiSpJHtZQmWX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      "]]}}, 
   {Dashing[{Small, Small}], 
    LineBox[{{-1, 0.}, {-1, 1.}, {1, 1.}, {1, 0.}}]}, {{}, {}, 
    {GrayLevel[0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJw12Hk0ld33AHClUpKkKJUxhIyRopctCa9CKqVSaKDMSUmETJEISeYhkqlr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      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["r", "a"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox[
      RowBox[{
        SubscriptBox["v", "z"], " ", "\"/\"", " ", "U"}], HoldForm], 
     TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{-1, 1}, {0, 2.05}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.5755480222534423`*^9, 3.575548459345888*^9, 3.5865829471832223`*^9, 
   3.5875545136281004`*^9, 3.5876064192827682`*^9, 3.5876316477102737`*^9, 
   3.5876318817853575`*^9, 3.5876321383221397`*^9, {3.5876322232563305`*^9, 
   3.5876322411434774`*^9}, 3.587632345482994*^9, 3.5876359628390245`*^9, 
   3.589077967296322*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Figure 16.2R  Central velocity (p. 272)\
\>", "Section",
 CellChangeTimes->{{3.4222373106523943`*^9, 3.422237313991394*^9}, {
  3.502180807027952*^9, 3.5021808100921273`*^9}, {3.5876322642252474`*^9, 
  3.5876322931347136`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"vz", "[", 
       RowBox[{"s", ",", "0"}], "]"}], ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", 
      RowBox[{"First", "[", "z", "]"}], ",", "10"}], "}"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"HoldForm", "[", 
        FractionBox["z", "d"], "]"}], ",", 
       RowBox[{"HoldForm", "[", 
        RowBox[{
         SubscriptBox["v", "z"], "\"\</\>\"", "U"}], "]"}]}], "}"}]}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"1", ",", "2.01"}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Black", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", "Black", "}"}], ",", "Dashed"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", "0.66"}]}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<pipes-vzpipe\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "200"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422199519625*^9, 3.422199611606*^9}, {
   3.4221997855109997`*^9, 3.422199800777*^9}, {3.422237174036394*^9, 
   3.422237175068394*^9}, {3.422240316316394*^9, 3.4222403210213943`*^9}, {
   3.422243526427394*^9, 3.4222435293503942`*^9}, 3.422518140236*^9, {
   3.422853072302413*^9, 3.422853097146413*^9}, {3.433126869991538*^9, 
   3.433126882328538*^9}, {3.433126917566538*^9, 3.433126947340538*^9}, {
   3.440311315005432*^9, 3.4403113601874323`*^9}, {3.479265733145*^9, 
   3.479265742297*^9}, {3.4850519376516557`*^9, 3.485051941425872*^9}, {
   3.501492781842841*^9, 3.5014928388451014`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755480224944706`*^9, 3.575548459603918*^9, 
  3.586582947351283*^9, 3.587554513998145*^9, 3.587606419597807*^9, 
  3.587635963091056*^9, 3.5890779674563665`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwV1Xc4lt8bAHAUmsabEVnve0gJLZVR7rtSCRVayggNGsgoJBSSEvp+8zMa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     "]]}, 
   {GrayLevel[0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQ7bST7euWmZ32DBDgcFpxrbKY9x4433xbzJWqjafg
/K6rKx/e+HQLzmfcv2+f5OXHcL5U28zExzyv4PyH1eX5D9M+IMxzsPi+oPcz
nH8l0izn9fdvcP6DN2u4XYJ+wfl53ZYlb4r+wvmL9jM+7c9mcIDx/fYXllxk
YILzN/2WlZTqY4bzdz7KMNGwZYXzm87bmcSdZ4Pz36VtXhXZzAHnC1mnmMXo
c8H530UWLGvbxw3n18+s6i3L54XzTyo/iOhj5ofzo3WKzm6NEYDzdVOFfi43
FoTzHyj8ENP7g+A/NdT9MO2qEJx/6ujy3ZMXCMP53BUHBKeki8D58+o1lMvt
ReF8Qa5rau84xOD8ay8vPXV/jOB/f56rv3SnOJz/I8Mve1qjBJzvbV+XlBUl
ifDfjM/el1Sk4Hw+AfYDtq8R/Pd3Mt93H5KG8zfqCAnmdcvA+Ucvcr4zSpWF
83f+dNvbbSAH53M6KIm/+I7gr77to6N+QR7OF6u9J/nXRwHOZzC3+x2+C8GX
YHi7cIuqIpy/oGuf4+1OBP/ifnP74m8IftG2y6ZXo5XgfMc9+z5a7Efwd+YE
cvNpK8P5C++xya3tR/DXMM7ovnYcwd+hovnz/38EHwAzzcal
     "]]}},
  AspectRatio->0.66,
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["z", "d"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox[
      RowBox[{
        SubscriptBox["v", "z"], " ", "\"/\"", " ", "U"}], HoldForm], 
     TraditionalForm]},
  AxesOrigin->{0, 1.},
  Method->{},
  PlotRange->{{0.0125, 10}, {1, 2.01}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.575548022499472*^9, 3.5755484596149178`*^9, 
  3.586582947353272*^9, 3.587554514003146*^9, 3.587606419602806*^9, 
  3.587635963106058*^9, 3.589077967459325*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Margin figure: Driving pressure (p. 272)\
\>", "Section",
 CellChangeTimes->{{3.422237300989394*^9, 3.422237302122394*^9}, {
  3.4228537250984125`*^9, 3.4228537271154127`*^9}, {3.422853931778413*^9, 
  3.422853939644413*^9}, {3.5021809078527193`*^9, 3.502180921106477*^9}, {
  3.587632376335698*^9, 3.5876324080475016`*^9}, {3.5876326363508945`*^9, 
  3.5876326386871767`*^9}}],

Cell[CellGroupData[{

Cell["Get data", "Subsection",
 CellChangeTimes->{{3.5876324126870604`*^9, 3.587632414094227*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"ReadList", "[", 
    RowBox[{"\"\<data/pipes.ppipe.data\>\"", ",", "Word", ",", 
     RowBox[{"RecordLists", "\[Rule]", "True"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.422197557093*^9, 3.4221975858450003`*^9}, {
   3.422198318259*^9, 3.422198331632*^9}, {3.422198600071*^9, 
   3.422198601049*^9}, {3.422198675572*^9, 3.422198679362*^9}, {
   3.4221990361140003`*^9, 3.422199041567*^9}, {3.422199661207*^9, 
   3.422199678507*^9}, {3.422237156232394*^9, 3.422237156684394*^9}, {
   3.4224247365747733`*^9, 3.4224247397107735`*^9}, {3.422516853466*^9, 
   3.4225168539779997`*^9}, {3.422851722028413*^9, 3.422851723281413*^9}, {
   3.4228539547824125`*^9, 3.422853962527413*^9}, 3.4309107814876003`*^9, {
   3.433126809002538*^9, 3.4331268102045383`*^9}, {3.485048406733699*^9, 
   3.485048408436796*^9}, {3.5053649845149393`*^9, 3.5053649858040133`*^9}, {
   3.5053687815781193`*^9, 3.50536878578636*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rp", "=", 
   RowBox[{
    RowBox[{"data", "[", 
     RowBox[{"[", "2", "]"}], "]"}], "//", "ToExpression"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radial", " ", "coordinates"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"z", "=", 
   RowBox[{
    RowBox[{"data", "[", 
     RowBox[{"[", "3", "]"}], "]"}], "//", "ToExpression"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"longitudinal", " ", "coordinates"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"qp", "=", 
   RowBox[{
    RowBox[{"Drop", "[", 
     RowBox[{"data", ",", "4"}], "]"}], "//", "ToExpression"}]}], 
  RowBox[{"(*", " ", 
   RowBox[{"pressure", " ", "data"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4221975986029997`*^9, 3.422197598986*^9}, {
  3.422197632599*^9, 3.4221976985290003`*^9}, {3.422197940203*^9, 
  3.422197962167*^9}, {3.422199050526*^9, 3.4221990977679996`*^9}, {
  3.422199219829*^9, 3.422199227591*^9}, {3.422199495073*^9, 
  3.422199512376*^9}, {3.422851816573413*^9, 3.422851830481413*^9}, {
  3.422852494045413*^9, 3.4228525034314127`*^9}, {3.4228525608204126`*^9, 
  3.422852576732413*^9}, {3.422852685593413*^9, 3.4228526889054127`*^9}, {
  3.422853986173413*^9, 3.422853986248413*^9}, {3.587632417718663*^9, 
  3.5876325103577785`*^9}}],

Cell["\<\
Symmetrize radial coordinates and pressure data\
\>", "Text",
 CellChangeTimes->{{3.587632515598405*^9, 3.5876325171985984`*^9}, {
  3.5876326461450715`*^9, 3.587632659454667*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"r", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"Reverse", "[", "rp", "]"}]}], ",", "rp"}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"q", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Reverse", "[", "#", "]"}], ",", "#"}], "]"}], "&"}], ",", 
     "qp"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.422852694831413*^9, 3.422852751788413*^9}, {
  3.422852943479413*^9, 3.422852960323413*^9}, {3.422853983314413*^9, 
  3.422853991693413*^9}}],

Cell["\<\
Interpolate symmetrized pressure\
\>", "Text",
 CellChangeTimes->{{3.5876325282549257`*^9, 3.587632530190157*^9}, {
  3.5876326644952717`*^9, 3.587632670254964*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"p", "=", 
  RowBox[{"ListInterpolation", "[", 
   RowBox[{"q", ",", 
    RowBox[{"{", 
     RowBox[{"z", ",", 
      RowBox[{"2", "r"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422199287601*^9, 3.4221992909969997`*^9}, {
   3.422199348712*^9, 3.422199370057*^9}, {3.422199471094*^9, 
   3.4221995040620003`*^9}, {3.4228518771674128`*^9, 3.4228518880514126`*^9}, 
   3.422851946842413*^9, 3.4228519844454126`*^9, {3.422852514739413*^9, 
   3.422852523467413*^9}, 3.4228530072964125`*^9, {3.422853995403413*^9, 
   3.4228540007354126`*^9}, 3.4850521477516727`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"InterpolatingFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0.0125`", ",", "9.9875`"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.975`"}], ",", "0.975`"}], "}"}]}], "}"}], 
    ",", "\<\"<>\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 CellChangeTimes->{3.575548022689495*^9, 3.5755484597849407`*^9, 
  3.5865829475893126`*^9, 3.587554514477202*^9, 3.587606419755824*^9, 
  3.5876359634801025`*^9, 3.5890779675563774`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot central velocity", "Subsection",
 CellChangeTimes->{{3.4222373106523943`*^9, 3.422237313991394*^9}, 
   3.502180938571476*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figa", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"p", "[", 
     RowBox[{"s", ",", "0"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", 
      RowBox[{"First", "[", "z", "]"}], ",", "10"}], "}"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"HoldForm", "[", 
        FractionBox["z", "d"], "]"}], ",", 
       RowBox[{"HoldForm", "[", "p", "]"}]}], "}"}]}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "4"}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422199519625*^9, 3.422199611606*^9}, {
   3.4221997855109997`*^9, 3.422199800777*^9}, {3.422237174036394*^9, 
   3.422237175068394*^9}, {3.422240316316394*^9, 3.4222403210213943`*^9}, {
   3.422243526427394*^9, 3.4222435293503942`*^9}, 3.422518140236*^9, {
   3.422853072302413*^9, 3.422853097146413*^9}, {3.422854124093413*^9, 
   3.422854127318413*^9}, {3.4228541611384125`*^9, 3.422854174691413*^9}, {
   3.422854211701413*^9, 3.422854217849413*^9}, 3.422856738949413*^9, {
   3.479265778104*^9, 3.4792658023380003`*^9}, {3.4850520357682676`*^9, 
   3.485052036400304*^9}, {3.502180983892068*^9, 3.5021810376411424`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwt0Xkw1Q0XB3D7en+VJdm3H6lsiULiHJKytD89yJJQ9r036tGCpBLKFpN0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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["z", "d"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox["p", HoldForm], TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0.0125, 10}, {0, 4}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.5755480227374997`*^9, 3.5755484598339453`*^9, 
  3.5865829476463203`*^9, 3.587554514600217*^9, 3.5876064198078327`*^9, 
  3.5876359637281313`*^9, 3.5890779675973825`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figa", ",", 
    RowBox[{"Graphics", "[", 
     RowBox[{"{", 
      RowBox[{"Black", ",", "Dashed", ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "3.2"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"10.0", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
     "]"}]}], "]"}], "//", "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<pipes-ppipe\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Tiny"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.422198019755*^9, 3.422198139676*^9}, {3.422198191025*^9,
    3.422198205168*^9}, {3.4221982403640003`*^9, 3.422198245216*^9}, 
   3.422199802226*^9, 3.4222371969853945`*^9, {3.4222385901323943`*^9, 
   3.422238590200394*^9}, {3.4222436482113943`*^9, 3.422243667104394*^9}, {
   3.422516897627*^9, 3.422516899099*^9}, {3.422853103538413*^9, 
   3.422853107114413*^9}, {3.422853682156413*^9, 3.422853686047413*^9}, {
   3.422856742271413*^9, 3.422856761158413*^9}, 3.423288195885417*^9, {
   3.4232884830214167`*^9, 3.423288486881417*^9}, 3.4309107819786*^9, {
   3.440312543266432*^9, 3.440312544851432*^9}, {3.440313415061432*^9, 
   3.440313415152432*^9}, {3.4792658136879997`*^9, 3.479265825138*^9}, {
   3.4850521642236147`*^9, 3.485052164733644*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755480229365234`*^9, 3.5755484600469713`*^9, 
  3.5865829482564*^9, 3.5875545148462467`*^9, 3.587606420019858*^9, 
  3.5876359639701605`*^9, 3.5890779679103966`*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJwt0Xkw1Q0XB3D7en+VJdm3H6lsiULiHJKytD89yJJQ9r036tGCpBLKFpN0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      "]]}}, 
   {GrayLevel[0], Dashing[{Small, Small}], LineBox[{{0, 3.2}, {10., 0}}]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox[
      FractionBox["z", "d"], HoldForm], TraditionalForm], 
    FormBox[
     TagBox["p", HoldForm], TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0.0125, 10}, {0, 4}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], Automatic}]], "Output",
 CellChangeTimes->{3.575548022966527*^9, 3.575548460052973*^9, 
  3.5865829482604494`*^9, 3.587554514851247*^9, 3.587606420024858*^9, 
  3.5876359639751625`*^9, 3.5890779679113855`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 16.5 Phenomenology of turbulent pipe flow (p. 274)\
\>", "Title",
 CellChangeTimes->{{3.423110611238*^9, 3.423110620193*^9}, {
  3.423288522236417*^9, 3.423288528911417*^9}, {3.440579084842554*^9, 
  3.4405790865045547`*^9}, {3.502181474613136*^9, 3.5021814882589164`*^9}, {
  3.587632694881919*^9, 3.5876326978232727`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Figure 16.4 Drag factor (L) and Friction factor (R) (p. 275)\
\>", "Section",
 CellChangeTimes->{{3.5021815703146095`*^9, 3.5021815725307364`*^9}, {
  3.587632712223998*^9, 3.587632728142909*^9}, {3.5876329562552795`*^9, 
  3.587632963135104*^9}, {3.5876339385911446`*^9, 3.587633942318592*^9}, {
  3.587634028862976*^9, 3.587634039886298*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.423110624076*^9, 3.423110628099*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(24\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.575548473207552*^9, 3.586582949103509*^9, 
  3.587554515502323*^9, 3.5876064203068924`*^9, 3.587635964389211*^9, 
  3.589077968242483*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Drag factor", "Subsection",
 CellChangeTimes->{{3.587633904227022*^9, 3.5876339062222605`*^9}}],

Cell["Constants", "Text",
 CellChangeTimes->{{3.502181598763237*^9, 3.5021815996922903`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"k", "=", "0.001"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"roughness", "/", "diameter"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"R1", "=", "2300"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"nominal", " ", "transition", " ", "to", " ", "turbulence"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"As", "=", "0.079"}], " ", ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"smooth", " ", "turbulence", " ", "coefficient"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Ar", "=", "0.028"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"rough", " ", "turbulence", " ", "coefficient"}], " ", 
   "*)"}]}]}], "Input",
 CellChangeTimes->{{3.423110809158*^9, 3.4231108232200003`*^9}, {
  3.423110859572*^9, 3.4231109360039997`*^9}, {3.423111058776*^9, 
  3.42311113474*^9}, {3.423111187605*^9, 3.423111188868*^9}, {
  3.423111305127*^9, 3.4231113329*^9}, {3.4231113885360003`*^9, 
  3.423111397109*^9}, {3.4231116257320004`*^9, 3.423111645749*^9}, {
  3.423111708983*^9, 3.423111710486*^9}, {3.423111873334*^9, 
  3.423111874519*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"R2", "=", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     FractionBox["As", "Ar"], ")"}], "4"], 
   FractionBox["1", "k"]}]}]], "Input"],

Cell[BoxData["63368.91041493129`"], "Output",
 CellChangeTimes->{3.575548473400572*^9, 3.5865829491704903`*^9, 
  3.5875545155653315`*^9, 3.587606420434908*^9, 3.5876359645372305`*^9, 
  3.5890779687574887`*^9}]
}, Open  ]],

Cell["Friction factor", "Text",
 CellChangeTimes->{{3.42311063575*^9, 3.4231106384820004`*^9}, {
  3.502181679491854*^9, 3.502181680131891*^9}, {3.5876339721911774`*^9, 
  3.5876339748945007`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Cf", "[", "R_", "]"}], "=", 
  RowBox[{"Piecewise", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       FractionBox["16", "R"], ",", " ", 
       RowBox[{"0", "<", "R", "\[LessEqual]", " ", "R1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"As", " ", 
        SuperscriptBox["R", 
         RowBox[{
          RowBox[{"-", "1"}], "/", "4"}]]}], ",", 
       RowBox[{"R1", "<", "R", "\[LessEqual]", "R2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Ar", " ", 
        SuperscriptBox["k", 
         RowBox[{"1", "/", "4"}]]}], ",", 
       RowBox[{"R2", "\[LessEqual]", " ", "R"}]}], "}"}]}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.423110695773*^9, 3.4231107119779997`*^9}, {
   3.4231107501210003`*^9, 3.4231107634119997`*^9}, {3.423110796133*^9, 
   3.423110802053*^9}, {3.423110998295*^9, 3.423111047317*^9}, {
   3.4231111451099997`*^9, 3.42311117575*^9}, {3.42311134271*^9, 
   3.423111373495*^9}, 3.423111407207*^9, {3.423111440918*^9, 
   3.4231114465039997`*^9}, 3.4231115054779997`*^9, {3.42311154919*^9, 
   3.423111621382*^9}, {3.4231117316870003`*^9, 3.4231117998459997`*^9}, {
   3.4231119257349997`*^9, 3.423111925975*^9}, 3.423111979335*^9}],

Cell[BoxData[
 TagBox[GridBox[{
    {"\[Piecewise]", GridBox[{
       {
        FractionBox["16", "R"], 
        RowBox[{"0", "<", "R", "\[LessEqual]", "2300"}]},
       {
        FractionBox["0.079`", 
         SuperscriptBox["R", 
          RowBox[{"1", "/", "4"}]]], 
        RowBox[{"2300", "<", "R", "\[LessEqual]", "63368.91041493129`"}]},
       {"0.004979182348108984`", 
        RowBox[{"63368.91041493129`", "\[LessEqual]", "R"}]},
       {"0", 
        TagBox["True",
         "PiecewiseDefault",
         AutoDelete->True]}
      },
      AllowedDimensions->{2, Automatic},
      Editable->True,
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{
       "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.84]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}},
      Selectable->True]}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxItemSize->{
    "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[0.35]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  "Piecewise",
  DeleteWithContents->True,
  Editable->False,
  SelectWithContents->True,
  Selectable->False]], "Output",
 CellChangeTimes->{3.575548473494583*^9, 3.5865829492064753`*^9, 
  3.5875545156363416`*^9, 3.587606420480914*^9, 3.5876359645942373`*^9, 
  3.589077968797494*^9}]
}, Open  ]],

Cell["Drag factor", "Text",
 CellChangeTimes->{{3.587633961136851*^9, 3.5876339837425623`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "R_", "]"}], "=", 
   RowBox[{
    FractionBox["R", "16"], 
    RowBox[{"Cf", "[", "R", "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4231118079849997`*^9, 3.423111860692*^9}, {
  3.4231137026619997`*^9, 3.4231137043719997`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figa", "=", 
  RowBox[{"LogLogPlot", "[", 
   RowBox[{
    RowBox[{"f", "[", "R", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"R", ",", "1", ",", 
      SuperscriptBox["10", "6"]}], "}"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Re", ",", "f"}], "}"}]}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"90", ",", 
         RowBox[{"1.1", " ", 
          SuperscriptBox["10", "6"]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0.7", ",", "600"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4231118637869997`*^9, 3.423111905668*^9}, {
  3.423113715861*^9, 3.423113744933*^9}, {3.423113779623*^9, 
  3.423113995606*^9}, {3.423116800535*^9, 3.423116818934*^9}, {
  3.479265949383*^9, 3.479265953894*^9}, {3.5014929995752945`*^9, 
  3.5014930055496364`*^9}}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJxTTMoPSmViYGBwAGIQXfO1eM3N15PsGKBg3ovTh3yjC+1h/G39Eq/Voxvh
fKfHdv9FoifC+Ry8QZ3c0Qvh/NcLo8+xRW+E82NzHhSwRB+E86ddVHJijr4I
5984zMJ/X/QRlL9gj8vK/yGL772Cy2dO7apeuvcjnM+0bWtdeuY3OP/07Cdv
Y47+gvPTHhr3H6j7D+d3Wnm85xJkcoDxv6f98DlvxQLn+x3ecdhrMhucb8tU
/eH+YQ443/SF85fcl1wOMPeJ/ZMR3GzLC5fv//b9sWAWP5xf2f/x+eRHAnC+
fnmW52wuITjfMOBJ63o5Ybh56o7P/OS9RODymcovP81OFIXzF8iWKJdMEIPz
o7Ob9p/YKQ7nJ62aaKZ1XgJuXhyjptXz35Jw+WWeByU2ikjD+UVP7yR9V5eB
841mNljODJKF849pu3/5HIbgR+3jXe8XjeA3P56lwpKC4F/R3cqfX4rglx56
+dRlBoK/43XQpA93EXyfOklf70cI/gOBB+zLniH4nGa5tdHvYfwFe6Ia2jOO
McjB5X+L7LGbo4zg29mpvnbPlIOrN/ooK8KQh5BXWyJmu6MIwefj4ujXqEHw
7119ZcTZj+Bf6ngcfXAygn/M+k5L5QwEf93Cs9deLkTw63I3VJ3aguAXKaxc
3LQTwU+7vPCM1T4E389ystzq4wi+05tu9+QzCL7Z/JYC6YsIvhxr2aHuWwi+
0I681873EXy27HSRP48R/F+yCbZbXiD47y5EpOW8RfBjCkrvvv2I4AMAbg/r
ig==
      "]], LineBox[CompressedData["
1:eJwBAQP+/CFib1JlAgAAAC8AAAACAAAAwO3Gl/P6HkCxcnwnst3fP8rpQuGU
+x5AKkNMmUHl3z++nDsKTQAfQE5TesLxDuA/sk80MwUFH0AFhU64QivgP6YC
LVy9CR9AvLYirpNH4D+OaB6uLRMfQCway5k1gOA/XjQBUg4mH0AM4RtxefHg
P/7LxpnPSx9Azm69HwHU4T8/+1EpUpcfQFSKAH0QmeM/grv4Q/SbH0DkC+kc
3bTjP8R7n16WoB9AcY3RvKnQ4z9I/OyT2qkfQImQovxCCOQ/UP2H/mK8H0C5
lkR8dXfkP2D/vdNz4R9AGaOIe9pV5T/BARW/yhUgQOa7EHqkEuc/4gWBaexf
IEBx7SB3OIzqP8qG2Ss26iBAKXyjydaD8D+sE3a0LYAhQHXJTv2jB/Q/NBmT
qisMIkCk6vzBl0/3P37IcdZnlSJAYAY1yQCH+j/Dg5TIUSojQP5pBXZ8BP4/
rrc3KEK1I0DAUOxZD6MAQJP3Hk7gSyRAbxCiy+lmAkAesIbhhNgkQBA62YXX
DARAaxKwqmdiJUD3YFXhf6oFQCTI7ey9ZCVAI4IOqIKxBUDefSsvFGclQFCj
x26FuAVAUOmms8BrJUCm5Tn8isYFQDTAnbwZdSVAUmoeF5biBUD8bYvOy4cl
QKpz50ysGgZAjslm8i+tJUBghnm42IoGQEd/pDSGryVAi6cyf9uRBkAANeJ2
3LElQLbI60XemAZAcqBd+4i2JUAMC17T46YGQFd3VATivyVAu49C7u7CBkAg
JUIWlNIlQBaZCyQF+wZAsoAdOvj3JUDNq52PMWsHQE44EZYm+iVAoNJ4o7xx
B0Dq7wTyVPwlQHT5U7dHeAdAIV/sqbEAJkAZRwrfXYUHQJA9uxlrCSZAZuJ2
LoqfB0As9a51mQsmQDoJUkIVpgdAx6yi0ccNJkALMC1WoKwHQP4biokkEiZA
sX3jfba5B0Ca033lUhQmQISkvpFBwAdANotxQYEWJkBYy5mlzMYHQDq5/BrK
GiZAZVU7MqfTB0B86lCH
      "]], LineBox[CompressedData["
1:eJwBQQK+/SFib1JlAgAAACMAAAACAAAA7Hcl+EwfJkDT8Ek4ceMHQNzYJ2mX
IyZAk3RT/Jr0B0BKt/bYUCwmQEvujruAFwhAKHSUuMM9JkDD4QU6TF0IQJ9n
CzePgyZAn6/hM3p0CUCHWj360xonQD97qUCN0QtAMfcw81avJ0Do7XckmSMO
QIEMpVngOShAlCEkX98mEEDLLV2GF9AoQChklLhNUxFAu8eVIFVcKUAImAXt
yGsSQG0LkPDQ5SlAbB/6jMB+E0AaW86G+noqQMa+drkTqRRAbSONiioGK0Bs
T/TAc78VQCpXowqYCCtA5rYgwU7EFUDoirmKBQsrQGIeTcEpyRVAYvLliuAP
K0BW7aXB39IVQFjBPouWGStAQotXwkvmFUBEX/CLAi0rQBrHusMjDRZAHJtT
jdpTK0DKPoHG01oWQNrOaQ1IVitARqatxq5fFkCXAoCNtVgrQMAN2saJZBZA
EmqsjZBdK0C23DLHP24WQAg5BY5GZytAonrkx6uBFkDz1raOsnorQHi2R8mD
qBZAsArNDiB9K0DyHXTJXq0WQG4+446NfytAboWgyTmyFkDopQ+PaIQrQGJU
+cnvuxZA3nRojx6OK0BO8qrKW88WQJyofg+MkCtAylnXyjbUFkBZ3JSP+ZIr
QETBA8sR2RZA1EPBj9SXK0A6kFzLx+IWQJJ31w9CmitAtveIy6LnFkBPq+2P
r5wrQDBftct97BZADN8DEB2fK0CqxuHLWPEWQMoSGpCKoStAJi4OzDP2FkDB
FPT8
      "]]}, {}}, {{}, {}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["Re", TraditionalForm], 
    FormBox["f", TraditionalForm]},
  AxesOrigin->{4.499809670330265, -0.35667494393873245`},
  CoordinatesToolOptions:>{"DisplayFunction" -> ({
      Exp[
       Part[#, 1]], 
      Exp[
       Part[#, 2]]}& ), "CopiedValueFunction" -> ({
      Exp[
       Part[#, 1]], 
      Exp[
       Part[#, 2]]}& )},
  FrameTicks->{{{{0., 
       FormBox["1", TraditionalForm]}, {1.6094379124341003`, 
       FormBox["5", TraditionalForm]}, {2.302585092994046, 
       FormBox["10", TraditionalForm]}, {3.912023005428146, 
       FormBox["50", TraditionalForm]}, {4.605170185988092, 
       FormBox["100", TraditionalForm]}, {6.214608098422191, 
       FormBox["500", TraditionalForm]}, {6.907755278982137, 
       FormBox["1000", TraditionalForm]}, {8.517193191416238, 
       FormBox["5000", TraditionalForm]}, {0.6931471805599453, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.0986122886681098`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.3862943611198906`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.791759469228055, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.9459101490553132`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.0794415416798357`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.1972245773362196`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.995732273553991, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {3.4011973816621555`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {3.6888794541139363`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.0943445622221, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.248495242049359, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.382026634673881, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.499809670330265, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}, {{0., 
       FormBox["\"\"", TraditionalForm]}, {1.6094379124341003`, 
       FormBox["\"\"", TraditionalForm]}, {2.302585092994046, 
       FormBox["\"\"", TraditionalForm]}, {3.912023005428146, 
       FormBox["\"\"", TraditionalForm]}, {4.605170185988092, 
       FormBox["\"\"", TraditionalForm]}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm]}, {6.907755278982137, 
       FormBox["\"\"", TraditionalForm]}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm]}, {0.6931471805599453, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.0986122886681098`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.3862943611198906`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.791759469228055, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.9459101490553132`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.0794415416798357`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.1972245773362196`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.995732273553991, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {3.4011973816621555`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {3.6888794541139363`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.0943445622221, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.248495242049359, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.382026634673881, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.499809670330265, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}}, {{{4.605170185988092, 
       FormBox["100", TraditionalForm]}, {6.907755278982137, 
       FormBox["1000", TraditionalForm]}, {9.210340371976184, 
       FormBox[
        TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {11.512925464970229`, 
       FormBox[
        TemplateBox[{"10", "5"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {13.815510557964274`, 
       FormBox[
        TemplateBox[{"10", "6"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.699514748210191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.85366542803745, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.987196820661973, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.104979856318357, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.903487552536127, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.308952660644293`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.596634733096073`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.819778284410283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.002099841204238`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.156250521031495`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.289781913656018`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.407564949312402`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.206072645530174`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.611537753638338`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.89921982609012, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.122363377404328`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.304684934198283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.458835614025542`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.592367006650065`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.710150042306449`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}, {{4.605170185988092, 
       FormBox["\"\"", TraditionalForm]}, {6.907755278982137, 
       FormBox["\"\"", TraditionalForm]}, {9.210340371976184, 
       FormBox["\"\"", TraditionalForm]}, {11.512925464970229`, 
       FormBox["\"\"", TraditionalForm]}, {13.815510557964274`, 
       FormBox["\"\"", TraditionalForm]}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.699514748210191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.85366542803745, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.987196820661973, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.104979856318357, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.903487552536127, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.308952660644293`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.596634733096073`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.819778284410283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.002099841204238`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.156250521031495`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.289781913656018`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.407564949312402`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.206072645530174`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.611537753638338`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.89921982609012, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.122363377404328`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.304684934198283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.458835614025542`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.592367006650065`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.710150042306449`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}}},
  GridLines->{None, None},
  Method->{},
  PlotRange->{{4.499809670330265, 13.9108207377686}, {-0.35667494393873245`, 
   6.396929655216146}},
  PlotRangeClipping->True,
  PlotRangePadding->{{None, None}, {None, None}},
  Ticks->{{{4.605170185988092, 
      FormBox["100", TraditionalForm]}, {6.907755278982137, 
      FormBox["1000", TraditionalForm]}, {9.210340371976184, 
      FormBox[
       TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {11.512925464970229`, 
      FormBox[
       TemplateBox[{"10", "5"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {13.815510557964274`, 
      FormBox[
       TemplateBox[{"10", "6"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {5.298317366548036, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.703782474656201, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.991464547107982, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.214608098422191, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.396929655216146, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.551080335043404, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.684611727667927, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.802394763324311, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {7.600902459542082, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.006367567650246, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.294049640102028, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.517193191416238, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.699514748210191, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.85366542803745, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.987196820661973, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {9.104979856318357, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {9.903487552536127, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.308952660644293`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.596634733096073`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.819778284410283`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.002099841204238`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.156250521031495`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.289781913656018`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.407564949312402`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.206072645530174`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.611537753638338`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.89921982609012, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.122363377404328`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.304684934198283`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.458835614025542`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.592367006650065`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.710150042306449`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}}, {{0., 
      FormBox["1", TraditionalForm]}, {1.6094379124341003`, 
      FormBox["5", TraditionalForm]}, {2.302585092994046, 
      FormBox["10", TraditionalForm]}, {3.912023005428146, 
      FormBox["50", TraditionalForm]}, {4.605170185988092, 
      FormBox["100", TraditionalForm]}, {6.214608098422191, 
      FormBox["500", TraditionalForm]}, {6.907755278982137, 
      FormBox["1000", TraditionalForm]}, {8.517193191416238, 
      FormBox["5000", TraditionalForm]}, {0.6931471805599453, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {1.0986122886681098`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {1.3862943611198906`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {1.791759469228055, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {1.9459101490553132`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {2.0794415416798357`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {2.1972245773362196`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {2.995732273553991, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {3.4011973816621555`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {3.6888794541139363`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {4.0943445622221, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {4.248495242049359, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {4.382026634673881, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {4.499809670330265, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.298317366548036, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.703782474656201, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.991464547107982, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.396929655216146, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.551080335043404, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.684611727667927, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.802394763324311, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {7.600902459542082, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.006367567650246, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.294049640102028, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}}}]], "Output",
 CellChangeTimes->{3.5755484736886063`*^9, 3.5865829493705096`*^9, 
  3.5875545158913713`*^9, 3.587606420695939*^9, 3.587635964823264*^9, 
  3.589077968989519*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"figb", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{"Black", ",", "Dotted", ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Log", "[", "R1", "]"}], ",", 
           RowBox[{"Log", "[", "0.7", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Log", "[", "R1", "]"}], ",", 
           RowBox[{"Log", "[", "500", "]"}]}], "}"}]}], "}"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Log", "[", "R2", "]"}], ",", 
           RowBox[{"Log", "[", "0.7", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Log", "[", "R2", "]"}], ",", 
           RowBox[{"Log", "[", "500", "]"}]}], "}"}]}], "}"}], "]"}], ",", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<laminar\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Log", "[", "500", "]"}], ",", 
          RowBox[{"Log", "[", "500", "]"}]}], "}"}]}], "]"}], ",", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<smooth\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{"1.2", " ", 
            SuperscriptBox["10", "4"]}], "]"}], ",", 
          RowBox[{"Log", "[", "500", "]"}]}], "}"}]}], "]"}], ",", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<rough\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{"2", " ", 
            SuperscriptBox["10", "5"]}], "]"}], ",", 
          RowBox[{"Log", "[", "500", "]"}]}], "}"}]}], "]"}]}], "}"}], 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4231142217650003`*^9, 3.423114224468*^9}, 
   3.423114254751*^9, {3.423114308553*^9, 3.423114320617*^9}, {
   3.423114387098*^9, 3.42311438788*^9}, {3.423114419976*^9, 
   3.423114431*^9}, {3.423114687866*^9, 3.423114724632*^9}, {
   3.423114768233*^9, 3.42311483465*^9}, {3.4231151727279997`*^9, 
   3.423115535578*^9}, {3.4231161950030003`*^9, 3.423116208898*^9}, {
   3.423116785675*^9, 3.423116785738*^9}, {3.423116833435*^9, 
   3.4231168367139997`*^9}, {3.479265945135*^9, 3.479265945935*^9}, 
   3.479265994336*^9}],

Cell["Plot drag factor", "Text",
 CellChangeTimes->{{3.587632976528714*^9, 3.5876329787189736`*^9}, {
  3.587633884607667*^9, 3.587633886590906*^9}, {3.5876340515827026`*^9, 
  3.5876340529108624`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figa", ",", "figb"}], "]"}], "//", "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<pipes-frictionfactor\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4231142051619997`*^9, 3.423114208836*^9}, {
   3.42311432562*^9, 3.423114326868*^9}, 3.423114692597*^9, {
   3.423115539429*^9, 3.423115552116*^9}, {3.423116613222*^9, 
   3.4231166221809998`*^9}, {3.423116685333*^9, 3.4231166858129997`*^9}, 
   3.423288733324417*^9, 3.4309107825336*^9, {3.440330784642432*^9, 
   3.440330792440432*^9}, {3.4405792787985544`*^9, 3.4405792844775543`*^9}, {
   3.4405793560615544`*^9, 3.4405793562855544`*^9}, {3.4792658916689997`*^9, 
   3.4792658951730003`*^9}, {3.501493093572671*^9, 3.501493105132332*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755484739436383`*^9, 3.586582949675539*^9, 
  3.5875545162634153`*^9, 3.587606420975973*^9, 3.587635965088296*^9, 
  3.589077969388569*^9}],

Cell[BoxData[
 GraphicsBox[{{{{}, {}, 
     {GrayLevel[0], LineBox[CompressedData["
1:eJxTTMoPSmViYGBwAGIQXfO1eM3N15PsGKBg3ovTh3yjC+1h/G39Eq/Voxvh
fKfHdv9FoifC+Ry8QZ3c0Qvh/NcLo8+xRW+E82NzHhSwRB+E86ddVHJijr4I
5984zMJ/X/QRlL9gj8vK/yGL772Cy2dO7apeuvcjnM+0bWtdeuY3OP/07Cdv
Y47+gvPTHhr3H6j7D+d3Wnm85xJkcoDxv6f98DlvxQLn+x3ecdhrMhucb8tU
/eH+YQ443/SF85fcl1wOMPeJ/ZMR3GzLC5fv//b9sWAWP5xf2f/x+eRHAnC+
fnmW52wuITjfMOBJ63o5Ybh56o7P/OS9RODymcovP81OFIXzF8iWKJdMEIPz
o7Ob9p/YKQ7nJ62aaKZ1XgJuXhyjptXz35Jw+WWeByU2ikjD+UVP7yR9V5eB
841mNljODJKF849pu3/5HIbgR+3jXe8XjeA3P56lwpKC4F/R3cqfX4rglx56
+dRlBoK/43XQpA93EXyfOklf70cI/gOBB+zLniH4nGa5tdHvYfwFe6Ia2jOO
McjB5X+L7LGbo4zg29mpvnbPlIOrN/ooK8KQh5BXWyJmu6MIwefj4ujXqEHw
7119ZcTZj+Bf6ngcfXAygn/M+k5L5QwEf93Cs9deLkTw63I3VJ3aguAXKaxc
3LQTwU+7vPCM1T4E389ystzq4wi+05tu9+QzCL7Z/JYC6YsIvhxr2aHuWwi+
0I681873EXy27HSRP48R/F+yCbZbXiD47y5EpOW8RfBjCkrvvv2I4AMAbg/r
ig==
       "]], LineBox[CompressedData["
1:eJwBAQP+/CFib1JlAgAAAC8AAAACAAAAwO3Gl/P6HkCxcnwnst3fP8rpQuGU
+x5AKkNMmUHl3z++nDsKTQAfQE5TesLxDuA/sk80MwUFH0AFhU64QivgP6YC
LVy9CR9AvLYirpNH4D+OaB6uLRMfQCway5k1gOA/XjQBUg4mH0AM4RtxefHg
P/7LxpnPSx9Azm69HwHU4T8/+1EpUpcfQFSKAH0QmeM/grv4Q/SbH0DkC+kc
3bTjP8R7n16WoB9AcY3RvKnQ4z9I/OyT2qkfQImQovxCCOQ/UP2H/mK8H0C5
lkR8dXfkP2D/vdNz4R9AGaOIe9pV5T/BARW/yhUgQOa7EHqkEuc/4gWBaexf
IEBx7SB3OIzqP8qG2Ss26iBAKXyjydaD8D+sE3a0LYAhQHXJTv2jB/Q/NBmT
qisMIkCk6vzBl0/3P37IcdZnlSJAYAY1yQCH+j/Dg5TIUSojQP5pBXZ8BP4/
rrc3KEK1I0DAUOxZD6MAQJP3Hk7gSyRAbxCiy+lmAkAesIbhhNgkQBA62YXX
DARAaxKwqmdiJUD3YFXhf6oFQCTI7ey9ZCVAI4IOqIKxBUDefSsvFGclQFCj
x26FuAVAUOmms8BrJUCm5Tn8isYFQDTAnbwZdSVAUmoeF5biBUD8bYvOy4cl
QKpz50ysGgZAjslm8i+tJUBghnm42IoGQEd/pDSGryVAi6cyf9uRBkAANeJ2
3LElQLbI60XemAZAcqBd+4i2JUAMC17T46YGQFd3VATivyVAu49C7u7CBkAg
JUIWlNIlQBaZCyQF+wZAsoAdOvj3JUDNq52PMWsHQE44EZYm+iVAoNJ4o7xx
B0Dq7wTyVPwlQHT5U7dHeAdAIV/sqbEAJkAZRwrfXYUHQJA9uxlrCSZAZuJ2
LoqfB0As9a51mQsmQDoJUkIVpgdAx6yi0ccNJkALMC1WoKwHQP4biokkEiZA
sX3jfba5B0Ca033lUhQmQISkvpFBwAdANotxQYEWJkBYy5mlzMYHQDq5/BrK
GiZAZVU7MqfTB0B86lCH
       "]], LineBox[CompressedData["
1:eJwBQQK+/SFib1JlAgAAACMAAAACAAAA7Hcl+EwfJkDT8Ek4ceMHQNzYJ2mX
IyZAk3RT/Jr0B0BKt/bYUCwmQEvujruAFwhAKHSUuMM9JkDD4QU6TF0IQJ9n
CzePgyZAn6/hM3p0CUCHWj360xonQD97qUCN0QtAMfcw81avJ0Do7XckmSMO
QIEMpVngOShAlCEkX98mEEDLLV2GF9AoQChklLhNUxFAu8eVIFVcKUAImAXt
yGsSQG0LkPDQ5SlAbB/6jMB+E0AaW86G+noqQMa+drkTqRRAbSONiioGK0Bs
T/TAc78VQCpXowqYCCtA5rYgwU7EFUDoirmKBQsrQGIeTcEpyRVAYvLliuAP
K0BW7aXB39IVQFjBPouWGStAQotXwkvmFUBEX/CLAi0rQBrHusMjDRZAHJtT
jdpTK0DKPoHG01oWQNrOaQ1IVitARqatxq5fFkCXAoCNtVgrQMAN2saJZBZA
EmqsjZBdK0C23DLHP24WQAg5BY5GZytAonrkx6uBFkDz1raOsnorQHi2R8mD
qBZAsArNDiB9K0DyHXTJXq0WQG4+446NfytAboWgyTmyFkDopQ+PaIQrQGJU
+cnvuxZA3nRojx6OK0BO8qrKW88WQJyofg+MkCtAylnXyjbUFkBZ3JSP+ZIr
QETBA8sR2RZA1EPBj9SXK0A6kFzLx+IWQJJ31w9CmitAtveIy6LnFkBPq+2P
r5wrQDBftct97BZADN8DEB2fK0CqxuHLWPEWQMoSGpCKoStAJi4OzDP2FkDB
FPT8
       "]]}, {}}, {{}, {}, {}}}, 
   {GrayLevel[0], Dashing[{0, Small}], 
    LineBox[NCache[{{Log[2300], -0.35667494393873245`}, {
       Log[2300], Log[500]}}, {{7.7406644019172415`, -0.35667494393873245`}, {
      7.7406644019172415`, 6.214608098422191}}]], 
    LineBox[NCache[{{11.056728648149408`, -0.35667494393873245`}, {
       11.056728648149408`, Log[500]}}, {{
      11.056728648149408`, -0.35667494393873245`}, {11.056728648149408`, 
      6.214608098422191}}]], 
    InsetBox["\<\"laminar\"\>", 
     NCache[{Log[500], Log[500]}, {6.214608098422191, 6.214608098422191}]], 
    InsetBox["\<\"smooth\"\>", 
     NCache[{9.392661928770137, Log[500]}, {9.392661928770137, 
      6.214608098422191}]], 
    InsetBox["\<\"rough\"\>", 
     NCache[{Log[200000], Log[500]}, {12.206072645530174`, 
      6.214608098422191}]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["Re", TraditionalForm], 
    FormBox["f", TraditionalForm]},
  AxesOrigin->{4.499809670330265, -0.35667494393873245`},
  CoordinatesToolOptions:>{"DisplayFunction" -> ({
      Exp[
       Part[#, 1]], 
      Exp[
       Part[#, 2]]}& ), "CopiedValueFunction" -> ({
      Exp[
       Part[#, 1]], 
      Exp[
       Part[#, 2]]}& )},
  FrameTicks->{{{{0., 
       FormBox["1", TraditionalForm]}, {1.6094379124341003`, 
       FormBox["5", TraditionalForm]}, {2.302585092994046, 
       FormBox["10", TraditionalForm]}, {3.912023005428146, 
       FormBox["50", TraditionalForm]}, {4.605170185988092, 
       FormBox["100", TraditionalForm]}, {6.214608098422191, 
       FormBox["500", TraditionalForm]}, {6.907755278982137, 
       FormBox["1000", TraditionalForm]}, {8.517193191416238, 
       FormBox["5000", TraditionalForm]}, {0.6931471805599453, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.0986122886681098`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.3862943611198906`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.791759469228055, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.9459101490553132`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.0794415416798357`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.1972245773362196`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.995732273553991, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {3.4011973816621555`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {3.6888794541139363`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.0943445622221, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.248495242049359, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.382026634673881, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.499809670330265, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}, {{0., 
       FormBox["\"\"", TraditionalForm]}, {1.6094379124341003`, 
       FormBox["\"\"", TraditionalForm]}, {2.302585092994046, 
       FormBox["\"\"", TraditionalForm]}, {3.912023005428146, 
       FormBox["\"\"", TraditionalForm]}, {4.605170185988092, 
       FormBox["\"\"", TraditionalForm]}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm]}, {6.907755278982137, 
       FormBox["\"\"", TraditionalForm]}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm]}, {0.6931471805599453, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.0986122886681098`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.3862943611198906`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.791759469228055, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {1.9459101490553132`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.0794415416798357`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.1972245773362196`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {2.995732273553991, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {3.4011973816621555`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {3.6888794541139363`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.0943445622221, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.248495242049359, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.382026634673881, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {4.499809670330265, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}}, {{{4.605170185988092, 
       FormBox["100", TraditionalForm]}, {6.907755278982137, 
       FormBox["1000", TraditionalForm]}, {9.210340371976184, 
       FormBox[
        TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {11.512925464970229`, 
       FormBox[
        TemplateBox[{"10", "5"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {13.815510557964274`, 
       FormBox[
        TemplateBox[{"10", "6"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.699514748210191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.85366542803745, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.987196820661973, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.104979856318357, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.903487552536127, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.308952660644293`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.596634733096073`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.819778284410283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.002099841204238`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.156250521031495`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.289781913656018`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.407564949312402`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.206072645530174`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.611537753638338`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.89921982609012, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.122363377404328`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.304684934198283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.458835614025542`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.592367006650065`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.710150042306449`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}, {{4.605170185988092, 
       FormBox["\"\"", TraditionalForm]}, {6.907755278982137, 
       FormBox["\"\"", TraditionalForm]}, {9.210340371976184, 
       FormBox["\"\"", TraditionalForm]}, {11.512925464970229`, 
       FormBox["\"\"", TraditionalForm]}, {13.815510557964274`, 
       FormBox["\"\"", TraditionalForm]}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.699514748210191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.85366542803745, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.987196820661973, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.104979856318357, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.903487552536127, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.308952660644293`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.596634733096073`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.819778284410283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.002099841204238`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.156250521031495`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.289781913656018`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.407564949312402`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.206072645530174`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.611537753638338`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.89921982609012, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.122363377404328`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.304684934198283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.458835614025542`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.592367006650065`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.710150042306449`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}}},
  GridLines->{None, None},
  Method->{},
  PlotRange->{{4.499809670330265, 13.9108207377686}, {-0.35667494393873245`, 
   6.396929655216146}},
  PlotRangeClipping->True,
  PlotRangePadding->{{None, None}, {None, None}},
  Ticks->{{{4.605170185988092, 
      FormBox["100", TraditionalForm]}, {6.907755278982137, 
      FormBox["1000", TraditionalForm]}, {9.210340371976184, 
      FormBox[
       TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {11.512925464970229`, 
      FormBox[
       TemplateBox[{"10", "5"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {13.815510557964274`, 
      FormBox[
       TemplateBox[{"10", "6"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {5.298317366548036, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.703782474656201, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.991464547107982, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.214608098422191, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.396929655216146, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.551080335043404, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.684611727667927, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.802394763324311, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {7.600902459542082, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.006367567650246, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.294049640102028, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.517193191416238, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.699514748210191, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.85366542803745, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.987196820661973, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {9.104979856318357, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {9.903487552536127, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.308952660644293`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.596634733096073`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.819778284410283`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.002099841204238`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.156250521031495`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.289781913656018`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.407564949312402`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.206072645530174`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.611537753638338`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.89921982609012, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.122363377404328`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.304684934198283`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.458835614025542`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.592367006650065`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.710150042306449`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}}, {{0., 
      FormBox["1", TraditionalForm]}, {1.6094379124341003`, 
      FormBox["5", TraditionalForm]}, {2.302585092994046, 
      FormBox["10", TraditionalForm]}, {3.912023005428146, 
      FormBox["50", TraditionalForm]}, {4.605170185988092, 
      FormBox["100", TraditionalForm]}, {6.214608098422191, 
      FormBox["500", TraditionalForm]}, {6.907755278982137, 
      FormBox["1000", TraditionalForm]}, {8.517193191416238, 
      FormBox["5000", TraditionalForm]}, {0.6931471805599453, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {1.0986122886681098`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {1.3862943611198906`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {1.791759469228055, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {1.9459101490553132`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {2.0794415416798357`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {2.1972245773362196`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {2.995732273553991, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {3.4011973816621555`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {3.6888794541139363`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {4.0943445622221, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {4.248495242049359, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {4.382026634673881, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {4.499809670330265, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.298317366548036, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.703782474656201, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.991464547107982, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.396929655216146, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.551080335043404, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.684611727667927, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.802394763324311, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {7.600902459542082, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.006367567650246, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.294049640102028, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}}}]], "Output",
 CellChangeTimes->{3.5755484740496516`*^9, 3.5865829502256174`*^9, 
  3.5875545164654417`*^9, 3.5876064210709844`*^9, 3.587635965190309*^9, 
  3.5890779693945694`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Friction factor", "Subsection",
 CellChangeTimes->{{3.4231156807539997`*^9, 3.423115710193*^9}, {
  3.5021817330689187`*^9, 3.502181734162981*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figc", "=", 
  RowBox[{"LogLogPlot", "[", 
   RowBox[{
    RowBox[{"Cf", "[", "R", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"R", ",", "1", ",", 
      SuperscriptBox["10", "6"]}], "}"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Re", ",", 
       SubscriptBox["C", "f"]}], "}"}]}], ",", " ", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"90", ",", 
         RowBox[{"1.1", " ", 
          SuperscriptBox["10", "6"]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0.003", ",", "0.22"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4231157135690002`*^9, 3.423115853382*^9}, 
   3.4231158940559998`*^9, {3.423116020263*^9, 3.4231161374309998`*^9}, {
   3.479265961704*^9, 3.4792659656870003`*^9}, {3.5014930271748734`*^9, 
   3.50149303411027*^9}}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJwBwQI+/SFib1JlAgAAACsAAAACAAAAKY+T/5glEUDHyFkA3jn4vy93akmb
ChJA3Wi1J+fN+78xUOSFrx4TQHNmzgwcDwDAJ0HmTh9KFEBfSNKe+2UCwGkj
6fKbYRVA4wzY5vSUBMCgHXQjdJAWQFEB7kel8gbAW2uCv8i5F0DHnAqATkUJ
wGKqkTYqzxhA1RopbhFwC8BeASk65/sZQM3IV3WLyQ3ApknBGLEUG0BdWYgy
H/sPwHLl3GL3JxxAeshf49UQEcAymYA5mVIdQDp8A7p3OxLAxitH9PNWHUDP
Dsp00j8SwFq+Da9OWx1AYqGQLy1EEsCD45okBGQdQIvGHaXiTBLA1C21D291
HUDcEDiQTV4SwHXC6eVEmB1AfaVsZiOBEsC461KS8N0dQMDO1RLPxhLATH4Z
TUviHUBUYZzNKcsSwOAQ4Aem5h1A6PNiiITPEsAJNm19W+8dQBEZ8P052BLA
WoCHaMYAHkBiYwrppOkSwPsUvD6cIx5AA/g+v3oME8A+PiXrR2keQEYhqGsm
UhPAMvEdFABuHkA61KCU3lYTwCakFj24ch5ALoeZvZZbE8AOCgiPKHweQBbt
ig8HZRPA3tXqMgmPHkDmuG2z53cTwNKI41vBkx5A2mtm3J98E8DGO9yEeZge
QM4eXwVYgRPArqHN1umhHkC3hFBXyIoTwH5tsHrKtB5AhlAz+6idE8ByIKmj
grkeQHoDLCRhohPAZtOhzDq+HkButiRNGacTwE45kx6rxx5AVhwWn4mwE8BC
7ItHY8weQErPDshBtRPANp+EcBvRHkA+ggfx+bkTwB4FdsKL2h5AJuj4QmrD
E8ASuG7rQ98eQBqb8WsiyBPABmtnFPzjHkAOTuqU2swTwPodYD206B5AAgHj
vZLRE8Du0FhmbO0eQPaz2+ZK1hPAXHB13e3xHkBkU/hdzNoTwC8LOCw=
      "]], LineBox[CompressedData["
1:eJwVxX1MjHEcAPC7dAgdz3PPr6l0z1X3e3DXKeeUeVlfYWw56pZllmqHQqzF
ZJOoWMIKlVrUaiSr9e5tpuNLra4LrcxLoamJyuWMUqsrL3989vE0xhv2OYhE
Ir9//o/Dzfm/JuRQ5NQ5+rufQctgUG/upBxKZi/fwn9h8HHhmjl6EQ+7V8Yb
9F8ZrN+uWyGR8OBmrIpIGmCwzEETYXLi4W3Wt5jyQQaz9surNTIeQgdikyRW
BiN1YoNU4GHT9ehS0w8Gp9uaC9qDedCIDb81EwwGToYuNeTzMPVcd03qzGLa
o/ENI9d5kCb3KAKkLDYlFUfmlfAg12TcjprP4ha7Nae7nIfAzPd3axkWQ6bO
TRsbeEjVp7UbXFiM/mN6dbSXB8nLdsd8nsWnYvLEQhQgbT8Yz2tZ7JOkHrBG
KUDeUbLRL5xFS3qX2n9IAdnMC7qjkMVKWeJdTYonzNjpPhTylUWda06Fep4X
tHQv7d+ukeEpc8Kb2KtecEjrOjmcIMPGM7nmUJU3zL+sGjE2yLD6QcCSoHve
4Kiye78Xc5gzJt/2KVgJY602f5WeQ3lteu/Zd0pYW3Yz5WQOh3FsbUWskcKe
8FX9h99xqDQPW017KQRqw2cWdHHYk6z25WIpLHJOXNzUzWHIYNkdjKPw+tmd
A24fOdRhUYPrMQqbl/l+b+7l0H744su2cxTUEmGM/8ZhVmvMT98qCiP1MqfO
KQ43ny7VpddQ6MjUqqamORSt7Ev8UEehan9o8BIRwSMluycz7lOIkV/KPOVA
MPx4mGPfEwpd5+ew6lkEFTTIJbeTgila7H52AcH6FI/V42MU0qU2H39vgttW
MXnCBIWwoYFdeiXBIZvjzzA7Ba/mvoy9lKBnlLW8TiTA4+Q3n7MXE7y87pFr
nJMAo1ZToU1N0Ge0+ljBXAEazQ8sMzUEWypvdLQ4C3CltG7cYxnBafcL55Ws
AD4Rt8K2+hG89iq538AJMBFQnLZnOUH/iwnrU10EMMsKak9oCXYG7SuqWShA
ni2758oKgqsf2t0sbgJYKiI9inUE/wIc1FVL
      "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQAWIQ/aZc9YePvJpD+a0VV/abih5wUDThzVRE8O/cUM+c
rozgC3vwK35RR/C9tn+7EaCD4O+cetSDyxTB1yiZsuOwLYL/5O358pUJCP78
dG7z/mYk8xOXzDA8j+C3R9n+uiylDucbfjf4HLYewW/kWRr5wFIDzj+tG9sm
fgHB3318qkJojCacn8s94cOFpwi+VPS5tl9VWgh55d4uLTZtOF9mdnDvrWAE
/5SQ1ISuhQg+AJw5nRw=
      "]]}, {}}, {{}, {}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["Re", TraditionalForm], 
    FormBox[
     SubscriptBox["C", "f"], TraditionalForm]},
  AxesOrigin->{4.499809670330265, -5.809142990314028},
  CoordinatesToolOptions:>{"DisplayFunction" -> ({
      Exp[
       Part[#, 1]], 
      Exp[
       Part[#, 2]]}& ), "CopiedValueFunction" -> ({
      Exp[
       Part[#, 1]], 
      Exp[
       Part[#, 2]]}& )},
  FrameTicks->{{{{-6.907755278982137, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.001\"", 0.001, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-6.214608098422191, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.002\"", 0.002, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-5.298317366548036, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.005\"", 0.005, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-4.605170185988092, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.010\"", 0.01, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-3.912023005428146, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.020\"", 0.02, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-2.995732273553991, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.050\"", 0.05, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-2.302585092994046, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.100\"", 0.1, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-1.6094379124341003`, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.200\"", 0.2, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-0.6931471805599453, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.500\"", 0.5, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-5.809142990314028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-5.521460917862246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-5.115995809754082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.961845129926823, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.8283137373023015`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.710530701645918, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-3.506557897319982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-3.2188758248682006`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.8134107167600364`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.6592600369327783`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.5257286443082556`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.4079456086518722`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-1.2039728043259361`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-0.916290731874155, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}, {{-6.907755278982137, 
       FormBox["\"\"", TraditionalForm]}, {-6.214608098422191, 
       FormBox["\"\"", TraditionalForm]}, {-5.298317366548036, 
       FormBox["\"\"", TraditionalForm]}, {-4.605170185988092, 
       FormBox["\"\"", TraditionalForm]}, {-3.912023005428146, 
       FormBox["\"\"", TraditionalForm]}, {-2.995732273553991, 
       FormBox["\"\"", TraditionalForm]}, {-2.302585092994046, 
       FormBox["\"\"", TraditionalForm]}, {-1.6094379124341003`, 
       FormBox["\"\"", TraditionalForm]}, {-0.6931471805599453, 
       FormBox["\"\"", TraditionalForm]}, {-5.809142990314028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-5.521460917862246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-5.115995809754082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.961845129926823, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.8283137373023015`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.710530701645918, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-3.506557897319982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-3.2188758248682006`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.8134107167600364`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.6592600369327783`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.5257286443082556`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.4079456086518722`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-1.2039728043259361`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-0.916290731874155, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}}, {{{4.605170185988092, 
       FormBox["100", TraditionalForm]}, {6.907755278982137, 
       FormBox["1000", TraditionalForm]}, {9.210340371976184, 
       FormBox[
        TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {11.512925464970229`, 
       FormBox[
        TemplateBox[{"10", "5"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {13.815510557964274`, 
       FormBox[
        TemplateBox[{"10", "6"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.699514748210191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.85366542803745, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.987196820661973, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.104979856318357, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.903487552536127, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.308952660644293`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.596634733096073`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.819778284410283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.002099841204238`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.156250521031495`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.289781913656018`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.407564949312402`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.206072645530174`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.611537753638338`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.89921982609012, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.122363377404328`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.304684934198283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.458835614025542`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.592367006650065`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.710150042306449`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}, {{4.605170185988092, 
       FormBox["\"\"", TraditionalForm]}, {6.907755278982137, 
       FormBox["\"\"", TraditionalForm]}, {9.210340371976184, 
       FormBox["\"\"", TraditionalForm]}, {11.512925464970229`, 
       FormBox["\"\"", TraditionalForm]}, {13.815510557964274`, 
       FormBox["\"\"", TraditionalForm]}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.699514748210191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.85366542803745, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.987196820661973, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.104979856318357, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.903487552536127, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.308952660644293`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.596634733096073`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.819778284410283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.002099841204238`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.156250521031495`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.289781913656018`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.407564949312402`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.206072645530174`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.611537753638338`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.89921982609012, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.122363377404328`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.304684934198283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.458835614025542`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.592367006650065`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.710150042306449`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}}},
  GridLines->{None, None},
  Method->{},
  PlotRange->{{4.499809670330265, 
   13.9108207377686}, {-5.809142990314028, -1.5141277326297755`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{None, None}, {None, None}},
  Ticks->{{{4.605170185988092, 
      FormBox["100", TraditionalForm]}, {6.907755278982137, 
      FormBox["1000", TraditionalForm]}, {9.210340371976184, 
      FormBox[
       TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {11.512925464970229`, 
      FormBox[
       TemplateBox[{"10", "5"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {13.815510557964274`, 
      FormBox[
       TemplateBox[{"10", "6"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {5.298317366548036, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.703782474656201, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.991464547107982, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.214608098422191, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.396929655216146, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.551080335043404, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.684611727667927, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.802394763324311, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {7.600902459542082, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.006367567650246, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.294049640102028, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.517193191416238, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.699514748210191, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.85366542803745, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.987196820661973, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {9.104979856318357, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {9.903487552536127, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.308952660644293`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.596634733096073`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.819778284410283`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.002099841204238`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.156250521031495`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.289781913656018`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.407564949312402`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.206072645530174`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.611537753638338`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.89921982609012, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.122363377404328`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.304684934198283`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.458835614025542`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.592367006650065`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.710150042306449`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}}, {{-6.907755278982137, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.001\"", 0.001, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-6.214608098422191, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.002\"", 0.002, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-5.298317366548036, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.005\"", 0.005, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-4.605170185988092, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.010\"", 0.01, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-3.912023005428146, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.020\"", 0.02, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-2.995732273553991, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.050\"", 0.05, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-2.302585092994046, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.100\"", 0.1, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-1.6094379124341003`, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.200\"", 0.2, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-0.6931471805599453, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.500\"", 0.5, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-5.809142990314028, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-5.521460917862246, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-5.115995809754082, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-4.961845129926823, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-4.8283137373023015`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-4.710530701645918, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-3.506557897319982, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-3.2188758248682006`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-2.8134107167600364`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-2.6592600369327783`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-2.5257286443082556`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-2.4079456086518722`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-1.2039728043259361`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-0.916290731874155, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}}}]], "Output",
 CellChangeTimes->{3.5755484742446733`*^9, 3.5865829503816643`*^9, 
  3.5875545173235426`*^9, 3.5876064212850103`*^9, 3.587635965388331*^9, 
  3.5890779696436415`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"figd", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{"Black", ",", "Dotted", ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Log", "[", "R1", "]"}], ",", 
           RowBox[{"Log", "[", 
            SuperscriptBox["10", 
             RowBox[{"-", "4"}]], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Log", "[", "R1", "]"}], ",", 
           RowBox[{"Log", "[", "0.2", "]"}]}], "}"}]}], "}"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Log", "[", "R2", "]"}], ",", 
           RowBox[{"Log", "[", 
            SuperscriptBox["10", 
             RowBox[{"-", "4"}]], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Log", "[", "R2", "]"}], ",", 
           RowBox[{"Log", "[", "0.2", "]"}]}], "}"}]}], "}"}], "]"}], ",", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<laminar\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Log", "[", "500", "]"}], ",", 
          RowBox[{"Log", "[", "0.2", "]"}]}], "}"}]}], "]"}], ",", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<smooth\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{"1.2", " ", 
            SuperscriptBox["10", "4"]}], "]"}], ",", 
          RowBox[{"Log", "[", "0.2", "]"}]}], "}"}]}], "]"}], ",", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<rough\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{"2", " ", 
            SuperscriptBox["10", "5"]}], "]"}], ",", 
          RowBox[{"Log", "[", "0.2", "]"}]}], "}"}]}], "]"}]}], "}"}], 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.423116144236*^9, 3.4231161619779997`*^9}, {
  3.423116249035*^9, 3.4231163541540003`*^9}, {3.47926593584*^9, 
  3.479265937863*^9}}],

Cell["Plot friction factor", "Text",
 CellChangeTimes->{{3.587634126799728*^9, 3.587634131006231*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figc", ",", "figd"}], "]"}], "//", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<pipes-frictioncoefficient\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.423116144236*^9, 3.4231161619779997`*^9}, {
   3.423116249035*^9, 3.423116273442*^9}, {3.423116363093*^9, 
   3.423116373604*^9}, {3.423116435446*^9, 3.42311643898*^9}, {
   3.423116680293*^9, 3.423116680389*^9}, {3.4232886207964168`*^9, 
   3.423288622019417*^9}, {3.423288693717417*^9, 3.4232886955994167`*^9}, 
   3.4309107840566*^9, {3.440330801993432*^9, 3.440330807770432*^9}, {
   3.4405792725585546`*^9, 3.440579289165554*^9}, {3.4405793297745543`*^9, 
   3.4405793300935545`*^9}, {3.479265911538*^9, 3.479265912885*^9}, {
   3.50149305579451*^9, 3.5014930732115064`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755484746427217`*^9, 3.58658295068466*^9, 
  3.5875545180596313`*^9, 3.5876064216270514`*^9, 3.587635965801383*^9, 
  3.5890779698406334`*^9}],

Cell[BoxData[
 GraphicsBox[{{{{}, {}, 
     {GrayLevel[0], LineBox[CompressedData["
1:eJwBwQI+/SFib1JlAgAAACsAAAACAAAAKY+T/5glEUDHyFkA3jn4vy93akmb
ChJA3Wi1J+fN+78xUOSFrx4TQHNmzgwcDwDAJ0HmTh9KFEBfSNKe+2UCwGkj
6fKbYRVA4wzY5vSUBMCgHXQjdJAWQFEB7kel8gbAW2uCv8i5F0DHnAqATkUJ
wGKqkTYqzxhA1RopbhFwC8BeASk65/sZQM3IV3WLyQ3ApknBGLEUG0BdWYgy
H/sPwHLl3GL3JxxAeshf49UQEcAymYA5mVIdQDp8A7p3OxLAxitH9PNWHUDP
Dsp00j8SwFq+Da9OWx1AYqGQLy1EEsCD45okBGQdQIvGHaXiTBLA1C21D291
HUDcEDiQTV4SwHXC6eVEmB1AfaVsZiOBEsC461KS8N0dQMDO1RLPxhLATH4Z
TUviHUBUYZzNKcsSwOAQ4Aem5h1A6PNiiITPEsAJNm19W+8dQBEZ8P052BLA
WoCHaMYAHkBiYwrppOkSwPsUvD6cIx5AA/g+v3oME8A+PiXrR2keQEYhqGsm
UhPAMvEdFABuHkA61KCU3lYTwCakFj24ch5ALoeZvZZbE8AOCgiPKHweQBbt
ig8HZRPA3tXqMgmPHkDmuG2z53cTwNKI41vBkx5A2mtm3J98E8DGO9yEeZge
QM4eXwVYgRPArqHN1umhHkC3hFBXyIoTwH5tsHrKtB5AhlAz+6idE8ByIKmj
grkeQHoDLCRhohPAZtOhzDq+HkButiRNGacTwE45kx6rxx5AVhwWn4mwE8BC
7ItHY8weQErPDshBtRPANp+EcBvRHkA+ggfx+bkTwB4FdsKL2h5AJuj4QmrD
E8ASuG7rQ98eQBqb8WsiyBPABmtnFPzjHkAOTuqU2swTwPodYD206B5AAgHj
vZLRE8Du0FhmbO0eQPaz2+ZK1hPAXHB13e3xHkBkU/hdzNoTwC8LOCw=
       "]], LineBox[CompressedData["
1:eJwVxX1MjHEcAPC7dAgdz3PPr6l0z1X3e3DXKeeUeVlfYWw56pZllmqHQqzF
ZJOoWMIKlVrUaiSr9e5tpuNLra4LrcxLoamJyuWMUqsrL3989vE0xhv2OYhE
Ir9//o/Dzfm/JuRQ5NQ5+rufQctgUG/upBxKZi/fwn9h8HHhmjl6EQ+7V8Yb
9F8ZrN+uWyGR8OBmrIpIGmCwzEETYXLi4W3Wt5jyQQaz9surNTIeQgdikyRW
BiN1YoNU4GHT9ehS0w8Gp9uaC9qDedCIDb81EwwGToYuNeTzMPVcd03qzGLa
o/ENI9d5kCb3KAKkLDYlFUfmlfAg12TcjprP4ha7Nae7nIfAzPd3axkWQ6bO
TRsbeEjVp7UbXFiM/mN6dbSXB8nLdsd8nsWnYvLEQhQgbT8Yz2tZ7JOkHrBG
KUDeUbLRL5xFS3qX2n9IAdnMC7qjkMVKWeJdTYonzNjpPhTylUWda06Fep4X
tHQv7d+ukeEpc8Kb2KtecEjrOjmcIMPGM7nmUJU3zL+sGjE2yLD6QcCSoHve
4Kiye78Xc5gzJt/2KVgJY602f5WeQ3lteu/Zd0pYW3Yz5WQOh3FsbUWskcKe
8FX9h99xqDQPW017KQRqw2cWdHHYk6z25WIpLHJOXNzUzWHIYNkdjKPw+tmd
A24fOdRhUYPrMQqbl/l+b+7l0H744su2cxTUEmGM/8ZhVmvMT98qCiP1MqfO
KQ43ny7VpddQ6MjUqqamORSt7Ev8UEehan9o8BIRwSMluycz7lOIkV/KPOVA
MPx4mGPfEwpd5+ew6lkEFTTIJbeTgila7H52AcH6FI/V42MU0qU2H39vgttW
MXnCBIWwoYFdeiXBIZvjzzA7Ba/mvoy9lKBnlLW8TiTA4+Q3n7MXE7y87pFr
nJMAo1ZToU1N0Ge0+ljBXAEazQ8sMzUEWypvdLQ4C3CltG7cYxnBafcL55Ws
AD4Rt8K2+hG89iq538AJMBFQnLZnOUH/iwnrU10EMMsKak9oCXYG7SuqWShA
ni2758oKgqsf2t0sbgJYKiI9inUE/wIc1FVL
       "]], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQAWIQ/aZc9YePvJpD+a0VV/abih5wUDThzVRE8O/cUM+c
rozgC3vwK35RR/C9tn+7EaCD4O+cetSDyxTB1yiZsuOwLYL/5O358pUJCP78
dG7z/mYk8xOXzDA8j+C3R9n+uiylDucbfjf4HLYewW/kWRr5wFIDzj+tG9sm
fgHB3318qkJojCacn8s94cOFpwi+VPS5tl9VWgh55d4uLTZtOF9mdnDvrWAE
/5SQ1ISuhQg+AJw5nRw=
       "]]}, {}}, {{}, {}, {}}}, 
   {GrayLevel[0], Dashing[{0, Small}], 
    LineBox[NCache[{{Log[2300], -Log[10000]}, {
       Log[2300], -1.6094379124341003`}}, {{
      7.7406644019172415`, -9.210340371976184}, {
      7.7406644019172415`, -1.6094379124341003`}}]], 
    LineBox[NCache[{{11.056728648149408`, -Log[10000]}, {
       11.056728648149408`, -1.6094379124341003`}}, {{
      11.056728648149408`, -9.210340371976184}, {
      11.056728648149408`, -1.6094379124341003`}}]], 
    InsetBox["\<\"laminar\"\>", 
     NCache[{Log[500], -1.6094379124341003`}, {
      6.214608098422191, -1.6094379124341003`}]], 
    InsetBox["\<\"smooth\"\>", {9.392661928770137, -1.6094379124341003`}], 
    InsetBox["\<\"rough\"\>", 
     NCache[{Log[200000], -1.6094379124341003`}, {
      12.206072645530174`, -1.6094379124341003`}]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["Re", TraditionalForm], 
    FormBox[
     SubscriptBox["C", "f"], TraditionalForm]},
  AxesOrigin->{4.499809670330265, -5.809142990314028},
  CoordinatesToolOptions:>{"DisplayFunction" -> ({
      Exp[
       Part[#, 1]], 
      Exp[
       Part[#, 2]]}& ), "CopiedValueFunction" -> ({
      Exp[
       Part[#, 1]], 
      Exp[
       Part[#, 2]]}& )},
  FrameTicks->{{{{-6.907755278982137, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.001\"", 0.001, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-6.214608098422191, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.002\"", 0.002, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-5.298317366548036, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.005\"", 0.005, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-4.605170185988092, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.010\"", 0.01, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-3.912023005428146, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.020\"", 0.02, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-2.995732273553991, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.050\"", 0.05, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-2.302585092994046, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.100\"", 0.1, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-1.6094379124341003`, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.200\"", 0.2, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-0.6931471805599453, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.500\"", 0.5, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
        TraditionalForm]}, {-5.809142990314028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-5.521460917862246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-5.115995809754082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.961845129926823, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.8283137373023015`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.710530701645918, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-3.506557897319982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-3.2188758248682006`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.8134107167600364`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.6592600369327783`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.5257286443082556`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.4079456086518722`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-1.2039728043259361`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-0.916290731874155, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}, {{-6.907755278982137, 
       FormBox["\"\"", TraditionalForm]}, {-6.214608098422191, 
       FormBox["\"\"", TraditionalForm]}, {-5.298317366548036, 
       FormBox["\"\"", TraditionalForm]}, {-4.605170185988092, 
       FormBox["\"\"", TraditionalForm]}, {-3.912023005428146, 
       FormBox["\"\"", TraditionalForm]}, {-2.995732273553991, 
       FormBox["\"\"", TraditionalForm]}, {-2.302585092994046, 
       FormBox["\"\"", TraditionalForm]}, {-1.6094379124341003`, 
       FormBox["\"\"", TraditionalForm]}, {-0.6931471805599453, 
       FormBox["\"\"", TraditionalForm]}, {-5.809142990314028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-5.521460917862246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-5.115995809754082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.961845129926823, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.8283137373023015`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-4.710530701645918, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-3.506557897319982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-3.2188758248682006`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.8134107167600364`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.6592600369327783`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.5257286443082556`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-2.4079456086518722`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-1.2039728043259361`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {-0.916290731874155, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}}, {{{4.605170185988092, 
       FormBox["100", TraditionalForm]}, {6.907755278982137, 
       FormBox["1000", TraditionalForm]}, {9.210340371976184, 
       FormBox[
        TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {11.512925464970229`, 
       FormBox[
        TemplateBox[{"10", "5"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {13.815510557964274`, 
       FormBox[
        TemplateBox[{"10", "6"}, "Superscript", SyntaxForm -> SuperscriptBox],
         TraditionalForm]}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.699514748210191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.85366542803745, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.987196820661973, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.104979856318357, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.903487552536127, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.308952660644293`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.596634733096073`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.819778284410283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.002099841204238`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.156250521031495`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.289781913656018`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.407564949312402`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.206072645530174`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.611537753638338`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.89921982609012, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.122363377404328`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.304684934198283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.458835614025542`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.592367006650065`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.710150042306449`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}, {{4.605170185988092, 
       FormBox["\"\"", TraditionalForm]}, {6.907755278982137, 
       FormBox["\"\"", TraditionalForm]}, {9.210340371976184, 
       FormBox["\"\"", TraditionalForm]}, {11.512925464970229`, 
       FormBox["\"\"", TraditionalForm]}, {13.815510557964274`, 
       FormBox["\"\"", TraditionalForm]}, {5.298317366548036, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.703782474656201, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {5.991464547107982, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.214608098422191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.396929655216146, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.551080335043404, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.684611727667927, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {6.802394763324311, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {7.600902459542082, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.006367567650246, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.294049640102028, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.517193191416238, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.699514748210191, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.85366542803745, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {8.987196820661973, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.104979856318357, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {9.903487552536127, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.308952660644293`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.596634733096073`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {10.819778284410283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.002099841204238`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.156250521031495`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.289781913656018`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {11.407564949312402`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.206072645530174`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.611537753638338`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {12.89921982609012, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.122363377404328`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.304684934198283`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.458835614025542`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.592367006650065`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}, {13.710150042306449`, 
       FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
        Thickness[0.001]}}}}},
  GridLines->{None, None},
  Method->{},
  PlotRange->{{4.499809670330265, 
   13.9108207377686}, {-5.809142990314028, -1.5141277326297755`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{None, None}, {None, None}},
  Ticks->{{{4.605170185988092, 
      FormBox["100", TraditionalForm]}, {6.907755278982137, 
      FormBox["1000", TraditionalForm]}, {9.210340371976184, 
      FormBox[
       TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {11.512925464970229`, 
      FormBox[
       TemplateBox[{"10", "5"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {13.815510557964274`, 
      FormBox[
       TemplateBox[{"10", "6"}, "Superscript", SyntaxForm -> SuperscriptBox], 
       TraditionalForm]}, {5.298317366548036, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.703782474656201, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {5.991464547107982, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.214608098422191, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.396929655216146, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.551080335043404, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.684611727667927, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {6.802394763324311, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {7.600902459542082, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.006367567650246, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.294049640102028, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.517193191416238, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.699514748210191, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.85366542803745, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {8.987196820661973, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {9.104979856318357, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {9.903487552536127, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.308952660644293`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.596634733096073`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {10.819778284410283`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.002099841204238`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.156250521031495`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.289781913656018`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {11.407564949312402`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.206072645530174`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.611537753638338`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {12.89921982609012, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.122363377404328`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.304684934198283`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.458835614025542`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.592367006650065`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {13.710150042306449`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}}, {{-6.907755278982137, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.001\"", 0.001, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-6.214608098422191, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.002\"", 0.002, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-5.298317366548036, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.005\"", 0.005, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-4.605170185988092, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.010\"", 0.01, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-3.912023005428146, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.020\"", 0.02, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-2.995732273553991, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.050\"", 0.05, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-2.302585092994046, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.100\"", 0.1, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-1.6094379124341003`, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.200\"", 0.2, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-0.6931471805599453, 
      FormBox[
       TagBox[
        InterpretationBox["\"0.500\"", 0.5, AutoDelete -> True], 
        NumberForm[#, {
          DirectedInfinity[1], 3}, NumberPadding -> {"", "0"}]& ], 
       TraditionalForm]}, {-5.809142990314028, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-5.521460917862246, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-5.115995809754082, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-4.961845129926823, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-4.8283137373023015`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-4.710530701645918, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-3.506557897319982, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-3.2188758248682006`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-2.8134107167600364`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-2.6592600369327783`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-2.5257286443082556`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-2.4079456086518722`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-1.2039728043259361`, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}, {-0.916290731874155, 
      FormBox["\"\"", TraditionalForm], {0.00375, 0.}, {
       Thickness[0.001]}}}}]], "Output",
 CellChangeTimes->{3.5755484747397337`*^9, 3.5865829507507224`*^9, 
  3.5875545182376547`*^9, 3.587606421712059*^9, 3.5876359659834037`*^9, 
  3.5890779699106917`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 16.7 Water pipe (p. 276)\
\>", "Section",
 CellChangeTimes->{{3.5021818337236757`*^9, 3.502181841051095*^9}, {
  3.5876341471831727`*^9, 3.5876341560632386`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Rey", "=", 
   SuperscriptBox["10", "4"]}], ";"}]], "Input",
 CellChangeTimes->{{3.5021818622513075`*^9, 3.5021818715728407`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", "Rey", "]"}]], "Input",
 CellChangeTimes->{{3.5021818759650917`*^9, 3.5021819059578075`*^9}}],

Cell[BoxData["4.937500000000001`"], "Output",
 CellChangeTimes->{3.5755484748287435`*^9, 3.5865829523889217`*^9, 
  3.5875545186216993`*^9, 3.5876064217960696`*^9, 3.5876359661444216`*^9, 
  3.5890779709737773`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 16.8 Barrel of wine, continued (p. 277)\
\>", "Section",
 CellChangeTimes->{{3.5021819910356736`*^9, 3.5021820036033926`*^9}, {
  3.5876353571523523`*^9, 3.58763536071878*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5876342134331226`*^9, 3.58763421707156*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(13\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5876354750224934`*^9, 3.587635966511466*^9, 
  3.589077971346816*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.5876353273927813`*^9, 3.587635333902562*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{
    FractionBox["1", "2"], "5", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"spout", " ", "radius"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L", "=", 
   RowBox[{"20", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"spout", " ", "length"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"h", "=", 
   RowBox[{"2", "Meter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"barrel", " ", "height"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Nu]", "=", 
   RowBox[{
    SuperscriptBox["10", 
     RowBox[{"-", "6"}]], 
    RowBox[{
     SuperscriptBox["Meter", "2"], "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"viscosity", " ", "of", " ", "water"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"k", "=", 
   RowBox[{"0.5", "Millimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"roughness", " ", "of", " ", "wood"}], " ", "*)"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.502182072000304*^9, 3.502182113149658*^9}, {
  3.5021821450624833`*^9, 3.5021821552380652`*^9}, {3.5021822600630608`*^9, 
  3.5021822674294825`*^9}, {3.5021824391783056`*^9, 3.50218243926031*^9}, {
  3.587635337286969*^9, 3.5876354182256794`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5876354229282455`*^9, 3.5876354238233533`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"U", "=", 
  RowBox[{
   RowBox[{
    SqrtBox[
     RowBox[{"2", 
      SubscriptBox["g", "0"], "h"}]], "//", "ToSI"}], "//", "N"}]}]], "Input",
 CellChangeTimes->{{3.502182116713862*^9, 3.502182137344042*^9}}],

Cell[BoxData[
 TemplateBox[{"6.2631142413339385`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.575548475041768*^9, 3.575548489523508*^9}, 
   3.5865829545181603`*^9, 3.5875545188807306`*^9, 3.5876064220030937`*^9, 
   3.5876354753425317`*^9, 3.587635966848506*^9, 3.5890779724619837`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Rey", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", "a", " ", "U"}], "\[Nu]"], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.5021821587612667`*^9, 3.502182181247553*^9}}],

Cell[BoxData["313155.7120666969`"], "Output",
 CellChangeTimes->{{3.575548475267796*^9, 3.575548489741534*^9}, 
   3.586582954661203*^9, 3.5875545191577635`*^9, 3.587606422219122*^9, 
   3.587635475559561*^9, 3.5876359671055374`*^9, 3.589077972602974*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["k", 
   RowBox[{"2", "a"}]], "//", "ToSI"}]], "Input",
 CellChangeTimes->{{3.5021822716157217`*^9, 3.5021822880786633`*^9}}],

Cell[BoxData["0.01`"], "Output",
 CellChangeTimes->{{3.575548475381809*^9, 3.5755484898405437`*^9}, 
   3.5865829547942247`*^9, 3.5875545194297953`*^9, 3.587606422316132*^9, 
   3.5876354756595716`*^9, 3.587635967217552*^9, 3.5890779728670497`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["C", "f"], "=", 
  RowBox[{
   RowBox[{"0.028", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["k", 
       RowBox[{"2", "a"}]], ")"}], 
     RowBox[{"1", "/", "4"}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.5021823012164145`*^9, 3.502182325977831*^9}}],

Cell[BoxData["0.008854377448471463`"], "Output",
 CellChangeTimes->{{3.5755484756258388`*^9, 3.575548489939556*^9}, 
   3.5865829548862295`*^9, 3.58755451963382*^9, 3.5876064224111433`*^9, 
   3.5876354757575817`*^9, 3.5876359673445654`*^9, 3.5890779730190673`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"2", 
  SubscriptBox["C", "f"], 
  RowBox[{"L", "/", "a"}]}]], "Input",
 CellChangeTimes->{{3.502182374594612*^9, 3.502182382886086*^9}}],

Cell[BoxData["0.1416700391755434`"], "Output",
 CellChangeTimes->{{3.575548475728853*^9, 3.5755484900385685`*^9}, 
   3.5865829551512527`*^9, 3.587554519749833*^9, 3.5876064225721645`*^9, 
   3.587635475854594*^9, 3.5876359674575815`*^9, 3.5890779736761513`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 16.6 Laminar cylindric flow (p. 277)\
\>", "Title",
 CellChangeTimes->{{3.5021825636274242`*^9, 3.5021825699557858`*^9}, {
  3.587635503376899*^9, 3.587635506191236*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 16.9 Burning bushing (p. 280)\
\>", "Section",
 CellChangeTimes->{{3.5021826981561184`*^9, 3.502182705235523*^9}, {
  3.587635538193075*^9, 3.5876355409114013`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CleanGlobal", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(9\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755484978975124`*^9, 3.5755486809094696`*^9, 
  3.586582955389291*^9, 3.5875545210309896`*^9, 3.5876064228321967`*^9, 
  3.587635967784618*^9, 3.5890779743162203`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Eta]", "=", 
   RowBox[{"10", " ", "Pascal", " ", "Second"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]", "=", 
   RowBox[{"1", 
    RowBox[{"Gram", "/", 
     SuperscriptBox["Centimeter", "3"]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"5", " ", "Centimeter"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"d", " ", "=", " ", 
   RowBox[{"1.", " ", "Millimeter"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalOmega]", "=", 
   RowBox[{"2", " ", 
    RowBox[{"\[Pi]", " ", "/", "Second"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5021827864471684`*^9, 3.502182806358307*^9}, 
   3.5755485442090693`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.587635633470505*^9, 3.587635634318607*^9}}],

Cell["Power density", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Epsilon]", "=", " ", 
  RowBox[{
   FractionBox[
    RowBox[{"\[Eta]", " ", 
     SuperscriptBox["\[CapitalOmega]", "2"], 
     SuperscriptBox["a", "2"]}], 
    SuperscriptBox["d", "2"]], "//", 
   RowBox[{"ToUnit", "[", 
    RowBox[{
     RowBox[{"Joule", "/", "Second"}], "/", 
     SuperscriptBox["Meter", "3"]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.50218295675591*^9, 3.5021829608491435`*^9}, {
  3.5755485197991405`*^9, 3.575548528242153*^9}, {3.575548632945717*^9, 
  3.5755486699221525`*^9}}],

Cell[BoxData[
 TemplateBox[{"986960.4401089358`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"J\"", "\[InvisibleSpace]", "\"/(\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"], "\[InvisibleSpace]", "\"s\"", 
            "\[InvisibleSpace]", "\")\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: joules per meter cubed second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Joules\"", 
        RowBox[{
          SuperscriptBox["\"Meters\"", "3"], " ", "\"Seconds\""}]]}], 
     "]"}]& )]], "Output",
 CellChangeTimes->{{3.5755486708542633`*^9, 3.5755486818035784`*^9}, 
   3.5865829561634035`*^9, 3.5875545222261314`*^9, 3.587606423651292*^9, 
   3.587635968809743*^9, 3.589077975827381*^9}]
}, Open  ]],

Cell["Kinematic viscosity", "Text",
 CellChangeTimes->{{3.587635590192315*^9, 3.587635594894878*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Nu]", "=", 
  RowBox[{
   RowBox[{
    FractionBox["\[Eta]", "\[Rho]"], "//", "ToSI"}], "//", "N"}]}]], "Input",
 CellChangeTimes->{{3.5021828880509796`*^9, 3.50218291043226*^9}}],

Cell[BoxData[
 TemplateBox[{"0.01`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"], "\[NegativeMediumSpace]", "\"/\"", 
            "\[InvisibleSpace]", "\"s\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox[
        SuperscriptBox["\"Meters\"", "2"], "\"Seconds\""]}], 
     "]"}]& )]], "Output",
 CellChangeTimes->{3.5755484990266466`*^9, 3.5755486819826*^9, 
  3.586582956310408*^9, 3.5875545227101903`*^9, 3.5876064238953233`*^9, 
  3.5876359691127777`*^9, 3.5890779780036564`*^9}]
}, Open  ]],

Cell["Reynolds number", "Text",
 CellChangeTimes->{{3.5876356006715717`*^9, 3.5876356030858593`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Rey", "=", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"\[CapitalOmega]", " ", "a", " ", "d"}], "\[Nu]"], "//", 
    "ToSI"}], "//", "N"}]}]], "Input",
 CellChangeTimes->{{3.5021828094644847`*^9, 3.502182868439858*^9}}],

Cell[BoxData["0.031415926535897934`"], "Output",
 CellChangeTimes->{3.575548499493705*^9, 3.575548682442655*^9, 
  3.58658295696647*^9, 3.587554523161244*^9, 3.5876064242033606`*^9, 
  3.587635969592839*^9, 3.589077979246814*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 16.10 Spindle vortex (p. 281)\
\>", "Section",
 CellChangeTimes->{{3.423305996595417*^9, 3.4233059984764166`*^9}, {
  3.5021836025728483`*^9, 3.5021836068430924`*^9}, {3.587635613439103*^9, 
  3.587635616351452*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.4233060010944166`*^9, 3.4233060037714167`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(8\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755484997707376`*^9, 3.5755487186269956`*^9, 
  3.5865829573575*^9, 3.5875545234262767`*^9, 3.5876064244623914`*^9, 
  3.5876359700108852`*^9, 3.589077979479891*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.5876358650892973`*^9, 3.587635866303443*^9}}],

Cell["Viscosity", "Text",
 CellChangeTimes->{{3.587635625617564*^9, 3.5876356269747267`*^9}, {
  3.587635676047616*^9, 3.5876356777748213`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Nu]", "=", 
   RowBox[{
    SuperscriptBox["10", 
     RowBox[{"-", "6"}]], " ", 
    RowBox[{
     SuperscriptBox["Meter", "2"], "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"kinematic", " ", "viscosity", " ", "of", " ", "water"}], " ", 
   "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"1", " ", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radius", " ", "of", " ", "spindle"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f", "=", 
   RowBox[{"10.", " ", 
    SuperscriptBox["Second", 
     RowBox[{"-", "1"}]]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"rotation", " ", "frequency"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4233064686134167`*^9, 3.4233064908364167`*^9}, {
  3.423306528498417*^9, 3.4233065379574165`*^9}, {3.4233066729174166`*^9, 
  3.423306685776417*^9}, {3.5876356553811345`*^9, 3.587635664673251*^9}, {
  3.5876357194618235`*^9, 3.587635878799944*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5876358823993745`*^9, 3.5876358832464757`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalOmega]", "=", " ", 
  RowBox[{"2", " ", "\[Pi]", " ", "f", 
   RowBox[{"(*", " ", 
    RowBox[{"circular", " ", "frequency"}], " ", "*)"}]}]}]], "Input",
 CellChangeTimes->{{3.4233064686134167`*^9, 3.4233064908364167`*^9}, {
  3.423306528498417*^9, 3.4233065379574165`*^9}, {3.4233066729174166`*^9, 
  3.423306685776417*^9}, {3.5876356553811345`*^9, 3.587635664673251*^9}, {
  3.5876357194618235`*^9, 3.587635878799944*^9}}],

Cell[BoxData[
 TemplateBox[{"62.83185307179586`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"per \"", "\[InvisibleSpace]", "\"second\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: reciprocal seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["1", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755484999207544`*^9, 3.575548718852024*^9, 3.5865829575335608`*^9, 
   3.5875545235922937`*^9, 3.5876064246044083`*^9, {3.587635775764581*^9, 
   3.587635836527869*^9}, 3.587635970753975*^9, 3.5890779795578527`*^9}]
}, Open  ]],

Cell["Depression of center", "Text",
 CellChangeTimes->{{3.587635903359889*^9, 3.587635907087336*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"z", "=", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[CapitalOmega]", "2"], 
      SuperscriptBox["a", "2"]}], 
     RowBox[{"2", 
      SubscriptBox["g", "0"]}]]}], "//", 
   RowBox[{"ToUnit", "[", "Centimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.423306499855417*^9, 3.423306548874417*^9}}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "2.012839124693827`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"cm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: centimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Centimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575548500173787*^9, 3.575548719079051*^9, 
  3.586582957701584*^9, 3.587554523822322*^9, 3.5876064248364353`*^9, 
  3.5876359715920753`*^9, 3.589077979691912*^9}]
}, Open  ]],

Cell["Reynolds number", "Text",
 CellChangeTimes->{{3.5876359128790307`*^9, 3.58763591563036*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Rey", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"\[CapitalOmega]", " ", 
     SuperscriptBox["a", "2"]}], "\[Nu]"], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4233065563754168`*^9, 3.4233065601454167`*^9}, {
  3.4233066404104166`*^9, 3.423306692739417*^9}}],

Cell[BoxData["6283.185307179587`"], "Output",
 CellChangeTimes->{3.5755485005288267`*^9, 3.5755487192690725`*^9, 
  3.5865829578445997`*^9, 3.587554524011346*^9, 3.587606425065463*^9, 
  3.5876359719031134`*^9, 3.5890779798019257`*^9}]
}, Open  ]],

Cell["Relative error", "Text",
 CellChangeTimes->{{3.587635978479906*^9, 3.5876359814712625`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox["4", "Rey"]], "Input",
 CellChangeTimes->{{3.423307234887417*^9, 3.423307238202417*^9}}],

Cell[BoxData["0.0006366197723675813`"], "Output",
 CellChangeTimes->{3.575548500564833*^9, 3.5755487193100777`*^9, 
  3.5865829578846073`*^9, 3.5875545240453525`*^9, 3.587606425098466*^9, 
  3.5876359719351163`*^9, 3.5890779798258867`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{944, 903},
WindowMargins->{{197, Automatic}, {Automatic, 35}},
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 208, 5, 65, "Title",
 InitializationCell->True],
Cell[778, 29, 426, 9, 62, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1241, 43, 325, 6, 65, "Title"],
Cell[CellGroupData[{
Cell[1591, 53, 119, 2, 41, "Input"],
Cell[1713, 57, 363, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2113, 71, 209, 5, 75, "Section"],
Cell[2325, 78, 26, 0, 30, "Text"],
Cell[2354, 80, 1097, 33, 125, "Input"],
Cell[3454, 115, 32, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[3511, 119, 215, 7, 62, "Input"],
Cell[3729, 128, 716, 17, 41, "Output"]
}, Open  ]],
Cell[4460, 148, 31, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[4516, 152, 154, 5, 59, "Input"],
Cell[4673, 159, 208, 3, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[4930, 168, 184, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[5139, 176, 125, 2, 41, "Input"],
Cell[5267, 180, 288, 8, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5592, 193, 154, 2, 36, "Subsection"],
Cell[5749, 197, 1034, 17, 41, "Input"],
Cell[CellGroupData[{
Cell[6808, 218, 1299, 30, 82, "Input"],
Cell[8110, 250, 5232, 140, 172, "Output"]
}, Open  ]],
Cell[13357, 393, 156, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[13538, 399, 215, 6, 41, "Input"],
Cell[13756, 407, 170, 4, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13963, 416, 238, 7, 41, "Input"],
Cell[14204, 425, 247, 6, 41, "Output"]
}, Open  ]],
Cell[14466, 434, 109, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[14600, 439, 409, 10, 41, "Input"],
Cell[15012, 451, 567, 16, 41, "Output"]
}, Open  ]],
Cell[15594, 470, 111, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[15730, 475, 398, 11, 56, "Input"],
Cell[16131, 488, 298, 5, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16478, 499, 111, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[16614, 504, 1260, 28, 83, "Input"],
Cell[17877, 534, 12988, 224, 266, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[30902, 763, 1105, 24, 62, "Input"],
Cell[32010, 789, 375, 7, 21, "Message"],
Cell[32388, 798, 13071, 225, 266, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[45508, 1029, 101, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[45634, 1034, 1832, 44, 82, "Input"],
Cell[47469, 1080, 76425, 1269, 266, "Output"]
}, Open  ]],
Cell[123909, 2352, 287, 9, 30, "Text"],
Cell[CellGroupData[{
Cell[124221, 2365, 752, 21, 41, "Input"],
Cell[124976, 2388, 224, 5, 212, "Output"]
}, Open  ]],
Cell[125215, 2396, 294, 9, 30, "Text"],
Cell[CellGroupData[{
Cell[125534, 2409, 738, 19, 56, "Input"],
Cell[126275, 2430, 6139, 107, 263, "Output"]
}, Open  ]],
Cell[132429, 2540, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[132555, 2545, 762, 14, 62, "Input"],
Cell[133320, 2561, 380, 7, 21, "Message"],
Cell[133703, 2570, 82090, 1363, 266, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[215866, 3941, 191, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[216082, 3949, 326, 6, 98, "Section"],
Cell[CellGroupData[{
Cell[216433, 3959, 59, 1, 41, "Input"],
Cell[216495, 3962, 434, 10, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[216966, 3977, 131, 3, 36, "Subsection"],
Cell[217100, 3982, 1693, 50, 126, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[218830, 4037, 100, 1, 36, "Subsection"],
Cell[218933, 4040, 32, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[218990, 4044, 417, 12, 60, "Input"],
Cell[219410, 4058, 1509, 38, 41, "Output"]
}, Open  ]],
Cell[220934, 4099, 31, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[220990, 4103, 139, 4, 56, "Input"],
Cell[221132, 4109, 345, 6, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[221526, 4121, 102, 1, 36, "Subsection"],
Cell[221631, 4124, 39, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[221695, 4128, 158, 3, 41, "Input"],
Cell[221856, 4133, 355, 7, 41, "Output"]
}, Open  ]],
Cell[222226, 4143, 93, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[222344, 4148, 458, 13, 57, "Input"],
Cell[222805, 4163, 1479, 37, 41, "Output"]
}, Open  ]],
Cell[224299, 4203, 111, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[224435, 4208, 564, 17, 56, "Input"],
Cell[225002, 4227, 1602, 40, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[226665, 4274, 187, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[226877, 4282, 59, 1, 41, "Input"],
Cell[226939, 4285, 551, 12, 21, "Message"]
}, Open  ]],
Cell[227505, 4300, 30, 0, 30, "Text"],
Cell[227538, 4302, 339, 11, 60, "Input"],
Cell[227880, 4315, 359, 10, 30, "Text"],
Cell[CellGroupData[{
Cell[228264, 4329, 786, 22, 58, "Input"],
Cell[229053, 4353, 718, 17, 60, "Output"]
}, Open  ]],
Cell[229786, 4373, 484, 13, 30, "Text"],
Cell[CellGroupData[{
Cell[230295, 4390, 710, 19, 60, "Input"],
Cell[231008, 4411, 4075, 124, 117, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[235120, 4540, 847, 22, 60, "Input"],
Cell[235970, 4564, 736, 19, 61, "Output"]
}, Open  ]],
Cell[236721, 4586, 94, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[236840, 4592, 966, 26, 60, "Input"],
Cell[237809, 4620, 760, 18, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[238606, 4643, 623, 19, 58, "Input"],
Cell[239232, 4664, 667, 16, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[239960, 4687, 236, 5, 65, "Title"],
Cell[CellGroupData[{
Cell[240221, 4696, 127, 2, 41, "Input"],
Cell[240351, 4700, 359, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[240747, 4714, 231, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[241003, 4723, 59, 1, 41, "Input"],
Cell[241065, 4726, 379, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[241481, 4740, 32, 0, 36, "Subsection"],
Cell[241516, 4742, 1226, 35, 141, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[242779, 4782, 95, 1, 36, "Subsection"],
Cell[242877, 4785, 32, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[242934, 4789, 224, 7, 58, "Input"],
Cell[243161, 4798, 758, 17, 41, "Output"]
}, Open  ]],
Cell[243934, 4818, 31, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[243990, 4822, 164, 5, 59, "Input"],
Cell[244157, 4829, 232, 3, 41, "Output"]
}, Open  ]],
Cell[244404, 4835, 33, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[244462, 4839, 345, 9, 56, "Input"],
Cell[244810, 4850, 776, 17, 41, "Output"]
}, Open  ]],
Cell[245601, 4870, 30, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[245656, 4874, 240, 6, 56, "Input"],
Cell[245899, 4882, 635, 14, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[246595, 4903, 212, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[246832, 4912, 59, 1, 41, "Input"],
Cell[246894, 4915, 430, 10, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[247361, 4930, 32, 0, 36, "Subsection"],
Cell[247396, 4932, 1917, 55, 217, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[249350, 4992, 99, 1, 36, "Subsection"],
Cell[249452, 4995, 35, 0, 30, "Text"],
Cell[249490, 4997, 108, 3, 56, "Input"],
Cell[249601, 5002, 36, 0, 30, "Text"],
Cell[249640, 5004, 121, 4, 41, "Input"],
Cell[249764, 5010, 150, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[249939, 5016, 271, 6, 41, "Input"],
Cell[250213, 5024, 688, 15, 41, "Output"]
}, Open  ]],
Cell[250916, 5042, 112, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[251053, 5047, 109, 3, 56, "Input"],
Cell[251165, 5052, 803, 18, 41, "Output"]
}, Open  ]],
Cell[251983, 5073, 31, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[252039, 5077, 139, 4, 56, "Input"],
Cell[252181, 5083, 288, 4, 41, "Output"]
}, Open  ]],
Cell[252484, 5090, 43, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[252552, 5094, 289, 8, 56, "Input"],
Cell[252844, 5104, 830, 20, 41, "Output"]
}, Open  ]],
Cell[253689, 5127, 39, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[253753, 5131, 217, 5, 41, "Input"],
Cell[253973, 5138, 655, 14, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[254689, 5159, 202, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[254916, 5168, 59, 1, 41, "Input"],
Cell[254978, 5171, 408, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[255423, 5185, 32, 0, 36, "Subsection"],
Cell[255458, 5187, 1118, 33, 125, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[256613, 5225, 99, 1, 36, "Subsection"],
Cell[256715, 5228, 32, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[256772, 5232, 154, 5, 58, "Input"],
Cell[256929, 5239, 768, 17, 41, "Output"]
}, Open  ]],
Cell[257712, 5259, 31, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[257768, 5263, 174, 5, 59, "Input"],
Cell[257945, 5270, 260, 3, 41, "Output"]
}, Open  ]],
Cell[258220, 5276, 33, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[258278, 5280, 296, 8, 56, "Input"],
Cell[258577, 5290, 799, 18, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[259437, 5315, 178, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[259640, 5323, 125, 2, 41, "Input"],
Cell[259768, 5327, 383, 9, 21, "Message"]
}, Open  ]],
Cell[260166, 5339, 524, 16, 85, "Input"],
Cell[CellGroupData[{
Cell[260715, 5359, 376, 11, 45, "Input"],
Cell[261094, 5372, 602, 13, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[261745, 5391, 386, 7, 75, "Section"],
Cell[CellGroupData[{
Cell[262156, 5402, 100, 1, 36, "Subsection"],
Cell[262259, 5405, 897, 15, 41, "Input"],
Cell[263159, 5422, 1399, 32, 82, "Input"],
Cell[264561, 5456, 211, 4, 30, "Text"],
Cell[264775, 5462, 543, 18, 62, "Input"],
Cell[265321, 5482, 128, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[265474, 5489, 547, 11, 41, "Input"],
Cell[266024, 5502, 670, 18, 41, "Output"]
}, Open  ]],
Cell[266709, 5523, 134, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[266868, 5530, 683, 14, 41, "Input"],
Cell[267554, 5546, 404, 7, 41, "Output"]
}, Open  ]],
Cell[267973, 5556, 233, 5, 30, "Text"],
Cell[CellGroupData[{
Cell[268231, 5565, 2520, 53, 82, "Input"],
Cell[270754, 5620, 71965, 1195, 266, "Output"]
}, Open  ]],
Cell[342734, 6818, 163, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[342922, 6824, 765, 21, 41, "Input"],
Cell[343690, 6847, 288, 6, 212, "Output"]
}, Open  ]],
Cell[343993, 6856, 217, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[344235, 6863, 471, 14, 43, "Input"],
Cell[344709, 6879, 6307, 110, 258, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[351053, 6994, 983, 22, 62, "Input"],
Cell[352039, 7018, 509, 10, 21, "Message"],
Cell[352551, 7030, 77669, 1290, 266, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[430281, 8327, 238, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[430544, 8336, 1714, 41, 104, "Input"],
Cell[432261, 8379, 350, 7, 21, "Message"],
Cell[432614, 8388, 5912, 110, 280, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[438575, 8504, 387, 7, 75, "Section"],
Cell[CellGroupData[{
Cell[438987, 8515, 98, 1, 36, "Subsection"],
Cell[439088, 8518, 977, 16, 41, "Input"],
Cell[440068, 8536, 1332, 32, 82, "Input"],
Cell[441403, 8570, 190, 4, 30, "Text"],
Cell[441596, 8576, 592, 19, 62, "Input"],
Cell[442191, 8597, 175, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[442391, 8605, 598, 12, 41, "Input"],
Cell[442992, 8619, 547, 16, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[443588, 8641, 137, 2, 36, "Subsection"],
Cell[CellGroupData[{
Cell[443750, 8647, 1272, 27, 83, "Input"],
Cell[445025, 8676, 3511, 67, 269, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[448573, 8748, 1366, 28, 62, "Input"],
Cell[449942, 8778, 354, 7, 21, "Message"],
Cell[450299, 8787, 3585, 68, 269, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[453957, 8863, 342, 6, 65, "Title"],
Cell[CellGroupData[{
Cell[454324, 8873, 354, 6, 75, "Section"],
Cell[CellGroupData[{
Cell[454703, 8883, 119, 2, 41, "Input"],
Cell[454825, 8887, 356, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[455218, 8901, 101, 1, 36, "Subsection"],
Cell[455322, 8904, 93, 1, 30, "Text"],
Cell[455418, 8907, 1177, 30, 102, "Input"],
Cell[CellGroupData[{
Cell[456620, 8941, 166, 6, 57, "Input"],
Cell[456789, 8949, 211, 3, 41, "Output"]
}, Open  ]],
Cell[457015, 8955, 197, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[457237, 8962, 1271, 31, 56, "Input"],
Cell[458511, 8995, 1987, 55, 104, "Output"]
}, Open  ]],
Cell[460513, 9053, 95, 1, 30, "Text"],
Cell[460611, 9056, 289, 8, 56, "Input"],
Cell[CellGroupData[{
Cell[460925, 9068, 962, 25, 65, "Input"],
Cell[461890, 9095, 24574, 464, 260, "Output"]
}, Open  ]],
Cell[486479, 9562, 2295, 59, 86, "Input"],
Cell[488777, 9623, 202, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[489004, 9630, 826, 14, 62, "Input"],
Cell[489833, 9646, 328, 7, 21, "Message"],
Cell[490164, 9655, 25383, 480, 260, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[515596, 10141, 153, 2, 36, "Subsection"],
Cell[CellGroupData[{
Cell[515774, 10147, 949, 25, 65, "Input"],
Cell[516726, 10174, 24576, 491, 259, "Output"]
}, Open  ]],
Cell[541317, 10668, 2029, 57, 87, "Input"],
Cell[543349, 10727, 102, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[543476, 10732, 846, 15, 41, "Input"],
Cell[544325, 10749, 331, 7, 21, "Message"],
Cell[544659, 10758, 25380, 507, 291, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[570100, 11272, 180, 4, 75, "Section"],
Cell[570283, 11278, 165, 4, 41, "Input"],
Cell[CellGroupData[{
Cell[570473, 11286, 126, 2, 41, "Input"],
Cell[570602, 11290, 215, 3, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[570866, 11299, 194, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[571085, 11307, 126, 2, 41, "Input"],
Cell[571214, 11311, 287, 8, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[571538, 11324, 100, 1, 36, "Subsection"],
Cell[571641, 11327, 1431, 43, 161, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[573109, 11375, 99, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[573233, 11380, 233, 7, 45, "Input"],
Cell[573469, 11389, 775, 17, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[574281, 11411, 207, 5, 56, "Input"],
Cell[574491, 11418, 255, 3, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[574783, 11426, 163, 4, 57, "Input"],
Cell[574949, 11432, 248, 3, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[575234, 11440, 315, 10, 58, "Input"],
Cell[575552, 11452, 265, 3, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[575854, 11460, 160, 4, 41, "Input"],
Cell[576017, 11466, 262, 3, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[576352, 11477, 188, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[576565, 11485, 183, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[576773, 11493, 77, 2, 41, "Input"],
Cell[576853, 11497, 385, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[577275, 11511, 32, 0, 36, "Subsection"],
Cell[577310, 11513, 732, 21, 125, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[578079, 11539, 95, 1, 36, "Subsection"],
Cell[578177, 11542, 29, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[578231, 11546, 532, 14, 59, "Input"],
Cell[578766, 11562, 930, 21, 41, "Output"]
}, Open  ]],
Cell[579711, 11586, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[579837, 11591, 205, 5, 56, "Input"],
Cell[580045, 11598, 813, 20, 41, "Output"]
}, Open  ]],
Cell[580873, 11621, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[580999, 11626, 253, 7, 56, "Input"],
Cell[581255, 11635, 229, 3, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[581545, 11645, 234, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[581804, 11654, 129, 2, 41, "Input"],
Cell[581936, 11658, 383, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[582356, 11672, 100, 1, 36, "Subsection"],
Cell[582459, 11675, 144, 2, 30, "Text"],
Cell[582606, 11679, 1133, 31, 104, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[583776, 11715, 99, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[583900, 11720, 457, 8, 41, "Input"],
Cell[584360, 11730, 753, 17, 41, "Output"]
}, Open  ]],
Cell[585128, 11750, 102, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[585255, 11755, 362, 11, 61, "Input"],
Cell[585620, 11768, 616, 13, 41, "Output"]
}, Open  ]],
Cell[586251, 11784, 98, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[586374, 11789, 291, 7, 59, "Input"],
Cell[586668, 11798, 235, 3, 41, "Output"]
}, Open  ]],
Cell[586918, 11804, 98, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[587041, 11809, 115, 2, 58, "Input"],
Cell[587159, 11813, 239, 3, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
