(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     96217,       2258]
NotebookOptionsPosition[     90204,       2052]
NotebookOutlinePosition[     90652,       2069]
CellTagsIndexPosition[     90609,       2066]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 20 Mechanical balances (p. 339)\
\>", "Title",
 InitializationCell->True,
 CellChangeTimes->{{3.5024405833570004`*^9, 3.5024405875629997`*^9}, {
  3.5311200324173994`*^9, 3.5311200355045757`*^9}, {3.587788691466926*^9, 
  3.587788694177292*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{"<<", "Lautrup`"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.4336752916522226`*^9, 3.4336752977412224`*^9}, 
   3.4792812836210003`*^9, 3.589038731718052*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 20.3 Momentum balance (p. 343)\
\>", "Title",
 CellChangeTimes->{{3.502440578749*^9, 3.502440593083*^9}, {
  3.5877887077859774`*^9, 3.5877887116814876`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Application: Launch of a small rocket (p. 344)\
\>", "Section",
 CellChangeTimes->{{3.502440604639*^9, 3.5024406081549997`*^9}, {
  3.5877887141787806`*^9, 3.5877887250971346`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(15\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.505290346593135*^9, 3.5052904386704016`*^9, 3.505290505630232*^9, 
   3.5755979197282963`*^9, 3.586583282394329*^9, 3.587788755585987*^9, 
   3.587788801954791*^9, 3.5878800787808113`*^9, 3.587881019874094*^9, {
   3.5878810563237457`*^9, 3.5878810679772353`*^9}, 3.58907861181471*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initial data", "Subsection"],

Cell["\<\
Parameters with SI units removed (so that the differential equation can be \
solved)\
\>", "Text",
 CellChangeTimes->{
  3.5877890116251965`*^9, {3.587881183661029*^9, 3.58788118969178*^9}, {
   3.587881329764736*^9, 3.5878813438604918`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["M", "0"], "=", 
   RowBox[{
    RowBox[{"1.", " ", "Kilogram"}], " ", "//", "NoUnit"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"Mass", " ", "of", " ", "rocket"}], " ", "*)"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["M", "1"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"200.", " ", "Gram"}], "//", "ToSI"}], "//", "NoUnit"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"Mass", " ", "of", " ", "fuel"}], " ", "*)"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{
    RowBox[{"50.", 
     FractionBox["Meter", "Second"]}], "//", "NoUnit"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"Gas", " ", "velocity"}], " ", "*)"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", 
   RowBox[{
    RowBox[{"AccelerationDueToGravity", "//", "ToSI"}], "//", "NoUnit"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["t", "1"], "=", 
   RowBox[{
    RowBox[{"3.", "Second"}], "//", "NoUnit"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"Burn", " ", "time"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{
    RowBox[{"1.2", " ", 
     FractionBox["Kilogram", 
      SuperscriptBox["Meter", "3"]]}], "//", "NoUnit"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"Air", " ", "density"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["d", "0"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"6.", " ", "Centimeter"}], "//", "ToSI"}], "//", "NoUnit"}]}], 
  " ", 
  RowBox[{"(*", " ", 
   RowBox[{"Rocket", " ", "diameter"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["C", "D"], "=", " ", "1."}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"Drag", " ", "coefficient"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"total", " ", "=", " ", 
   RowBox[{
    RowBox[{"15", " ", "Second"}], " ", "//", "NoUnit"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"total", " ", "time", " ", "of", " ", "flight"}], " ", "*)"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.433680892228223*^9, 3.4336808994062223`*^9}, {
   3.505290429564881*^9, 3.5052905016050014`*^9}, {3.5877887868618956`*^9, 
   3.5877887988994184`*^9}, {3.587788974245408*^9, 3.587789020067198*^9}, 
   3.5877893273547783`*^9, {3.5877896098972397`*^9, 3.5877896118424864`*^9}, {
   3.5878807918449574`*^9, 3.587880799316927*^9}, {3.5878808976265335`*^9, 
   3.5878810101578584`*^9}, 3.587881052174218*^9}],

Cell["Mass loss rate", "Text",
 CellChangeTimes->{3.5877890752571306`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "=", 
  FractionBox[
   RowBox[{
    SubscriptBox["M", "0"], "-", 
    SubscriptBox["M", "1"]}], 
   SubscriptBox["t", "1"]]}]], "Input"],

Cell[BoxData["0.26666666666666666`"], "Output",
 CellChangeTimes->{
  3.5052903466261373`*^9, {3.5052904387414055`*^9, 3.505290462993793*^9}, 
   3.5052905057002354`*^9, 3.5755979199513235`*^9, 3.5865832825053253`*^9, 
   3.5877887556929836`*^9, 3.587788802104836*^9, 3.58778907007246*^9, 
   3.5877892599892807`*^9, 3.587789331443239*^9, 3.587789616540039*^9, 
   3.587790085864995*^9, 3.587880078990909*^9, 3.5878805776385884`*^9, 
   3.5878810200030823`*^9, {3.58788105645484*^9, 3.587881068109255*^9}, 
   3.5890786119217663`*^9}]
}, Open  ]],

Cell["Rocket cross section", "Text",
 CellChangeTimes->{{3.587788856074549*^9, 3.5877888768021994`*^9}, {
  3.5877890841212482`*^9, 3.5877890862085075`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["A", "0"], "=", 
  RowBox[{
   FractionBox["1", "4"], "\[Pi]", " ", 
   SuperscriptBox[
    SubscriptBox["d", "0"], "2"]}]}]], "Input",
 CellChangeTimes->{3.5877888522351046`*^9}],

Cell[BoxData["0.0028274333882308137`"], "Output",
 CellChangeTimes->{
  3.587788852508128*^9, 3.587789089075919*^9, 3.5877892600172863`*^9, 
   3.5877893314693537`*^9, 3.5877896165721064`*^9, 3.587790085895035*^9, 
   3.587880079028841*^9, 3.587880577669566*^9, 3.587881020032187*^9, {
   3.587881056478759*^9, 3.5878810681382356`*^9}, 3.589078611943754*^9}]
}, Open  ]],

Cell["Takeoff parameter (must be >1)", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox[
  RowBox[{"U", " ", "Q"}], 
  RowBox[{
   SubscriptBox["M", "0"], 
   SubscriptBox["g", "0"]}]]], "Input"],

Cell[BoxData["1.3596216173039044`"], "Output",
 CellChangeTimes->{
  3.5052903466541386`*^9, 3.505290438778408*^9, 3.505290505727237*^9, 
   3.575597920004331*^9, 3.586583282534371*^9, 3.587788755715987*^9, 
   3.5877888021297956`*^9, 3.5877890991671114`*^9, 3.58778926012132*^9, 
   3.587789331630302*^9, 3.587789616746108*^9, 3.587790086060995*^9, 
   3.587880079196862*^9, 3.587880577834612*^9, 3.5878810200571003`*^9, {
   3.5878810565027575`*^9, 3.5878810681632595`*^9}, 3.589078611965761*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution with no drag", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"M", "[", "t_", "]"}], "=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"t", "<", 
      SubscriptBox["t", "1"]}], ",", 
     RowBox[{
      SubscriptBox["M", "0"], "-", 
      RowBox[{"Q", " ", "t"}]}], ",", 
     SubscriptBox["M", "1"]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4336814401702223`*^9, 3.4336814592982225`*^9}, {
  3.5877900122998114`*^9, 3.5877900197126713`*^9}, {3.5878804870850334`*^9, 
  3.5878804872600374`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"thrust", "[", "t_", "]"}], "=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"t", "<", 
      SubscriptBox["t", "1"]}], ",", 
     FractionBox[
      RowBox[{"U", " ", "Q", " "}], 
      RowBox[{"M", "[", "t", "]"}]], ",", "0"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.433679331030223*^9, 3.4336793630142226`*^9}, {
  3.4336808235202227`*^9, 3.4336808475692225`*^9}, {3.4336815174422226`*^9, 
  3.433681519148223*^9}, {3.587789980857782*^9, 3.5877899858954396`*^9}, {
  3.5877901272461624`*^9, 3.587790155817793*^9}, {3.5878801511901045`*^9, 
  3.5878801538134475`*^9}, {3.5878802685650997`*^9, 3.587880340638266*^9}, {
  3.5878803912127686`*^9, 3.5878804222176886`*^9}, {3.587880551301238*^9, 
  3.587880559373294*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gg", "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["g", "0"], "//", "ToSI"}], "//", "NoUnit"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5878802051940007`*^9, 3.587880220602975*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"z", ",", "v"}], "}"}], "=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"z", ",", "v"}], "}"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"z", ",", "v"}], "}"}], "/.", 
      RowBox[{
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"z", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"v", "[", "t", "]"}]}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"v", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             RowBox[{"-", "gg"}], "+", 
             RowBox[{"thrust", "[", "t", "]"}]}]}], ",", 
           RowBox[{
            RowBox[{"z", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
           RowBox[{
            RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"z", ",", "v"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", 
           RowBox[{"total", "//", "NoUnit"}]}], "}"}]}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4336808823852224`*^9, 3.433680883122223*^9}, {
  3.4336809513442225`*^9, 3.433680954084223*^9}, {3.433680986016223*^9, 
  3.4336810299222226`*^9}, {3.587789994098469*^9, 3.587789996128712*^9}, {
  3.5877900359787426`*^9, 3.5877900815864377`*^9}, {3.587880066486287*^9, 
  3.5878800717170053`*^9}, {3.58788012291749*^9, 3.5878801362781982`*^9}, {
  3.58788018507839*^9, 3.5878802252375546`*^9}, {3.587880601313572*^9, 
  3.58788063311168*^9}, {3.5878808703830137`*^9, 3.5878808864610634`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results with no drag", "Subsection",
 CellChangeTimes->{{3.587881088012794*^9, 3.587881089219943*^9}, {
  3.587881205948848*^9, 3.587881207732051*^9}}],

Cell["Velocity at end of burn", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["v", "1"], "=", 
  RowBox[{"v", "[", 
   SubscriptBox["t", "1"], "]"}]}]], "Input",
 CellChangeTimes->{3.4336820778562226`*^9}],

Cell[BoxData["51.05194965351798`"], "Output",
 CellChangeTimes->{
  3.505290346702142*^9, 3.5052904389094152`*^9, 3.5052905057792397`*^9, 
   3.5755979201803503`*^9, 3.5865832825773344`*^9, 3.5877887558950105`*^9, 
   3.5877888021677847`*^9, 3.5877893447439485`*^9, 3.587789620291605*^9, 
   3.587790088705367*^9, {3.5877901345491276`*^9, 3.587790159975316*^9}, 
   3.5878800793829203`*^9, 3.587880492001636*^9, 3.5878810682142677`*^9, 
   3.5890786120107718`*^9}]
}, Open  ]],

Cell["Height at end of burn", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["z", "1"], "=", 
  RowBox[{"z", "[", 
   SubscriptBox["t", "1"], "]"}]}]], "Input"],

Cell[BoxData["45.516156268414086`"], "Output",
 CellChangeTimes->{
  3.5052903467281427`*^9, 3.505290438933417*^9, 3.505290505797241*^9, 
   3.5755979202193556`*^9, 3.5865832825993376`*^9, 3.587788755925029*^9, 
   3.5877888021857967`*^9, 3.5877893447699614`*^9, 3.5877896203115077`*^9, 
   3.5877900887293696`*^9, {3.5877901345721235`*^9, 3.587790159998294*^9}, 
   3.5878800794119177`*^9, 3.587880492026635*^9, 3.587881068239262*^9, 
   3.5890786120317507`*^9}]
}, Open  ]],

Cell["Time at maximum height", "Text",
 CellChangeTimes->{{3.433681086570223*^9, 3.4336810937922225`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["t", "max"], "=", 
  RowBox[{
   RowBox[{"FindMaximum", "[", 
    RowBox[{
     RowBox[{"z", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       SubscriptBox["t", "1"], ",", 
       SubscriptBox["t", "1"], ",", "total"}], "}"}]}], "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"2", ",", "1", ",", "2"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4336811013142223`*^9, 3.4336811765102224`*^9}, {
  3.433681231025223*^9, 3.433681242405223*^9}, {3.433681278676223*^9, 
  3.4336812796792226`*^9}}],

Cell[BoxData["8.20585002110651`"], "Output",
 CellChangeTimes->{
  3.5052903467461443`*^9, 3.505290439044423*^9, 3.5052905058202424`*^9, 
   3.575597920314369*^9, 3.5865832826213675`*^9, 3.587788755960023*^9, 
   3.5877888022028384`*^9, 3.5877893450299435`*^9, 3.5877896205286045`*^9, 
   3.5877900889463825`*^9, {3.5877901347912054`*^9, 3.587790160218361*^9}, 
   3.5878800797269506`*^9, 3.5878804922536373`*^9, 3.5878810682632656`*^9, 
   3.5890786120527782`*^9}]
}, Open  ]],

Cell["Maximum height", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["z", "max"], "=", 
  RowBox[{"z", "[", 
   SubscriptBox["t", "max"], "]"}]}]], "Input",
 CellChangeTimes->{{3.4336813147712226`*^9, 3.4336813234942226`*^9}}],

Cell[BoxData["178.40055463019308`"], "Output",
 CellChangeTimes->{
  3.5052903467631445`*^9, 3.505290439072425*^9, 3.5052905058292427`*^9, 
   3.575597920354374*^9, 3.586583282643343*^9, 3.587788755983014*^9, 
   3.587788802224826*^9, 3.5877893452480135`*^9, 3.5877896207435675`*^9, 
   3.587790089192437*^9, {3.587790135006178*^9, 3.5877901604343767`*^9}, 
   3.587880079947997*^9, 3.587880492475705*^9, 3.5878810682802687`*^9, 
   3.589078612073782*^9}]
}, Open  ]],

Cell["Time at impact on ground", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["t", "2"], "=", 
  RowBox[{"t", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"z", "[", "t", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "total", ",", 
       SubscriptBox["t", "max"], ",", "total"}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.433681188266223*^9, 3.4336812181902227`*^9}, {
  3.4336812554302225`*^9, 3.433681267466223*^9}}],

Cell[BoxData["14.237731009768435`"], "Output",
 CellChangeTimes->{
  3.505290346781146*^9, 3.5052904391394286`*^9, 3.5052905058382435`*^9, 
   3.5755979203933773`*^9, 3.5865832826653457`*^9, 3.587788756002021*^9, 
   3.5877888022428274`*^9, 3.5877893457100472`*^9, 3.587789621211621*^9, 
   3.587790089640481*^9, {3.5877901354602327`*^9, 3.587790160884449*^9}, 
   3.5878800804870596`*^9, 3.58788049295176*^9, 3.5878810682932763`*^9, 
   3.589078612095742*^9}]
}, Open  ]],

Cell["Velocity at impact", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"v", "[", 
  SubscriptBox["t", "2"], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "59.15254515342824`"}]], "Output",
 CellChangeTimes->{
  3.5052903468041472`*^9, 3.5052904391734304`*^9, 3.5052905058472443`*^9, 
   3.5755979204403815`*^9, 3.586583282690392*^9, 3.5877887560200253`*^9, 
   3.587788802260806*^9, 3.5877893460231113`*^9, 3.587789621500657*^9, 
   3.5877900899264765`*^9, {3.587790135745273*^9, 3.5877901611604366`*^9}, 
   3.587880080754091*^9, 3.587880493249848*^9, 3.587881068305282*^9, 
   3.5890786121177483`*^9}]
}, Open  ]],

Cell["Plot height as function of time", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1a", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"z", "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      SubscriptBox["t", "2"]}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Black", ",", 
       RowBox[{"Dashing", "[", 
        RowBox[{"{", "0.02", "}"}], "]"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.390123871918355*^9, {3.468069437790251*^9, 3.468069465774852*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], Dashing[{0.02}], LineBox[CompressedData["
1:eJwV13c8Vt8fAHCbMkLKfp77PMreIrPzMSozKkWR/TxGiehrJWRkNESJJNmr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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 14.237731009768435`}, {0., 178.4005446245149}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.505290346828149*^9, 3.5052904392334337`*^9, 3.505290505863245*^9, 
   3.5755979205553956`*^9, 3.5865832827193527`*^9, 3.5877887560950317`*^9, 
   3.5877888022888346`*^9, 3.5877893467121572`*^9, 3.587789622153775*^9, 
   3.587790090579558*^9, {3.587790136382416*^9, 3.5877901618005395`*^9}, 
   3.587880081595251*^9, 3.587880493898878*^9, 3.5878810686392803`*^9, 
   3.589078612145753*^9}]
}, Open  ]],

Cell["Plot velocity as function of time", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig2a", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"v", "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      SubscriptBox["t", "2"]}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Black", ",", 
       RowBox[{"Dashing", "[", 
        RowBox[{"{", "0.02", "}"}], "]"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.3901238760863547`*^9, {3.4680694701110992`*^9, 3.4680694746433587`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], Dashing[{0.02}], LineBox[CompressedData["
1:eJwVlXk01P0Xx8dgZlCTtYjKvo4l24T4XEvMlElSKipLZEnJXiTbIxKKlBZZ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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 14.237731009768435`}, {-59.15254230394977, 
    51.04487419610525}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.50529034684815*^9, 3.5052904392964373`*^9, 3.5052905058882465`*^9, 
   3.575597920596403*^9, 3.586583282741403*^9, 3.5877887561180377`*^9, 
   3.587788802308795*^9, 3.5877893474082823`*^9, 3.587789622781851*^9, 
   3.5877900912066545`*^9, {3.5877901370183907`*^9, 3.5877901624286976`*^9}, 
   3.5878800823063335`*^9, 3.587880494548951*^9, 3.5878810686813297`*^9, 
   3.5890786121657915`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution with drag", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"z", ",", "v"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.433681472280223*^9, 3.4336814791262226`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"drag", "[", "t_", "]"}], "=", 
   RowBox[{
    RowBox[{"Sign", "[", 
     RowBox[{"v", "[", "t", "]"}], "]"}], 
    FractionBox[
     RowBox[{
      FractionBox["1", "2"], 
      SubscriptBox["\[Rho]", "0"], 
      SubscriptBox["A", "0"], 
      SuperscriptBox[
       RowBox[{"v", "[", "t", "]"}], "2"], 
      SubscriptBox["C", "D"]}], 
     RowBox[{"M", "[", "t", "]"}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4336813884752226`*^9, 3.4336814880352225`*^9}, {
   3.4336815709162226`*^9, 3.4336816151662226`*^9}, 3.4336816650392227`*^9, {
   3.4336818575752225`*^9, 3.4336818627312226`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"z", ",", "v"}], "}"}], "=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"z", ",", "v"}], "}"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"z", ",", "v"}], "}"}], "/.", 
      RowBox[{
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"z", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"v", "[", "t", "]"}]}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"v", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             RowBox[{"-", 
              SubscriptBox["g", "0"]}], "+", 
             RowBox[{"thrust", "[", "t", "]"}], "-", 
             RowBox[{"drag", "[", "t", "]"}]}]}], ",", 
           RowBox[{
            RowBox[{"z", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
           RowBox[{
            RowBox[{"v", "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"z", ",", "v"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "total"}], "}"}]}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4336816437762227`*^9, 3.4336816706342225`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results with drag", "Subsection",
 CellChangeTimes->{{3.5878811257476115`*^9, 3.587881126683771*^9}, {
  3.587881216724241*^9, 3.5878812182194376`*^9}}],

Cell["Velocity at end of burn", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["v", "1"], "=", 
  RowBox[{"v", "[", 
   SubscriptBox["t", "1"], "]"}]}]], "Input",
 CellChangeTimes->{3.4336821278042226`*^9}],

Cell[BoxData["44.910874500624544`"], "Output",
 CellChangeTimes->{
  3.505290346889152*^9, 3.5052904393634415`*^9, 3.5052905059242487`*^9, 
   3.5755979207224164`*^9, 3.586583282791362*^9, 3.587788756165041*^9, 
   3.5877888023488665`*^9, 3.587789347692278*^9, 3.5877896230078945`*^9, 
   3.5877900914506707`*^9, {3.5877901372394686`*^9, 3.5877901627446384`*^9}, 
   3.5878800825493293`*^9, 3.58788049478296*^9, 3.5878810687243233`*^9, 
   3.5878811182386656`*^9, 3.589078612209809*^9}]
}, Open  ]],

Cell["Height at end of burn", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["z", "1"], "=", 
  RowBox[{"z", "[", 
   SubscriptBox["t", "1"], "]"}]}]], "Input"],

Cell[BoxData["43.22399929280769`"], "Output",
 CellChangeTimes->{
  3.505290346909153*^9, 3.5052904393924427`*^9, 3.50529050595325*^9, 
   3.5755979207624197`*^9, 3.5865832828173647`*^9, 3.587788756184044*^9, 
   3.5877888023728547`*^9, 3.58778934788134*^9, 3.587789623028891*^9, 
   3.587790091471678*^9, {3.587790137263424*^9, 3.587790162768654*^9}, 
   3.587880082574336*^9, 3.5878804948089848`*^9, 3.5878810687473*^9, 
   3.587881118261661*^9, 3.589078612231759*^9}]
}, Open  ]],

Cell["Time at maximum height", "Text",
 CellChangeTimes->{{3.433681086570223*^9, 3.4336810937922225`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["t", "max"], "=", 
  RowBox[{
   RowBox[{"FindMaximum", "[", 
    RowBox[{
     RowBox[{"z", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       SubscriptBox["t", "1"], ",", 
       SubscriptBox["t", "1"], ",", "total"}], "}"}]}], "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"2", ",", "1", ",", "2"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4336811013142223`*^9, 3.4336811765102224`*^9}, {
  3.433681231025223*^9, 3.433681242405223*^9}, {3.433681278676223*^9, 
  3.4336812796792226`*^9}}],

Cell[BoxData["6.19945044915879`"], "Output",
 CellChangeTimes->{
  3.505290346933155*^9, 3.505290439424445*^9, 3.5052905059712515`*^9, 
   3.5755979208144264`*^9, 3.5865832828374104`*^9, 3.5877887562030487`*^9, 
   3.5877888023908443`*^9, 3.587789348171337*^9, 3.587789623242914*^9, 
   3.5877900916827393`*^9, {3.5877901374794908`*^9, 3.587790162981681*^9}, 
   3.5878800827943606`*^9, 3.587880495028037*^9, 3.587881068767338*^9, 
   3.587881118283681*^9, 3.5890786122518044`*^9}]
}, Open  ]],

Cell["Maximum height", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["z", "max"], "=", 
  RowBox[{"z", "[", 
   SubscriptBox["t", "max"], "]"}]}]], "Input",
 CellChangeTimes->{{3.4336813147712226`*^9, 3.4336813234942226`*^9}}],

Cell[BoxData["102.73825159187803`"], "Output",
 CellChangeTimes->{
  3.505290346952156*^9, 3.505290439466447*^9, 3.5052905059802513`*^9, 
   3.575597920865433*^9, 3.5865832828584137`*^9, 3.5877887562230473`*^9, 
   3.5877888024078517`*^9, 3.587789348525435*^9, 3.587789623459944*^9, 
   3.5877900918947277`*^9, {3.5877901376984916`*^9, 3.587790163194716*^9}, 
   3.5878800830523844`*^9, 3.5878804952450495`*^9, 3.587881068777338*^9, 
   3.587881118310685*^9, 3.5890786122728047`*^9}]
}, Open  ]],

Cell["Time at impact on ground", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["t", "2"], "=", 
  RowBox[{"t", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"z", "[", "t", "]"}], "\[Equal]", "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "total", ",", 
       SubscriptBox["t", "max"], ",", "total"}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.433681188266223*^9, 3.4336812181902227`*^9}, {
  3.4336812554302225`*^9, 3.433681267466223*^9}}],

Cell[BoxData["11.461543331605517`"], "Output",
 CellChangeTimes->{
  3.5052903469741573`*^9, 3.50529043951445*^9, 3.505290505989252*^9, 
   3.575597920909438*^9, 3.5865832828834157`*^9, 3.587788756242037*^9, 
   3.587788802425852*^9, 3.58778935253194*^9, 3.587789623903021*^9, 
   3.5877900923407793`*^9, {3.587790138141551*^9, 3.58779016363975*^9}, 
   3.5878800835124636`*^9, 3.5878804957081203`*^9, 3.587881068787321*^9, 
   3.5878811183326283`*^9, 3.589078612293767*^9}]
}, Open  ]],

Cell["Velocity at impact", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"v", "[", 
  SubscriptBox["t", "2"], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "30.88360076458038`"}]], "Output",
 CellChangeTimes->{
  3.5052903470011587`*^9, 3.5052904395554523`*^9, 3.5052905060022526`*^9, 
   3.5755979209634447`*^9, 3.5865832829034185`*^9, 3.5877887562610564`*^9, 
   3.5877888024488773`*^9, 3.5877893528059597`*^9, 3.5877896241750293`*^9, 
   3.587790092607815*^9, {3.5877901384146605`*^9, 3.587790163917797*^9}, 
   3.587880083781491*^9, 3.587880495986184*^9, 3.58788106879733*^9, 
   3.587881118359659*^9, 3.5890786123138123`*^9}]
}, Open  ]],

Cell["Plot height", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1b", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"z", "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      SubscriptBox["t", "2"]}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4680694844809217`*^9, 3.4680694889941797`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwV13k0VV8bB3AkZMh83WvKeImMFyW0HyUyVDLeg5IpKQ2GkPiRVIiQhJJ5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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 11.461543331605517`}, {0., 102.73824182975942`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.5052903470291605`*^9, 3.5052904396144557`*^9, 3.5052905060162535`*^9, 
   3.575597921046455*^9, 3.5865832829353795`*^9, 3.587788756288058*^9, 
   3.5877888024738345`*^9, 3.5877893534280376`*^9, 3.5877896248071127`*^9, 
   3.587790093235978*^9, {3.587790139048685*^9, 3.587790164553886*^9}, 
   3.5878800844385605`*^9, 3.587880496627226*^9, 3.5878810688123016`*^9, 
   3.5878811183866515`*^9, 3.589078612342775*^9}]
}, Open  ]],

Cell["Plot velocity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig2b", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"v", "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      SubscriptBox["t", "2"]}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.4680694966166153`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwVVHk41PsbHZpmJksIWbJMhinGlmEylu/ntRdyu7aSilJCpCw/spcW4qpk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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 11.461543331605517`}, {-30.88360036313559, 
    44.8529894232302}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.5052903470541615`*^9, 3.5052904396794596`*^9, 3.505290506034255*^9, 
   3.5755979210944624`*^9, 3.5865832829594*^9, 3.587788756308037*^9, 
   3.587788802490861*^9, 3.587789354054089*^9, 3.587789625449153*^9, 
   3.587790093864014*^9, {3.5877901396827374`*^9, 3.5877901651879735`*^9}, 
   3.58788008514165*^9, 3.587880497272314*^9, 3.587881068830346*^9, 
   3.587881118406665*^9, 3.5890786123618007`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Figure 20.1 Vertical flight of fireworks rocket (p. 345)\
\>", "Subsection",
 CellChangeTimes->{{3.5024406512130003`*^9, 3.502440652858*^9}, {
  3.587881240757329*^9, 3.5878812575164576`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"fig1a", ",", "fig1b", ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       RowBox[{"HoldForm", "[", "z", "]"}]}], "}"}]}]}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<balance-rocketheight\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.3901238943493547`*^9, {3.4336754335342226`*^9, 3.4336754461732225`*^9}, {
   3.433675642511223*^9, 3.4336756474762225`*^9}, {3.4336756831722226`*^9, 
   3.4336756992082224`*^9}, {3.468069405542406*^9, 3.468069430463832*^9}, {
   3.479281312854*^9, 3.47928132363*^9}, {3.501495857452756*^9, 
   3.501495917525192*^9}, {3.503464392905958*^9, 3.5034643936570015`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.505290347073163*^9, 3.5052904397074614`*^9, 
  3.5052905060512557`*^9, 3.5755979213294907`*^9, 3.586583283101432*^9, 
  3.587788756436077*^9, 3.58778880261689*^9, 3.587880085232687*^9, 
  3.5878810689603467`*^9, 3.5890786124958367`*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], Dashing[{0.02}], LineBox[CompressedData["
1:eJwV13c8Vt8fAHCbMkLKfp77PMreIrPzMSozKkWR/TxGiehrJWRkNESJJNmr
EtnZK2VEkqwkVELZo6zf/f11X+/Xveecez/nfM7nXJKDxykKHQ0NTQItDc3/
rwOPnoaNB98/QtOwV3RlzEkrtldngG/iCvLnGTHeyqdFZh3XBxbGb6DW3s1L
ki9JyFJmSM+iLwZtqKSenZtRQkXHH1uOjsehKc0L6lXV2qjU7x1HnNVDNERS
0blXb4Qq89dadPuSUQ7T+yc7tOaoZujgtVXjp4izeJat5agVGmMsSOobT0eF
txpipfrtkQOhcUmGmIUCuo4qBD+mokmVgRORVjno+L6spuCli4hqOpc/lpiH
1F/UsLvMeaIpZyZG9b4CJN9eHL1L7ipyCxG2u8/5AildoGlcqfRFs0mHqmeN
X6Jle19i0J1raN2puLJ7vBh9CWFe72MNRWoLmtRNmhJ0sfSV6V9qOAq4/nav
BLEUiSa5JnJO3USbCV8vh1qVI53kafo669tIi3xR6KV/BarskpbLNYtFQYWr
74YTK1GDe9mzB8pxiKaN/aBy32tUI+ZzzTTmAdI+/ajXfrEala8MaS+dfIhC
vx4IuctZi8yYSoIOyyQhhnWN4Z/G9SjJw4lllZCCwnnK9LvGG9H6dcK+934Z
KGso4OnElSak/FHN2/B5JmpNg9W/NM0oJjhAYvZXFmKS6cwUJbagenZ43x6W
i6KOTmwHW71BQfmyJiJKL1Aea/6Zh9Nv0Negkbn1ykL07sPl58/925BCV7ll
9oEitNvm37nBxLeIIvzYrK2lGN324SpT7GtHU+8aTv3QK0PPNT+zGjh2oPLj
pdNWiuWoi+6Jg+1iBxoZfdmjcbACccSKc97m7EIaFrp0JwWr0L1cdPG7cTfS
/mF15rJBLTKcLefsHP+AAi3tpJXONaMAiXfvreR7UVB3TKr37hZUQB2+PXO9
F/E/xmr46lsQ67cdFlb+j8i88X7nU8U3qPOTPq2haR86rVPPvEv3Hdrktqof
TOlDaWcvJvfvaUfSZu7XXaf70NK3O0GiY+3oTnvc36iIT2hV/rOe9a1OdKJ+
aKGtuh+d1XDWOMbUg4I2Z19a7PqMas6p8s0G9KBCtR33n2c/oxHOzwkOCz1o
T6nINNPiZ8Qclqn5buID6sm7NH5UfBAxsoZXlox+RKfit3ubHwwjpujOxFey
Ayi0mzPu9PgwSprbFDdpHECv2ERMJ+RGUMGlQvqBM4Nob+TxDrqOEbQtUNH6
JnwI9V2716xNO4rGrc26tP+NoGeKXENfVcYQ691sixryN6R3UqAq98IYMmme
WmMK/IZGPESSPMLHEMV93eh8/zfE9kL57M6HMTQiQrIl3h1Hl8TO9xLcvyG+
Qzy22rsnkYxQZvuFzHEU6Deh0K/8E3mFeTfdbxtHxmJJL/ge/UTl07qv382M
I4Ynd7bNtn4iqJrMVz40gVgk0j/mvJ1Cp86KRrO3TqAD/SqKtBenkc+9/OO1
PybRA8HrO+jrb/R6zR8t7f6OaIU0Pp049Qft2BgelpD7joq5P5o4vPmDomRm
RBN8v6MDFmqTycVz6HG7NJM7yw/UoGc0rdI+jxoYilqEJH+izi3T9+pvFhHD
pZCaUyd+Ino/fRcSyxIy+GhWGuX1E1U6ewrGGy6h3vSFzOXXP1F7S3Tj3g9L
6PsRpbBOoylk0G/QnT25jHb7l2sHuv9Cm6jFxld0DZ3+XVM3XDSDGGJEqz34
NtHruvCvVn0zaFnpvz9/3DYR+Z4xzcjaDFoVDxmnq9tEi4oj2iNHZlFxZCE2
Q9lC9/w2WkY6Z1ENlhPSUbONOunVO75M/UaZYU48ac00IHJneMaDcx71/KkO
Of+OHvr/Q5ty5HlkztVwy3WTHmJsMtnmleaRp3bY4w45BliUuyjjaTGPFPUq
fkYnMkBT77/LnqnzyFr8F/OaGyPY8QksekovII9Bx8cfScyQkmm5fkV/Ed0V
f1ha9GE3DDi4yJRYLaLUpiGBNR5W4CH7OSxfXkS+T5ndCixZ4VZaYqdPwiLq
zTlx+dw3Vgh40p92bXwRCTLVNE6uscG5RHODiMAlxNvvJMOhvgf23zJLTipa
RlHFPh3vd3PDKUO77sHmZZSWTGbMV+KGu7s8GQQ/LyOvZdH4BmtuYIyKvfxk
axntBHfezXvJDcvh73UyDFfQn9T7lx0s90JvkPHMs8kVpMV1rL6ojAdivfQ1
6/nWUKLCzXhKOi+Q+6uvV0iuoRMSfH95u3mhVE2u/qXmGioVi/Jh2uKFAZr9
Oml2a2jD+FR40jk+wGInjt/IW0OiT1OHHu7jh6JnQad1VddRpefNDplHAtAz
WerWZvEXZbDx3jLrEQYHffHn9a5/EX97g9AiIwGWnz3+XXHtL1o32/erX4MA
fN6hnnlP/yKj7TOSF/MIYEdr5hP98y/6XWgp2BpOhHnhmRtGvv9QHLfWbfVs
DLgsSI+6H26g7a1zQw0RZJCOgx39gg30YnWPcUYuGY512FKaajeQniU/bdE7
MgSgVIWyyQ0E+bsmzThEYFxMsD1ZcRMxvE088+KRCLxa59mgvN9E/3bV9q5V
HoBTySwXNhm3kbnPIwGyoBhc6hNr/o9/G7m1dPh+1xGDmxzHJeakt9EnkVcM
w25i8Do0YuXb6W2kkDt+80K1GJAv0t1tS99GKSVtwm9sxGFRY7MuXmsHWYQY
CJ95KQHv/vjee6BOAzYjrWwzftJgQ23tEtGjgfSmB68icqRh+Qs3a4kJDZgl
acdY9EkD1vUi4oMdDQxF3i9Pl5cBv2cTARyRNBBd/OJs/4wMiLqYUqI+0oC2
vkri0iU5CP0mqh54iRa6aNZ4RkwUgO/cVV9WH1og1oc9s3BRgMKextLkYFrI
FP3xdjFUAQbrreWq4mkhK9eOFFWpAPKp8QdWK2mh3CDcXOmgIoye3+bwZKQD
N9oLks8ZlEC979OEYyod2FQyt8t+OQQbwmdyT+TSgdO+Dej9ewhqnPvc1Iro
IKuuxiJhvzIc2ehd4GiiAwaZ9pkAM2XQJvfsVH2ng+7D47Ztrcqg79kuyClD
D67k7ojwchU4y1Z/uqaaHsYEp8yTi1WB9+wR3twWengoPlbi16sKA09rh+K6
6EEkNMX32pIqnFeqsXf+Sg+9BWwjf5TV4ML5Kg9uegYYq776Na5GDZzySm65
GDIAX7wf01yPOnjp5TXzDDLAkas9RYa8WuDrf52/YZwBsmr7TBBoQWDhKY+L
swwwHvalkOKqBTf5tgSadhhgbpfLsH6NFjyaPenlIcoI/8WdLPF0OgINDzaw
dm9GyDJX2xVRh4Dj+4mgYA4mWMktz54x0YaCiBXVaV1mmI5YMTkSqQdDtzd5
FoyZQXU43MUqXQ92P6BfWDvDDGTOj/pJ1XrglsGVz+jCDDEROySveT2QrJfh
J91mhvnLRmPPrI5CwTrlr0UfM5A+6sq7qB6Dgov9Va1OLNC+lX1xP50+5J+s
UE+P2AUnu05Vxd40wud3ruNfNSsQAuINurhOwq6djEU/Mgc8WHm6vXXFHEKc
vp+aleOEJ5zU6xSKBWwUq/gWUrkgtWdzJd/uPNw1051oTOSGzBfXqfX7LkDE
oZh9x0r3AqfApbwZV1vYDLkbPzLFA7Y8AVtjI3bwmbvr0wTLfvBJdztk89ke
FNnfOW4BLxRxsaQ3tztAtmWImos3H6hpvXG1eeIIgb+ufNKN4we517vZQ/Wd
wCGb7Y7gQ36ge2F6WtHICY7b5+otJfND6trTrjETJ+AeHCnJyOKHQw8cbFRP
O0Hu2+PxtJX88GnaY/7DBSfozRU2rR/lhxn2Z/2FXk4gTn33Vl1aAFylMp/t
eewEH8exGoU2AShoTvZ1/+EE7yP7VCo6BCCE3qUtfMoJ3kpHFWv2CICwK9xM
nnaCOp+5nOODAvAqm/tk4x8nKNhdF3dhRgA0JzUFN1edIFTRyjl6jyD0+eTd
EGWmgHxoAvc3C0HgXhx+eEKUAndIu13vTQnCBygdn7GlQGfJiPjOrCCwZSy7
P7SnANuxl1PuC4LAtfzZFzlSIMbN3MXonyB4u0Uy36FSIKok1ZmJTQiy0qL3
73enQOhRJWqgnBAUKwouffWngI+rtaOLjxAE/PbKORVPgbINWZHPAULQ68zF
1HWfAst3aCeOBgvBUu+pyaMJFPB+leMgEiUEcdfqhJSSKOC5MW8/+kgIVPar
Xd94QgG3OxF25rVCIHulelM3nwK2xYUXgEEYOmuXV0rqKABqz6l7dgnDVeUZ
IYEGCpAb8z1G2YXBPtgyKKiRApM9WTcCeYVhd0zivHYL3t98cnalpDCkBuTG
l7yjwFW5yN/yJ4Wh5S1Hl3Yf/n0vbK+TUoWBk39DYPcUBVyVL9ycz8DrxmDH
vcO/KGBYez62PlcY/l7cq+04TQH2rjPpF4qFYZ2Z+WTZLAXiZ41aH7UIg7Jg
iKzeAgVSpFTZuWeEoaiw6ivbPwoU53M+oVMlQHa0oaDhbirkCYi5BmsS4BKB
+ECWlQqpt7SUN4EA72c2b3CxUeG2u1vXigEBqPJ9Nb3sVHBRaNmasiLAV7uF
Qh0uKhCqfG26gwmwavMjfICXCvsk70oahxPg6vUTJ7L4qMD2OGv1bRQB7vbV
377MT4V/gR/uNsUR4IF0ncu2ABU+gVR9WSYBpsMknrAS8PHejhJT2ghQ3f16
bViECmFqKzN8nQRY8C3nij1AhYAC1sqEHgI0WVanwUH8fW6rmsUOEuBwy5jw
U1Eq6JrFh4TOEMBNJ/mBgQQ+3uej31z2EGFluM3bXJYKC/rWL77vJcI4u8Xo
H9xTVV7+DnxEuOg3Vxsph7/P46dc1iQinFjTGCmVp0KR7V9tUyUi2KToJmwq
UsH55/MMZQsiZIWx/9U+TAWaE38quK2JQI3vvfEed1KpXNecHRHe+WWrn1Ol
QvuNV2v5bkTw7Lyq6apGBVmhSmNCEBF0YqtCvDWo0Bb6134jlAjMVr+jF3Db
/VL3HYgkgvu2bLO7JhXiymvT4+OIwN853GqnRYVls+ZVpmwiLIcOsyogKtyp
YGCbzCdCDWtQQhpuUcIxUmMhEUQuyZpyABUsZ94aXaskQl9J5alJ3NUR79Pm
OohQZeOdHKBDhdDXg4YDi0SIJ9+QNjtKBQGSoF3ZGhFsD9JWZOAuibT+L36T
CJ07d68s4fYqmDMVpcfgpkafT+wxKmT5egWMsWEQ/J6hqfg4FeYl9zKa7sHg
xN2lzhXcWqMlsbVcGNQenGdW1adCv95KVvJ+DAKHuTkqcLNw+3WbY3h7h+3z
Tw2oYN7Kd76ZjIHZV6sffbjT/aomFQ5i0KTUls5iSAX1r//+ckji4zu25bnh
vhn/OPy6NAaDR6yXk3B/PKq5Z1YWg86FbY9W3JeeXz/wTgmD+FH3NT4jKlTa
El4eVsGAyqdOD7gZ99ar5ahiMCTBgyi4U/1pTMO0MMj6uHCsAPe0dPrgAsJg
/eIKdztulTFtJzsdDP4MM+2Zwt19LMxf6zgGVTqnEwnGVKjzLMlJM8Cg0mCr
VAX38+SJj/TGGKRxBu01wf24ZS+t8wkMPBI/vbLHHfNHV7bdDAPRUfr7V3H7
8V21kjmNQeskS2kEbmedrKh7ZzAweTa5LwH32Ut9ZUsWGNQoJFZm4NZ7yDBx
9jwGPH5YSiFuxYZDnK+tMTgdFNJQiZs07aQlbIvBNb2Kg424OXkS3ELsMbjV
3tLRhntbqzVxwhGDUM78V524Z51XWo5RMXDY7zTUjXs47uBivgsGkiMrOh9w
t1efIbJfxONlbzPbg7vye4SxpzsGXrlP+t/jztlT7v/RA4PF5yW0HbgT1H7k
qHhhcN4rzb0Vd7jj/r5HVzEoWHMUqMPtdecY7ZYPBhOa28xluO0rfGTt/DGg
1/FQKcBt9i3HqvkaBqzMFdlPcB9h/RwlGoTB5q1PZ2Jxyygzl0eHYNDX034s
GLeQ7eGJ2VAMEj499HPHvTvamdMsAoMjyWoz53D/fZWoVRKJwXtCSaoe7qmR
Nrf9MRgcd6S/L4O7n2k90f82BnnOkm95cLfKi7eO3MVgQUr8yD98vjPCo4iZ
9zHQWM6drcd9r7DSmOkhBprrEgfTcAcPTPm7JmFwsCY0NQi3tbRBn9wTDF6G
VZ5WwW101p/2/lN8fUcmxO7BrR6SL7uajsfT0IDtJ75eeft2RdfkYJDNpfz2
Hm6mbbVyYj4GP3jcmR1xL4u5TYQ+w4Bz4FqkEu7ea+1aBkUYCKYKG3bj+XJb
5NbipyoM9kp4DS7g+RVgUk1Uq8HjIWVHLMft6jtjnFKHwc7vAzl+uI91GOU6
NOPr4ZWizzqer7TebNZznRhEqNtiE3h+z6doRp/qxuOjSe/+CPfom0vlZR/w
+d26/s8Ed41AF2dgP95fw9rnYnx/8Gm608oyhsHG0c0UJz28vZ1hn844Bjf+
OU9y43bdYZoInMQgxb07p14Xj49mCM3CFAYPY+NSeHBrl3tpDC5gENc0bvRc
G68fBRbF+fQkyJ2srIo/gue7Pk/DBCMJ7oSOFYnj3vnR816YhQQsX+lFavH9
cVHEcDaOjQRNH7X8vuH75+cnmmIB+0ig+TzoBy++v2bEkZ4YipHAjP9Qrwa+
XwvJjz4LlyBBQhqvf5UKvl7fJ7+ukyLBlR+rycq4o1h5BhTlSfDrRZ6phDIV
3COY9gqqkWBCajBwG68Hqv4zUTOGJFiZcP9hi9eTbruyq7cvkyBRj7q6jdcv
gQDmvFVPEnDnhXdp4na6f27YzpsE/DdEhv3IVFhv3dJW9iNB6/3xRzMYFchS
x/aM3iBBsgY1u1oYj+dKf75cAgk0EsKWGfH6KhSzPtpbTQJzz/3euixUoGYa
cmvVkYAYoSugz0yF4pqUo7kNJLi6Z2HNkAmf3zl4fq2VBLXhXWeNGKjgYR7t
f6CbBF0LtzJUaKjQRBDg8RnHv+cTwatijQIuJRoG/LvIMNWasmD/nQJa2gI2
51nJ8IAvc1N7kgLc3etej9nJ8GFyxw+boEDtdFmKMDcZ1n/VtA6M4ffJ8vMk
QTLQ5+q9VBmhQPW9A4mSMmTYPRvTlNtLAY7L7N81TpLhS/riLxv8/DSxMfM3
8DQZgkXfEtlrKVAZ3c5Rd4YMrw4sv6mqpuDn30j8p4YMrXdMXTmqKFAxQntb
z5EMedw821klFLAzWlU0+Y8Mtj8/OkfmUaBE/GuIzSMyxNXIapLiKCApY6L6
4DEZlNp/62THUiBDoXru3RMy3IqNbRe7S4F76ok2yhlkoGhfsxa/RYHLxqaa
bM/IUKK9FMMbQQFxz/q1qhoyNJZSuAv9KJBa8fTyvjEypDqZd1Tb4OfTY3bW
naIi4DBj+f0cfr7tbXVUVZQQAcvoKw/PHKCA4FFnniQpEXz9XAk3JVPgue7l
Tkd5ERiNEec9QqBAFwRqbqiJAN8DT16m/RTg1EgSkjQRgYYJ1zBtRgo8lOsZ
ifQWgUlf2l3UcSdI54MLug0iEOzd0k5JdAI3dq1GL9MDsCT9KyJp3RG8y5MM
n00fALJA+/NKeUeQDeU6nHLtIHzZzybKau0A/606t5XtEgUb+aQ/BG972CQI
Xx/IFIXZoNhmESs7UHabSyhREoP4weirln42UOiK9SV2i4F5dV/o1V4rKOGS
K89yEYeim1lMJ2nOgdvI+Q3XNXEo+BSjRJQ+C/ZW3ymq8RIwIPJy7vXEKdi9
dVupniwJjM09i2VHTOHIce7sy0WSIN54LKzxP314dXRVQ8xQClb2pi9FPgcY
oK4pJw1KQd8++t/7GRWB+1ZgKd+IFFzKDaP5UacAxi9plR6NSkFDrVF1tp8C
NK6xyidPSMFVv1eiazPyUBCFSab8lgIzk9L85kw5uFZgQEijlYaijgPCL5Sl
gPDnMVOehDSUfjGeC9AhgyU3KUJcWhrE5eSmdF6RIF4lhz5fVho4Y9LkZkkk
YA4uoilQkoZlK/mCcRoM5ve0/numhfc3zpJSFyAIjQp//rw8KQ0uzZtpsvbc
sHnG213eXBqfzwynKU0uUAlYnyk6Kw2YH7XViY8TCprofhVbSUPDwr0awyes
EH+ad6KEIg3yr6eHbnnSQadviv0hF/x+DO8hiQM0wJRCGit1w9vT0cYmFm6g
a5NSX8o8pMGuhl1d228RlbMUW6l44c/vjQs0qphB89IqQ+VXcRusuUsEjiPJ
k9WWh32lgUYm5Z5qUC9y+g8+V/jjpoHadsbFI/8Dz656Wg==
      "]]}}, {{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJwV13k0VV8bB3AkZMh83WvKeImMFyW0HyUyVDLeg5IpKQ2GkPiRVIiQhJJ5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      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["t", TraditionalForm], 
    FormBox[
     TagBox["z", HoldForm], TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 14.237731009768435`}, {0., 178.4005446245149}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.5052903470881634`*^9, 3.5052904397554636`*^9, 
  3.5052905060572557`*^9, 3.5755979213334894`*^9, 3.586583283104417*^9, 
  3.5877887564390364`*^9, 3.5877888026198735`*^9, 3.587880086217803*^9, 
  3.587881068963332*^9, 3.5890786124988885`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"fig2a", ",", "fig2b", ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"t", ",", "u"}], "}"}]}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "60"}], ",", "60"}], "}"}]}]}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<balance-rocketvelocity\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.3901238979973545`*^9, {3.433675449287223*^9, 3.433675457650223*^9}, {
   3.4336756318802223`*^9, 3.4336756362762227`*^9}, 3.4336756760202227`*^9, {
   3.4680693650340896`*^9, 3.468069388953458*^9}, {3.479281356999*^9, 
   3.479281358502*^9}, {3.501495861845007*^9, 3.5014958626850553`*^9}, {
   3.5014959226624856`*^9, 3.501495930348925*^9}, {3.5034643968571844`*^9, 
   3.503464397585226*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.505290347113165*^9, 3.5052904398344684`*^9, 
  3.505290506088258*^9, 3.57559792157852*^9, 3.586583283251424*^9, 
  3.5877887566781354`*^9, 3.5877888027618904`*^9, 3.587880086339819*^9, 
  3.5878810691063747`*^9, 3.5890786126428523`*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], Dashing[{0.02}], LineBox[CompressedData["
1:eJwVlXk01P0Xx8dgZlCTtYjKvo4l24T4XEvMlElSKipLZEnJXiTbIxKKlBZZ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      "]]}}, {{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJwVVHk41PsbHZpmJksIWbJMhinGlmEylu/ntRdyu7aSilJCpCw/spcW4qpk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      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["t", TraditionalForm], 
    FormBox["u", TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{-60, 60},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.5052903471331663`*^9, 3.5052904398864713`*^9, 
  3.5052905060942583`*^9, 3.5755979215825195`*^9, 3.586583283254446*^9, 
  3.5877887566811213`*^9, 3.587788802764894*^9, 3.587880087315903*^9, 
  3.587881069109376*^9, 3.5890786126459055`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 20.5 Kinetic energy balance (p. 348)\
\>", "Title",
 CellChangeTimes->{{3.502440833697*^9, 3.5024408408269997`*^9}, {
  3.5878813195973434`*^9, 3.5878813228358326`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 20.1  Barrel of wine (p. 350)\
\>", "Section",
 CellChangeTimes->{{3.50244086459*^9, 3.502440874843*^9}, {
  3.5878817066128116`*^9, 3.5878817103643103`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(20\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755979632135143`*^9, 3.5865832834884915`*^9, 
  3.5877887568971004`*^9, 3.5878817650542994`*^9, 3.5878819668140354`*^9, 
  3.5890786128758874`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["a", "0"], "=", 
   RowBox[{
    FractionBox["1", "2"], " ", "Meter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radius", " ", "of", " ", "barrel"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"h", "=", 
   RowBox[{"2", " ", "Meter"}]}], " ", 
  RowBox[{"(*", 
   RowBox[{"height", " ", "of", " ", "barrel"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{
    FractionBox["1", "2"], "*", "5", " ", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radius", " ", "of", " ", "pipe"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L", "=", 
   RowBox[{"20", " ", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"length", " ", "of", " ", "pipe"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"T", "=", 
   RowBox[{"0.5", " ", "Second"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"time", " ", "of", " ", "observation"}], " ", "*)"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.587881862453702*^9, 3.587881914437399*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5878819256448107`*^9, 3.5878819265319643`*^9}}],

Cell["Cross sections of barrel and spout", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"A", "=", 
   RowBox[{
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["a", "2"]}], "//", "N"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["A", "0"], "=", 
   RowBox[{
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox[
      SubscriptBox["a", "0"], "2"]}], "//", "N"}]}], ";"}]}], "Input"],

Cell["Effective Length", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["L", "0"], "=", 
  RowBox[{
   RowBox[{"L", "+", 
    RowBox[{"h", " ", 
     FractionBox["A", 
      SubscriptBox["A", "0"]]}]}], "//", 
   RowBox[{"ToUnit", "[", "Centimeter", "]"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"20.5`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"cm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: centimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Centimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575597963887597*^9, 3.5865832837285233`*^9, 
  3.587788757625225*^9, 3.5878817655243587`*^9, 3.587881966964054*^9, 
  3.589078612985899*^9}]
}, Open  ]],

Cell["Exit velocity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"U", "=", 
  RowBox[{
   RowBox[{
    SqrtBox[
     RowBox[{"2", 
      SubscriptBox["g", "0"], "h"}]], "//", "ToSI"}], "//", "N"}]}]], "Input",
 CellChangeTimes->{{3.502440888275*^9, 3.502440888574*^9}}],

Cell[BoxData[
 TemplateBox[{"6.2631142413339385`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755979640446153`*^9, 3.586583283801489*^9, 
  3.5877887582853146`*^9, 3.587881765606369*^9, 3.5878819670291076`*^9, 
  3.589078613056883*^9}]
}, Open  ]],

Cell["Rise time", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Tau]", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", 
     SubscriptBox["L", "0"]}], "U"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.06546264114011704`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"s\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Seconds\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575597964181631*^9, 3.5865832838715477`*^9, 
  3.5877887585693426`*^9, 3.5878817656733723`*^9, 3.587881967094103*^9, 
  3.5890786131319284`*^9}]
}, Open  ]],

Cell["96% rise time", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"2", "\[Tau]"}]], "Input",
 CellChangeTimes->{{3.433912341501*^9, 3.433912345646*^9}, {3.43391238219*^9, 
  3.433912382781*^9}}],

Cell[BoxData[
 TemplateBox[{"0.1309252822802341`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"s\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Seconds\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575597964217636*^9, 3.5865832839595547`*^9, 
  3.587788759697486*^9, 3.587881765696402*^9, 3.5878819671141176`*^9, 
  3.5890786131609144`*^9}]
}, Open  ]],

Cell["Spilled wine in rise time", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"V", "=", 
  RowBox[{
   RowBox[{"A", " ", "U", " ", "\[Tau]", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"Cosh", "[", 
      FractionBox["T", "\[Tau]"], "]"}], "]"}]}], "//", 
   RowBox[{"ToUnit", "[", "Liter", "]"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"5.5907917791120445`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"L\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: liters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Liters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575597964554677*^9, 3.5865832841175284`*^9, 
  3.5877887601055417`*^9, 3.5878817658203974`*^9, 3.587881967235132*^9, 
  3.5890786132899413`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{971, 988},
WindowMargins->{{87, Automatic}, {3, Automatic}},
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 263, 6, 65, "Title",
 InitializationCell->True],
Cell[833, 30, 308, 7, 62, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1178, 42, 176, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[1379, 50, 191, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[1595, 58, 59, 1, 41, "Input"],
Cell[1657, 61, 506, 11, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2200, 77, 34, 0, 36, "Subsection"],
Cell[2237, 79, 252, 6, 30, "Text"],
Cell[2492, 87, 2504, 73, 235, "Input"],
Cell[4999, 162, 74, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[5098, 167, 165, 6, 58, "Input"],
Cell[5266, 175, 534, 8, 41, "Output"]
}, Open  ]],
Cell[5815, 186, 157, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[5997, 192, 218, 7, 56, "Input"],
Cell[6218, 201, 358, 5, 41, "Output"]
}, Open  ]],
Cell[6591, 209, 46, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[6662, 213, 134, 5, 58, "Input"],
Cell[6799, 220, 498, 7, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[7346, 233, 43, 0, 36, "Subsection"],
Cell[7392, 235, 494, 14, 41, "Input"],
Cell[7889, 251, 781, 17, 58, "Input"],
Cell[8673, 270, 223, 7, 41, "Input"],
Cell[8899, 279, 1728, 43, 102, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10664, 327, 157, 2, 36, "Subsection"],
Cell[10824, 331, 39, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[10888, 335, 166, 5, 41, "Input"],
Cell[11057, 342, 464, 7, 41, "Output"]
}, Open  ]],
Cell[11536, 352, 37, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[11598, 356, 122, 4, 41, "Input"],
Cell[11723, 362, 463, 7, 41, "Output"]
}, Open  ]],
Cell[12201, 372, 106, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[12332, 377, 556, 15, 41, "Input"],
Cell[12891, 394, 465, 7, 41, "Output"]
}, Open  ]],
Cell[13371, 404, 30, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[13426, 408, 196, 5, 41, "Input"],
Cell[13625, 415, 455, 7, 41, "Output"]
}, Open  ]],
Cell[14095, 425, 40, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[14160, 429, 450, 12, 41, "Input"],
Cell[14613, 443, 460, 7, 41, "Output"]
}, Open  ]],
Cell[15088, 453, 34, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[15147, 457, 76, 2, 41, "Input"],
Cell[15226, 461, 477, 8, 41, "Output"]
}, Open  ]],
Cell[15718, 472, 47, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[15790, 476, 493, 14, 41, "Input"],
Cell[16286, 492, 9171, 157, 252, "Output"]
}, Open  ]],
Cell[25472, 652, 49, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[25546, 656, 499, 14, 41, "Input"],
Cell[26048, 672, 5043, 90, 243, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[31140, 768, 40, 0, 36, "Subsection"],
Cell[31183, 770, 168, 4, 41, "Input"],
Cell[31354, 776, 644, 18, 66, "Input"],
Cell[32001, 796, 1330, 38, 102, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[33368, 839, 158, 2, 36, "Subsection"],
Cell[33529, 843, 39, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[33593, 847, 166, 5, 41, "Input"],
Cell[33762, 854, 486, 7, 41, "Output"]
}, Open  ]],
Cell[34263, 864, 37, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[34325, 868, 122, 4, 41, "Input"],
Cell[34450, 874, 470, 7, 41, "Output"]
}, Open  ]],
Cell[34935, 884, 106, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[35066, 889, 556, 15, 41, "Input"],
Cell[35625, 906, 481, 7, 41, "Output"]
}, Open  ]],
Cell[36121, 916, 30, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[36176, 920, 196, 5, 41, "Input"],
Cell[36375, 927, 483, 7, 41, "Output"]
}, Open  ]],
Cell[36873, 937, 40, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[36938, 941, 450, 12, 41, "Input"],
Cell[37391, 955, 474, 7, 41, "Output"]
}, Open  ]],
Cell[37880, 965, 34, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[37939, 969, 76, 2, 41, "Input"],
Cell[38018, 973, 502, 8, 41, "Output"]
}, Open  ]],
Cell[38535, 984, 27, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[38587, 988, 348, 9, 41, "Input"],
Cell[38938, 999, 9540, 163, 252, "Output"]
}, Open  ]],
Cell[48493, 1165, 29, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[48547, 1169, 322, 9, 41, "Input"],
Cell[48872, 1180, 4631, 84, 240, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[53552, 1270, 202, 4, 36, "Subsection"],
Cell[CellGroupData[{
Cell[53779, 1278, 798, 17, 62, "Input"],
Cell[54580, 1297, 424, 8, 21, "Message"],
Cell[55007, 1307, 17881, 302, 262, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72925, 1614, 916, 21, 62, "Input"],
Cell[73844, 1637, 424, 8, 21, "Message"],
Cell[74271, 1647, 8745, 153, 252, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[83089, 1808, 187, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[83301, 1816, 176, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[83502, 1824, 59, 1, 41, "Input"],
Cell[83564, 1827, 366, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83967, 1841, 32, 0, 36, "Subsection"],
Cell[84002, 1843, 1216, 38, 174, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[85255, 1886, 99, 1, 36, "Subsection"],
Cell[85357, 1889, 50, 0, 30, "Text"],
Cell[85410, 1891, 348, 12, 62, "Input"],
Cell[85761, 1905, 32, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[85818, 1909, 237, 8, 58, "Input"],
Cell[86058, 1919, 566, 13, 41, "Output"]
}, Open  ]],
Cell[86639, 1935, 29, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[86693, 1939, 227, 7, 45, "Input"],
Cell[86923, 1948, 723, 17, 41, "Output"]
}, Open  ]],
Cell[87661, 1968, 25, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[87711, 1972, 153, 5, 56, "Input"],
Cell[87867, 1979, 576, 13, 41, "Output"]
}, Open  ]],
Cell[88458, 1995, 29, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[88512, 1999, 151, 3, 41, "Input"],
Cell[88666, 2004, 573, 13, 41, "Output"]
}, Open  ]],
Cell[89254, 2020, 41, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[89320, 2024, 256, 7, 56, "Input"],
Cell[89579, 2033, 573, 13, 73, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
