(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     93931,       3039]
NotebookOptionsPosition[     80515,       2581]
NotebookOutlinePosition[     80984,       2599]
CellTagsIndexPosition[     80941,       2596]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 29 Subsonic Flight (p. 513)\
\>", "Title",
 CellChangeTimes->{
  3.481345961228077*^9, {3.502943125433*^9, 3.502943127461*^9}, {
   3.5883990523778625`*^9, 3.588399057120472*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{"<<", "Lautrup`"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.4812768617306*^9, 3.481276866227857*^9}, 
   3.5890387849497375`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 29.1 Aircraft controls (p. 513)\
\>", "Title",
 CellChangeTimes->{{3.502943150652*^9, 3.502943164196*^9}, {
  3.5883990703361864`*^9, 3.5883990752647705`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 29.1  Boing 747-200 (p. 515)\
\>", "Section",
 CellChangeTimes->{
  3.502943190584*^9, {3.5883990914408484`*^9, 3.588399100432*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(12\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.575626386628087*^9, 3.586606251260314*^9, 3.5866573710937767`*^9, 
   3.5866608198158116`*^9, {3.58839933866623*^9, 3.588399358024691*^9}, 
   3.5890790906521997`*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"374000.", " ", "Kilogram"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"maximum", " ", "mass"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"T", "=", 
   RowBox[{"973000.0", " ", "Newton"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"maximum", " ", "thrust"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"290", " ", 
    RowBox[{"Kilometer", "/", "Hour"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"takeoff", " ", "speed"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{
  3.481345452364972*^9, {3.4813454833437443`*^9, 3.481345484204793*^9}, 
   3.481345523539043*^9, {3.4813459786300726`*^9, 3.481345978813083*^9}, {
   3.4813460417036805`*^9, 3.481346045276885*^9}, {3.5883991092180853`*^9, 
   3.588399162448845*^9}, {3.588399350449778*^9, 3.5883993547143507`*^9}, {
   3.5883994134547296`*^9, 3.588399416377103*^9}}],

Cell["Horizontal acceleration", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g", "=", 
  RowBox[{
   FractionBox["T", "M"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"2.6016042780748663`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"s\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters per second squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", 
        SuperscriptBox["\"Seconds\"", "2"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.57562638675209*^9, 3.5866062513833294`*^9, 3.5866573712407556`*^9, 
   3.586660819940826*^9, {3.58839933898127*^9, 3.588399358157712*^9}, 
   3.5890790907872553`*^9}]
}, Open  ]],

Cell["Relative to standard gravity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["g", 
   SubscriptBox["g", "0"]], "//", "ToSI"}]], "Input"],

Cell[BoxData["0.26528980621056797`"], "Output",
 CellChangeTimes->{
  3.575626386809074*^9, 3.586606251441346*^9, 3.586657371325756*^9, 
   3.5866608199978266`*^9, {3.588399339035318*^9, 3.5883993582147136`*^9}, 
   3.58907909084725*^9}]
}, Open  ]],

Cell["Takeoff length", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"L", "=", 
  RowBox[{
   FractionBox[
    SuperscriptBox["U", "2"], 
    RowBox[{"2", "g"}]], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"1247.1530711938385`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756263869071255`*^9, 3.586606251531348*^9, 3.5866573714177656`*^9, 
   3.5866608200928044`*^9, {3.588399339073282*^9, 3.5883993583047266`*^9}, 
   3.589079090952248*^9}]
}, Open  ]],

Cell["Takeoff time", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   FractionBox["U", "g"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"30.96380038826082`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"s\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Seconds\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756263869870977`*^9, 3.5866062516083994`*^9, 3.586657371494776*^9, 
   3.5866608201718297`*^9, {3.5883993390993147`*^9, 3.5883993583807344`*^9}, 
   3.589079091037288*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 29.3 Steady flight (p. 517)\
\>", "Title",
 CellChangeTimes->{{3.5029433654119997`*^9, 3.502943371076*^9}, {
  3.5883994692328167`*^9, 3.5883994721202593`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 29.2 Boeing 747 (p. 518)\
\>", "Section",
 CellChangeTimes->{{3.502943225348*^9, 3.502943225975*^9}, {
  3.5029433921800003`*^9, 3.5029433964519997`*^9}, {3.588399310800735*^9, 
  3.5883993168484592`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.588399304120854*^9, 3.5883993070252113`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(6\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5883993334465685`*^9, 3.588399422179839*^9, 
  3.589079091226267*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"374000.0", " ", "Kilogram"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"maximum", " ", "mass"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"T", "=", 
   RowBox[{"973000.0", " ", "Newton"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"maximum", " ", "thrust"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{
  3.481345452364972*^9, {3.4813454833437443`*^9, 3.481345484204793*^9}, 
   3.481345523539043*^9, {3.4813459786300726`*^9, 3.481345978813083*^9}, {
   3.4813460417036805`*^9, 3.481346045276885*^9}, 3.5029434257530003`*^9, {
   3.5883993810416136`*^9, 3.5883994070489173`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"k", "=", 
  RowBox[{
   FractionBox["T", 
    RowBox[{"M", " ", 
     SubscriptBox["g", "0"]}]], "//", "ToSI"}]}]], "Input"],

Cell[BoxData["0.26528980621056797`"], "Output",
 CellChangeTimes->{3.575626399678691*^9, 3.586606251727373*^9, 
  3.5866573716087923`*^9, 3.5866608202878704`*^9, 3.5883993335615816`*^9, 
  3.5883994223158565`*^9, 3.589079091357313*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Theta]", "=", 
  RowBox[{
   RowBox[{"ArcSin", "[", "k", "]"}], "//", "ToSI"}]}]], "Input"],

Cell[BoxData["0.2685044861383903`"], "Output",
 CellChangeTimes->{3.5756263996957254`*^9, 3.5866062517423744`*^9, 
  3.5866573716228323`*^9, 3.586660820302853*^9, 3.5883993335766554`*^9, 
  3.5883994223348618`*^9, 3.589079091379328*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox["\[Theta]", "Degree"]], "Input"],

Cell[BoxData["15.384173836058679`"], "Output",
 CellChangeTimes->{3.575626399710725*^9, 3.586606251760418*^9, 
  3.5866573716398354`*^9, 3.5866608203168383`*^9, 3.588399333589628*^9, 
  3.588399422348891*^9, 3.589079091396329*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 29.3 g-factors (p. 520)\
\>", "Section",
 CellChangeTimes->{{3.5029435244519997`*^9, 3.502943532276*^9}, {
  3.502943714133*^9, 3.502943715492*^9}, {3.588399486232977*^9, 
  3.5883994890243893`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(5\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5756263998757596`*^9, 3.586606251923397*^9, 
  3.5866573717858534`*^9, 3.5866608204628916`*^9, 3.588399609394672*^9, 
  3.5890790915833025`*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"900.", " ", 
    RowBox[{"Kilometer", "/", "Hour"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "typical", " ", "speed", " ", "of", " ", "commercial", " ", "jet"}], " ", 
   "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Beta]", "=", 
   RowBox[{"15.", " ", "Degree"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "maximal", " ", "bank", " ", "angle", " ", "for", " ", "passenger", " ", 
    "comfort"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.502943546524*^9, 3.5029435528380003`*^9}, {
  3.5883995168018913`*^9, 3.5883995337370105`*^9}, {3.588399565306023*^9, 
  3.5883996032418394`*^9}, {3.5883996997381306`*^9, 3.5883997084492054`*^9}}],

Cell["\<\
Effective (centrifugal plus earth\[CloseCurlyQuote]s) gravity for various \
bank angles\
\>", "Text",
 CellChangeTimes->{{3.588399639338425*^9, 3.588399679792567*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sec", "[", 
  RowBox[{"60", " ", "Degree"}], "]"}]], "Input",
 CellChangeTimes->{3.502943614954*^9}],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.575626399934764*^9, 3.586606251984492*^9, 
  3.5866573718718634`*^9, 3.5866608205249033`*^9, 3.58839960947669*^9, 
  3.5890790916893606`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sec", "[", 
  RowBox[{"84.3", " ", "Degree"}], "]"}]], "Input",
 CellChangeTimes->{{3.502943621561*^9, 3.502943622361*^9}}],

Cell[BoxData["10.068490935399732`"], "Output",
 CellChangeTimes->{3.5756263999487677`*^9, 3.586606252003407*^9, 
  3.586657371890852*^9, 3.586660820542904*^9, 3.588399609493642*^9, 
  3.58907909170632*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sec", "[", "\[Beta]", "]"}]], "Input",
 CellChangeTimes->{3.502943626217*^9}],

Cell[BoxData["1.035276180410083`"], "Output",
 CellChangeTimes->{3.5756263999617724`*^9, 3.586606252018448*^9, 
  3.58665737190487*^9, 3.5866608205588923`*^9, 3.588399609510663*^9, 
  3.58907909172132*^9}]
}, Open  ]],

Cell["Clean turn radius", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"R", "=", 
  RowBox[{
   FractionBox[
    SuperscriptBox["U", "2"], 
    RowBox[{
     SubscriptBox["g", "0"], 
     RowBox[{"Tan", "[", "\[Beta]", "]"}]}]], "//", 
   RowBox[{"ToUnit", "[", "Kilometer", "]"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"23.785204475845966`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5756264000527983`*^9, 3.5866062521134233`*^9, 
  3.5866573719978385`*^9, 3.5866608206909094`*^9, 3.588399609598691*^9, 
  3.589079091826377*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"2", "R"}]], "Input"],

Cell[BoxData[
 TemplateBox[{"47.57040895169193`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575626400066739*^9, 3.58660625216243*^9, 
  3.586657372017873*^9, 3.586660820762928*^9, 3.5883996096146493`*^9, 
  3.5890790918463945`*^9}]
}, Open  ]],

Cell["Time of full turn", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"2", "\[Pi]", " ", "R"}], "U"], "//", 
   RowBox[{"ToUnit", "[", "Minute", "]"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"9.9631231527265`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"min\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: minutes"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Minutes\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5756264001437483`*^9, 3.5866062522444763`*^9, 
  3.586657372095851*^9, 3.5866608208498983`*^9, 3.588399609691659*^9, 
  3.5890790919323473`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 29.4 Estimating lift (p. 520)\
\>", "Title",
 CellChangeTimes->{{3.502943674006*^9, 3.5029436840360003`*^9}, {
  3.5883997491134267`*^9, 3.5883997522407675`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 29.5 and 29.7: Cessna 150 (p. 522 and p. 526)\
\>", "Section",
 CellChangeTimes->{{3.502943783383*^9, 3.502943789316*^9}, {
  3.588399785424048*^9, 3.5883997882083893`*^9}, {3.5884018636649857`*^9, 
  3.5884018831764917`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(5\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.575626496017787*^9, 3.5866062524104614`*^9, 3.5866573722458706`*^9, 
   3.586660821057925*^9, 3.588399965775894*^9, {3.5884000206888695`*^9, 
   3.588400068589954*^9}, 3.5884022358152647`*^9, 3.589079092139372*^9}]
}, Open  ]],

Cell["Parameters", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"L", "=", 
   RowBox[{"10.17", "  ", "Meter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"wing", " ", "span"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"A", "=", 
   RowBox[{"14.8", " ", 
    SuperscriptBox["Meter", "2"]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"wing", " ", "area"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"681", " ", "Kilogram"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"maximum", " ", "take"}], "-", 
    RowBox[{"off", " ", "mass"}]}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"180.", " ", 
    RowBox[{"Kilometer", "/", "Hour"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"cruise", " ", "speed"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Nu]", "=", 
   RowBox[{"1.5*^-5", " ", 
    RowBox[{
     SuperscriptBox["Meter", "2"], "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"air", " ", "viscosity"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["c", "S"], "=", 
   RowBox[{"343", " ", 
    RowBox[{"Meter", "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"speed", " ", "of", " ", "sound"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{"1.2", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"air", " ", "density"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["p", "0"], "=", 
   RowBox[{"1", "Atmosphere"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"air", " ", "pressure"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Alpha]", "0"], "=", 
   RowBox[{
    RowBox[{"-", "2"}], "Degree"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"zero", "-", 
    RowBox[{"lift", " ", "angle", " ", "of", " ", "attack"}]}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Alpha]", "stall"], "=", 
   RowBox[{"16", "Degree"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"stall", " ", "angle"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{
  3.502943972685*^9, {3.502945725115*^9, 3.5029457265699997`*^9}, {
   3.588399797994622*^9, 3.588399880810154*^9}, {3.588399994625599*^9, 
   3.5884000095134497`*^9}, {3.5884019065694923`*^9, 
   3.5884019355281153`*^9}, {3.5884019686673546`*^9, 3.5884019897950506`*^9}}],

Cell["Average chord length", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"c", "=", 
  RowBox[{
   FractionBox["A", "L"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"1.4552605703048183`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575626496241846*^9, 3.5866062526125245`*^9, 3.5866573724429355`*^9, 
   3.586660821288954*^9, 3.588399966006925*^9, {3.588400020902896*^9, 
   3.5884000688039813`*^9}, 3.588402236038333*^9, 3.589079092555467*^9}]
}, Open  ]],

Cell["Aspect ratio", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["L", "c"], "//", "ToSI"}]], "Input"],

Cell[BoxData["6.988439189189188`"], "Output",
 CellChangeTimes->{
  3.57562649627882*^9, 3.586606252653508*^9, 3.586657372483901*^9, 
   3.5866608213370037`*^9, 3.58839996603793*^9, {3.588400020945902*^9, 
   3.588400068842986*^9}, 3.5884022360782948`*^9, 3.58907909267548*^9}]
}, Open  ]],

Cell["Wing load", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]p", "=", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"M", " ", 
      SubscriptBox["g", "0"]}], "A"]}], "//", 
   RowBox[{"ToUnit", "[", "Pascal", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.390206772611786*^9, 3.390206792116786*^9}, 
   3.50294668944*^9}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "451.23842229729723`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"Pa\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pascals"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Pascals\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264963728323`*^9, 3.5866062527525077`*^9, 3.586657372586913*^9, 
   3.586660821464976*^9, 3.58839996613297*^9, {3.5884000210409136`*^9, 
   3.588400068935998*^9}, 3.5884022361713085`*^9, 3.5890790931235437`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["\[CapitalDelta]p", 
   SubscriptBox["p", "0"]], " ", "100", " ", "Percent"}]], "Input",
 CellChangeTimes->{{3.5883999427209654`*^9, 3.588399984106276*^9}, {
  3.588400014546096*^9, 3.5884001221217546`*^9}}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "0.4453376978014283`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"%\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: percent"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Percent\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575626496392834*^9, 3.586606252768529*^9, 3.5866573726049147`*^9, 
   3.586660821504981*^9, {3.5883999450072556`*^9, 3.5883999661539683`*^9}, {
   3.588400021437968*^9, 3.588400122587826*^9}, 3.5884022362353554`*^9, 
   3.589079093219549*^9}]
}, Open  ]],

Cell["Reynolds number", "Text",
 CellChangeTimes->{{3.5884028594575305`*^9, 3.5884028710599575`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Rey", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"U", " ", "c"}], "\[Nu]"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData["4.850868567682728`*^6"], "Output",
 CellChangeTimes->{
  3.5756264964988475`*^9, 3.5866062528745728`*^9, 3.5866573727089715`*^9, 
   3.586660821616011*^9, 3.588399966256955*^9, {3.5884000215489783`*^9, 
   3.58840004659416*^9}, 3.5884022363383665`*^9, 3.5890790933955727`*^9}]
}, Open  ]],

Cell["Mach number", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ma", "=", 
  RowBox[{
   FractionBox["U", 
    SubscriptBox["c", "S"]], "//", "ToSI"}]}]], "Input"],

Cell[BoxData["0.1457725947521866`"], "Output",
 CellChangeTimes->{
  3.575626496563856*^9, 3.5866062529395657`*^9, 3.5866573727739415`*^9, 
   3.586660821681003*^9, 3.5883999663200245`*^9, {3.5884000216130266`*^9, 
   3.588400046658217*^9}, 3.5884022364033766`*^9, 3.58907909347554*^9}]
}, Open  ]],

Cell["Velocity difference across the wing (29.19)", "Text",
 CellChangeTimes->{{3.5884002455445614`*^9, 3.588400277440484*^9}, {
  3.5884003075053253`*^9, 3.5884003183366833`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[CapitalDelta]v", "ab"], "=", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["\[CapitalDelta]p", 
     RowBox[{
      SubscriptBox["\[Rho]", "0"], "U"}]]}], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"7.52064037162162`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575626496738921*^9, 3.586606253211559*^9, 3.5866573729539585`*^9, 
   3.586660822064103*^9, 3.5883999664919844`*^9, {3.5884000217720127`*^9, 
   3.588400046821189*^9}, 3.5884022365463753`*^9, 3.589079093651601*^9}]
}, Open  ]],

Cell["Lift", "Text",
 CellChangeTimes->{{3.5884022089449043`*^9, 3.588402209551957*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"lift", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Rho]", "0"], "U", " ", "L", " ", "c", " ", 
    SubscriptBox["\[CapitalDelta]v", "ab"]}], "//", 
   RowBox[{"ToUnit", "[", "Newton", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.502947542116*^9, 3.502947564408*^9}, {3.502947599382*^9,
    3.502947628889*^9}, 3.5884020235393515`*^9}],

Cell[BoxData[
 TemplateBox[{"6678.3286499999995`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"N\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: newtons"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Newtons\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575626499044208*^9, 3.5866062611085925`*^9, 3.5866573760573883`*^9, 
   3.5866608250914326`*^9, 3.588399968786277*^9, {3.5884000239463215`*^9, 
   3.5884000489574633`*^9}, 3.5884006586819143`*^9, 3.5884007289818873`*^9, 
   3.588401683305102*^9, 3.588401799793913*^9, 3.588402025111494*^9, 
   3.5884021236260157`*^9, 3.588402236794401*^9, 3.5890790941556654`*^9}]
}, Open  ]],

Cell["Check balance", "Text",
 CellChangeTimes->{{3.588402216888912*^9, 3.588402221528532*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox["lift", 
  RowBox[{"M", " ", 
   SubscriptBox["g", "0"]}]]], "Input",
 CellChangeTimes->{{3.5884021039635572`*^9, 3.588402192193721*^9}}],

Cell[BoxData["1.`"], "Output",
 CellChangeTimes->{{3.588402111409457*^9, 3.588402133919317*^9}, {
   3.5884021655013585`*^9, 3.5884021930348577`*^9}, 3.5884022369064193`*^9, 
   3.589079094475706*^9}]
}, Open  ]],

Cell["Lift coefficient", "Text",
 CellChangeTimes->{{3.588402229000453*^9, 3.588402231304751*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["C", "L"], "=", 
  RowBox[{
   FractionBox["lift", 
    RowBox[{
     FractionBox["1", "2"], 
     SubscriptBox["\[Rho]", "0"], 
     SuperscriptBox["U", "2"], "A"}]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.502946785988*^9, 3.5029467930959997`*^9}, {
  3.502946872249*^9, 3.50294690208*^9}}],

Cell[BoxData["0.30082561486486487`"], "Output",
 CellChangeTimes->{
  3.5756264993052607`*^9, 3.586606261876688*^9, 3.5866573762663746`*^9, 
   3.5866608253594646`*^9, 3.5883999689973025`*^9, {3.588400024181351*^9, 
   3.5884000491635275`*^9}, 3.5884006589690094`*^9, 3.588400729233918*^9, 
   3.588401683515142*^9, 3.5884018000029383`*^9, 3.588402063674428*^9, 
   3.588402237142495*^9, 3.589079094707736*^9}]
}, Open  ]],

Cell["Lift slope", "Text",
 CellChangeTimes->{{3.5884022786327515`*^9, 3.5884022802169266`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Lambda]", "=", 
  FractionBox[
   RowBox[{"2", "\[Pi]"}], 
   RowBox[{"1", "+", 
    RowBox[{"2", " ", 
     FractionBox["c", "L"]}]}]]}]], "Input",
 CellChangeTimes->{{3.502947003792*^9, 3.50294703723*^9}}],

Cell[BoxData["4.885126050187191`"], "Output",
 CellChangeTimes->{
  3.5756264995242343`*^9, 3.586606261971699*^9, 3.5866573763054223`*^9, 
   3.5866608254174724`*^9, 3.588399969069312*^9, {3.5884000242213173`*^9, 
   3.5884000492315364`*^9}, 3.588400659037958*^9, 3.588400729276947*^9, 
   3.588401683560113*^9, 3.588401800040903*^9, 3.5884022371824737`*^9, 
   3.5890790947567463`*^9}]
}, Open  ]],

Cell["Angle of attack difference", "Text",
 CellChangeTimes->{{3.588402329648199*^9, 3.588402352136104*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]\[Alpha]", "=", 
  FractionBox[
   SubscriptBox["C", "L"], "\[Lambda]"]}]], "Input",
 CellChangeTimes->{{3.502947073755*^9, 3.5029471035439997`*^9}, {
   3.502947246909*^9, 3.502947259517*^9}, 3.50294729235*^9}],

Cell[BoxData["0.06157990843518514`"], "Output",
 CellChangeTimes->{
  3.5756264996122484`*^9, 3.586606265596116*^9, 3.586657376320424*^9, 
   3.5866608254394755`*^9, 3.5883999690893145`*^9, {3.5884000242393475`*^9, 
   3.588400049250497*^9}, 3.5884006590560036`*^9, 3.588400729295883*^9, 
   3.58840168357714*^9, 3.5884018000569057`*^9, 3.5884022372034793`*^9, 
   3.589079094776746*^9}]
}, Open  ]],

Cell["True angle of attack", "Text",
 CellChangeTimes->{{3.588402382608938*^9, 3.5884023875925694`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Alpha]", "=", 
  RowBox[{
   SubscriptBox["\[Alpha]", "0"], "+", "\[CapitalDelta]\[Alpha]", 
   " "}]}]], "Input",
 CellChangeTimes->{{3.5029471360690002`*^9, 3.502947147465*^9}, {
  3.5029472182200003`*^9, 3.502947231255*^9}, {3.502947327575*^9, 
  3.502947348553*^9}, {3.5029474027320004`*^9, 3.502947405182*^9}}],

Cell[BoxData["0.02667332339529855`"], "Output",
 CellChangeTimes->{
  3.5756264997723265`*^9, 3.5866062657921963`*^9, 3.586657376349413*^9, 
   3.5866608255124855`*^9, 3.5883999691273174`*^9, {3.5884000242713623`*^9, 
   3.588400049286502*^9}, 3.588400659088969*^9, 3.5884007293329306`*^9, 
   3.588401683612156*^9, 3.588401800088952*^9, 3.588402237235442*^9, 
   3.5890790947957087`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Alpha]", " ", "/", "Degree"}]], "Input",
 CellChangeTimes->{{3.588402505577586*^9, 3.5884025335150843`*^9}}],

Cell[BoxData["1.528268856138166`"], "Output",
 CellChangeTimes->{{3.588402526685266*^9, 3.5884025339902315`*^9}, 
   3.589079094811751*^9}]
}, Open  ]],

Cell["Lift coefficient at stall", "Text",
 CellChangeTimes->{{3.5884025917285566`*^9, 3.5884025983523703`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["C", "Lstall"], "=", 
  RowBox[{"\[Lambda]", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["\[Alpha]", "stall"], "-", 
     SubscriptBox["\[Alpha]", "0"]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.502947927716*^9, 3.502947981898*^9}}],

Cell[BoxData["1.5347076111128202`"], "Output",
 CellChangeTimes->{
  3.5756265007163773`*^9, 3.58660626632026*^9, 3.5866573765484667`*^9, 
   3.586660825983569*^9, 3.588399969164324*^9, {3.588400024305331*^9, 
   3.588400049320506*^9}, 3.588400659124013*^9, 3.588400729367936*^9, 
   3.5884016836531606`*^9, 3.5884018001209545`*^9, 3.5884022372734604`*^9, 
   3.589079094829715*^9}]
}, Open  ]],

Cell["Stall velocity", "Text",
 CellChangeTimes->{{3.5884026355290737`*^9, 3.5884026376403637`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["U", "stall"], "=", 
  RowBox[{
   SqrtBox[
    FractionBox["lift", 
     RowBox[{
      FractionBox["1", "2"], 
      SubscriptBox["C", "Lstall"], 
      SubscriptBox["\[Rho]", "0"], "A"}]]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.5029480046809998`*^9, 3.502948076259*^9}}],

Cell[BoxData[
 TemplateBox[{"22.13678705526845`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756265008993998`*^9, 3.5866062664682627`*^9, 3.5866573781546197`*^9, 
   3.586660826506641*^9, 3.588399969386352*^9, {3.5884000244263573`*^9, 
   3.5884000494445224`*^9}, 3.588400659247025*^9, 3.5884007294989276`*^9, 
   3.5884016837791753`*^9, 3.58840180024998*^9, 3.5884022374095044`*^9, 
   3.5884026046171107`*^9, 3.589079094999735*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["U", "stall"], "//", 
  RowBox[{"ToUnit", "[", 
   RowBox[{"Kilometer", "/", "Hour"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5029480848719997`*^9, 3.502948099417*^9}}],

Cell[BoxData[
 TemplateBox[{"79.69243339896641`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"km\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]",
             "\"h\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers per hour"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilometers\"", "\"Hours\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756265013374557`*^9, 3.586606266548269*^9, 3.586657379061767*^9, 
   3.5866608265996265`*^9, 3.588399969459361*^9, {3.5884000244853964`*^9, 
   3.588400049501529*^9}, 3.5884006593050303`*^9, 3.5884007295679207`*^9, 
   3.5884016838651857`*^9, 3.588401800308*^9, 3.588402237469513*^9, 
   3.589079095086775*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 29.6 Boeing 747-400 (p. 522)\
\>", "Section",
 CellChangeTimes->{{3.502944342326*^9, 3.5029443489969997`*^9}, {
  3.5884002879608765`*^9, 3.588400304391942*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(21\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.5756264968998976`*^9, 3.5866062538276362`*^9, 3.586657373124013*^9, 
   3.586660822272077*^9, 3.5883999666430035`*^9, {3.5884000219580307`*^9, 
   3.5884000469732647`*^9}, 3.588400656682679*^9, 3.588400726928627*^9, 
   3.589079095308815*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"L", "=", 
   RowBox[{"64.", " ", "Meter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"wing", " ", "span"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"A", "=", 
   RowBox[{"520.", " ", 
    SuperscriptBox["Meter", "2"]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"wing", " ", "area"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"900.", " ", 
    RowBox[{"Kilometer", "/", "Hour"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"cruising", " ", "speed"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"z", "=", 
   RowBox[{"10.", " ", "Kilometer"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"cruising", " ", "height"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["p", "0"], "=", 
   RowBox[{"1", " ", "Atmosphere"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"ground", " ", "air", " ", "pressure"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["h", "2"], "=", 
   RowBox[{"30.", " ", "Kilometer"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"atmospheric", " ", "scale", " ", "height"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"400000.", " ", "Kilogram"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"maximal", " ", "mass", " ", "of", " ", "aircraft"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Eta]", "0"], "=", 
   RowBox[{"1.8", " ", 
    SuperscriptBox["10", 
     RowBox[{"-", "5"}]], "Pascal", " ", "Second"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"ground", " ", "viscosity", " ", "of", " ", "air"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{"1.2", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"ground", " ", "density", " ", "of", " ", "air"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["T", "0"], "=", 
   RowBox[{
    RowBox[{"30", "Celsius"}], "//", "ToSI"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"ground", " ", "temperature"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Gamma]", "=", 
   FractionBox["7", "5"]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"adiabatic", " ", "index"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.481345602018532*^9, 3.4813456037696323`*^9}, {
   3.502944380112*^9, 3.502944393916*^9}, {3.502944423998*^9, 
   3.502944482974*^9}, {3.502944524766*^9, 3.5029445276619997`*^9}, {
   3.502944565387*^9, 3.502944566413*^9}, {3.502944600848*^9, 
   3.5029446622539997`*^9}, 3.502945304157*^9, {3.502945443986*^9, 
   3.502945469521*^9}, {3.588400325930646*^9, 3.588400547960905*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Ground level", "Subsection",
 CellChangeTimes->{{3.5884008416162276`*^9, 3.5884008871280513`*^9}, {
  3.58840092291248*^9, 3.5884009246407423`*^9}}],

Cell["Chord length", "Text",
 CellChangeTimes->{{3.502945299126*^9, 3.5029453010290003`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"c", "=", 
  RowBox[{
   FractionBox["A", "L"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"8.125`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575626497107924*^9, 3.5866062547317495`*^9, 3.586657373542034*^9, 
   3.586660822504125*^9, 3.58839996684803*^9, {3.588400022126094*^9, 
   3.588400047163232*^9}, 3.5884006568797245`*^9, 3.5884007271286535`*^9, 
   3.5890790956918674`*^9}]
}, Open  ]],

Cell["Aspect ratio", "Text",
 CellChangeTimes->{{3.502945578598*^9, 3.5029455809639997`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["L", "c"], "//", "ToSI"}]], "Input"],

Cell[BoxData["7.876923076923077`"], "Output",
 CellChangeTimes->{
  3.575626497148929*^9, 3.5866062548047595`*^9, 3.5866573736140814`*^9, 
   3.586660822552144*^9, 3.58839996689009*^9, {3.5884000221660767`*^9, 
   3.5884000472022753`*^9}, 3.5884006569196897`*^9, 3.5884007271696177`*^9, 
   3.589079095739899*^9}]
}, Open  ]],

Cell["Wing loading", "Text",
 CellChangeTimes->{{3.5029449607060003`*^9, 3.502944970324*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]p", "=", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"M", " ", 
      SubscriptBox["g", "0"]}], "A"]}], "//", 
   RowBox[{"ToUnit", "[", "Pascal", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.390206772611786*^9, 3.390206792116786*^9}, 
   3.502946714751*^9}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "7543.576923076923`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"Pa\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: pascals"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Pascals\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264972629447`*^9, 3.5866062550118437`*^9, 3.5866573743661785`*^9, 
   3.58666082275414*^9, 3.5883999670160513`*^9, {3.5884000222860727`*^9, 
   3.5884000473132515`*^9}, 3.588400657031726*^9, 3.588400727283636*^9, 
   3.589079096268933*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["\[CapitalDelta]p", "Atmosphere"], "100", " ", 
  "Percent"}]], "Input",
 CellChangeTimes->{{3.5029450066099997`*^9, 3.502945065435*^9}, {
  3.5884006119300194`*^9, 3.5884006142993307`*^9}}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "7.444931579646605`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"%\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: percent"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Percent\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264973279514`*^9, 3.5866062551158547`*^9, 3.586657374430185*^9, 
   3.586660822872153*^9, 3.5883999670900607`*^9, {3.5884000223590918`*^9, 
   3.588400047379263*^9}, 3.588400614844385*^9, 3.5884006570957355`*^9, 
   3.588400727349636*^9, 3.5890790963479433`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cruising altitude", "Subsection",
 CellChangeTimes->{{3.588400789913662*^9, 3.588400794281145*^9}}],

Cell["Pressure at cruising altitude", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"p", "=", 
  RowBox[{
   SubscriptBox["p", "0"], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", 
      FractionBox["z", 
       SubscriptBox["h", "2"]]}], ")"}], 
    RowBox[{"\[Gamma]", "/", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.24192491286747445`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"atm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: atmospheres"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Atmospheres\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264973779583`*^9, 3.586606255620904*^9, 3.586657374484193*^9, 
   3.586660822929208*^9, 3.5883999671530724`*^9, {3.5884000224090877`*^9, 
   3.588400047436308*^9}, 3.588400657144718*^9, 3.5884007274006467`*^9, 
   3.589079096475979*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["\[CapitalDelta]p", "p"], "100", "Percent"}]], "Input",
 CellChangeTimes->{{3.502945070975*^9, 3.502945080007*^9}, {
  3.588400721081891*^9, 3.5884007232261543`*^9}}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "30.77372847386292`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"%\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: percent"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Percent\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264974439664`*^9, 3.5866062564599667`*^9, 3.586657374548159*^9, 
   3.586660823043206*^9, 3.5883999672220774`*^9, {3.5884000224831*^9, 
   3.5884000475023146`*^9}, 3.5884006572107687`*^9, {3.5884007235812016`*^9, 
   3.588400727466721*^9}, 3.5890790965719967`*^9}]
}, Open  ]],

Cell["Density", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Rho]", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Rho]", "0"], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["z", 
        SubscriptBox["h", "2"]]}], ")"}], 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"\[Gamma]", "-", "1"}], ")"}]}]]}], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.43546484316145395`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"kg\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"m\"", "3"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: kilograms per meter cubed"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Kilograms\"", 
        SuperscriptBox["\"Meters\"", "3"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264975109744`*^9, 3.5866062565930376`*^9, 3.5866573746122236`*^9, 
   3.586660823138186*^9, 3.5883999672911034`*^9, {3.588400022535116*^9, 
   3.588400047554302*^9}, 3.5884006572617517`*^9, 3.5884007275197315`*^9, 
   3.5890790967719564`*^9}]
}, Open  ]],

Cell["Temperature", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"T", "=", 
  RowBox[{
   SubscriptBox["T", "0"], 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FractionBox["z", 
      SubscriptBox["h", "2"]]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"202.1`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"K\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kelvins"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kelvins\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264975649815`*^9, 3.5866062567400017`*^9, 3.586657374663173*^9, 
   3.5866608232162027`*^9, 3.588399967347093*^9, {3.588400022593111*^9, 
   3.5884000476062884`*^9}, 3.588400657311779*^9, 3.588400727570668*^9, 
   3.589079096852009*^9}]
}, Open  ]],

Cell["Sound speed", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["c", "S"], "=", 
  RowBox[{
   SqrtBox[
    RowBox[{"\[Gamma]", " ", 
     FractionBox["p", "\[Rho]"]}]], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"280.72821969537256`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264976659946`*^9, 3.5866062569720736`*^9, 3.586657374763186*^9, 
   3.5866608236263285`*^9, 3.58839996745211*^9, {3.588400022689123*^9, 
   3.5884000477103014`*^9}, 3.5884006574087963`*^9, 3.5884007276666765`*^9, 
   3.5890790969960318`*^9}]
}, Open  ]],

Cell["Mach number", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ma", "=", 
  RowBox[{
   FractionBox["U", 
    SubscriptBox["c", "S"]], "//", "ToSI"}]}]], "Input"],

Cell[BoxData["0.8905410374179099`"], "Output",
 CellChangeTimes->{
  3.5756264977300024`*^9, 3.586606257100047*^9, 3.5866573748302364`*^9, 
   3.5866608236952887`*^9, 3.5883999675211153`*^9, {3.588400022753173*^9, 
   3.5884000477763104`*^9}, 3.5884006574727926`*^9, 3.588400727732689*^9, 
   3.5890790971000013`*^9}]
}, Open  ]],

Cell["Viscosity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Eta]", "=", 
  RowBox[{
   SubscriptBox["\[Eta]", "0"], 
   SqrtBox[
    FractionBox["T", 
     SubscriptBox["T", "0"]]]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.00001469693845669907`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"s\"", "\[ThinSpace]", "\"Pa\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: second pascals"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       RowBox[{"\"Pascals\"", " ", "\"Seconds\""}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264977900434`*^9, 3.5866062574521337`*^9, 3.5866573748882456`*^9, 
   3.5866608237543077`*^9, 3.5883999675851216`*^9, {3.5884000227951365`*^9, 
   3.588400047816315*^9}, 3.5884006575128045`*^9, 3.588400727773732*^9, 
   3.5890790971650476`*^9}]
}, Open  ]],

Cell["Kinematic viscosity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Nu]", "=", 
  RowBox[{
   FractionBox["\[Eta]", "\[Rho]"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.00003375`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{
            SuperscriptBox["\"m\"", "2"], "\[NegativeMediumSpace]", "\"/\"", 
            "\[InvisibleSpace]", "\"s\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters squared per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox[
        SuperscriptBox["\"Meters\"", "2"], "\"Seconds\""]}], 
     "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264978870635`*^9, 3.586606257580119*^9, 3.5866573749842663`*^9, 
   3.5866608238533106`*^9, 3.5883999676871367`*^9, {3.588400022906151*^9, 
   3.5884000478993473`*^9}, 3.5884006575948157`*^9, 3.588400727856744*^9, 
   3.5890790973000317`*^9}]
}, Open  ]],

Cell["Reynolds number", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["Rey", "c"], "=", 
  RowBox[{
   FractionBox[
    RowBox[{"U", " ", "c"}], "\[Nu]"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData["6.018518518518519`*^7"], "Output",
 CellChangeTimes->{
  3.575626497990035*^9, 3.5866062577401695`*^9, 3.5866573750862303`*^9, 
   3.5866608239813347`*^9, 3.58839996779515*^9, {3.588400023009164*^9, 
   3.5884000480023413`*^9}, 3.588400657697813*^9, 3.5884007279607573`*^9, 
   3.589079097436099*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[CapitalDelta]v", "ab"], "=", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["\[CapitalDelta]p", 
     RowBox[{
      SubscriptBox["\[Rho]", "0"], "U"}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4813465535869584`*^9, 3.4813465862958293`*^9}}],

Cell[BoxData[
 TemplateBox[{"25.145256410256412`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575626498135091*^9, 3.58660625791619*^9, 3.5866573752322493`*^9, 
   3.5866608241243443`*^9, 3.5883999679431715`*^9, {3.588400023153182*^9, 
   3.5884000481433954`*^9}, 3.588400657867853*^9, 3.5884007281377363`*^9, 
   3.589079097812088*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[CapitalDelta]v", "ab"], "=", 
  RowBox[{
   RowBox[{"-", 
    FractionBox["\[CapitalDelta]p", 
     RowBox[{"\[Rho]", " ", "U"}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.502945908073*^9, 3.502945917821*^9}}],

Cell[BoxData[
 TemplateBox[{"69.29217861364803`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756264983141155`*^9, 3.586606258249245*^9, 3.586657375375304*^9, 
   3.5866608242663293`*^9, 3.5883999681121902`*^9, {3.5884000232972*^9, 
   3.588400048312378*^9}, 3.58840065801787*^9, 3.588400728309758*^9, 
   3.5890790979881535`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 29.5 Estimating drag (p. 527)\
\>", "Title",
 CellChangeTimes->{{3.502948194502*^9, 3.502948201428*^9}, {
  3.588402661745412*^9, 3.5884026648887672`*^9}, {3.5884026953047633`*^9, 
  3.5884026953686647`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Example 29.8 Cessna 150 (p. 528)\
\>", "Section",
 CellChangeTimes->{{3.5029482364370003`*^9, 3.502948242565*^9}, {
  3.5884026761932554`*^9, 3.5884026864565344`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5029563740550003`*^9, 3.502956377366*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(20\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.575626539250216*^9, 3.586606266996322*^9, 
  3.586657379598835*^9, 3.586660826747675*^9, 3.5884027019644794`*^9, 
  3.5890790982281427`*^9}]
}, Open  ]],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.588403061488179*^9, 3.5884030642086143`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["Rey", "c"], "=", 
   RowBox[{"5", "\[Times]", 
    SuperscriptBox["10", "6"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"180.", " ", 
    RowBox[{"Kilometer", "/", "Hour"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"c", "=", 
   RowBox[{"1.5", "Meter"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5029482520889997`*^9, 3.502948280409*^9}, {
   3.502948356074*^9, 3.5029483563269997`*^9}, {3.502948469626*^9, 
   3.502948480969*^9}, {3.5029572073380003`*^9, 3.50295721694*^9}, 
   3.50295730737*^9}],

Cell["Laminar  skin drag coefficient", "Text",
 CellChangeTimes->{{3.5884030678250012`*^9, 3.58840308006455*^9}, {
  3.588403111248528*^9, 3.5884031143369045`*^9}, {3.588403410641531*^9, 
  3.588403411672662*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["C", "Dlam"], "=", 
  RowBox[{
   RowBox[{"2.65", " ", 
    SuperscriptBox[
     SubscriptBox["Rey", "c"], 
     RowBox[{
      RowBox[{"-", "1"}], "/", "2"}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.502956189408*^9, 3.502956189803*^9}}],

Cell[BoxData["0.0011851160280748885`"], "Output",
 CellChangeTimes->{3.575626539389272*^9, 3.5866062703047085`*^9, 
  3.586657380578971*^9, 3.586660826843688*^9, 3.5884027021116114`*^9, 
  3.5890790983812*^9}]
}, Open  ]],

Cell["\<\
Turbulent skin drag coefficient\
\>", "Text",
 CellChangeTimes->{{3.5884034075581408`*^9, 3.5884034188966093`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["C", "Dturb"], "=", 
  RowBox[{
   RowBox[{"0.063", " ", 
    SuperscriptBox[
     SubscriptBox["Rey", "c"], 
     RowBox[{
      RowBox[{"-", "1"}], "/", "7"}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.502956198204*^9, 3.5029561986029997`*^9}}],

Cell[BoxData["0.0069557639361450185`"], "Output",
 CellChangeTimes->{3.575626539405307*^9, 3.5866062703647537`*^9, 
  3.586657380646953*^9, 3.586660826857654*^9, 3.588402702126526*^9, 
  3.5890790984122066`*^9}]
}, Open  ]],

Cell["\<\
Laminar boundary layer thickness (28.22)\
\>", "Text",
 CellChangeTimes->{{3.588403122624958*^9, 3.5884031260404053`*^9}, {
  3.588403317824791*^9, 3.588403354704427*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Delta]", "lam"], "=", 
  RowBox[{
   RowBox[{"5", " ", 
    SuperscriptBox[
     SubscriptBox["Rey", "c"], 
     RowBox[{
      RowBox[{"-", "1"}], "/", "2"}]], "c"}], "//", 
   RowBox[{"ToUnit", "[", "Millimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.502948466205*^9, 3.502948466635*^9}}],

Cell[BoxData[
 TemplateBox[{"3.3541019662496843`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"mm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: millimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Millimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575626539354229*^9, 3.586606269756675*^9, 
  3.5866573798618684`*^9, 3.5866608268087597`*^9, 3.5884027020794935`*^9, 
  3.589079098499175*^9}]
}, Open  ]],

Cell["\<\
Turbulent boundary layer thickness (28.45)\
\>", "Text",
 CellChangeTimes->{{3.588403357539798*^9, 3.588403394928569*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Delta]", "turb"], "=", 
  RowBox[{
   RowBox[{"0.16", " ", 
    SuperscriptBox[
     SubscriptBox["Rey", "c"], 
     RowBox[{
      RowBox[{"-", "1"}], "/", "7"}]], "c"}], "//", 
   RowBox[{"ToUnit", "[", "Millimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.502948503134*^9, 3.5029485035299997`*^9}}],

Cell[BoxData[
 TemplateBox[{"26.4981483281715`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"mm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: millimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Millimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575626539373238*^9, 3.586606269985668*^9, 
  3.586657380541954*^9, 3.5866608268266816`*^9, 3.588402702095524*^9, 
  3.5890790986041956`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 29.9 Cessna 150 Induced drag (p. 530)\
\>", "Section",
 CellChangeTimes->{{3.502956416165*^9, 3.502956423476*^9}, {
   3.588403469433015*^9, 3.588403477281014*^9}, 3.5884037420007486`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5029563740550003`*^9, 3.502956377366*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(8\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.575626539600278*^9, 3.586606272097989*^9, 
  3.586657382362197*^9, 3.5866608270056915`*^9, 3.5890790988592205`*^9}]
}, Open  ]],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.5884037492005396`*^9, 3.588403750872759*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["C", "L"], "=", "0.3"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"lift", " ", "coefficient"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"L", "=", 
    RowBox[{"10.17", "Meter"}]}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"wing", " ", "span"}], " ", "*)"}], ";"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"c", "=", 
   RowBox[{"1.5", "Meter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"chord", " ", "length"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.502956460906*^9, 3.502956509564*^9}, {3.502956562283*^9,
    3.502956562634*^9}, 3.502956873498*^9, {3.588403480377418*^9, 
   3.5884034833777514`*^9}, {3.5884035336251545`*^9, 3.588403559625472*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["C", "Dind"], "=", 
  RowBox[{
   FractionBox["c", 
    RowBox[{"\[Pi]", " ", "L", " "}]], 
   SuperscriptBox[
    SubscriptBox["C", "L"], "2"]}]}]], "Input"],

Cell[BoxData["0.004225352471466249`"], "Output",
 CellChangeTimes->{3.5756265396432686`*^9, 3.5866062723079567`*^9, 
  3.5866573825021987`*^9, 3.5866608270477185`*^9, 3.5890790989402723`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 29.10 Cessna 150 (p. 530)\
\>", "Section",
 CellChangeTimes->{{3.5029566256219997`*^9, 3.502956646324*^9}, {
  3.588403873657347*^9, 3.588403878937049*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5029563740550003`*^9, 3.502956377366*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(3\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.5756265398022847`*^9, {3.5756267186457434`*^9, 3.5756267410965605`*^9}, 
   3.586606272524984*^9, 3.5866573828922644`*^9, 3.586660827191736*^9, 
   3.5884038948950186`*^9, {3.5884039432392015`*^9, 3.5884039571539693`*^9}, {
   3.5884039885429173`*^9, 3.5884039960408707`*^9}, 3.5890790991882625`*^9}]
}, Open  ]],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.588403870128872*^9, 3.588403871513094*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{"1.2", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["U", "glide"], "=", 
   RowBox[{"110", " ", 
    RowBox[{"Kilometer", "/", "Hour"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"A", "=", 
   RowBox[{"14.8", " ", 
    SuperscriptBox["Meter", "2"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"681.", " ", "Kilogram"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Alpha]", "0"], "=", 
   RowBox[{
    RowBox[{"-", "2"}], "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Lambda]", "=", "4.9"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5029567385220003`*^9, 3.502956776972*^9}, {
   3.502956817723*^9, 3.502956867867*^9}, {3.502956986829*^9, 
   3.5029570210220003`*^9}, 3.502957357869*^9, 3.5756267379652233`*^9, {
   3.588403829225705*^9, 3.588403863896121*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"LD", "=", "8.4"}], ";"}]], "Input",
 CellChangeTimes->{{3.5756267074693384`*^9, 3.5756267139882298`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Gamma]", "=", 
  RowBox[{"ArcCot", "[", "LD", "]"}]}]], "Input",
 CellChangeTimes->{{3.5884039390016775`*^9, 3.588403939505726*^9}}],

Cell[BoxData["0.11848995915813776`"], "Output",
 CellChangeTimes->{
  3.5756265398652945`*^9, {3.5756267187547555`*^9, 3.5756267412026253`*^9}, 
   3.586606274604286*^9, 3.586657383742324*^9, 3.5866608272577033`*^9, 
   3.5884038950530415`*^9, {3.588403943386222*^9, 3.5884039573099756`*^9}, {
   3.5884039886959715`*^9, 3.5884039961939163`*^9}, 3.589079099331291*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Gamma]", "/", "Degree"}]], "Input"],

Cell[BoxData["6.78897457443879`"], "Output",
 CellChangeTimes->{
  3.575626539883295*^9, {3.5756267187678013`*^9, 3.5756267412195897`*^9}, 
   3.586606277879712*^9, 3.5866573837744007`*^9, 3.586660827274705*^9, 
   3.588403895069062*^9, {3.588403943404212*^9, 3.588403957325982*^9}, {
   3.588403988712968*^9, 3.588403996208918*^9}, 3.5890790993553195`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["C", "Lglide"], "=", 
  FractionBox[
   RowBox[{"M", " ", 
    SubscriptBox["g", "0"], " ", 
    RowBox[{"Cos", "[", "\[Gamma]", "]"}]}], 
   RowBox[{
    FractionBox["1", "2"], 
    SubscriptBox["\[Rho]", "0"], 
    SuperscriptBox[
     SubscriptBox["U", "glide"], "2"], "A"}]]}]], "Input",
 CellChangeTimes->{3.3902068265397863`*^9, 3.5884039929865*^9}],

Cell[BoxData["0.7998684614612198`"], "Output",
 CellChangeTimes->{
  3.575626540184334*^9, {3.5756267190627937`*^9, 3.575626741511613*^9}, 
   3.5866062790528035`*^9, 3.586657384570476*^9, 3.586660827591804*^9, 
   3.588403895380119*^9, {3.588403943717235*^9, 3.588403957584983*^9}, {
   3.5884039889640045`*^9, 3.588403996463961*^9}, 3.5890790999164*^9}]
}, Open  ]],

Cell["Glide angle of attack", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Alpha]", "glide"], "=", 
  RowBox[{
   SubscriptBox["\[Alpha]", "0"], "+", 
   FractionBox[
    SubscriptBox["C", "Lglide"], "\[Lambda]"]}]}]], "Input",
 CellChangeTimes->{{3.502956926127*^9, 3.502956927326*^9}, 
   3.5884039842023945`*^9}],

Cell[BoxData["0.12833187648281133`"], "Output",
 CellChangeTimes->{
  3.5756265402013755`*^9, {3.5756267190797963`*^9, 3.5756267415276155`*^9}, 
   3.5866062791558924`*^9, 3.5866573847034807`*^9, 3.5866608276648035`*^9, 
   3.5884038953961077`*^9, {3.5884039437312727`*^9, 3.5884039576000395`*^9}, {
   3.5884039889820147`*^9, 3.5884039964819794`*^9}, 3.5890791001163807`*^9}]
}, Open  ]],

Cell["To degrees", "Text",
 CellChangeTimes->{{3.5884040064642363`*^9, 3.5884040109688206`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Alpha]", "glide"], "/", "Degree"}]], "Input"],

Cell[BoxData["7.352874899459272`"], "Output",
 CellChangeTimes->{
  3.575626540218378*^9, {3.5756267190958023`*^9, 3.5756267415446167`*^9}, 
   3.5866062793168774`*^9, 3.586657384781444*^9, 3.5866608276887636`*^9, 
   3.588403895411087*^9, {3.588403943746224*^9, 3.5884039576150293`*^9}, {
   3.5884039889989724`*^9, 3.588403996496942*^9}, 3.5890791001713867`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 29.11 Cessna 150:  Breguet\[CloseCurlyQuote]s range equation (p. 531)\
\>", "Section",
 CellChangeTimes->{{3.5029580894300003`*^9, 3.50295809226*^9}, {
  3.588404117049241*^9, 3.588404134784525*^9}, {3.5884041747865896`*^9, 
  3.588404180120282*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.502957408281*^9, 3.502957412008*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(11\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.5756265403843584`*^9, 3.575626797608657*^9, 3.5866062803879967`*^9, 
   3.586657385370589*^9, 3.5866608281539536`*^9, 3.588404161488924*^9, {
   3.588404457373515*^9, 3.5884044724324226`*^9}, 3.58907910041943*^9}]
}, Open  ]],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.588404156513302*^9, 3.5884041582725344`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"681.", " ", "Kilogram"}]}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{"maximal", " ", "mass"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"LD", "=", "5."}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"lift", " ", "to", " ", "drag", " ", "ratio"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", "=", 
   RowBox[{"16.", 
    RowBox[{"Kilogram", "/", "Hour"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"rate", " ", "of", " ", "fuel", " ", "consumption"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalDelta]M", "=", 
   RowBox[{"61", " ", "Kilogram"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"fuel", " ", "capacity"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"180.", " ", 
    RowBox[{"Kilometer", "/", "Hour"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"cruising", " ", "speed"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{{3.502958106618*^9, 3.5029581347*^9}, {
  3.5029582657869997`*^9, 3.50295826604*^9}, {3.502958324525*^9, 
  3.502958325833*^9}, {3.58840416892188*^9, 3.588404356074647*^9}, {
  3.588404389689928*^9, 3.5884043907140036`*^9}, {3.5884044405073366`*^9, 
  3.588404445795049*^9}}],

Cell["Thrust in level flight", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"T", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"M", " ", 
     SubscriptBox["g", "0"]}], "LD"], "//", 
   RowBox[{"ToUnit", "[", "Newton", "]"}]}]}]], "Input",
 CellChangeTimes->{3.588404465961621*^9}],

Cell[BoxData[
 TemplateBox[{"1335.6657300000002`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"N\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: newtons"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Newtons\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575626540576382*^9, 3.575626797777678*^9, 3.586606281060104*^9, 
   3.586657385597594*^9, 3.5866608283468924`*^9, 3.588404161680952*^9, {
   3.588404457583539*^9, 3.5884044726224213`*^9}, 3.5890791006684494`*^9}]
}, Open  ]],

Cell["Specific fuel consumption", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"F", " ", 
     SubscriptBox["g", "0"]}], "T"], "//", "ToSI"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"0.00003263175069342469`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"per \"", "\[InvisibleSpace]", "\"second\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: reciprocal seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["1", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575626540732402*^9, 3.575626797912695*^9, 3.5866062812200756`*^9, 
   3.5866573864616933`*^9, 3.5866608286139326`*^9, 3.5884041618309717`*^9, {
   3.5884044576865544`*^9, 3.5884044727584524`*^9}, 3.5890791008995366`*^9}]
}, Open  ]],

Cell["Endurance", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    FractionBox["1", "f"], "LD", " ", 
    RowBox[{"Log", "[", 
     FractionBox["M", 
      RowBox[{"M", "-", "\[CapitalDelta]M"}]], "]"}]}], "//", 
   RowBox[{"ToUnit", "[", "Hour", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5884044690499973`*^9, 3.5884044944352193`*^9}}],

Cell[BoxData[
 TemplateBox[{"3.9941853714478426`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"h\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: hours"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Hours\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756265408344145`*^9, 3.575626798044753*^9, 3.586606281820151*^9, 
   3.586657387273814*^9, 3.5866608288259497`*^9, 3.588404161918979*^9, 
   3.5884044078932304`*^9, {3.588404457748561*^9, 3.5884044951973324`*^9}, 
   3.5890791018436384`*^9}]
}, Open  ]],

Cell["Range", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", "=", 
  RowBox[{
   RowBox[{"U", " ", "t"}], "//", 
   RowBox[{"ToUnit", "[", "Kilometer", "]"}]}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"718.9533668606117`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5756265409134645`*^9, 3.57562679811272*^9, 3.5866062822682457`*^9, 
   3.5866573874378223`*^9, 3.58666082923497*^9, 3.588404161993977*^9, {
   3.5884044577945414`*^9, 3.5884044728974705`*^9}, 3.589079101955653*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 29.8 The distant laminar wake (p. 541)\
\>", "Title",
 CellChangeTimes->{{3.502958465366*^9, 3.502958472164*^9}, {
  3.5884045386098347`*^9, 3.5884045423202887`*^9}}],

Cell[CellGroupData[{

Cell["Flow inside the wake (p. 542)", "Section",
 CellChangeTimes->{{3.502958480358*^9, 3.502958483572*^9}, {
  3.58840455441785*^9, 3.588404565368242*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CleanGlobal", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(10\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.575626820688555*^9, 3.5866062834984193`*^9, 3.5866573876868124`*^9, 
   3.58665754753188*^9, {3.5866576757159758`*^9, 3.5866577246151657`*^9}, 
   3.5866608293859997`*^9, 3.589079102186641*^9}]
}, Open  ]],

Cell["Basic structure of inside solution", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "=", 
   RowBox[{
    FractionBox["1", "x"], 
    RowBox[{"Exp", "[", 
     RowBox[{
      RowBox[{"-", "U"}], " ", 
      FractionBox[
       RowBox[{" ", 
        RowBox[{
         SuperscriptBox["y", "2"], "+", "  ", 
         SuperscriptBox["z", "2"]}]}], 
       RowBox[{"4", " ", "\[Nu]", " ", "x"}]]}], "]"}]}]}], ";"}]], "Input"],

Cell["Check that it satisfies the field equation", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"0", "==", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"U", " ", 
      RowBox[{
       SubscriptBox["\[PartialD]", "x"], "f"}]}], "-", 
     RowBox[{"\[Nu]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["\[PartialD]", 
          RowBox[{"y", ",", "y"}]], "f"}], "+", 
        RowBox[{
         SubscriptBox["\[PartialD]", 
          RowBox[{"z", ",", "z"}]], "f"}]}], ")"}]}]}], ")"}]}], "//", 
  "Simplify"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.5756268207566214`*^9, 3.586606289330151*^9, 3.5866573883899374`*^9, 
   3.5866575476039143`*^9, {3.586657675748009*^9, 3.5866577246851716`*^9}, 
   3.586660829407006*^9, 3.5890791022576504`*^9}]
}, Open  ]],

Cell["The three components of the velocity field", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["u", "x"], "=", 
   RowBox[{
    SubscriptBox["A", "x"], "f"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["u", "y"], "=", 
   RowBox[{
    SubscriptBox["A", "y"], "f"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["u", "z"], "=", 
   RowBox[{
    SubscriptBox["A", "z"], "f"}]}], ";"}]}], "Input"],

Cell["Velocity potential", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalPsi]", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"2", "\[Nu]"}], "U"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{
          SubscriptBox["A", "y"], "y"}], "+", 
         RowBox[{
          SubscriptBox["A", "z"], "z"}]}], 
        RowBox[{
         SuperscriptBox["y", "2"], "+", 
         SuperscriptBox["z", "2"]}]]}], "+", " ", 
      FractionBox[
       SubscriptBox["A", "x"], 
       RowBox[{"2", "x"}]]}], ")"}], 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      RowBox[{"Exp", "[", 
       RowBox[{
        RowBox[{"-", "U"}], " ", 
        FractionBox[
         RowBox[{" ", 
          RowBox[{
           SuperscriptBox["y", "2"], "+", "  ", 
           SuperscriptBox["z", "2"]}]}], 
         RowBox[{"4", " ", "\[Nu]", " ", "x"}]]}], "]"}]}], ")"}]}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5029585926280003`*^9, 3.5029585937019997`*^9}, {
  3.586657682797916*^9, 3.586657704266618*^9}}],

Cell["Check that it satisfies the field equation", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"0", "==", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "x"], 
     SubscriptBox["u", "x"]}], "+", 
    RowBox[{
     SubscriptBox["\[PartialD]", "y"], 
     SubscriptBox["u", "y"]}], "+", 
    RowBox[{
     SubscriptBox["\[PartialD]", "z"], 
     SubscriptBox["u", "z"]}], "+", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      RowBox[{"y", ",", "y"}]], "\[CapitalPsi]"}], "+", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      RowBox[{"z", ",", "z"}]], "\[CapitalPsi]"}]}]}], "//", 
  "Simplify"}]], "Input",
 CellChangeTimes->{{3.502958605283*^9, 3.5029586059630003`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.5756268207985687`*^9, 3.586606291789403*^9, 3.586657390843264*^9, 
   3.586657548206006*^9, {3.586657676183034*^9, 3.5866577253052025`*^9}, 
   3.5866608294370165`*^9, 3.589079102364664*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Drag and lift", "Subsection",
 CellChangeTimes->{{3.502958642358*^9, 3.502958645587*^9}}],

Cell["Pressure", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]p", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Rho]", "0"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "U"}], 
       RowBox[{
        SubscriptBox["\[PartialD]", "x"], "\[CapitalPsi]"}]}], "+", 
      RowBox[{"\[Nu]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[PartialD]", 
           RowBox[{"y", ",", "y"}]], "\[CapitalPsi]"}], "+", 
         RowBox[{
          SubscriptBox["\[PartialD]", 
           RowBox[{"z", ",", "z"}]], "\[CapitalPsi]"}]}], ")"}]}]}], ")"}]}], 
   "//", "Simplify"}]}]], "Input",
 CellChangeTimes->{{3.5029586070480003`*^9, 3.50295860815*^9}, {
  3.502958664943*^9, 3.502958667006*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Nu]", " ", 
   SubscriptBox["A", "x"], " ", 
   SubscriptBox["\[Rho]", "0"]}], 
  SuperscriptBox["x", "2"]]], "Output",
 CellChangeTimes->{
  3.5756268208236113`*^9, 3.5866062925985107`*^9, 3.586657391648321*^9, 
   3.586657548854082*^9, {3.58665767686812*^9, 3.5866577259612846`*^9}, 
   3.586660829462994*^9, 3.5890791024117126`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Assumptions", "=", 
   RowBox[{
    FractionBox["U", 
     RowBox[{"x", " ", "\[Nu]"}]], ">", "0"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drag", "=", 
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["\[Rho]", "0"]}], "U", " ", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     RowBox[{"-", "\[Infinity]"}], "\[Infinity]"], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", 
        RowBox[{"-", "\[Infinity]"}], "\[Infinity]"], 
       RowBox[{
        SubscriptBox["u", "x"], 
        RowBox[{"\[DifferentialD]", "y"}]}]}], ")"}], 
     RowBox[{"\[DifferentialD]", "z"}]}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "4"}], " ", "\[Pi]", " ", "\[Nu]", " ", 
  SubscriptBox["A", "x"], " ", 
  SubscriptBox["\[Rho]", "0"]}]], "Output",
 CellChangeTimes->{
  3.5756268226197977`*^9, 3.5866062944617386`*^9, 3.586657397954156*^9, 
   3.5866575518514633`*^9, {3.586657678414357*^9, 3.5866577274825215`*^9}, 
   3.5866608317942734`*^9, 3.5890791041349435`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pressure and lift (p. 545)", "Section",
 CellChangeTimes->{{3.5884046000566463`*^9, 3.588404604225155*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(12\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{
  3.5756268230239077`*^9, 3.5866062952358356`*^9, 3.586657398990271*^9, 
   3.5866575520504503`*^9, {3.586657678629348*^9, 3.5866577276795354`*^9}, 
   3.586660834300646*^9, 3.5884046106019726`*^9, 3.589079104365958*^9}]
}, Open  ]],

Cell[TextData[{
 "Velocity potential (for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"D", "=", "0"}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"L", "=", "1"}], ",", " ", 
    RowBox[{"and", " ", Cell["U=1"]}]}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 ")"
}], "Text",
 CellChangeTimes->{{3.5884046988171787`*^9, 3.588404710792695*^9}, {
  3.5884047425387197`*^9, 3.5884047944592934`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalPsi]", "=", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"4", "\[Pi]", " ", "r"}]], 
    RowBox[{"Cos", "[", "\[Phi]", "]"}], " ", 
    RowBox[{"Cot", "[", 
     FractionBox["\[Theta]", "2"], "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{3.5029587125369997`*^9}],

Cell["Pressure", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]p", "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", "\[Theta]", "]"}], 
       RowBox[{
        SubscriptBox["\[PartialD]", "r"], "\[CapitalPsi]"}]}], "-", 
      RowBox[{
       RowBox[{"Sin", "[", "\[Theta]", "]"}], 
       FractionBox["1", "r"], 
       RowBox[{
        SubscriptBox["\[PartialD]", "\[Theta]"], "\[CapitalPsi]"}]}]}], 
     ")"}]}], "//", "Simplify"}]}]], "Input",
 CellChangeTimes->{{3.502958713414*^9, 3.502958738148*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    RowBox[{"Cos", "[", "\[Phi]", "]"}], " ", 
    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
   RowBox[{"4", " ", "\[Pi]", " ", 
    SuperscriptBox["r", "2"]}]]}]], "Output",
 CellChangeTimes->{
  3.5756268230909014`*^9, 3.586606295428935*^9, 3.586657399758355*^9, 
   3.5866575520864525`*^9, {3.5866576786643486`*^9, 3.586657727714554*^9}, 
   3.5866608345997095`*^9, 3.5884046106869907`*^9, 3.5890791044019833`*^9}]
}, Open  ]],

Cell["Contribution to lift", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Pi]"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", 
       RowBox[{"2", "\[Pi]"}]], 
      RowBox[{"\[CapitalDelta]p", " ", 
       SuperscriptBox["r", 
        RowBox[{"2", " "}]], 
       RowBox[{"Cos", "[", "\[Phi]", "]"}], 
       RowBox[{"\[DifferentialD]", "\[Phi]"}]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"Sin", "[", "\[Theta]", "]"}], "2"], 
    RowBox[{"\[DifferentialD]", "\[Theta]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.502958742521*^9, 3.502958745439*^9}}],

Cell[BoxData[
 FractionBox["1", "3"]], "Output",
 CellChangeTimes->{
  3.575626823739008*^9, 3.5866062961710105`*^9, 3.5866574006814985`*^9, 
   3.58665755263754*^9, {3.586657679216459*^9, 3.5866577282685747`*^9}, 
   3.5866608361448197`*^9, 3.5884046108359675`*^9, 3.5890791050219994`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{794, 740},
WindowMargins->{{0, Automatic}, {Automatic, 23}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 197, 5, 65, "Title"],
Cell[767, 29, 280, 7, 62, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1084, 41, 177, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[1286, 49, 150, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[1461, 57, 59, 1, 41, "Input"],
Cell[1523, 60, 388, 10, 21, "Message"]
}, Open  ]],
Cell[1926, 73, 26, 0, 30, "Text"],
Cell[1955, 75, 1071, 28, 102, "Input"],
Cell[3029, 105, 39, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[3093, 109, 98, 3, 56, "Input"],
Cell[3194, 114, 806, 19, 41, "Output"]
}, Open  ]],
Cell[4015, 136, 44, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[4084, 140, 97, 3, 56, "Input"],
Cell[4184, 145, 237, 4, 41, "Output"]
}, Open  ]],
Cell[4436, 152, 30, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[4491, 156, 144, 5, 61, "Input"],
Cell[4638, 163, 602, 14, 41, "Output"]
}, Open  ]],
Cell[5255, 180, 28, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[5308, 184, 98, 3, 58, "Input"],
Cell[5409, 189, 605, 14, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6075, 210, 178, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[6278, 218, 223, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[6526, 227, 127, 2, 41, "Input"],
Cell[6656, 231, 286, 8, 21, "Message"]
}, Open  ]],
Cell[6957, 242, 26, 0, 30, "Text"],
Cell[6986, 244, 786, 20, 82, "Input"],
Cell[CellGroupData[{
Cell[7797, 268, 148, 5, 58, "Input"],
Cell[7948, 275, 235, 3, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8220, 283, 117, 3, 41, "Input"],
Cell[8340, 288, 236, 3, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8613, 296, 59, 1, 58, "Input"],
Cell[8675, 299, 230, 3, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[8954, 308, 217, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[9196, 317, 59, 1, 41, "Input"],
Cell[9258, 320, 361, 9, 21, "Message"]
}, Open  ]],
Cell[9634, 332, 26, 0, 30, "Text"],
Cell[9663, 334, 846, 21, 82, "Input"],
Cell[10512, 357, 177, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[10714, 365, 124, 3, 41, "Input"],
Cell[10841, 370, 191, 3, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11069, 378, 147, 3, 41, "Input"],
Cell[11219, 383, 205, 3, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11461, 391, 101, 2, 41, "Input"],
Cell[11565, 395, 205, 3, 41, "Output"]
}, Open  ]],
Cell[11785, 401, 33, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[11843, 405, 247, 8, 61, "Input"],
Cell[12093, 415, 582, 13, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12712, 433, 44, 1, 41, "Input"],
Cell[12759, 436, 576, 13, 41, "Output"]
}, Open  ]],
Cell[13350, 452, 33, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[13408, 456, 168, 5, 56, "Input"],
Cell[13579, 463, 574, 13, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14214, 483, 180, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[14419, 491, 244, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[14688, 500, 59, 1, 41, "Input"],
Cell[14750, 503, 434, 10, 21, "Message"]
}, Open  ]],
Cell[15199, 516, 26, 0, 30, "Text"],
Cell[15228, 518, 2658, 79, 248, "Input"],
Cell[17889, 599, 36, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[17950, 603, 98, 3, 56, "Input"],
Cell[18051, 608, 644, 14, 41, "Output"]
}, Open  ]],
Cell[18710, 625, 28, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[18763, 629, 74, 2, 56, "Input"],
Cell[18840, 633, 277, 4, 41, "Output"]
}, Open  ]],
Cell[19132, 640, 25, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[19182, 644, 309, 9, 56, "Input"],
Cell[19494, 655, 665, 14, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20196, 674, 245, 5, 58, "Input"],
Cell[20444, 681, 712, 16, 41, "Output"]
}, Open  ]],
Cell[21171, 700, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[21297, 705, 129, 4, 56, "Input"],
Cell[21429, 711, 290, 4, 41, "Output"]
}, Open  ]],
Cell[21734, 718, 27, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[21786, 722, 123, 4, 58, "Input"],
Cell[21912, 728, 286, 4, 41, "Output"]
}, Open  ]],
Cell[22213, 735, 180, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[22418, 741, 223, 7, 58, "Input"],
Cell[22644, 750, 796, 18, 41, "Output"]
}, Open  ]],
Cell[23455, 771, 88, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[23568, 776, 361, 8, 41, "Input"],
Cell[23932, 786, 798, 16, 41, "Output"]
}, Open  ]],
Cell[24745, 805, 95, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[24865, 810, 164, 4, 58, "Input"],
Cell[25032, 816, 200, 3, 41, "Output"]
}, Open  ]],
Cell[25247, 822, 98, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[25370, 827, 341, 10, 65, "Input"],
Cell[25714, 839, 410, 6, 41, "Output"]
}, Open  ]],
Cell[26139, 848, 96, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[26260, 853, 233, 7, 64, "Input"],
Cell[26496, 862, 386, 6, 41, "Output"]
}, Open  ]],
Cell[26897, 871, 108, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[27030, 876, 250, 5, 56, "Input"],
Cell[27283, 883, 387, 6, 41, "Output"]
}, Open  ]],
Cell[27685, 892, 104, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[27814, 897, 341, 7, 41, "Input"],
Cell[28158, 906, 388, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[28583, 917, 134, 2, 41, "Input"],
Cell[28720, 921, 139, 2, 41, "Output"]
}, Open  ]],
Cell[28874, 926, 111, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[29010, 931, 279, 8, 41, "Input"],
Cell[29292, 941, 382, 6, 41, "Output"]
}, Open  ]],
Cell[29689, 950, 100, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[29814, 955, 324, 10, 79, "Input"],
Cell[30141, 967, 924, 20, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31102, 992, 209, 5, 41, "Input"],
Cell[31314, 999, 899, 20, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[32262, 1025, 179, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[32466, 1033, 59, 1, 41, "Input"],
Cell[32528, 1036, 463, 11, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[33028, 1052, 32, 0, 36, "Subsection"],
Cell[33063, 1054, 2959, 85, 281, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36059, 1144, 154, 2, 36, "Subsection"],
Cell[36216, 1148, 93, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[36334, 1153, 98, 3, 56, "Input"],
Cell[36435, 1158, 658, 15, 41, "Output"]
}, Open  ]],
Cell[37108, 1176, 93, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[37226, 1181, 74, 2, 56, "Input"],
Cell[37303, 1185, 313, 5, 41, "Output"]
}, Open  ]],
Cell[37631, 1193, 93, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[37749, 1198, 310, 9, 56, "Input"],
Cell[38062, 1209, 692, 15, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38791, 1229, 228, 5, 58, "Input"],
Cell[39022, 1236, 735, 16, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[39806, 1258, 105, 1, 36, "Subsection"],
Cell[39914, 1261, 45, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[39984, 1265, 310, 11, 59, "Input"],
Cell[40297, 1278, 685, 15, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41019, 1298, 204, 4, 58, "Input"],
Cell[41226, 1304, 737, 16, 41, "Output"]
}, Open  ]],
Cell[41978, 1323, 23, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[42026, 1327, 350, 12, 59, "Input"],
Cell[42379, 1341, 890, 20, 41, "Output"]
}, Open  ]],
Cell[43284, 1364, 27, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[43336, 1368, 189, 7, 58, "Input"],
Cell[43528, 1377, 661, 15, 41, "Output"]
}, Open  ]],
Cell[44204, 1395, 27, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[44256, 1399, 172, 6, 66, "Input"],
Cell[44431, 1407, 829, 19, 41, "Output"]
}, Open  ]],
Cell[45275, 1429, 27, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[45327, 1433, 123, 4, 58, "Input"],
Cell[45453, 1439, 317, 5, 41, "Output"]
}, Open  ]],
Cell[45785, 1447, 25, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[45835, 1451, 161, 6, 66, "Input"],
Cell[45999, 1459, 786, 18, 41, "Output"]
}, Open  ]],
Cell[46800, 1480, 35, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[46860, 1484, 112, 3, 56, "Input"],
Cell[46975, 1489, 902, 22, 41, "Output"]
}, Open  ]],
Cell[47892, 1514, 31, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[47948, 1518, 151, 5, 56, "Input"],
Cell[48102, 1525, 314, 5, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48453, 1535, 293, 8, 58, "Input"],
Cell[48749, 1545, 825, 19, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49611, 1569, 262, 7, 58, "Input"],
Cell[49876, 1578, 820, 19, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[50769, 1605, 226, 5, 65, "Title"],
Cell[CellGroupData[{
Cell[51020, 1614, 177, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[51222, 1622, 124, 2, 41, "Input"],
Cell[51349, 1626, 358, 9, 21, "Message"]
}, Open  ]],
Cell[51722, 1638, 94, 1, 30, "Text"],
Cell[51819, 1641, 600, 16, 82, "Input"],
Cell[52422, 1659, 213, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[52660, 1666, 289, 9, 41, "Input"],
Cell[52952, 1677, 209, 3, 41, "Output"]
}, Open  ]],
Cell[53176, 1683, 125, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[53326, 1690, 296, 9, 41, "Input"],
Cell[53625, 1701, 211, 3, 41, "Output"]
}, Open  ]],
Cell[53851, 1707, 181, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[54057, 1715, 337, 10, 41, "Input"],
Cell[54397, 1727, 582, 13, 41, "Output"]
}, Open  ]],
Cell[54994, 1743, 132, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[55151, 1750, 346, 10, 41, "Input"],
Cell[55500, 1762, 578, 13, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[56127, 1781, 206, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[56358, 1789, 124, 2, 41, "Input"],
Cell[56485, 1793, 332, 8, 21, "Message"]
}, Open  ]],
Cell[56832, 1804, 94, 1, 30, "Text"],
Cell[56929, 1807, 768, 21, 82, "Input"],
Cell[CellGroupData[{
Cell[57722, 1832, 197, 7, 55, "Input"],
Cell[57922, 1841, 191, 2, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[58162, 1849, 174, 4, 75, "Section"],
Cell[CellGroupData[{
Cell[58361, 1857, 124, 2, 41, "Input"],
Cell[58488, 1861, 520, 11, 21, "Message"]
}, Open  ]],
Cell[59023, 1875, 92, 1, 30, "Text"],
Cell[59118, 1878, 1158, 33, 164, "Input"],
Cell[60279, 1913, 140, 3, 41, "Input"],
Cell[CellGroupData[{
Cell[60444, 1920, 158, 3, 41, "Input"],
Cell[60605, 1925, 368, 5, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61010, 1935, 61, 1, 41, "Input"],
Cell[61074, 1938, 357, 5, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61468, 1948, 394, 12, 65, "Input"],
Cell[61865, 1962, 355, 5, 41, "Output"]
}, Open  ]],
Cell[62235, 1970, 37, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[62297, 1974, 282, 8, 57, "Input"],
Cell[62582, 1984, 376, 5, 41, "Output"]
}, Open  ]],
Cell[62973, 1992, 96, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[63094, 1997, 87, 2, 41, "Input"],
Cell[63184, 2001, 364, 5, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[63597, 2012, 268, 5, 75, "Section"],
Cell[CellGroupData[{
Cell[63890, 2021, 119, 2, 41, "Input"],
Cell[64012, 2025, 434, 10, 21, "Message"]
}, Open  ]],
Cell[64461, 2038, 94, 1, 30, "Text"],
Cell[64558, 2041, 1411, 39, 142, "Input"],
Cell[65972, 2082, 38, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[66035, 2086, 227, 7, 56, "Input"],
Cell[66265, 2095, 646, 14, 41, "Output"]
}, Open  ]],
Cell[66926, 2112, 41, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[66992, 2116, 148, 5, 56, "Input"],
Cell[67143, 2123, 762, 17, 41, "Output"]
}, Open  ]],
Cell[67920, 2143, 25, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[67970, 2147, 331, 9, 56, "Input"],
Cell[68304, 2158, 672, 15, 41, "Output"]
}, Open  ]],
Cell[68991, 2176, 21, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[69037, 2180, 139, 4, 41, "Input"],
Cell[69179, 2186, 654, 14, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[69894, 2207, 184, 4, 65, "Title"],
Cell[CellGroupData[{
Cell[70103, 2215, 156, 2, 75, "Section"],
Cell[CellGroupData[{
Cell[70284, 2221, 77, 2, 41, "Input"],
Cell[70364, 2225, 414, 10, 21, "Message"]
}, Open  ]],
Cell[70793, 2238, 50, 0, 30, "Text"],
Cell[70846, 2240, 376, 13, 60, "Input"],
Cell[71225, 2255, 58, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[71308, 2259, 486, 17, 41, "Input"],
Cell[71797, 2278, 250, 4, 41, "Output"]
}, Open  ]],
Cell[72062, 2285, 58, 0, 30, "Text"],
Cell[72123, 2287, 403, 15, 82, "Input"],
Cell[72529, 2304, 34, 0, 30, "Text"],
Cell[72566, 2306, 1018, 34, 62, "Input"],
Cell[73587, 2342, 58, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[73670, 2346, 631, 20, 41, "Input"],
Cell[74304, 2368, 246, 4, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74587, 2377, 95, 1, 36, "Subsection"],
Cell[74685, 2380, 24, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[74734, 2384, 724, 22, 41, "Input"],
Cell[75461, 2408, 376, 9, 57, "Output"]
}, Open  ]],
Cell[75852, 2420, 155, 5, 57, "Input"],
Cell[CellGroupData[{
Cell[76032, 2429, 517, 16, 56, "Input"],
Cell[76552, 2447, 376, 8, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[76989, 2462, 113, 1, 75, "Section"],
Cell[CellGroupData[{
Cell[77127, 2467, 59, 1, 41, "Input"],
Cell[77189, 2470, 437, 10, 21, "Message"]
}, Open  ]],
Cell[77641, 2483, 468, 16, 30, "Text"],
Cell[78112, 2501, 310, 9, 57, "Input"],
Cell[78425, 2512, 24, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[78474, 2516, 543, 16, 56, "Input"],
Cell[79020, 2534, 473, 11, 58, "Output"]
}, Open  ]],
Cell[79508, 2548, 36, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[79569, 2552, 613, 17, 58, "Input"],
Cell[80185, 2571, 290, 5, 88, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
