(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    122103,       2401]
NotebookOptionsPosition[    120034,       2326]
NotebookOutlinePosition[    120667,       2350]
CellTagsIndexPosition[    120580,       2345]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Package: NDSolveBV", "Title",
 CellChangeTimes->{{3.505115765799651*^9, 3.5051157724460306`*^9}, {
  3.505118408454802*^9, 3.5051184106459274`*^9}, {3.505118997680504*^9, 
  3.5051189985745544`*^9}, {3.588480786071603*^9, 3.58848079287144*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"BeginPackage", "[", "\"\<Lautrup`NDSolveBV`\>\"", "]"}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"Unprotect", "[", "NDSolveBV", "]"}], ";"}], "\[IndentingNewLine]",
   "\[IndentingNewLine]", 
  RowBox[{"(*", " ", "messages", " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"NDSolveBV", "::", "\"\<usage\>\""}], "=", 
   "\"\<Numerical differential equation boundary value \
solver.\\n\\nNDSolveBV[eqs,vars,intv,init,opts] \
where\\n\\n\\teqs\\tdifferential equations and boundary values: \
eqs=(odes,bva,bvb)\\n\\tvars\\tthe dependent variables \\n\\tintv\\tthe \
integration variable and its solution interval with an intermediary joining \
point\\n\\tinit\\tthe initial values for unknown \
parameters\\n\\topts\\tvarious options\\n\\t\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"NDSolveBV", "::", "\"\<lengtherror\>\""}], "=", 
   "\"\<The number of equations, boundary conditions left and rigth, \
variables, and initial conditions must be the same (`1`)\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"NDSolveBV", "::", "\"\<structureerror\>\""}], "=", 
   "\"\<The boundary values bva and bvb must refer to a and b, respectively\>\
\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"NDSolveBV", "::", "\"\<intervalerror\>\""}], "=", 
   "\"\<Interval must be of the form {x,a,c,b} with a < c < b\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"NDSolveBV", "::", "\"\<equationerror\>\""}], " ", "=", 
    "\"\<Equations must be of the form eqs={odes,bva,bvb}\>\""}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "query", " ", "for", " ", "list", " ", "of", " ", "numeric", " ", 
    "values"}], " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"NumericListQ", "[", "x_", "]"}], ":=", 
    RowBox[{"And", "@@", 
     RowBox[{"NumericQ", "/@", "x"}]}]}], ";"}], "\[IndentingNewLine]", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{"the", " ", "function", " ", "itself"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"NDSolveBV", "[", 
     RowBox[{
     "eqs_", ",", "vars_", ",", "intv_", ",", "init_", ",", "opts___"}], 
     "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "x", ",", "a", ",", "c", ",", "b", ",", "n", ",", "ivars", ",", "aBV", 
        ",", "bBV", ",", "odes", ",", "cvals", ",", "solac", ",", "solbc", 
        ",", "sol", ",", "roots", ",", "args", ",", "varsac", ",", "varsbc", 
        ",", "fun", ",", "sub", ",", "int", ",", "optsNDSolveBV", ",", 
        "optsNDSolve", ",", "optsFindRoot", ",", 
        "optsFunctionInterpolation"}], "}"}], ",", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"check", " ", "interval", " ", "structure"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "intv", "]"}], "\[NotEqual]", "4"}], ",", 
         RowBox[{
          RowBox[{"Message", "[", 
           RowBox[{"NDSolveBV", "::", "\"\<intervalerror\>\""}], "]"}], ";", 
          RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"x", "=", 
        RowBox[{
        "intv", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], ";", 
       RowBox[{"a", "=", 
        RowBox[{
        "intv", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], ";", 
       RowBox[{"c", "=", 
        RowBox[{
        "intv", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], ";", 
       RowBox[{"b", "=", 
        RowBox[{
        "intv", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}]}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"(", 
           RowBox[{"a", "\[LessEqual]", "c", "\[LessEqual]", "b"}], ")"}]}], 
         ",", 
         RowBox[{
          RowBox[{"Message", "[", 
           RowBox[{"NDSolveBV", "::", "\"\<intervalerror\>\""}], "]"}], ";", 
          RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]",
        "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "check", " ", "the", " ", "number", " ", "of", " ", "equations", " ", 
         "and", " ", "their", " ", "structure"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "eqs", "]"}], "\[NotEqual]", "3"}], ",", 
         RowBox[{
          RowBox[{"Message", "[", 
           RowBox[{
            RowBox[{"NDSolveBV", "::", "\"\<equationerror\>\""}], ",", "n"}], 
           "]"}], ";", 
          RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"odes", "=", 
        RowBox[{"eqs", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"bva", "=", 
        RowBox[{"eqs", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"bvb", "=", 
        RowBox[{"eqs", "[", 
         RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{
         "check", " ", "the", " ", "numbers", " ", "of", " ", "equations"}], 
         ",", " ", 
         RowBox[{
         "boundary", " ", "values", " ", "left", " ", "and", " ", "right"}], 
         ",", " ", 
         RowBox[{"and", " ", "initial", " ", "values"}]}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"n", "=", 
        RowBox[{"Length", "[", "vars", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Length", "[", "odes", "]"}], "\[Equal]", 
            RowBox[{"Length", "[", "bva", "]"}], "\[Equal]", 
            RowBox[{"Length", "[", "bvb", "]"}], "\[Equal]", 
            RowBox[{"Length", "[", "init", "]"}], "==", "n"}], ")"}]}], ",", 
         RowBox[{
          RowBox[{"Message", "[", 
           RowBox[{
            RowBox[{"NDSolveBV", "::", "\"\<lengtherror\>\""}], ",", "n"}], 
           "]"}], ";", 
          RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Length", "[", 
             RowBox[{"Cases", "[", 
              RowBox[{"bva", ",", 
               RowBox[{
                RowBox[{"_", "[", "a", "]"}], "\[Equal]", "_"}]}], "]"}], 
             "]"}], "\[Equal]", 
            RowBox[{"Length", "[", 
             RowBox[{"Cases", "[", 
              RowBox[{"bvb", ",", 
               RowBox[{
                RowBox[{"_", "[", "b", "]"}], "\[Equal]", "_"}]}], "]"}], 
             "]"}], "\[Equal]", "n"}], ")"}]}], ",", 
         RowBox[{
          RowBox[{"Message", "[", 
           RowBox[{"NDSolveBV", "::", "\"\<structureerror\>\""}], "]"}], ";", 
          RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]",
        "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"get", " ", "options"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"optsNDSolve", "=", 
        RowBox[{"Sequence", "@@", 
         RowBox[{"FilterRules", "[", 
          RowBox[{
           RowBox[{"Flatten", "[", 
            RowBox[{"{", "opts", "}"}], "]"}], ",", 
           RowBox[{"Options", "[", "NDSolve", "]"}]}], "]"}]}]}], ";", 
       RowBox[{"optsFindRoot", "=", 
        RowBox[{"Sequence", "@@", 
         RowBox[{"FilterRules", "[", 
          RowBox[{
           RowBox[{"Flatten", "[", 
            RowBox[{"{", "opts", "}"}], "]"}], ",", 
           RowBox[{"Options", "[", "FindRoot", "]"}]}], "]"}]}]}], ";", 
       RowBox[{"optsNDSolveBV", "=", 
        RowBox[{"Sequence", "@@", 
         RowBox[{"FilterRules", "[", 
          RowBox[{
           RowBox[{"Flatten", "[", 
            RowBox[{"{", 
             RowBox[{"opts", ",", 
              RowBox[{"Options", "[", "NDSolveBV", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Options", "[", "NDSolveBV", "]"}]}], "]"}]}]}], ";", 
       RowBox[{"optsFunctionInterpolation", "=", 
        RowBox[{"Sequence", "@@", 
         RowBox[{"FilterRules", "[", 
          RowBox[{
           RowBox[{"Flatten", "[", 
            RowBox[{"{", "opts", "}"}], "]"}], ",", 
           RowBox[{"Options", "[", "FunctionInterpolation", "]"}]}], 
          "]"}]}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"the", " ", "unknown", " ", "boundary", " ", "values"}], " ", 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{"ivars", "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"First", "[", "#1", "]"}], "&"}], ")"}], "/@", "init"}]}], 
       ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "integrate", " ", "from", " ", "a", " ", "to", " ", "c", " ", "and", 
         " ", "from", " ", "c", " ", "to", " ", "b"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"solac", "[", 
         RowBox[{"args_", "?", "NumericListQ"}], "]"}], ":=", 
        RowBox[{"First", "[", 
         RowBox[{"NDSolve", "@@", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"odes", "\[Union]", "bva"}], ",", "vars", ",", 
              RowBox[{"{", 
               RowBox[{"x", ",", "a", ",", "c"}], "}"}], ",", "optsNDSolve"}],
              "}"}], "/.", "\[InvisibleSpace]", 
            RowBox[{"MapThread", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"#1", "\[Rule]", "#2"}], "&"}], ",", 
              RowBox[{"{", 
               RowBox[{"ivars", ",", "args"}], "}"}]}], "]"}]}], ")"}]}], 
         "]"}]}], ";", 
       RowBox[{
        RowBox[{"solbc", "[", 
         RowBox[{"args_", "?", "NumericListQ"}], "]"}], ":=", 
        RowBox[{"First", "[", 
         RowBox[{"NDSolve", "@@", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"odes", "\[Union]", "bvb"}], ",", "vars", ",", 
              RowBox[{"{", 
               RowBox[{"x", ",", "c", ",", "b"}], "}"}], ",", "optsNDSolve"}],
              "}"}], "/.", "\[InvisibleSpace]", 
            RowBox[{"MapThread", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"#1", "\[Rule]", "#2"}], "&"}], ",", 
              RowBox[{"{", 
               RowBox[{"ivars", ",", "args"}], "}"}]}], "]"}]}], ")"}]}], 
         "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"construct", " ", "solution"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"cvals", "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"#1", "[", "c", "]"}], "&"}], ")"}], "/@", "vars"}]}], ";",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"sol", "[", 
         RowBox[{"args_", "?", "NumericListQ"}], "]"}], ":=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"cvals", "/.", "\[InvisibleSpace]", 
           RowBox[{"solac", "[", "args", "]"}]}], ")"}], "-", 
         RowBox[{"(", 
          RowBox[{"cvals", "/.", "\[InvisibleSpace]", 
           RowBox[{"solbc", "[", "args", "]"}]}], ")"}]}]}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "determine", " ", "the", " ", "roots", " ", "for", " ", "the", " ", 
         "unknown", " ", "boundary", " ", "values"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"roots", "=", 
        RowBox[{"FindRoot", "@@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"sol", "[", "ivars", "]"}], ",", "init", ",", 
           "optsFindRoot"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"args", "=", 
        RowBox[{"ivars", "/.", "\[InvisibleSpace]", "roots"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"varsac", "=", 
        RowBox[{"solac", "[", "args", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"varsbc", "=", 
        RowBox[{"solbc", "[", "args", "]"}]}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{"output", " ", "the", " ", "two"}], "-", 
         RowBox[{"part", " ", "solution"}]}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"fun", "[", "f_", "]"}], ":=", 
        RowBox[{
         RowBox[{"Evaluate", "[", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"#1", "<", "c"}], ",", 
            RowBox[{"Evaluate", "[", 
             RowBox[{
              RowBox[{"f", "[", "#1", "]"}], "/.", "\[InvisibleSpace]", 
              "varsac"}], "]"}], ",", 
            RowBox[{"Evaluate", "[", 
             RowBox[{
              RowBox[{"f", "[", "#1", "]"}], "/.", "\[InvisibleSpace]", 
              "varsbc"}], "]"}]}], "]"}], "]"}], "&"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"sub", "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"#1", "\[Rule]", 
            RowBox[{"fun", "[", "#1", "]"}]}], "&"}], ")"}], "/@", "vars"}]}],
        ";", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"sub", ",", "roots"}], "}"}]}]}], "]"}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"End", "[", "]"}], ";"}], "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"NDSolveBV", ",", 
    RowBox[{"{", 
     RowBox[{"Protected", ",", "ReadProtected"}], "}"}]}], "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"EndPackage", "[", "]"}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.39009901766658*^9, {3.4163793959890003`*^9, 3.416379526049*^9}, {
   3.41637958058*^9, 3.416379609008*^9}, {3.5037312546883755`*^9, 
   3.503731312944707*^9}, {3.5037313526349773`*^9, 3.503731372050088*^9}, {
   3.503731403367879*^9, 3.503731449085494*^9}, {3.5037314936480427`*^9, 
   3.503731510926031*^9}, {3.503731608600618*^9, 3.503731610094703*^9}, {
   3.5037316475038433`*^9, 3.50373169256242*^9}, {3.5037317357388897`*^9, 
   3.5037317720669675`*^9}, {3.5037318219878225`*^9, 3.503731828076171*^9}, {
   3.5037319086507797`*^9, 3.503731926747815*^9}, {3.5037319656110373`*^9, 
   3.503731966071064*^9}, {3.5037319971288404`*^9, 3.5037320323188534`*^9}, {
   3.503732184298546*^9, 3.503732195799204*^9}, {3.5037322480261908`*^9, 
   3.5037322547665763`*^9}, {3.5037323079926205`*^9, 3.503732351878131*^9}, {
   3.5037323853900476`*^9, 3.503732429281558*^9}, {3.5037325925978994`*^9, 
   3.5037327564622717`*^9}, {3.5037329333703904`*^9, 3.50373301430402*^9}, {
   3.503733308016819*^9, 3.503733415141946*^9}, {3.5037334721102047`*^9, 
   3.503733535639838*^9}, {3.5037336278781137`*^9, 3.5037336327323914`*^9}, {
   3.5037336668453426`*^9, 3.503733704505497*^9}, {3.503733743980755*^9, 
   3.503733747096933*^9}, {3.503733906271037*^9, 3.5037339629782805`*^9}, {
   3.5037340742696457`*^9, 3.503734101699215*^9}, {3.5037342085203247`*^9, 
   3.503734429569968*^9}, {3.503734508055457*^9, 3.503734527367562*^9}, {
   3.503735772631787*^9, 3.5037358093048844`*^9}, {3.5037359041173077`*^9, 
   3.503735985691973*^9}, {3.5037360197069187`*^9, 3.5037360592221785`*^9}, {
   3.5037361079339647`*^9, 3.503736114828359*^9}, 3.5037361930308323`*^9, {
   3.503736227522805*^9, 3.5037362522192173`*^9}, {3.5037363484507217`*^9, 
   3.503736475921013*^9}, {3.503736525855869*^9, 3.503736533795323*^9}, {
   3.5037367308675947`*^9, 3.503736741082179*^9}, 3.503736779769392*^9, {
   3.503736848853343*^9, 3.5037368570078096`*^9}, {3.5037369988319216`*^9, 
   3.503737011538648*^9}, {3.5037370885160513`*^9, 3.5037370900551395`*^9}, {
   3.503737135596744*^9, 3.5037371735129128`*^9}, {3.503737219110521*^9, 
   3.503737223185754*^9}, 3.5037372689333706`*^9, 3.5037373199862905`*^9, {
   3.5037374909620695`*^9, 3.5037374932412*^9}, {3.5037375323914394`*^9, 
   3.5037375364066687`*^9}, {3.5037405436896753`*^9, 
   3.5037405770735855`*^9}, {3.505117661044052*^9, 3.505117673170746*^9}, {
   3.505118431799137*^9, 3.505118457359599*^9}, {3.5051185105096393`*^9, 
   3.505118537148163*^9}, {3.5051186271103086`*^9, 3.50511864724146*^9}, {
   3.5051186854566455`*^9, 3.505118706361841*^9}, {3.5051187452020626`*^9, 
   3.5051188190662875`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Solve differential equations with boundary values\
\>", "Section",
 CellChangeTimes->{{3.5051157783033657`*^9, 3.505115781829568*^9}, {
  3.505118395255047*^9, 3.5051183978711967`*^9}, {3.588480816855461*^9, 
  3.5884808621432147`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "NDSolveBV"}]], "Input",
 CellChangeTimes->{{3.5051157878879137`*^9, 3.505115794128271*^9}}],

Cell[BoxData[
 StyleBox["\<\"Numerical differential equation boundary value \
solver.\\n\\nNDSolveBV[eqs,vars,intv,init,opts] where\\n\\n\teqs\t\
differential equations and boundary values: eqs=(odes,bva,bvb)\\n\tvars\tthe \
dependent variables \\n\tintv\tthe integration variable and its solution \
interval with an intermediary joining point\\n\tinit\tthe initial values for \
unknown parameters\\n\topts\tvarious options\\n\t\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.5884809595946383`*^9},
 CellTags->"Info3588488159-1683493"]
}, Open  ]],

Cell["\<\
The n ordinary differential equations (odes) must be of first order. Higher \
order ordinary differential equations must first be reduced to first order.  \
There must be precisely 2n boundary values, n on the left (bva) of the \
integration interval and n on the right (bvb). Only n boundary values need to \
be known at the outset. The n unknowns  will be determined by the solutions.  \
There must be precisely n dependent variables (vars). The integration \
interval (intv) must be  of the form {x,a,c,b} , including a middle point \
where the two partial solutions from each end are joined. For each unknown \
boundary value, there must be an initial value to be used by FindRoot. for \
joining the solutions. \
\>", "Text",
 CellChangeTimes->{{3.505117334854396*^9, 3.5051176074779882`*^9}, {
  3.5051176850634265`*^9, 3.5051177642789574`*^9}, {3.5051178147118416`*^9, 
  3.5051181064345274`*^9}, {3.505118137272291*^9, 3.5051183119432817`*^9}}],

Cell[CellGroupData[{

Cell["Options", "Subsection",
 CellChangeTimes->{{3.5051168261993017`*^9, 3.505116827190359*^9}, {
  3.50511741991026*^9, 3.505117421022324*^9}}],

Cell["\<\
There are (at this time) no native options to NDSolveBV. Various options may \
be given for NDSolve, FindRoot, and FunctionInterpolation\
\>", "Text",
 CellChangeTimes->{{3.5051169588878913`*^9, 3.5051169700455294`*^9}, {
  3.505117013799032*^9, 3.505117044022761*^9}, {3.5051170824629593`*^9, 
  3.505117100582996*^9}, {3.5051171417033477`*^9, 3.5051171860308833`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "NDSolve", "]"}]], "Input",
 CellChangeTimes->{{3.505117105705289*^9, 3.505117110511564*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"AccuracyGoal", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"Compiled", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"DependentVariables", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"DiscreteVariables", "\[Rule]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"EvaluationMonitor", "\[Rule]", "None"}], ",", 
   RowBox[{"InterpolationOrder", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"MaxStepFraction", "\[Rule]", 
    FractionBox["1", "10"]}], ",", 
   RowBox[{"MaxSteps", "\[Rule]", "10000"}], ",", 
   RowBox[{"MaxStepSize", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"Method", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"NormFunction", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"PrecisionGoal", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"StartingStepSize", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"StepMonitor", "\[Rule]", "None"}], ",", 
   RowBox[{"WorkingPrecision", "\[Rule]", "MachinePrecision"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.505118914372739*^9, 3.5884809596446557`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FindRoot", "]"}]], "Input",
 CellChangeTimes->{{3.5051171133557262`*^9, 3.505117117671973*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"AccuracyGoal", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"Compiled", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"DampingFactor", "\[Rule]", "1"}], ",", 
   RowBox[{"Evaluated", "\[Rule]", "True"}], ",", 
   RowBox[{"EvaluationMonitor", "\[Rule]", "None"}], ",", 
   RowBox[{"Jacobian", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"MaxIterations", "\[Rule]", "100"}], ",", 
   RowBox[{"Method", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"PrecisionGoal", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"StepMonitor", "\[Rule]", "None"}], ",", 
   RowBox[{"WorkingPrecision", "\[Rule]", "MachinePrecision"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5051189143867397`*^9, 3.5884809596606407`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FunctionInterpolation", "]"}]], "Input",
 CellChangeTimes->{{3.505117120691146*^9, 3.505117128951618*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"InterpolationOrder", "\[Rule]", "3"}], ",", 
   RowBox[{"InterpolationPrecision", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"AccuracyGoal", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"PrecisionGoal", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"InterpolationPoints", "\[Rule]", "11"}], ",", 
   RowBox[{"MaxRecursion", "\[Rule]", "6"}]}], "}"}]], "Output",
 CellChangeTimes->{3.50511891440174*^9, 3.5884809596726475`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simple example", "Subsection",
 CellChangeTimes->{{3.505117203006854*^9, 3.5051172046619487`*^9}, {
  3.5051172761190357`*^9, 3.5051172784861717`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sol", "=", 
  RowBox[{"NDSolveBV", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"f", "'"}], "[", "x", "]"}], "\[Equal]", 
         RowBox[{"g", "[", "x", "]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"g", "'"}], "[", "x", "]"}], "\[Equal]", 
         SuperscriptBox["x", "2"]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"f", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
        RowBox[{
         RowBox[{"g", "[", "0", "]"}], "==", "u"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"f", "[", "1", "]"}], "\[Equal]", "1"}], ",", 
        RowBox[{
         RowBox[{"g", "[", "1", "]"}], "==", "v"}]}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"f", ",", "g"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "0.5", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"u", ",", "0.2", ",", "0.3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"v", ",", "1", ",", "1.1"}], "}"}]}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5037311097970877`*^9, 3.5037311554977016`*^9}, {
   3.503732167426581*^9, 3.5037321713698063`*^9}, {3.503732479090407*^9, 
   3.50373247948343*^9}, {3.5037328041860013`*^9, 3.503732852873786*^9}, 
   3.5037331928822336`*^9, {3.503733227298202*^9, 3.5037332380388165`*^9}, {
   3.503733448826873*^9, 3.503733449106889*^9}, {3.503733571745903*^9, 
   3.5037335719209137`*^9}, {3.503734469395246*^9, 3.503734477841729*^9}, {
   3.5037373647148485`*^9, 3.5037373884822083`*^9}, {3.50373741876294*^9, 
   3.5037374232741985`*^9}, 3.5037375561547985`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"f", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"#1", "<", "0.5`"}], ",", 
          RowBox[{
           TagBox[
            RowBox[{"InterpolatingFunction", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"0.`", ",", "0.5`"}], "}"}], "}"}], 
              ",", "\<\"<>\"\>"}], "]"}],
            False,
            Editable->False], "[", "#1", "]"}], ",", 
          RowBox[{
           TagBox[
            RowBox[{"InterpolatingFunction", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"0.5`", ",", "1.`"}], "}"}], "}"}], 
              ",", "\<\"<>\"\>"}], "]"}],
            False,
            Editable->False], "[", "#1", "]"}]}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"g", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"#1", "<", "0.5`"}], ",", 
          RowBox[{
           TagBox[
            RowBox[{"InterpolatingFunction", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"0.`", ",", "0.5`"}], "}"}], "}"}], 
              ",", "\<\"<>\"\>"}], "]"}],
            False,
            Editable->False], "[", "#1", "]"}], ",", 
          RowBox[{
           TagBox[
            RowBox[{"InterpolatingFunction", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"0.5`", ",", "1.`"}], "}"}], "}"}], 
              ",", "\<\"<>\"\>"}], "]"}],
            False,
            Editable->False], "[", "#1", "]"}]}], "]"}], "&"}], ")"}]}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Rule]", "0.9166666744659522`"}], ",", 
     RowBox[{"v", "\[Rule]", "1.2499999841748708`"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5051189144317417`*^9, 3.5884809597026644`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"f", "[", "x", "]"}], ",", 
      RowBox[{"g", "[", "x", "]"}]}], "}"}], "/.", "\[InvisibleSpace]", 
    RowBox[{"sol", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.503737432669736*^9, 3.5037374469495525`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVjnk4FHgAhpm0OgltTIWdcpQKXSravkqrA20mrWyMmFKrIUmHVgdLLFlW
FuVIOVbJUYYSIXS4hyGEwRjGjDE/1m2L9Mf3vM/3PO8fL83pPP00RUZGxnJ2
33jodF9dsejYLpniODWzZda75EyXu3dQt8AuzKJhkesmbH4QHZtN3QdhkF6q
gLUX5tVNz+Oo1uAMu4VzWHTYnBp/HU49hY1q1YHvWE5oNwgwDaF6IixwMOAN
ywMJ3/98O4jqh5fxpRpFLF/4zMvW/ZMagYx86ZNCVjiO2z7MHBxKgsvN4Xit
zYlYOONjt18jE4aGb26E334KbStpX2kwG1YBGoXpjc9xpWYHokfyoFfI/uBZ
+QKqbCWerGwR0hkZXaWsAuTyTatqd5bghuvVKN7LYpS4xK2wlZQhrnzyIjEo
xajOj4rpru9Re3+nCfF+C8pk2pIV2hUQDetGUj+/h9HMydyjGVVYc37E3Naq
AlV+W2re6dVCO/6TzrWYKmiG9U8+lnLwuzBmjqlaLfrHO3iNhvUot3mqk57D
QZI4v0LhLy6U90iNyyfqoKD+LxEIGrCtSS75sREXd37LcU7Q+oi06Ovbaxwa
MLY7TXD2ZhNoPx05SB41wnNTGXtZaTMurn/1i2vdR3RklAXX0T4h/dcVqQeW
N+N/l4aYtddaQWHPL2IwWrBuj/y0c34bjjj6JItCPmH/I9qZZ6o8cPamrvZs
bsUVdYdVXXYd0DjHowQrtGO95NjCJstOmOQ+U1q0mwcVxcF5ZVOduGR7It0i
pANyRTGZq2O74IzQKSVmJ85tyW+Zb87HLS/76qzFXUg76JRQOMbHxp7QKcfc
Loz2L1/SG9mN+qHSZe3WfOQc3T8nYp8AqnezHihQunHzgP8Ddu/s39gnVUzu
Bq39UHOifw+c9f2CDMwEMN7mF0Hd1ItKGZ7w3pgAdhq1k7LcXuRF5Y29je3B
aEUI391HCHFw5dV8ei+i5RiedK0+lOgrlnpM90KFl/OIUtaHkSVRxyJThZh7
3cd6xwURGH9vdwezDzKPGZl8ZTGymBFtwSoieNzPDFEuEUPNxt34S4MIr9cF
zlNy6wdXs4vh+Y8YSoc81vsqSOBRz2x1NO9H2wYO16tAgjyDVlNHFQmy4qEl
YAwgRU4j4QNHAvtMIyX16QH84KbzUC1kALf8H/q0PZHC1LiO4m0nRXqfeoyC
BYGdpKNcjkYQGOd/kneYQPIHW9dLi+AUfUA7w4ogxUy5ZECXYGVBQZalDcFg
Q8TTJn2CoNAT7+44ETA/9Npm7CQ4a3RvaIEXgUi8T/PMcYJVfksPyKcQuFFT
E74LI5je7r24KZVgSnMiyfcuQctAd31KGsHqNlvhdCRBmE22vdkzgsOX6DoT
cQQzevSL/gUE8utm62e9trrQ+DlcAqPLmXMF7wle3h5nchsJXFqnB5iVBBEm
DmsTmwmKrRqXCmoIzJP12Xt5BJwLqho9s96rq9XlPiIC+qtdNHE3QeSGraFW
EoIh98sz54QEF/ix1jRCUERJWykVE6yxYHUUjxBIXyxYOjxEICfbkBQ2TnCk
dav00ihBZ46Jy8kpAn6LvfzkBEGBS6KB4ReC5GxfB+/PBFGaC0dnZggChJb/
feNXAv6CIw==
     "]], LineBox[CompressedData["
1:eJwV1nc81d8fB3CSHymkaBgho8hqCunNVVGpvrZrZY+sCBkVt1wrN4rEtfde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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0, 1}, {0., 1.2499999637667092`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.5051189144547434`*^9, 3.5884809597266474`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Harder example", "Subsection",
 CellChangeTimes->{{3.5051172434881697`*^9, 3.5051172454862843`*^9}, {
  3.5051172851815543`*^9, 3.5051172852885604`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   SuperscriptBox["10", 
    RowBox[{"-", "8"}]]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"b", "=", "10"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"c", "=", "0.3"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Beta]", "=", "0.188"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ss", "=", "0.1"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"mm", "=", "0.9"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"kk", "=", 
   SuperscriptBox["10", 
    RowBox[{"-", "12"}]]}], ";"}]}], "Input",
 CellChangeTimes->{3.5037387366623197`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", ":=", 
   RowBox[{"NDSolveBV", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"u", "'"}], "[", "x", "]"}], "\[Equal]", 
          RowBox[{"v", "[", "x", "]"}]}], ",", 
         RowBox[{
          RowBox[{
           SuperscriptBox["v", "\[Prime]",
            MultilineFunction->None], "[", "x", "]"}], "\[Equal]", 
          RowBox[{
           RowBox[{"-", "\[Alpha]"}], 
           FractionBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", 
               SuperscriptBox["\[ExponentialE]", 
                RowBox[{"-", "x"}]]}], ")"}], " ", 
             RowBox[{
              SuperscriptBox["u", "\[Prime]",
               MultilineFunction->None], "[", "x", "]"}]}], 
            RowBox[{"x", " ", 
             RowBox[{"g", "[", "x", "]"}]}]]}]}], ",", 
         RowBox[{
          RowBox[{
           SuperscriptBox["g", "\[Prime]",
            MultilineFunction->None], "[", "x", "]"}], "\[Equal]", 
          RowBox[{
           SuperscriptBox["\[Beta]", "2"], " ", 
           FractionBox[
            SuperscriptBox[
             RowBox[{"u", "[", "x", "]"}], "2"], 
            SuperscriptBox["x", "2"]]}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"u", "[", "a", "]"}], "\[Equal]", 
          RowBox[{"a", " ", "s"}]}], ",", 
         RowBox[{
          RowBox[{"v", "[", "a", "]"}], "\[Equal]", "s"}], ",", 
         RowBox[{
          RowBox[{"g", "[", "a", "]"}], "\[Equal]", "m"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"u", "[", "b", "]"}], "\[Equal]", "1"}], ",", 
         RowBox[{
          RowBox[{"v", "[", "b", "]"}], "\[Equal]", "k"}], ",", 
         RowBox[{
          RowBox[{"g", "[", "b", "]"}], "\[Equal]", "1"}]}], "}"}]}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", "v", ",", "g"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "a", ",", "c", ",", "b"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"s", ",", "ss", ",", 
         RowBox[{"1.01", "ss"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"m", ",", "mm", ",", 
         RowBox[{"1.01", "mm"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "kk", ",", 
         RowBox[{"1.01", "kk"}]}], "}"}]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5037376612118073`*^9, 3.503737776435398*^9}, 
   3.503737808220216*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"u", "[", 
         SuperscriptBox["r", "2"], "]"}], "r"], "/.", "\[InvisibleSpace]", 
       RowBox[{
       "sol", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "2", ",", "10"}], "}"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", 
     SqrtBox["a"], ",", 
     SqrtBox["b"]}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1nk0VXsbB3BjHBV1GyRFSTQcwo2EfHdKt5lESYNyEbmk9KorQ0oTEpKh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     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwV13lUDf0fB/BSqRuVHmmRFiSlhIS0eM/TZtciiaiQQkm7HmkhFcqWNZIK
iScqpNBDiFK06LbIbbnL3KW6U7TQ/pvfH3PmvM6Z8/nO+c5n+c7cvUdc9k+R
kJAYpq//32s+FtxaZTkbPbmU9za7ZShOUXoSmeSM8u2stPKZfjgz1fRnRpE3
Nnkm79w/GY4Q2w9/Pd0UhL9llw9PhCWAz06+a6keh3/S4+5dWnIVf+/tbJYY
u4BoE7+RpGn3EVVol/mRfQchF1Wvv+5/DvXvEdK5zHx0PzlTwKgvB8tYqU83
tQy/Yuvk561shEX2I7fh9Ep8HnmXKXOsHdWisYPFqMPz7IBdBQQXN1Wzor85
1eF03J25NU5cHLK1vyTeU4eV2Wc2dXpzIX/7XMn8U3WYGZr1hh3HxTonbdmL
lXX4+fJ+onsZF+UvcO+Acz3y10QnKVvzUBp/sl1j3zcUzJE+dWUxicdacluP
JzIRHtXcZNDFx7GMW4ah15iIORFutG+Ij3U6SyQO5TDR6zO9NHWKABxdt8fb
PzFx9MDQ+frZAqjq3ZNdPrURVctHRF83CBCziCgVJTTCvXx3fVmuAE6rIhds
T2hCWY7/p3O7hNAqmTa2+WoT7BKV2gL9hOgyv/PN7n4TkpXJRfYhQpyy+BRr
+rEJh/6TC6xPEuKltUqrgkwz1MqC7h54KsQ8u4Lz5aeaEXbi6KVUGRH6nYS/
l51qQfeKP3msuyIYey/4HXO1BeNTXsWbPhHB98jeoeqcFrSfnsmKKhGh9Rxr
YP/nFiD1gaboiwjvq+p+pil+B/ta5jyTQRFS7V52S6Z9x/ZCzc8+tl1Yan62
o/5xK5JrqoqvN3Xh0NqKdu23raiMi7Da19GFe27S7f51rTgr8+7+PGEX1MNj
WDL9rfBq/mt59HAXJgtDvq82/wG+ityNLM1u1Bh5NGS9/4HBpeqNj3Z347CO
cUVwMwvlmf5n21q6oX3tmkSgkIVj/IXu0zq7UTdd0uLQMAv5WcVjiwXdMBtp
zNur2YYtI4vanAe7McaMTd3q1Ya1sQtOqSj3IOVsw+4VgjY09Lp8P+PQg0eD
xwb+DLXDrsD5u9vDHuwKIBcPynagpEFpnmF+DxS5jn4/1TvwbvgGr+95D0Lq
9FpFFh148Wlzt3dZD1b/W/O2NYb20fhLH5p64LP0kr2nVCcCOMdVb0iK0U/y
ZhDJnXjY9OZFjqMYG6qVlYwvd+JS2xdNcqsY2QVrFNRvdYJa2vx7trsYLlE3
GH2POuFmVGF42FuMghmbpmRWdaL9Y4FEXpAYgZbP+icZbOQV2t2/fkGMT7qd
P7uV2WDdVqtyvCyG9lSFvmYNNmpHU4smr4nxtc63J9+QjVmWnoMOGWIY+84m
vdaz8TCMnxmaJ0bXxRNNb0+z8S1KT2FDhRg2EU+Y/15kI6fpyGhclRg3PX58
u36DjR9X10kVfBVjvf6K2iO5bMQ6xIQMNYiR+1pYoVPJxj7Wr20LOsXw5Tu+
jJXl4Nlms6tKv8V4U3282F+Jg9Mp8wOrhsVQLXxYtF2NAxsjs6fRY2J8jJJ6
umQhB9vmjF76KklBT7n4UYcDB8KMu1tmTqfAsdRORyIHSUorXS11KAQJ7zxp
Os/B1Izk8fy5FCauzH13+DoHFb5DOnP0KMym9Pi3cjnImf1Xd6sBha2ZRkv/
VHGQ5hbp0b+Mjrfpsc2FBg4izJTP65nR8YZNtumzOCiNvR28eSWFFBfTY65i
Dm65Mg8kWlAol15dXqDEhVOzcvdVGwouha+a1qpzEa8lE5xgR6Fzt5WoXZeL
Z/d+tAc4UBh7AUUFUy4MZgycnb+BwoqDDu4HXbnwvfi2cakzhZyvW3vmpnHR
1VrZX7ebgtmxxomSLC62RypL2nhReK+/XdnpERc9ybXe/3pTaI/duTLmNRdj
rDQzv30U1Ez3xLW2cdEvd3U0/gAdr52TGsznYmXz34dKDtLxkn3uy/VyseTt
gC95iIITz69qpQQPKAxaYHiYwulrgSqX5/GQftvB0z2Ygqptn/4iIx6yL5zu
cQ6hcK83ePW75TzUxIdY2IVSeLcu3LPXjod9z2+YzAyn4w0OBSVu5qE9IJfd
S7s9KzJey40Hez3ZwI8RFEZGonI3+vEwHCO52COSXu/B+CvOER7epXQHaP5D
r+ca+/WfSB48Qt9lMmmbPj7568EZHkKnHJ1cFkWhbIe0DFJ5KHTRNW+ivWVq
olrTTR4C6tbFhR6n4O91xko6j4dzR1090qIp/Ck6uZTxnIfIjEJtjRgKp6Yf
11Ms5cHBBlQq7YyXgdPVv/Bg5Pjwc1gsBaMZByS0mDxI1p5gs2iX+O4ZmMvi
YfX4SS3EUfg205Vl1MNDtZGbxi/aXoc21y0d4EGRNZNrc4JCT5lD+YoxHtzv
/Eeeo/2PGlFiIU1i3N1Qv4H21MDVeZhOwj971cO/TlK4XG6aaadCwvlLQ+gm
2rqaxlfWzyGR39F/No724+AFp7fokZC6c6P/CW2LSu3jW41JhCd9fdpMu0Jb
PcjdjMREfe7XEdqu4co+u61IPAjbuF49ngK7Wt59rx2JsvnsRUtpB86T3uS3
iYTW1syjNrRHI8cR4Epiz6oqM0fap2uHlgfvImFf8cjPjbaKft/CCB8SgtIb
s9xpZx0XaUYFkJh/fdDWlbZJA0cpLoxE183FfzbQfmXIkko4TuIOs3iVFe21
cY2/z5wicVJDVc6ANrOppvt8CgklRmOgIu09iys7Ll8hEaqYEtxLvz8V/67h
RjoJib1ts6ppR7W+qrh9j8Qt+Yg9WbTllj1/nZ1Hwq5DYWcI7atJj/MfPCeR
ctpPZg3tee05d/NKSeiedDooTTvfLPN6YTkJlzlZyR/p/bdKTkt+8YVE1BH9
kBO0P7NTY18zSZR8+DbXnLabeUpoGYtEad/fd0T09w0iYzyqekh02M5RtaY9
bhnpWDtAIrd6WL2Dzo+zqcG2zDEScj9jxqJo34OPUft0PhYcqY/MofNr2bXd
OlwVPoiF3ppmtN/0uM0UzuFjv7XG01I6P5vT1o/+NOZj2sRw7Rs6f336bHqH
zPg4rGu6x5x2n4MVd9SKj4OpRSN5dL7LD5hUy2zmQ8/c2CPpGIU1W1RuaRzm
4+xy3j5Jup6q7ylc0A7nY72Z9eetRym4j06Nnx/Nx1fRhHcWXX8hucOHFp/j
Y01RkrUJXZ85Uzos/n7Mh/PhDHslur57p/dWaxXRz7syFAi6/s3VJneNlPLR
NNPQOyCIQpWRTuyzL3wIlSZfFgdSELt6fVjQw8cjplahhj/dXx50bJQ3EmDc
f1mwIt2Pogt7WXxTAbZ5GfcP76Hw6fXk4Q8WAlhoxTmw6f7lXqdz8Th97lmz
7E3oXU8Kx4e9mNRBAQZnWAcM7aDwYWPnrgb6HJSgJFL44UhhulufOL9AgB7o
chO20PnuLRGbUiLA7yxhk/FmCvww3Sz7SgGih30Nj9D9VT7DmywWCGA4DYKP
dD926es8nK4vhEhh/rq35nR/vsKO2X9PCHvBjFWTWnQ9+zroifOECPBu/6Y4
h+5fq/79HFYkhNHbvTGzZ1Noaw1TOfWJ9oNmVX1VCq26co+yhUIcr2mvVVSk
0Jhn0thhLIJKQ9qVReNiVJVHGe0sEuGK/AT7SQs936911rH/E2GW1sbJ9U1i
fPazjzj4SYSY1LBqNj0vK+WV3kU2i5Au0XxCtpaef07ZbteHRXjLUeAafRSj
jFV5gmndBd7SN2GsAjGKBma1bPnUhd+LNQwnEsQwC/rlaVfThcOLzPL84sV4
2l1DrqbPXUkHvKtq4sR4wknq1+N3wcwriLwaJUZO3bDiqEw3/DS2DMkGi3Et
r83+gX03vpSrXo3xECPS5/6zyfJuaCQezqoxEcOKaXah4EMPjM8V3Hxc24Pd
JxI8mrvpPI5YwDcTd+Pt+Xcvfp3qQ9rYRtOm8S7kphdRRSd+ImZtR8UDtS54
XrIp/HH5FwZ3ReowtEQQfyzP/u9yP3L+G7FOWSnEenaH9pSHA7Dj9wa12Qqg
F1ZvbfFyEM92cmx5nny0OJQkmxQPwVFeTepZKomdEuY+CVW/URexbLlRGQ9F
w3n+N6r/oOTIB7s+GR4uHPoz607FME6yyxwpd3oOvmdtkPs+AvxOv5n5gIMX
O148WdwyCo1S67dx0ziQajG4XdQzBsmDwgW9/mzUfQ02WTY2jld9n5vCWzoR
IVVAHRiegESGQPO6WycGL3xIl5wqQWx6lZlvcLcDmm8efjdOlCAufNbPTSlp
h6TBqKBeSpKon9KXldzehs0yaZYGKZJEefx++dgJFhJPFubsmjaFcB72nO5j
zoJ+qOp2m6tTCL/+LvOSgB94KjWkPVVViviVV8CzLGnFRC1RYntFirAy9DF8
r9SKAqWWa6Za0sSN8VfqrD3f4dgrSplMlybisz2XOFW0wNVmXWX0Qhni6esd
siULW6BRr9wYnk97l0PbsdRmVE9uG95oMpUQlDV7RSk0g5vF39tQPJX4ribY
kRHfBCH3J9fcXJZIEHAN4+Sa8HjPgZXqL2QJ/gXLsy+vNKLFxagmd40cEVub
1LhxTiP6uAei1d7IEZe/XJaKK2BivbRVnFOZHKHPtXHJfcJEtp5S/Jn3csTs
R+z+ujwmXPcXnR79JEeIdW4umf+QiWK+5JWOWjmiyrI6sTqL/o8S3Xz0gC1H
ZP45n2V0mQmF3i/Nq2QYhIKEL2UQwYSvUmZrkCyDmHZ0INgjjIk3S0LbHjIY
RENHvf35ECaCgjS4mooMoqx/bvtAIBPMn/upSVUGIVv3rOajHxPpAxPSlQYM
osVy2+2IHUwMqnyTlTRiENG7lOY82c7ElhX35S0WMwhnqRYRuY0JiYiNM/KW
MYhgc4S4uTCx7/d1zYsWDGLFKiUzy41MlKr5a3+2YhDGGwUG4euZUDFfM3cK
GESn8/yA/LVMfIrk6YfZMgj/qJA2PTsmdNKKDR/bM4hSF49p3jZMRL48a8xf
yyBeJWxNukUwUf999xKdDQxCQ9LBuXkNE4tGlpq6b2IQ118ZHJxpzcT/AAux
9iM=
     "]]}, 
   {Hue[0.1421359549995791, 0.6, 0.6], LineBox[CompressedData["
1:eJwV13k8VPv/B3BLlgmRFKWUkhQVomz1OuFS6UbloksqRZZKSssvJUpK2oVE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     "]]}, 
   {Hue[0.37820393249936934`, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1nk4FfsfB3CyHkQLImUpKUXRIqHeUxSJFC4V7RG6IZX8iKhIImWLFFJc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     "]]}, 
   {Hue[0.6142719099991583, 0.6, 0.6], LineBox[CompressedData["
1:eJwVlnk0lO0bx+2MkiKyZCmyhBKSF/k+opS9Em+plCLqFVkqiZStVUVSaKNV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     "]]}, 
   {Hue[0.8503398874989481, 0.6, 0.6], LineBox[CompressedData["
1:eJwVlnk4lV0Xxs2cJg2IlDRQiRLHEOl+QilpNvWqRCk0UEm9r7d6DRkqlUKh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     "]]}, 
   {Hue[0.08640786499873876, 0.6, 0.6], LineBox[CompressedData["
1:eJwVlnc41t8bx22eklEZUVK0JYkI9f7ITBIq7UGRaBMaRoVKIRV9MxqSEEJG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     "]]}, 
   {Hue[0.3224758424985268, 0.6, 0.6], LineBox[CompressedData["
1:eJwVlHk0Vd0bx8+R6RpKKokMUZLITYpIzy5KSaVolCJvZagokjeNopSMoTKW
KyFT3kyFCleu8Q6Ha57JkHnm4nd+f5y112ftvZ519n6e72fDZaeTV/gwDKsl
v/+vVfT0CB19Gcg4afBncscZyH61ItX9+QmgjbAab/EFwAtBrdHoTGtQZMg7
ZfpHwW3DIskMU2f4ePZ8izmRDD3tvjR96cfg4/pf3YhLHuy73MbFeAHwR53C
TdtVAR5fjN7T22Pg1TrPBwtdjfCTVzyLOb8Hm2/WUdYFTVD0/pqywvB70Lc/
y3/8TTMwepPdLAc/wJFE+4Gfxq1AuOvIEX00WBvt0WB2oh0G3po60Ds+Ad/Z
60KN0V0gXe/Gn0CkwczM+P7RuX7YN5ORq6+ZDiLKVQHcKwMw9VTtlrJrOgRI
mQwcZA7ApYh1bWML6VDu/TDmO+0vaJVN5wetzIC64QcL9UZDwN389d9q3a/A
MJn2d80eAaVO9VGT5zkw8ffCIDdlHLg34xK0KnJgeqw8vr5hHF7NyVrLrMyF
XjENnyChCZhcKVrdF54LsiPHa5WsJ6AU9SX7pH6DnyMqHnTxSbgR/dGupCYP
Ejuqdnadn4Im9RUjisE/4eNt1XcN32cgIzzxhGnGT6BPaeYKNs/Ac2Gj/+6y
f0Ja3m7tkYUZ0Opyd6uS/AW+qz5cn987Cz4RHTyP178AwWTzRP4saItkUbgh
hXAtxTH0WdociLibOfJlFkK6i1R2UMUctHb3V2jUFIK0Ay/Vsm8OXv5SCPRa
UwQFt6rC1JXmoc39hZRWWBGIrb6oWRI4D75/Lij7vSmGfiehynkbHlifmvbK
yS6GCNkFPkEPHuwqCurp5BaDEz89lPWaB23RJYl60nTwls8LTikm909Tqb1v
6ZC8MznqofICdBQvMzAMLwGj5KfWGTULkKMVHX3zWwk0m+sm9vcvgN97XSy8
oQTEZtI+/VlaAN37N4pGZH5D+sLzdOqWRfDfwT0cHfEbsLbbVIF/F0EvNun0
bGQpPKfY/zsovgRYa4z607xSuJPhuXOnwhLQZUP5xJtKYcugUoQxdQnMQh+l
KsoygPEx8HzliSW46mMheOgdAySEfF7bBi9B0M3FzLDQMrheU4pm+DF0+vO4
r2JmGZjZcqeWRDG0vrfXJokoA/njmpNDkhiKt+GIF6wqh29LsseeKmIozyLh
SndQOSTKfajw0cNQr97JNdr+FSAem6Oi7ICh1LvGA/kpFSC9U+l0qhOGXL7u
+WVcWQG1nkMVSncwtKix+YalWCU84BuWKH2EodUbeMWeLytBQN7TTygEQw1W
o+FiSZUwfNH7adtbDMWE9ziHMSpBvppeSIvCkNpq1vok4SroMH1N64vHEAjF
u7KeVcG2NR62bjkY4jeKMLGMrwKh5TS1j3kYKnscqNhNr4L7UToGP35iyGLu
XsUMfzXc7jEqzSzFkMPg8Y2KT6tB3IBRlcbFEFXtwFxibDVc1V51bU8jhiav
6jF3FFaDn+k7k8wWDCFnyQIaxoRbj1sdr3djqLyPZ58NTGBzV7iNj2IoXOrD
A7YZE7rXhzVsnCTrGx4IGrRhgtItYR00gyGRKL8cZS8mnF7iOO9cJO/L2F6x
N5QJZ74cjFqB4yhxqqb1XDwTDLvrLhHLcHTITF4osJQJ6O+EsTwFR9IPCmU+
1zMhcuXzHJoojv4kXt1W0s+EhImkc6uX4+jZsrRTPDEWnHdVrMiWxFFxFsTZ
nWDBqv3auaKyOArp7Mx+epkFWe6nxobW48hWwqc82oUFInd3BxfI44jPgTlG
hLKAS30So6yEI3aYq+BIPAt48y9P5Srj6EORtIxoDgsU1tsK7dmEIyRns29f
AwtWWjnMCKviSMJE4NSFARZIppd6mW/BUatbot1dHgtartnRfNVwlEo7ej9Y
nA08L/rdL1tx9JA5GpAiz4a01VoqdHUcHVsIpZVqsuHg0VpOiQaO5NT0sjsR
G+JT0uOztuHo7+mWssUTbOhrPUsP0cRR3lPPlnW2bHiun2VuQ8XRq3SVMW1X
NoReq3NZvx1Hls1lAmbe5PmxF4dKSFYTcVrnGMaG5g3Wkxe1cDS3a5XGs09s
UOT/Ht1DcpltNvqQwwZ7P7q91Q4cvQu0tMhjsMGlO8ejiGS7/KVr3AY2RA6H
Tklr40i3n+YxNsAGXKxs3opk4bWHAsQX2BCR5pkSTDLX8G+s6nIOFD5M2ZZN
crxzYJahAgeKrWZiy0l2i9Iuu0jlQN/l39uYJB8sq2v+dx8HrG9ITBaTvGb6
/mjISQ7kjRyWSiK5S3mDQLotB8asLv33iOSvZnTpclcORHuODR4g2euBvXqP
NwcWgi17eeT/mieJI/wNB4J2JdLjSFbifjFfn8CBpuGWBAOSx5advqaTy4Er
skoZv8n7F1Ln7p0s44DMoT24IclBVtH+Nxo5EHw8LiuNfD/rl/tjff5yoNz0
4bg4ydTsnkzaAgdMaRH1VuT7Y10vGQXLCYgq1/GNIftTLaHZXK9AAL3zsCqH
7N9Nh7v8EvsJUKgKey9B9nfvG1npreYEyLYf/C5D9n958Y+tB/8hwHH9RbSW
nI8UOWHz+88I8NpfJ9ZLzte96IgtLmEEOMd9n2JuJvOhoIk5xBMQsp/hlqqC
ow7F0ylnSgjwPE+LNt6II6mNcUI7BGvAZfez85PkfHfG6bRskaoBwT1hOd5y
OErfVP5VUaUGtvMpuYiSeTDZPGaz/GANrP2uajMtTc6nGsrr866Bi+GrCHMy
T0c+c4LbQmtgn6VGYZAEmUf1a/bcj2T9uM0VdDJ/GRr+UnR6DZxJkwmR+H8+
qU3O7wVqQfKkl/cOMs9mOu6bznjXAl+6RWAh6Qu5HFHe0dBayJTeuHh9GEP9
ujFso4+1MPr17MjyQQx56ZU80qLXwjlPv++7ezGUa7C6QVyAC9L9PVQN0kdK
Run+xV5cMHQ4VBhF+mzcrHd6u1cd5O2TaaKS/lS33jT9MLQObp39fS+F9OtV
p8tT5fF1cEJmVlg5iPSPX9PEFUYdDN7+bDL7AkOFZczRd8vrYUgxn2vhgaFg
o9wB/F09UD5rGDReIP2o+7KVldIAMuZBoZmyGLqhoP77FrcJ9j2gUi6sW4Lx
7i4J5NsGtaxTuy9l8aBDXz4SnnWAdDYfg3pvDhwvvdjDn9wFR7dprLy3cwbi
+Vr19qX0gHVCTZfo0SloC2l/eCWuF77mfkuvoU1A5sSaumMl/aB8+MkPlRXj
sIfQDkgv+gvWjnJvB4pHweqJtyV3YAg8Xq3ss/IdgR/+v7LGvEZAZOiOv/78
ECREZg5lPhmFU3+STO0MBuFi0P4vja/HgLH4T2ay/wAM0otj81+Pw8WB88dY
Kv1wuL1Vni9xAuzbnZwESnthoyvLQC93EkaOyNACgv5A3cEc323ZU2As/C1T
5mUPnMd0//Eum4bOQ/NFRt7dkDmb7Pi2fAYOvQnx2RzfBQEOM2tifs+CXODs
K5OmTthV2GQiXD8HpjkxgiKSnZB1LitVo24emsoVuk1sO2BZnWpU5l8eBEQf
ePjov3ZgVt7atp23AH9HDMr1V7WD27L0IbvZRRCvhVdxz9tgMqAoEhfE0NrG
UG18ohVkCxLr1Z9hyFp2E9rCawFcdf4Pi5w7LeMLipZrW+CowDt91Vek5/2G
hAM0muGZ55f4C6J86M+5NQItl5pAxUXqzP5QPtSxS4553K8RMpZNyQtKLUM6
M5oWNzkNsFiNcgxDlqF86ne1uyoNkL6iLkxLjh91mi1ccnath+PDfa+WIvnR
0NKd+ejaOrDYf6j0wWYB1EctQXzadbCOtbLmTpoACqJF1qJILpQvnZo9sk0Q
VQ4h8b1ruND5oecyJ1sQbXDqkn/yohZ6O0c7dXWFkOSAyI92sVpIsbHbJZ0l
hOSVA1I5b2qg7uTWqoS9wghLMmdoydfASKfdg7UFwsgjv3+F0xcCDvPveWz2
Uxhxbm84+zqNgNiNK56+KBRG2ga/JrNTCLC4kukzXyKM3iq0UfmSCMjuwUNa
q8l6x9bFhccS8LAvPOlTuzDKZ3g7NL4mQHy4gqsjQEElFy3qI9wIuLrifYOz
EAVpRBbElrkSUKDp0pxIoaDftb8KZm+TnnJe1ym7nIKyTukPnXUigBi9MrQk
RUGxKSGqsnYERE4s8peqUhCNUT6deo6AydVsIXwrBQHFp63jDAHHdn4U0dOg
oF0ST2TWniYAczsikbydgla7cJ48PkmA7fQb2UA9CpLV+CB75ggBeWsd5Rl7
KChfcsDX7zABq3X3buADCpL3LDEtNiagxL1LxdWQgo4NNf3QMiI9/C57S8oB
Chq4M3jLgfSye+5L9R5jCmqJ1HONRQSw6q00FUwoSEs/raRhLwFqc1Sts6YU
dIW6y36VAQH/A3T9WWA=
     "]]}, 
   {Hue[0.5585438199983166, 0.6, 0.6], LineBox[CompressedData["
1:eJwVlGk41F8bx2fmJ4ysEaKkohARiZD7aCNUkq20IUKFIvyJkqVsiVCJKUu2
kG0sFcky2c3w0yBli0j2Pdszz4tznevz4r6uc77LvcPa2diWRCAQ2lnn/3dz
Te4rNU0xSG46kC7CZg3FYXw5no/OwtWdCX5PBV9AMLvyFIV6FTY+vyv1ezoV
7hyt2pRv6AKLWjsZRKsiGOoLTdYUfQA+CrfL7gjQQMe6l0lYiYDTC09V1lva
wTvv2JuavteQx2b34/DpQahYqV4iuLyBDyW8cV/GB6HqzfVd2yfeAGMx6IzH
kyGoG85ytxxLBLc6wUP2Db8B91Tbho8kg2tibvGsxh8YfWHoWNOfBnfDdHqY
Y+Mg2unOlo6/B2Ub6WMOdTOgs5hfqqmYCxWX6M2BfLMw7y93e5dbLogsnPtJ
NJ2FK6+29E6v5kK15yQm+nMWlOsXyiIF8sFDkLfzw/AcMPcU/teiXgh6h4pc
F3oWIKxg7/5in0K4O7ReILh1ERAkD1MqC4Eo/c4hynwRMs2izJ0MqXCRVMJV
2rgI9wJdVHmuFgG/i7/cx/dLsHNAfkr/UQnYtt8N2XV+GZhOKenKjSXQaCiz
9j50GcL+iV8VEygF60NVzzLKlmFOYGPLSFwpeBd6mPzevgK1aCTrcc4HaLJ0
zkj5sQK3KG/tae2fwMxjDA1rr8FOuW2SOWJl4D2g35FlvwbfqNHMmCtloJ9k
7Pc3cg1Qk/8J25EyiJ6ce9LbvwabVqyk2FbLwS+70YHquw7d8nyTklEVkOrj
H+B8hoDy4zLOGuZXwA4tC/ujVwjoEeexAo/WChAmZOUNOBGQ8i9P9+ZNX8CX
3a4nJJyAHr/qX/F+9gV80skjPrUEdICriMyMroSfPcwAXVUi4vI0ukGiVsJr
aZ1V7iNE1DP4p1GhvRJmzjpEppwmopAv258GbK6CI67tHgbXiajXM1hYObYK
2nR5qvxiiSj098Vd4c+rgXe7YOr3v0R01XQhoKS4GlbtuCJG54noYFXk0ACz
Ghglp4dbCSTUS6FlaIjWwMci+iFZIRI6aKakNPyiBmRcroqvqpMQd3V9pGBp
DSxTQm0ZOiTUt992RruzBvy7qZT7+iQUxhNXFLOFBuxcHNtuWJJQfzV2+Ggc
DTITm18GeZNQiTKF4vSBBoOH5RqV/Uko/I06Ia6LBiactYYfQ0hI/d6tqkmx
r5DvfvKm9UsSeqLCPEl59RW8bePX+akkpJGUabYUXwvm0teKFIZIiNDzWt7/
Uy34VuPdKaMkVCMeQ+LproV0Jc6J1UkSMoq5nyMpXgfDsVl0s2USsntswq73
sg4+74m4lMmLIfmak92MkjqIrSnJURPE0BQR8i076mDoljgzXQRD97xlLzuL
1MO4Kt8LNUkMRTqtUWNj6qG14oEFjxKGzN7NhEpSWfw6svmzCoa2Dg9bZeL1
kBPsOGmuhqFUqzaecsEGuCCzzqehjaGblNoBXZUGMOoa8wrQwdD+72WlDOMG
+EoJcaIew9Ank3TbwcgGmPjQQW/Qx9DDyARN57wGyMoUS887hSHd5iiBJXoD
fAlS7fU1wlCrrk8ZN38j3GlUP0s3xdCwhvHmA08aITLp7ajJFQzleOiOlmU3
wk77wdfeVhhyLdT6otvUCHbBPhahNhhaU9hzy5K7CTR5k97YXcdQtePWo4N7
m0DLSOOFsgOGgtMEtjgbNIH1qbeXhx0xJLRjpfphSBMwP1wT3eSEoa5LU3Hc
mU2wQspxDnLG0Ou4IZfYuiZYPvDxwqALhq4xv5+QHGmCqak4DsU7GJITYmzN
5GyGhDTbtzauGJowok2ryDTDXtLba/5uGCoM/1hbptsM0lV/3J/cxZBXfS5F
93oznGlg4wp0xxBwpLoxgpohhk9D/7oHhtiOvdK3TG2GCxZ7bVU9MVT/4Knk
YE0zDJ1jBk+wOKIscN5psBkeTPgMx/yHIZN/Xo2LbC3w+cWfHBkvDImpuSQ9
lGqBL+qxHOks7nG19eQ+1gJHR1a3CXljKCX3wulYmxboFRiRuMVix7EzUpL+
LTC/0UG9gMVKcsf/ZSS1wM4TCi+GWDxnp0FXqWyBokfU6xz3MPQhWTG1rK8F
goOS6JtZjFw2lScT6OB+tmlVgMXJvDwZ9vx0qCToiq+w5tmzOaIVJOlgsSvZ
FGexvQHp/rQiHfLP0drjWNwwsuJQDHRAWGHVGRbHCSf6tBrRYUfxtdQ/rPc7
Hj0eOWZFh8yq6CO3WazhMpLC6UoHrvuLzQOs/3MlhJfsCqADjfbX5wiLu+r2
N2rH0EHJUMIvnKVXxnx7z/lUOhw2LttYw9LXc5fXjFsxHSqavIxHWPrrGUlw
PK2lg/XbTcH/WP6I+lSKveukg0SBTc8iy7/fGXb7aH/oUHOKJ+gXy98g7L3p
CjcDeNM+hwfcZvVF6ZyDiAQDpuIIxodY+ZC+tHBPWZEB+dq6ct2sPFUXQYr9
WQYoSosx/97AUPTAQLG/NQMu2I3zWLLyZ8P/uIHiyoBd/REiJfYYIjnSp/EY
BuhEWZuDLUvfbVY6Ol0MUBKYTXp0CUP8+htML44yIE9f3eW2Jctf9wx7jxUG
ZJjP/mdwHkO+9KmIbIlW0FyNXqKbsPrn//DnFptWEEsuMNxtgKGw3N3TB9xa
YZ+4U2WsHoYsf9RvMApsBQ434cbF4xj6d1BQISitFW7YpGpGIQyp/0n2nh5t
Bc6GZ/zbVVl5NqoRbXBrAwpZSKJKHEMBPg7yQ4FtEJdc8PiKKIbOZfIg4vM2
wLUUFyeEMDSNmV1XK22D/sAAzzEeVr6Kh6jJq22wpDW1fnmdhLK3cZ67F4RD
WppvgUYvCXlRXsm6xuIg/pFt4VY3CeltVyQ4puKwZBKV/aKDtX8lzbLNaTg8
NTvVidNJSFgqhUOFvR303hvyjFaQkK8c+jQS2A4XOJu53r5m7Us1T2nzwG9Q
cmvL8rw5Cc0YDS/sD+iARe9fklUlRCR/VXrBN6YDuDloasIFRGTnbD3fkNoB
uFipq202EXWFd8/a1nXAYaZd1mIiEVXW06de8naCuOyq1WwIEUUdKx0lvuwE
rsKwb2cvEpGSekgPI7sL0gpu6kmtEtCt7fJfbzO7IUrqcU68BgHNDP7iR6G9
MCPin+zxYBX6NSXiIagfsh9mER5O/IMbV4K12LJ+Qa40B6bitgippB4Nnewh
SJz7KLjkNA+90X2+tinDEG3IG7xInQXq7OaO07Q/8EsiF0mLzoAWfiAit+ov
dN8ZexhYNwWX/AItmaPj8Lerh/I0fBI+P/lSNB0wCcsJCzctVschPZ46TvWb
Ar1/Xume2mNwOfJI3vdn07Aqx472RYzCWE11UtmzGaCf0D67ac8fONnXI0HK
mAXjHeaeUrXDIOXGOKxROge6fk3NlMjf0HGiJHRf8Tywd1hE7wwZggsE9WuB
9QvwSiorRy9wEKhLWTdeNCyCmKpXnGzqL4hwXNz8+usS9MpmZRl0D8DBym59
zs5/4HHaMoFr0wAUnS/KUehYhr6FXnUDm37AOmQSqH9X4Oba+Xf3C/qA3nR7
3/6VVZD1VvqlKdgH7ljuuP3SGgwGMONSHvXCXERVPJGdgCilcseJsz0gXp7R
KR9EQN71/PqyKz+BKLP8m4EREaPCebelyE84teGlpkwYESUefMIdofADgh7m
pV7cSEI9fBYcP690w25XYfMjMSQkNHqPcSb8O+Rj8xLswhgycOMzdWrrgrUW
VHI0mtWDboO9Hru7IJevI1Z5Gxs6t8/iqotbJ5yZGAlbj2dDCQZ5y5RvHWBy
RK/WZ88GNJ47ikgHOmALQ6D97vsNqDOy8BuKZ0LDuumSwT52dOWVEY/2ZiYM
JA5ZtxWzoznVcQm/4G8wPDA1oK7OgXh5BT73cX+DbCv7g6JFHEjpe1RO2/N2
6DDe25yuzYneJZjWKUu0w+SAvY9IOSfaGj3K55yHw0k2rQdGFZzo0vGdFs/e
45AkxecfXMmJcjZWzhVn42BiS328TONE5KFeJVImDsVDxOieFk6UzyaWEpeE
g+9IXGZaHycSsQ5y/P4MB56JRqbaBjIaLzfpfOWOgx3fmy4XDjJiWypPqnfD
oVzR9UcGmYxMlSrLl+7g4OKyZUCcl4wq0jXHLZxxwKdsx9eFyahvJlpG3B6H
+Nk1tloZMtJaaFjIOY/DnFArB3EvGXWqPO7tN8fhtOpbLg0FMvqm5icmYoYD
wd2AP2s/GdGj2/weGONgs/Bc/KkGGZ08mihuboDDJ5EbEnVaZLR592ho+Ekc
hNS1d5CAjFKiaIbVujjQPH/tdjtKRtpL3Z+Vj+Gw/WWxbPZxMhL1HbvteAQH
z9IQ+SFdMjJM0nBLQjgwOi8pbtcno+HD72ld2jjI/VNStjAkI1ulgw6Ch3H4
Hw2HSUM=
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.505118915137782*^9, 3.58848096080575*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", 
        SuperscriptBox["r", "2"], "]"}], "/.", "\[InvisibleSpace]", 
       RowBox[{
       "sol", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "2", ",", "10"}], "}"}]}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", 
     SqrtBox["a"], ",", 
     SqrtBox["b"]}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV03k0ldsbB/BjPkf5oZAomn4Sx5jMPK8hV6ZwqYRElLIqSq5KE4eEkrFE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     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJwV03s41OkXAPAxbjNKaNFFqK2VMimzioycN7Uq6UIWUYgUFZmlQdTWuoaN
cc01JdcoZGbM/iIJKUoXFq0WsRp8zdcl99vv7Y/3eZ/Pc857nuec87wbXC5b
uVEpFEo4Pt/vt3UlaQastRBFSJ8MdiFAEK3wyD/cErgPD4m/+5YMczST5wxq
PyxdDcXmm52KupnnDhqmakvh2L/te7GyzMIbbFSSrkZhR7i2O2X/4guWVX7D
Mdh3g4eLEkz8YTaw1jbx+/ts6myoQRAoTTsK07D7e6KyWatvQK2q68ps7DG2
qnm58h/Qc6DC5SH2gsS9EYZSCAQpWhWWY9PidJJz5MPAc5fWQBW28o/8PRpy
EdDvtUXzNbZmGepLkomEq47HLP7G3mraGKkgGQ2rUyK9e7H3unS3UeZjIJJP
zaS4EqAVsPOZ8XwsuPoo5yliM55fXRkwy4VLQ8O5G7ANLKWFo5PxwGJmR5ph
G6eYL9edSIDojZZe9timPTFOF8YTocrtwsHL2EfZa2V6yWSYHrvSlYZtLXQ6
qSm+AzHPdqQ/wT4pkVPkQKTApfKvR5uwz3K3n2gRpUHs0JtEylkCAkv3Z9X1
3IW1Rz7m+GJXz9fOULyz4IZP84oE7BdZ5zdqkllwzKTkUjn2K1ERx2H4Hoh2
XJOZxm7xN1BvGcgGqmNMcaQbAW3rPpmNuT+AoS+SwlLsT9VB3oqiB7CbNy3o
wO6m1byw6M8Bv7TUGMY5AobuWFyo+5IHMzy1rM/YYmMyvvdMPoQeHty7/DwB
I93cSomefGBujm5lYU9qtymadBVA026H9xnYVOEZAf+fh8COc1T2dCdgdQdH
Kr/lMegvz7hteAHPf7pMyNpeAptyHDwCsSeDt7I3+pbAJQUrh2fYTmlruscW
SkBFyi7Y/CIBzNdTlVylMtCOF7R7XsL9bC4PaDYsB88GBZbIi4DoJzp6gmvl
kBy0YGt8mQAE2aLMmnKwsz8aysUutImz9bLgwR1FpgJ4ExAU6r1T3pkPrWa2
H3LZBPzYyxg1D6+AAwap2fm+uL7Xg3xmUwWEWflGyl/B9WfVnNcqCaFYxyPA
B3tCaVnzQKoQbmSc8TTlENCABooiHv0FgbpXMgg/Ajwzc9zrW5/CkN3cHXYg
AZ0MhZH1cdXwyXo8bncwAWWpBZYWZdWws648gIcdTtv/xO9DNTyaHNRnhuB+
+/w5b1c+B6ass5ZuKP5PaV/mA+Ofw775r33bwwnQl+PT2xJqoObcw0P2UQRE
fT218c/kWtjrvdLaLIEAo/uFNjPpDaDj8etOm3wCKF13GcFPG6DzwJtXi9h1
aolU+c4GCLeylsgvIOB44u+P1qu9AgXnWpv5QgLORVjLHEx5BYrnI8KLigng
ei3ykhJfQ6y1jcvWJwSIjKxU9G83QWfU7emkKgIaB+Y9BPAOBOPCPLk2AorV
aSeCwlpg9Hx2RpfSMIwfF03phbSD4sHbUsZHhsFTk/GS3dYJ092hPloxOP5f
nyKK6obkqXhR3dth+MLSSIewL1DstOOao6wYLjrdMpYq6oMU+9J/ryIx5FK7
jPYW98M2m8Pnt7DF0J3Qc93tgQiM9gc/O5EuBt43lfaj9YPwg32Q6vp6MRi3
6MeUvCBg3f9KR3JEYjh9M9ShbUgMZ05UBCbRSXh2+zl/LGQEdnGSXnb8REJ+
Ok/MuzkK000s0wVjEhy5pqX/xI+B3nWHddQTJAzX1d6vjB8H6zhPOzk3Eg71
dGlQC74BWqEVpc8hYZPv+z1GwgnIc2Wpz4eQ0G5WEaUrmIRBz30/b+OSYE8x
PBv6egpGzHTT+9JJ4M0UXbzTOA0t3WH703NJiLkwrXL35QwcNpxI2fSYhF01
nea0jllY1nvAwktAAv8k/9G29jnI3LxO1amKBMl27QweMQ9isrJ6Yx0J796w
dfXmF2AwVv6beiMJHMkSsfvMIujFOjrovCNhIuZFuoQMBdW8KfT6qZUEtaqC
DkYYBfXbhvzn1EGChPbc1/eSEmhernUN4zMJR6RTWNrREmiyqcZ8rJuEsD9K
c08toyKDh2ctt/WRoOWjamuaSEWW8ZnObl9JKJOc1JBRlUQ8v9LaLYMkLDaj
in0JkshqsfJpEUFCiUJ7ElNdCjlBozqTJOEYORC9lC6F3GIv73EeJcHa9GDD
tc3SKJpD+Z07TsKa90qtVx5LI3apBzd2goTGpV9nDuvKoDqJvD6LKRJ67/W7
fBTIoA5Htv+KGRJEvaO9hoaySO/PrOikWRKKz7jvWs2XRb99XsVcmsP7sNJ5
m29CQ3bZqqpeCySM9LpfW1VFQ5x7C6ceL+J9ShnfOF5NQ41Gg43l2Pc3KQTf
qqGhQI7b7r+wrd14EXP1NBT6t5xiHbagXyKhq5mGXpayij5hXx9ILczroSHK
3dQs6SUS5MmmNgNpOgru7/Kzxz6nkPXJW5aOAk5zhE7YVdt9PhfQ6ejlQMHM
WWxv7zW9aivoqPlg/5XL2C2jbuIlVTrimeeeDsFO/7Yo1aBNR1rWdoPF2BPK
H2QldOjo1McWlSfYR3fmyBlto6NwzxmowKZwDisW6dHRabndsTXYrlPJarFG
dDQXEbK+Dfvpqosar4zpaH2w/t5ObGVDkw1UwPn1Ec492PX+fVq+++jIw2Rd
2hC2ZopgS/EvdGRy3Z4/gu0vjGT0H6AjI3XWuwns9x2nt2ua01FDVqVoFnvr
7A6mnQUdXWQQS0vY/wd6WkFe
     "]]}, 
   {Hue[0.1421359549995791, 0.6, 0.6], LineBox[CompressedData["
1:eJwV03k01OsbAHAzg2aULKXNRcUpSlxFHKae9yKVVHK1SEVRiYSSFK327Gu2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     "]]}, 
   {Hue[0.37820393249936934`, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1Hk8VVsbB3AzpwyplChD0c0QDUpX8ttpvGlOM5GQlzJE5ZYGc0VJkbHI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     "]]}, 
   {Hue[0.6142719099991583, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1Hs8VVkbB3D3zlFCE1EhkQwG9UpCfpuMaTCV0hghXYwxNRm9NaWiIrdi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     "]]}, 
   {Hue[0.8503398874989481, 0.6, 0.6], LineBox[CompressedData["
1:eJwV03k4lekbB3CEzklIE5UmWlSKqIYI9X2TJK1iUiFpG8NY0jItSkrJUvYQ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     "]]}, 
   {Hue[0.08640786499873876, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1Hk4VdsbB3Azp5ShkiYqJSFRiXD77sZboiiRJA3qli6pVCoqMlNSIg1K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     "]]}, 
   {Hue[0.3224758424985268, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1Hk4lN0bB3BjiSnRolCiRIslSwuhvg/RIm+rpSJLq1evre3VRgslO0mk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     "]]}, 
   {Hue[0.5585438199983166, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1HlYjN0bB/D2mrRYKmsLJVQqSZv4PiLRG1lDoWTr1auFKEKhRKK9RCih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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{None, None},
  AxesOrigin->{0, 0.2},
  Method->{},
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5051189158038206`*^9, 3.5884809619339314`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{1097, 740},
WindowMargins->{{257, Automatic}, {Automatic, 30}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Report"}, "StandardReport.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3588488159-1683493"->{
  Cell[17972, 432, 549, 9, 194, "Print",
   CellTags->"Info3588488159-1683493"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3588488159-1683493", 120475, 2339}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 250, 3, 99, "Title"],
Cell[820, 27, 16733, 388, 1501, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[17578, 419, 246, 5, 88, "Section"],
Cell[CellGroupData[{
Cell[17849, 428, 120, 2, 40, "Input"],
Cell[17972, 432, 549, 9, 194, "Print",
 CellTags->"Info3588488159-1683493"]
}, Open  ]],
Cell[18536, 444, 961, 14, 106, "Text"],
Cell[CellGroupData[{
Cell[19522, 462, 145, 2, 37, "Subsection"],
Cell[19670, 466, 380, 6, 30, "Text"],
Cell[CellGroupData[{
Cell[20075, 476, 132, 2, 40, "Input"],
Cell[20210, 480, 1046, 21, 132, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21293, 506, 135, 2, 40, "Input"],
Cell[21431, 510, 745, 15, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22213, 530, 146, 2, 40, "Input"],
Cell[22362, 534, 473, 9, 61, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[22884, 549, 157, 2, 37, "Subsection"],
Cell[CellGroupData[{
Cell[23066, 555, 1776, 46, 64, "Input"],
Cell[24845, 603, 2067, 63, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26949, 671, 443, 12, 40, "Input"],
Cell[27395, 685, 6841, 120, 252, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[34285, 811, 159, 2, 37, "Subsection"],
Cell[34447, 815, 532, 19, 161, "Input"],
Cell[34982, 836, 2616, 75, 117, "Input"],
Cell[CellGroupData[{
Cell[37623, 915, 610, 18, 60, "Input"],
Cell[38236, 935, 45125, 757, 254, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83398, 1697, 582, 17, 48, "Input"],
Cell[83983, 1716, 35999, 604, 242, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
