(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     42305,       1012]
NotebookOptionsPosition[     40479,        948]
NotebookOutlinePosition[     40955,        966]
CellTagsIndexPosition[     40912,        963]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Package: VectorAnalysis", "Title",
 CellChangeTimes->{{3.50512116435043*^9, 3.505121184083559*^9}, {
  3.5884839799513063`*^9, 3.5884839864311533`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"BeginPackage", "[", "\"\<Lautrup`VectorAnalysis`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Unprotect", "[", "\"\<`*\>\"", "]"}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"default", " ", "values"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"$CoordinateSystem", "=", 
    RowBox[{"{", 
     RowBox[{"\"\<Cartesian\>\"", ",", "3"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"$Coordinates", "=", 
     RowBox[{"Map", "[", 
      RowBox[{"Symbol", ",", 
       RowBox[{"{", 
        RowBox[{"\"\<x\>\"", ",", "\"\<y\>\"", ",", "\"\<z\>\""}], "}"}]}], 
      "]"}]}], ";"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"UnitTensor", "=", 
    RowBox[{"IdentityMatrix", "[", "3", "]"}]}], ";"}], "\[IndentingNewLine]",
   "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"backwards", " ", "compatibility"}], " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{"SetCoordinates", "[", 
   RowBox[{"sys_Symbol", "[", "coords__Symbol", "]"}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Unprotect", "[", 
      RowBox[{"$Coordinates", ",", "UnitTensor", ",", "$CoordinateSystem"}], 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$Coordinates", "=", 
      RowBox[{"List", "[", "coords", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"UnitTensor", "=", 
      RowBox[{"IdentityMatrix", "[", 
       RowBox[{"Length", "[", "$Coordinates", "]"}], "]"}]}], ";", 
     RowBox[{"$CoordinateSystem", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SymbolName", "[", "sys", " ", "]"}], ",", 
        RowBox[{"Length", "[", "$Coordinates", "]"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Protect", "[", 
      RowBox[{"$Coordinates", ",", "UnitTensor", ",", "$CoordinateSystem"}], 
      "]"}], ";", "\[IndentingNewLine]", "$CoordinateSystem"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Coordinates", "[", "]"}], ":=", 
  "$Coordinates"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CoordinateRanges", "[", "]"}], ":=", 
   RowBox[{"CoordinateChartData", "[", 
    RowBox[{
    "$CoordinateSystem", ",", "\"\<CoordinateRangeAssumptions\>\"", ",", 
     "$Coordinates"}], "]"}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"my", " ", "Grad"}], ",", " ", "Div", ",", " ", "Curl", ",", " ", 
    RowBox[{"and", " ", "Laplacian"}]}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "shdw"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"`Grad", "[", "expression_", "]"}], ":=", 
  RowBox[{"System`Grad", "[", 
   RowBox[{"expression", ",", "$Coordinates", ",", "$CoordinateSystem"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"`Div", "[", "expression_", "]"}], ":=", 
  RowBox[{"System`Div", "[", 
   RowBox[{"expression", ",", "$Coordinates", ",", "$CoordinateSystem"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"`Curl", "[", "expression_", "]"}], ":=", 
  RowBox[{"System`Curl", "[", 
   RowBox[{"expression", ",", "$Coordinates", ",", "$CoordinateSystem"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"`Laplacian", "[", "expression_", "]"}], ":=", 
  RowBox[{"System`Laplacian", "[", 
   RowBox[{"expression", ",", "$Coordinates", ",", "$CoordinateSystem"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"On", "[", 
    RowBox[{"General", "::", "shdw"}], "]"}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"clean", " ", "up"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Protect", "[", "\"\<`*\>\"", "]"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"SetAttributes", "[", 
     RowBox[{
      RowBox[{"Evaluate", "[", 
       RowBox[{"Names", "[", "\"\<`*\>\"", "]"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"Protected", ",", "ReadProtected"}], "}"}]}], "]"}], ";"}], 
   "*)"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"EndPackage", "[", "]"}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]"}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.4245305969969997`*^9, 3.424530719448*^9, 3.424530768264*^9, {
   3.424573161638814*^9, 3.424573162296814*^9}, {3.4245756708338137`*^9, 
   3.4245756786678143`*^9}, {3.424575986143814*^9, 3.424575989651814*^9}, {
   3.424576051574814*^9, 3.424576067372814*^9}, {3.424576098886814*^9, 
   3.4245761165688143`*^9}, 3.424576181784814*^9, {3.424576509633814*^9, 
   3.424576517670814*^9}, {3.424576587525814*^9, 3.424576593828814*^9}, {
   3.4245769953668137`*^9, 3.424576998986814*^9}, {3.4245771626168137`*^9, 
   3.424577166797814*^9}, {3.424577417270814*^9, 3.424577428776814*^9}, {
   3.424577559828814*^9, 3.424577562836814*^9}, {3.4245776271798143`*^9, 
   3.424577631089814*^9}, {3.424577670018814*^9, 3.424577700982814*^9}, {
   3.424577747100814*^9, 3.424577766012814*^9}, {3.424577815015814*^9, 
   3.4245778157418137`*^9}, {3.424578858599814*^9, 3.4245788792418137`*^9}, {
   3.424578931115814*^9, 3.424578931869814*^9}, {3.424579175704814*^9, 
   3.424579187242814*^9}, {3.424579532512814*^9, 3.424579533520814*^9}, {
   3.424579681290814*^9, 3.424579683320814*^9}, {3.424579847466814*^9, 
   3.4245798497608137`*^9}, {3.424579997830814*^9, 3.4245800367678137`*^9}, {
   3.424580362250814*^9, 3.424580364598814*^9}, {3.424580412954814*^9, 
   3.4245804346428137`*^9}, {3.424580927773814*^9, 3.424580931045814*^9}, {
   3.424581006515814*^9, 3.4245810074778137`*^9}, {3.424581052091814*^9, 
   3.424581055224814*^9}, {3.4245811437858143`*^9, 3.424581151122814*^9}, {
   3.4245812458858137`*^9, 3.424581248414814*^9}, {3.424581396207814*^9, 
   3.424581397463814*^9}, {3.424581453103814*^9, 3.424581455691814*^9}, {
   3.424581487949814*^9, 3.424581491372814*^9}, {3.4245815292088137`*^9, 
   3.424581532170814*^9}, {3.424581717159814*^9, 3.4245817193518143`*^9}, {
   3.4245827145368137`*^9, 3.424582718376814*^9}, {3.424582949622814*^9, 
   3.424582956104814*^9}, {3.4245830140178137`*^9, 3.4245830150698137`*^9}, {
   3.4245830568198137`*^9, 3.4245830591468143`*^9}, {3.424583099052814*^9, 
   3.424583122124814*^9}, {3.4245832067268143`*^9, 3.4245832235368137`*^9}, {
   3.4245833555508137`*^9, 3.424583363014814*^9}, {3.4245834096028137`*^9, 
   3.424583420099814*^9}, {3.424583472699814*^9, 3.424583550598814*^9}, {
   3.4245838110108137`*^9, 3.424583837916814*^9}, {3.424586874303814*^9, 
   3.424587147237814*^9}, {3.4245872527338142`*^9, 3.424587324973814*^9}, {
   3.4245875138528137`*^9, 3.424587517540814*^9}, {3.424587564505814*^9, 
   3.4245876324708138`*^9}, {3.4245876796608143`*^9, 3.424587711348814*^9}, {
   3.424587792309814*^9, 3.424587798104814*^9}, {3.424587832089814*^9, 
   3.424587857810814*^9}, {3.424587955306814*^9, 3.4245879731088142`*^9}, 
   3.424588118747814*^9, {3.424588225685814*^9, 3.424588229133814*^9}, {
   3.424588322641814*^9, 3.424588344565814*^9}, {3.4245883860618143`*^9, 
   3.424588389442814*^9}, {3.424588444302814*^9, 3.424588447473814*^9}, {
   3.424588516174814*^9, 3.424588537565814*^9}, {3.424588787722814*^9, 
   3.424588821159814*^9}, {3.424588870280814*^9, 3.424588880770814*^9}, 
   3.424588949610814*^9, {3.424590525294814*^9, 3.424590709658814*^9}, {
   3.4245907545218143`*^9, 3.4245907555428143`*^9}, {3.4245907856678143`*^9, 
   3.424590828121814*^9}, {3.4245909184958143`*^9, 3.424590925546814*^9}, {
   3.424591010123814*^9, 3.4245910213858137`*^9}, {3.4245910524328136`*^9, 
   3.424591115016814*^9}, 3.424591145815814*^9, {3.424591403591814*^9, 
   3.424591443691814*^9}, {3.424591643321814*^9, 3.4245916538778143`*^9}, {
   3.4245916996208143`*^9, 3.424591729422814*^9}, {3.424591866591814*^9, 
   3.4245918768128138`*^9}, {3.424591946290814*^9, 3.424591947428814*^9}, {
   3.424592108308814*^9, 3.424592120232814*^9}, {3.4245922095098143`*^9, 
   3.424592211367814*^9}, {3.424592250664814*^9, 3.424592266039814*^9}, {
   3.424592307739814*^9, 3.424592384217814*^9}, {3.424592459715814*^9, 
   3.424592480990814*^9}, {3.424592559775814*^9, 3.424592661019814*^9}, {
   3.424592717556814*^9, 3.424592822840814*^9}, 3.424592856430814*^9, {
   3.424593569197814*^9, 3.424593575070814*^9}, {3.424593632140814*^9, 
   3.424593634217814*^9}, {3.424593734419814*^9, 3.424593739998814*^9}, {
   3.424593939398814*^9, 3.4245939432538137`*^9}, {3.424594541869814*^9, 
   3.424594550107814*^9}, {3.424594637486814*^9, 3.4245946628708143`*^9}, {
   3.424594704322814*^9, 3.424594711683814*^9}, {3.424594788927814*^9, 
   3.424594794843814*^9}, {3.424594950862814*^9, 3.424594953404814*^9}, {
   3.4245950144708138`*^9, 3.424595048183814*^9}, {3.424595156346814*^9, 
   3.4245951888008137`*^9}, {3.4245952423588142`*^9, 
   3.4245952449178143`*^9}, {3.424595331047814*^9, 3.424595386864814*^9}, {
   3.424857827889*^9, 3.424857860177*^9}, {3.424860129585*^9, 
   3.424860136455*^9}, {3.424860207277*^9, 3.4248602113050003`*^9}, {
   3.424860248483*^9, 3.4248603426549997`*^9}, {3.424860669459*^9, 
   3.4248606763859997`*^9}, 3.4248619930629997`*^9, {3.4746314561014347`*^9, 
   3.474631476234586*^9}, {3.4746318982247224`*^9, 3.4746319174978247`*^9}, {
   3.474631968351733*^9, 3.474632028506174*^9}, {3.4746321583356*^9, 
   3.474632158888632*^9}, {3.474632257068247*^9, 3.4746322572842593`*^9}, 
   3.479138134208*^9, {3.479955805704*^9, 3.479955857184*^9}, {
   3.479955908148*^9, 3.479955922308*^9}, {3.4808399929512663`*^9, 
   3.4808400163736057`*^9}, {3.5051212187025385`*^9, 
   3.5051212215497017`*^9}, {3.505121257212742*^9, 3.5051212687234*^9}, 
   3.5051213105407915`*^9, {3.505121627219905*^9, 3.5051216324972067`*^9}, {
   3.5051216758576865`*^9, 3.505121722686365*^9}, 3.5054095655522633`*^9, {
   3.5054096304189734`*^9, 3.5054096973658023`*^9}, {3.5054098938960433`*^9, 
   3.5054099074328175`*^9}, {3.505410136031893*^9, 3.505410142329253*^9}, {
   3.5054101777122765`*^9, 3.5054101854007163`*^9}, {3.505410727906746*^9, 
   3.5054108027620277`*^9}, 3.5757709704777412`*^9, {3.575771043981995*^9, 
   3.575771071142421*^9}, {3.575771200242626*^9, 3.5757712250577374`*^9}, {
   3.57577211501044*^9, 3.575772127689087*^9}, {3.5757721654697747`*^9, 
   3.575772189040764*^9}, {3.5757722605377436`*^9, 3.5757722807542486`*^9}, {
   3.575772468452819*^9, 3.5757725002128534`*^9}, {3.575772533667049*^9, 
   3.575772555466787*^9}, {3.5757725856425776`*^9, 3.5757725992742863`*^9}, {
   3.57577337312944*^9, 3.5757734516893215`*^9}, {3.5757736050715384`*^9, 
   3.575773618338219*^9}, {3.575773710970922*^9, 3.575773749280673*^9}, {
   3.5757820095216775`*^9, 3.5757820289460664`*^9}, {3.575782091929936*^9, 
   3.57578215736218*^9}, {3.575782217106677*^9, 3.575782229498205*^9}, {
   3.5757824854773893`*^9, 3.5757824894768915`*^9}, 3.575782700932444*^9, {
   3.575782756384366*^9, 3.5757827592097225`*^9}, 3.57578281164935*^9, {
   3.5757828828012476`*^9, 3.5757828835733795`*^9}, {3.5757829219372063`*^9, 
   3.5757829233463326`*^9}, {3.57578297317663*^9, 3.5757829788663273`*^9}, {
   3.5757830825123544`*^9, 3.575783097432214*^9}, {3.5757831344078717`*^9, 
   3.5757831351769533`*^9}, 3.575783182104863*^9, {3.575783221528798*^9, 
   3.5757832603537035`*^9}, {3.575783297216335*^9, 3.5757833230045385`*^9}, {
   3.5757833559136453`*^9, 3.5757833624795036`*^9}, {3.57578345001649*^9, 
   3.575783490887605*^9}, {3.575783626934719*^9, 3.575783694820218*^9}, {
   3.57578376026746*^9, 3.575783834413787*^9}, {3.575783897755729*^9, 
   3.5757839403920794`*^9}, 3.5757840477435503`*^9, {3.575784197935422*^9, 
   3.5757842001426735`*^9}, {3.5757842822069597`*^9, 
   3.5757842921682577`*^9}, {3.575784372334277*^9, 3.575784374060494*^9}, 
   3.5757844792627*^9, 3.5757846970540886`*^9, {3.575784753903186*^9, 
   3.575784755727415*^9}, {3.575784876103573*^9, 3.5757848946259127`*^9}, {
   3.5757851149035573`*^9, 3.5757851355661097`*^9}, {3.5757852224790473`*^9, 
   3.575785245069907*^9}, {3.5757852751986504`*^9, 3.5757853242038403`*^9}, {
   3.575785359230198*^9, 3.5757853664221015`*^9}, {3.5757856182497606`*^9, 
   3.575785701992276*^9}, {3.5757857510844545`*^9, 3.575785796816182*^9}, 
   3.5757858648617077`*^9, {3.5757859119336357`*^9, 3.5757859489412575`*^9}, {
   3.5757860112781277`*^9, 3.575786016981786*^9}, {3.5757861705491056`*^9, 
   3.5757861965923853`*^9}, {3.5757862383245926`*^9, 
   3.5757863285259056`*^9}, {3.575786485250588*^9, 3.575786516068463*^9}, {
   3.5757865717244434`*^9, 3.57578663551753*^9}, {3.5757867121290736`*^9, 
   3.5757867301703386`*^9}, {3.57578708757822*^9, 3.575787088890437*^9}, {
   3.575787147745799*^9, 3.5757871577080264`*^9}, {3.5757872168614936`*^9, 
   3.575787220030898*^9}, {3.575787331006783*^9, 3.575787332958085*^9}, {
   3.5757873862397194`*^9, 3.575787414644312*^9}, {3.5757874776812677`*^9, 
   3.5757875174242344`*^9}, {3.5757875501153502`*^9, 
   3.5757875678335247`*^9}, {3.5757929365199523`*^9, 
   3.5757929512068443`*^9}, {3.5757938038148804`*^9, 
   3.5757939441655335`*^9}, {3.5757940552374253`*^9, 3.57579405925996*^9}, {
   3.5758002894992933`*^9, 3.5758003048661757`*^9}, {3.575800601292389*^9, 
   3.5758006244333367`*^9}, 3.575800667180702*^9, {3.5758011914435053`*^9, 
   3.575801192018568*^9}, {3.5758542681613345`*^9, 3.575854295908818*^9}, {
   3.575854348989506*^9, 3.575854352004885*^9}, {3.5758578296693754`*^9, 
   3.5758578570528107`*^9}, {3.5758579915296946`*^9, 
   3.5758579955562415`*^9}, {3.575861713805101*^9, 3.5758617171755686`*^9}, {
   3.575861830763774*^9, 3.575861837766696*^9}, {3.5758653352258444`*^9, 
   3.575865340800477*^9}, {3.575865737578355*^9, 3.5758657642886686`*^9}, {
   3.575867316680621*^9, 3.5758673508969193`*^9}, {3.575867384058082*^9, 
   3.5758674363656473`*^9}, 3.5758708643010454`*^9, {3.575871099417591*^9, 
   3.575871099873654*^9}, {3.5758712474361873`*^9, 3.5758712514247065`*^9}, {
   3.575871650964901*^9, 3.5758716554124203`*^9}, {3.57587175502789*^9, 
   3.575871799291489*^9}, {3.5758718721386366`*^9, 3.5758718750480137`*^9}, {
   3.5758719170132265`*^9, 3.575871938743957*^9}, 3.5758720098068843`*^9, 
   3.575872113043844*^9, {3.5758721498214607`*^9, 3.5758721541600733`*^9}, {
   3.5758723382751455`*^9, 3.5758724120234385`*^9}, {3.5759471430160885`*^9, 
   3.575947160641301*^9}, {3.5759532831101503`*^9, 3.5759532854013653`*^9}, {
   3.5759533357067366`*^9, 3.575953350516569*^9}, {3.575953385072904*^9, 
   3.575953386799152*^9}, {3.5759534462816086`*^9, 3.5759534828922057`*^9}, {
   3.5759535834077854`*^9, 3.5759535951613193`*^9}, {3.5759536789538174`*^9, 
   3.575953680315005*^9}, {3.575953716321474*^9, 3.5759537442479925`*^9}, {
   3.5759540827725353`*^9, 3.5759540975143833`*^9}, {3.5759559810628996`*^9, 
   3.575956097975561*^9}, {3.5759561416230536`*^9, 3.5759562099345922`*^9}, {
   3.575956287927425*^9, 3.575956366222213*^9}, {3.575958997402666*^9, 
   3.5759590118844213`*^9}, {3.575959060996586*^9, 3.5759590914314556`*^9}, {
   3.5759605692990255`*^9, 3.5759605824956865`*^9}, {3.575967541781523*^9, 
   3.575967569592969*^9}, {3.5759677019326277`*^9, 3.5759677374930515`*^9}, {
   3.575967988652628*^9, 3.5759680394399652`*^9}, {3.575968183024032*^9, 
   3.5759682322862277`*^9}, {3.57596832781223*^9, 3.575968348346775*^9}, {
   3.575968535823338*^9, 3.5759685391827607`*^9}, 3.5759691733263674`*^9, 
   3.5759692136384587`*^9, {3.575969275542195*^9, 3.5759692837672653`*^9}, {
   3.5759694252509775`*^9, 3.575969438481638*^9}, {3.5759695576696444`*^9, 
   3.5759695603569975`*^9}, {3.575969703116871*^9, 3.57596971728269*^9}, {
   3.575969792099084*^9, 3.575969812161601*^9}, 3.5759698511494837`*^9, {
   3.575969946098431*^9, 3.575969986294485*^9}, {3.575970105166352*^9, 
   3.57597013721743*^9}, {3.575970169078377*^9, 3.575970175255153*^9}, {
   3.575970214190096*^9, 3.575970241405499*^9}, {3.57597031228636*^9, 
   3.5759703452205343`*^9}, {3.5759703937085867`*^9, 
   3.5759704105567017`*^9}, {3.5759704580096807`*^9, 
   3.5759704783812375`*^9}, {3.575970524842078*^9, 3.57597053176996*^9}, {
   3.57597058213035*^9, 3.5759706619853125`*^9}, {3.575970719203499*^9, 
   3.575970749235276*^9}, {3.575970871740689*^9, 3.5759709001762533`*^9}, {
   3.5759709502675314`*^9, 3.575970972060256*^9}, {3.575971110157545*^9, 
   3.5759711587276897`*^9}, {3.575972253082058*^9, 3.5759723437254815`*^9}, {
   3.575972400655589*^9, 3.5759724190669575`*^9}, {3.575972450607894*^9, 
   3.575972476537117*^9}, {3.575972541745346*^9, 3.5759725475050707`*^9}, {
   3.5759725788259964`*^9, 3.575972615570574*^9}, {3.5759726522902255`*^9, 
   3.575972681585907*^9}, {3.5759727745135765`*^9, 3.575972790066509*^9}, {
   3.575972827000123*^9, 3.5759728717997584`*^9}, {3.575972914484108*^9, 
   3.57597296040891*^9}, {3.5764005601278577`*^9, 3.57640056715374*^9}, {
   3.5764007587258005`*^9, 3.5764007859431906`*^9}, {3.576487440684676*^9, 
   3.5764874444121847`*^9}, 3.576488057159309*^9, 3.5764882036027746`*^9, 
   3.5766596369928613`*^9, {3.576659756884245*^9, 3.5766597571412773`*^9}, 
   3.5766597918474417`*^9, {3.576659999210323*^9, 3.5766601845805635`*^9}, 
   3.5766602319282446`*^9, {3.5766602767836275`*^9, 3.5766602834534254`*^9}, {
   3.5766603293879375`*^9, 3.5766603569912496`*^9}, {3.5766606061591473`*^9, 
   3.57666063393148*^9}, {3.576660903691848*^9, 3.576660906000123*^9}, {
   3.576661046363965*^9, 3.5766610465559874`*^9}, {3.576661182582308*^9, 
   3.5766611860157213`*^9}, {3.5766619794089174`*^9, 3.57666199443472*^9}, {
   3.5766622278087206`*^9, 3.576662234496524*^9}, {3.5766624101545997`*^9, 
   3.576662410369627*^9}, {3.5766624458608837`*^9, 3.576662448872244*^9}, {
   3.576662492049426*^9, 3.576662509544527*^9}, {3.576662626958613*^9, 
   3.576662630313018*^9}, {3.5766627665093565`*^9, 3.576662801579565*^9}, {
   3.5766628443676996`*^9, 3.576662878323775*^9}, {3.576662963340974*^9, 
   3.5766629672084384`*^9}, {3.5766631526756926`*^9, 3.576663253228757*^9}, 
   3.5766633152251964`*^9, {3.5766633974420614`*^9, 3.5766634160682955`*^9}, {
   3.5766634734321775`*^9, 3.576663576307521*^9}, {3.5766636401841865`*^9, 
   3.576663658332363*^9}, {3.5766638346415167`*^9, 3.576663837301838*^9}, {
   3.5766638817371693`*^9, 3.5766639557160463`*^9}, {3.576663989081048*^9, 
   3.576664039753129*^9}, {3.5766642004624104`*^9, 3.5766642335683837`*^9}, {
   3.5766642739332266`*^9, 3.576664284393482*^9}, {3.5766643569591875`*^9, 
   3.5766643594424863`*^9}, 3.576664399183256*^9, {3.5766644494682884`*^9, 
   3.57666445273368*^9}, {3.5766645911562886`*^9, 3.576664610458605*^9}, {
   3.5766646496773105`*^9, 3.576664669234658*^9}, {3.576664761498727*^9, 
   3.576664842063394*^9}, {3.576664951999585*^9, 3.5766649543908706`*^9}, {
   3.586580622391356*^9, 3.5865806271349*^9}, {3.5885762160698156`*^9, 
   3.588576216932927*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Usage: VectorAnalysis ", "Title",
 CellChangeTimes->{{3.5051211898468885`*^9, 3.505121198279371*^9}, 
   3.5054098225979652`*^9, {3.570780383275138*^9, 3.570780385673443*^9}, 
   3.575800011142474*^9, {3.588483998311614*^9, 3.588484002279169*^9}}],

Cell[BoxData[
 RowBox[{"HoldSubscripts", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5758007706156955`*^9, 3.5758007738781004`*^9}}],

Cell[CellGroupData[{

Cell["Example", "Subsection",
 CellChangeTimes->{{3.505121835526819*^9, 3.5051218371819143`*^9}, 
   3.5054098225989656`*^9, 3.575800011144286*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetCoordinates", "[", 
  RowBox[{"Cartesian", "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.505121735679108*^9, 3.505121737392206*^9}, {
   3.505121772161195*^9, 3.505121783472842*^9}, {3.505121919721635*^9, 
   3.5051219254169607`*^9}, 3.5054098225999656`*^9, 3.575800011145286*^9, 
   3.575800437694886*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Cartesian\"\>", ",", "3"}], "}"}]], "Output",
 CellChangeTimes->{
  3.5758004379509087`*^9, 3.5758005081596947`*^9, 3.575800542677038*^9, 
   3.575800699418748*^9, {3.57580076454593*^9, 3.5758007797308598`*^9}, 
   3.575800844066902*^9, 3.5758542573459888`*^9, 3.5758708262753477`*^9, 
   3.575947208436384*^9, 3.5759472992237053`*^9, 3.575947346171597*^9, 
   3.575947720498623*^9, 3.5759532273010693`*^9, 3.5759533655204997`*^9, 
   3.57595339977977*^9, {3.5759534627437077`*^9, 3.575953497875084*^9}, 
   3.5759536070457544`*^9, 3.575953697043105*^9, 3.588484019596352*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "Subscript", "]"}]], "Input",
 CellChangeTimes->{{3.5758010104518104`*^9, 3.5758010161485405`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Constant", ",", "HoldAllComplete", ",", "NHoldRest"}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.5758010164805555`*^9, 3.575854257362006*^9, 3.575870826294271*^9, 
   3.5759472084533377`*^9, 3.575947299250746*^9, 3.5759473461976004`*^9, 
   3.5759477205156364`*^9, 3.5759532273190975`*^9, 3.575953365539469*^9, 
   3.575953399798729*^9, {3.5759534627596664`*^9, 3.5759534978940516`*^9}, 
   3.575953607066871*^9, 3.575953697061096*^9, 3.588484019613351*^9}]
}, Open  ]],

Cell["Define vector", "Text",
 CellChangeTimes->{{3.505121825711258*^9, 3.505121827654369*^9}, 
   3.5054098226009655`*^9, 3.575800011145286*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["U", "x"], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{
      SubscriptBox["U", "y"], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{
      SubscriptBox["U", "z"], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.505121549872481*^9, 3.505121612906086*^9}, 
   3.505409822601966*^9, 3.575800011145286*^9}],

Cell["Calculate its gradient", "Text",
 CellChangeTimes->{{3.505121843183257*^9, 3.505121846414442*^9}, 
   3.505409822601966*^9, 3.575800011145286*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Grad", "[", "U", "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5051218118254633`*^9, 3.50512181455262*^9}, {
   3.505121931272296*^9, 3.5051219341204586`*^9}, 3.505409822601966*^9, 
   3.575800011145286*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "x"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "x"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "x"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]},
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "y"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "y"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "y"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]},
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "z"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "z"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "z"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5758002290746946`*^9, 3.575800418100417*^9, 3.5758005083997836`*^9, 
   3.575800543001109*^9, 3.5758007056785374`*^9, {3.575800764598899*^9, 
   3.5758007841253476`*^9}, 3.5758008440929103`*^9, 3.575854257554002*^9, 
   3.575870826393324*^9, 3.5759472084773245`*^9, 3.57594729928375*^9, 
   3.575947346231645*^9, 3.575947720563652*^9, 3.575953227396125*^9, 
   3.5759533655634727`*^9, 3.5759533998497767`*^9, {3.575953462782673*^9, 
   3.575953497955077*^9}, 3.5759536071167784`*^9, 3.575953697114069*^9, 
   3.588484019755392*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsection",
 CellChangeTimes->{{3.505121835526819*^9, 3.5051218371819143`*^9}, 
   3.5054098226059656`*^9, 3.5758000111472864`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetCoordinates", "[", 
   RowBox[{"Cylindrical", "[", 
    RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.505121735679108*^9, 3.505121737392206*^9}, {
   3.505121772161195*^9, 3.505121783472842*^9}, {3.505121919721635*^9, 
   3.5051219254169607`*^9}, {3.505121959185892*^9, 3.505121967209351*^9}, 
   3.5054098226059656`*^9, 3.5758000111472864`*^9, {3.575947577597694*^9, 
   3.575947578149768*^9}}],

Cell["Define vector", "Text",
 CellChangeTimes->{{3.505121825711258*^9, 3.505121827654369*^9}, 
   3.505409822606966*^9, 3.5758000111472864`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["U", "r"], "[", 
      RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], ",", 
     RowBox[{
      SubscriptBox["U", "\[Phi]"], "[", 
      RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], ",", 
     RowBox[{
      SubscriptBox["U", "z"], "[", 
      RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.505121549872481*^9, 3.505121612906086*^9}, {
   3.5051219793650465`*^9, 3.5051219925157986`*^9}, 3.505409822607966*^9, 
   3.5758000111472864`*^9, {3.575947582990389*^9, 3.5759475867258406`*^9}, {
   3.575947642830891*^9, 3.5759476439070234`*^9}}],

Cell["Calculate its gradient", "Text",
 CellChangeTimes->{{3.505121843183257*^9, 3.505121846414442*^9}, 
   3.505409822607966*^9, 3.5758000111472864`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Grad", "[", "U", "]"}], "//", "Expand"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5051218118254633`*^9, 3.50512181455262*^9}, {
   3.505121931272296*^9, 3.5051219341204586`*^9}, {3.5051220070976324`*^9, 
   3.5051220081206913`*^9}, 3.505409822607966*^9, 3.5707797312417884`*^9, 
   3.5758000111472864`*^9, 3.575947341935065*^9, {3.5759474564465003`*^9, 
   3.5759474580046415`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "r"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          SubscriptBox["U", "\[Phi]"], "[", 
          RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], "r"]}], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          SubscriptBox["U", "r"], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], "r"]}], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "r"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}]},
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "\[Phi]"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], 
      RowBox[{
       FractionBox[
        RowBox[{
         SubscriptBox["U", "r"], "[", 
         RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], "r"], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          SubscriptBox["U", "\[Phi]"], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], "r"]}], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "\[Phi]"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}]},
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "z"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         SubscriptBox["U", "z"], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}], "r"], 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "z"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Phi]", ",", "z"}], "]"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5758007646479006`*^9, 3.575800844125882*^9, 3.575854257610012*^9, 
   3.5758708264383087`*^9, 3.575947208525357*^9, 3.5759472993267603`*^9, 
   3.5759473462806144`*^9, 3.5759474584857016`*^9, 3.5759475940317464`*^9, 
   3.5759476478494773`*^9, 3.575947720610614*^9, 3.5759532274490876`*^9, 
   3.5759533655884333`*^9, 3.5759533998967733`*^9, {3.5759534628196993`*^9, 
   3.5759534980031013`*^9}, 3.575953607162812*^9, 3.5759536971621084`*^9, 
   3.5884840198273835`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsection",
 CellChangeTimes->{{3.505121835526819*^9, 3.5051218371819143`*^9}, 
   3.505409822611966*^9, 3.5758000111482863`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetCoordinates", "[", 
   RowBox[{"Spherical", "[", 
    RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.505121735679108*^9, 3.505121737392206*^9}, {
   3.505121772161195*^9, 3.505121783472842*^9}, {3.505121919721635*^9, 
   3.5051219254169607`*^9}, {3.505121959185892*^9, 3.505121967209351*^9}, {
   3.5051220569224825`*^9, 3.5051220852661037`*^9}, 3.5054098226129665`*^9, 
   3.5758000111482863`*^9}],

Cell["Define vector", "Text",
 CellChangeTimes->{{3.505121825711258*^9, 3.505121827654369*^9}, 
   3.5054098226129665`*^9, 3.5758000111482863`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"U", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["U", "r"], "[", 
      RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], ",", 
     RowBox[{
      SubscriptBox["U", "\[Theta]"], "[", 
      RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], ",", 
     RowBox[{
      SubscriptBox["U", "\[Phi]"], "[", 
      RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.505121549872481*^9, 3.505121612906086*^9}, {
   3.5051219793650465`*^9, 3.5051219925157986`*^9}, {3.505122088460286*^9, 
   3.5051221085554357`*^9}, 3.5054098226139665`*^9, 3.5758000111492863`*^9}],

Cell["Calculate its gradient", "Text",
 CellChangeTimes->{{3.505121843183257*^9, 3.505121846414442*^9}, 
   3.5054098226149664`*^9, 3.5758000111492863`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Grad", "[", "U", "]"}], "//", "Expand"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5051218118254633`*^9, 3.50512181455262*^9}, {
   3.505121931272296*^9, 3.5051219341204586`*^9}, {3.5051220070976324`*^9, 
   3.5051220081206913`*^9}, 3.5054098226149664`*^9, 3.5758000111492863`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "r"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          SubscriptBox["U", "\[Theta]"], "[", 
          RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], "r"]}], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          SubscriptBox["U", "r"], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], "r"]}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          SubscriptBox["U", "\[Phi]"], "[", 
          RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], "r"]}], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"Csc", "[", "\[Theta]", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           SubscriptBox["U", "r"], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}]}], "r"]}]},
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "\[Theta]"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], 
      RowBox[{
       FractionBox[
        RowBox[{
         SubscriptBox["U", "r"], "[", 
         RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], "r"], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          SubscriptBox["U", "\[Theta]"], 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], "r"]}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          RowBox[{"Cot", "[", "\[Theta]", "]"}], " ", 
          RowBox[{
           SubscriptBox["U", "\[Phi]"], "[", 
           RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}]}], "r"]}], 
       "+", 
       FractionBox[
        RowBox[{
         RowBox[{"Csc", "[", "\[Theta]", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           SubscriptBox["U", "\[Theta]"], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}]}], "r"]}]},
     {
      RowBox[{
       SuperscriptBox[
        SubscriptBox["U", "\[Phi]"], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         SubscriptBox["U", "\[Phi]"], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], "r"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SubscriptBox["U", "r"], "[", 
         RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], "r"], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"Cot", "[", "\[Theta]", "]"}], " ", 
         RowBox[{
          SubscriptBox["U", "\[Theta]"], "[", 
          RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}]}], "r"], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"Csc", "[", "\[Theta]", "]"}], " ", 
         RowBox[{
          SuperscriptBox[
           SubscriptBox["U", "\[Phi]"], 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"r", ",", "\[Theta]", ",", "\[Phi]"}], "]"}]}], "r"]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.57580076469195*^9, 3.575800844168911*^9, 
  3.5758542576540537`*^9, 3.575870826493329*^9, 3.575947208569371*^9, 
  3.575947299389779*^9, 3.575947346328618*^9, 3.575947720652623*^9, 
  3.575953227497094*^9, 3.5759533656164765`*^9, 3.5759533999467745`*^9, 
  3.5759534628596897`*^9, 3.575953498038065*^9, 3.5759536072058206`*^9, 
  3.575953697204132*^9, 3.5884840198723917`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{886, 723},
WindowMargins->{{Automatic, 149}, {Automatic, 32}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Report"}, "StandardReport.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 158, 2, 99, "Title"],
Cell[728, 26, 19064, 316, 741, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[19829, 347, 253, 3, 99, "Title"],
Cell[20085, 352, 132, 2, 40, "Input"],
Cell[CellGroupData[{
Cell[20242, 358, 147, 2, 37, "Subsection"],
Cell[CellGroupData[{
Cell[20414, 364, 377, 7, 40, "Input"],
Cell[20794, 373, 620, 10, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21451, 388, 141, 2, 40, "Input"],
Cell[21595, 392, 500, 9, 40, "Output"]
}, Open  ]],
Cell[22110, 404, 145, 2, 30, "Text"],
Cell[22258, 408, 522, 15, 40, "Input"],
Cell[22783, 425, 152, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[22960, 431, 257, 5, 40, "Input"],
Cell[23220, 438, 3594, 108, 77, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[26863, 552, 149, 2, 37, "Subsection"],
Cell[27015, 556, 480, 9, 40, "Input"],
Cell[27498, 567, 145, 2, 30, "Text"],
Cell[27646, 571, 701, 18, 40, "Input"],
Cell[28350, 591, 154, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[28529, 597, 446, 9, 40, "Input"],
Cell[28978, 608, 4069, 121, 124, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[33096, 735, 147, 2, 37, "Subsection"],
Cell[33246, 739, 492, 10, 40, "Input"],
Cell[33741, 751, 147, 2, 30, "Text"],
Cell[33891, 755, 679, 17, 40, "Input"],
Cell[34573, 774, 156, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[34754, 780, 344, 7, 40, "Input"],
Cell[35101, 789, 5338, 154, 181, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
